/***************************************************************************
                          capture.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef	__CAPTURE_H
#define	__CAPTURE_H


#include	"ksnuffle.h"

/*  Define's for the event filters. The first three are for the being,	*/
/*  end and freeze filters; the remaining ones are for the five ways	*/
/*  that the begin and end filters can be used.				*/

#define	TR_FILTER	(0x01)
#define	TR_START	(0x02)
#define	TR_STOP		(0x04)
#define	TR_FREEZE	(0x08)

#define	TR_ALL		(0x10)
#define	TR_TOSTOP	(0x20|TR_STOP )
#define	TR_FROMSTART	(0x30|TR_START)
#define	TR_ONESHOT	(0x40|TR_STOP|TR_START)
#define	TR_REPEAT	(0x50|TR_STOP|TR_START)


/*  Define's used when passing data between capture processes and the	*/
/*  main ksnuffle process. Most of these are actually only used with	*/
/*  remote sniffer.							*/

#define	RS_HELLO	('O')
#define	RS_STOP		('H')
#define	RS_IFACE	('I')
#define	RS_FILTER	('F')
#define	RS_BEGIN	('B')
#define	RS_END		('E')
#define	RS_MODE		('M')
#define	RS_SET		('S')
#define	RS_GO		('G')
#define	RS_QUERY	('Q')
#define	RS_PACKET	('P')
#define	RS_ERROR	('E')
#define	RS_TIME		('T')
#define	RS_SETERR	('s')
#define	RS_GOERR	('g')
#define	RS_STATE	('t')
#define	RS_FATAL	('f')
#define	RS_SNAP		('L')
#define	RS_PREFILTER	('p')
#define	RS_EVENT_0	('0')	/* ... RS_EVENT_5 = RS_EVENT_0 + 5	*/


/*  MState								*/
/*  Monitor state. displayed in the tool bar ...			*/

enum	MState
{
	MSUnset,			/* Configuration not yet set	*/
	MSSet,				/* Configuration has been set	*/
	MSCleared,			/* Configuration cleared	*/
	MSReset,			/* Configuration reset		*/
	MSWaiting,			/* Waiting for begin event	*/
	MSRunning,			/* Running and displaying	*/
	MSHalted,			/* Halted after end event	*/
	MSStopped,			/* Stopped			*/
	MSError,			/* Error occurred		*/
	MSClosed			/* Used when monitor is closed	*/
}	;

/*  CapBase								*/
/*  This class handles the bottom-level packet triggers and filtering.	*/
/*  It is separated out so that it can be used in both the main		*/
/*  ksnuffle program and in the remote sniffer rsnuffle by dint of the	*/
/*  CapLocal and CapRemote classes					*/

class	CapBase
{
public	:
	CapBase		*next	;

private	:
	NetPCap		*netpcap;	/* Network packet capture	*/
	int		mode	;	/* Trigger mode			*/
	BPFPROG	  	program	;	/* Monitor program		*/
	BPFPROG		start	;	/* Start trigger filter		*/
	BPFPROG		stop	;	/* Start trigger filter		*/

	struct
	{	bool	set	;
		BPFPROG	event	;
	}	events[NEVENT]	;

	int		minev	;	/* Minimum event number		*/
	int		maxev	;	/* Minimum event number		*/
	MState		state	;	/* Captured state		*/

	char		ebuf[PCAP_ERRBUF_SIZE] ;

	bool	CheckStart	(PktInfo *) ;
	bool	CheckStop	(PktInfo *) ;
	bool	CheckAccept	(PktInfo *) ;
	bool	CheckEvent	(PktInfo *, int) ;
	void	SendPktData	(PktInfo *, int) ;

	virtual	void	gotPacket (PktInfo *) = 0 ;
	virtual	void	gotState  (MState   ) = 0 ;

public	:

		 CapBase () ;
	virtual	~CapBase () ;

	bool	Build		(cchar *, int, TrigInf *, EventInf *, cchar *) ;
	bool	Execute		()	    ;
	void	Callback	(PktInfo *) ;
	void	Finish		()	    ;
	BPFI32	getNMask 	() ;
	int	getLType 	() ;


	inline	cchar	*getError   () { return ebuf    ; } ;
	inline	MState	 getState   () { return state   ; } ;
	inline	NetPCap *getNetPCap () { return netpcap ; } ;
}	;


/*  CapLocal								*/
/*  Derived CapBase class for local packet capture			*/

class	CapLocal : public CapBase
{
	KNDPCapLoc	*pcaploc ;

	virtual	void	gotPacket	(PktInfo *)	;
	virtual	void	gotState	(MState   )	;

public:

	CapLocal (KNDPCapLoc *) ;
       ~CapLocal () ;
}	;

/*  CapRemote								*/
/*  Derived CapBase class for local packet capture			*/

class	CapRemote : public CapBase
{
	int	sock	;

	virtual	void	gotPacket	(PktInfo *)	;
	virtual	void	gotState	(MState   )	;

public:

	CapRemote (int) ;
       ~CapRemote () ;
}	;

/*  CapCompile compiles a filter program. It is defined outside of the	*/
/*  "Capture" class in order that it can be used to verify a filter or	*/
/*  event trigger.							*/
extern	bool	CapCompile	(pcap_t	*, BPFPROG *, const char *, int, ulong) ;

#endif	/* __CAPTURE_H */
