/***************************************************************************
                          dec_ip.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<stdlib.h>
#include	<unistd.h>
#include	<fcntl.h>

#include	"knd_pcap.h"
#include	"nethdrs.h"



#define	TCPHSIZ	919
static	TCPHASH	*tcp_seq_hash[TCPHSIZ]	;
extern	bool	vonly			;


/*  putchar	: Put character into buffer				*/
/*  into	: char *&	: Buffer pointer			*/
/*  ilen	: int &		: Buffer space left			*/
/*  ch		: char		: Character in question			*/
/*  (returns)	: void		:					*/

inline	void	putchar
	(	char	*&info,
		int	&ilen,
		int	ch
	)
{
	if (ilen > 2)
	{
		*info++	 = ch	;
		ilen	-= 1	;
	}
}

/*  putchar	: Put string into buffer				*/
/*  into	: char *&	: Buffer pointer			*/
/*  ilen	: int &		: Buffer space left			*/
/*  str		: char *	: String in question			*/
/*  (returns)	: void		:					*/

inline	void	putstr
	(	char	*&info,
		int	&ilen,
		char	*str
	)
{
	int	slen	= strlen (str) ;

	if (slen > ilen - 2) slen = ilen - 2 ;
	if (slen <= 0) return ;

	memcpy	(info, str, slen) ;
	info	+= slen	;
	ilen	-= slen	;
}


/*  TCPInfo	: Extract TCP/IP information				*/
/*  ip		: IP *		: IP header address			*/
/*  tcp		: TCP_HDR *	: TCP header address			*/
/*  tlen	: int		: TCP data length			*/
/*  pktinfo	: PktInfo *	: Packet information			*/
/*  (returns)	: TCPHASH *	: TCP conversation			*/

static	TCPHASH	*TCPInfo
	(	IP	*ip,
		TCP_HDR	*tcp,
		int	tlen,
		PktInfo	*pktinfo
	)
{
	int	flags	= tcp->th_flags	      ;
	ulong	seq	= ntohl (tcp->th_seq) ;
	ulong	ack	= ntohl (tcp->th_ack) ;
	ushort	win	= ntohs (tcp->th_win) ;
	ushort	urp	= ntohs (tcp->th_urp) ;
	TCPHASH	*thp	= NULL		      ;
	int	hlen	;
	//ulong	thseq	;
	//ulong	thack	;
	char	tb[128]	;

	char	*info	= pktinfo->info	;
	int	ilen	= sizeof(pktinfo->info) ;

	if ((flags & (TH_SYN|TH_FIN|TH_RST|TH_PUSH)) != 0)
	{	if ((flags & TH_SYN ) != 0) putchar (info, ilen,  'S') ;
		if ((flags & TH_FIN ) != 0) putchar (info, ilen,  'F') ;
		if ((flags & TH_RST ) != 0) putchar (info, ilen,  'R') ;
		if ((flags & TH_PUSH) != 0) putchar (info, ilen,  'P') ;
	}
	else	putchar	(info, ilen, '.') ;

	putchar	(info, ilen, ' ') ;

	ulong	saddr	= *(ulong *)(&ip->ip_src) ;
	ulong	daddr	= *(ulong *)(&ip->ip_dst) ;
	ulong	sport	= tcp->th_sport ;
	ulong	dport	= tcp->th_dport ;
	int	hashval	;
	TCPADDR	tcpaddr	;
	bool	rev	;


	/* The source and destination are arbitrarily ordered so that	*/
	/* we get a single entry for each TCP/IP stream that passes our	*/
	/* way.								*/
	if ((sport < dport) || ((sport == dport) && (saddr < daddr)))
	{
		tcpaddr.src	= saddr	;
		tcpaddr.dst	= daddr	;
		tcpaddr.portp	= (sport << 16) | dport ;
		rev		= false	;
	}
	else
	{	tcpaddr.src	= daddr	;
		tcpaddr.dst	= saddr	;
		tcpaddr.portp	= (dport << 16) | sport ;
		rev		= true	;
	}

	hashval	= tcpaddr.portp % TCPHSIZ ;

	/* Scan the hash table to see if we already know about this	*/
	/* stream. If not then create a new entry.			*/
	for (thp = tcp_seq_hash[hashval] ; thp != NULL ; thp = thp->next)
		if (memcmp (&thp->tcpaddr, &tcpaddr, sizeof(TCPADDR)) == 0)
			break	;

	if (thp == NULL)
	{
		thp = (TCPHASH *)calloc(1, sizeof(TCPHASH)) ;
		thp->next = tcp_seq_hash[hashval] ;
		tcp_seq_hash[hashval] = thp	  ;
		memcpy	   (&thp->tcpaddr, &tcpaddr, sizeof(TCPADDR)) ;
		setMemUsed (sizeof(TCPHASH))	  ;
	}

	/* If this is the SYN-ACK packet at the start of a new TCP/IP	*/
	/* connection then fill in the initial details. The "memcpy" is	*/
	/* present just in case we re-use an entry.			*/
	if ((flags & (TH_ACK|TH_SYN)) == (TH_ACK|TH_SYN))
	{
		memcpy	(&thp->tcpaddr, &tcpaddr, sizeof(TCPADDR)) ;

		/* Note the initial sequence and acknowledgment values	*/
		/* (which depends on whether we have a reversed		*/
		/* ordering).						*/
		if (rev)
			thp->ack = seq, thp->seq = ack - 1 ;
		else	thp->seq = seq, thp->ack = ack - 1 ;
	}

	/* Adjust the sequence and acknowledgement numbers by the	*/
	/* stored base values (again, depends on the ordering).		*/
	if (rev)
		seq -= thp->ack, ack -= thp->seq ;
	else	seq -= thp->seq, ack -= thp->ack ;

	//thseq	= thp->seq ;
	//thack	= thp->ack ;

	pktinfo->rev = rev ;

	hlen	 = tcp->th_off * 4 ;
	tlen	-= hlen	;

	if ((tlen > 0) || ((flags & (TH_SYN|TH_FIN|TH_RST)) != 0))
	{	sprintf	(tb, " %lu:%lu(%d)", seq, seq + tlen, tlen) ;
		putstr	(info, ilen, tb) ;
	}
	if ((flags & TH_ACK) != 0)
	{	sprintf	(tb, " ack %lu", ack) ;
		putstr	(info, ilen, tb) ;
	}

	sprintf	(tb, " win %hu", win) ;
	putstr	(info, ilen, tb) ;
	
	if ((flags & TH_URG) != 0)
	{	sprintf	(tb, " urg %hu", urp) ;
		putstr	(info, ilen, tb) ;
	}

	*info	= 0 	;
	return	thp	;
}


/*  pkt_ip	: Decode in internet (IP) packet			*/
/*  iface	: cchar *	: Interface name			*/
/*  pktinfo	: PktInfo *	: Results structure			*/
/*  _p		: char *	: Actual packet data			*/
/*  (returns)	: void		:					*/

void	pkt_ip
	(	cchar	*iface,
		PktInfo	*pktinfo,
		char	*_p
	)
{
	IP	*ip	= (IP *)_p ;
	char	*appd	= NULL	   ;

	int	off	;
	int	hlen	;
	int	dlen	;
	char	*data	;

	TCP_HDR	*tcp	;
	UDP_HDR	*udp	;

	hlen	= ip->ip_hl * 4      ;	/* IP header length		*/
	dlen	= ntohs (ip->ip_len) ;	/* Total datagram length ...	*/
	dlen   -= hlen		     ;	/* ... hence IP data length	*/
	off	= ntohs (ip->ip_off) ;	/* Fragment offset and flags	*/
	data	= (char *)ip + hlen  ;	/* Start of data		*/

	pktinfo->ip	= ip	     ;
	pktinfo->ipdata	= NULL	     ;

	/* If the low-order 13 bits of the fragment offset are all	*/
	/* zero then this is the first fragmment, so we have all the	*/
	/* information needed.						*/
	if ((off & 0x1fff) == 0)
	{
		pktinfo->datlen	 = dlen	 ;

		sprintf	(pktinfo->size, "%4d,%4d",
					pktinfo->piHdr.pktlen,
					pktinfo->datlen) ;

		switch (ip->ip_p)
		{
			case IPPROTO_TCP  :
				tcp  = (TCP_HDR *)(pktinfo->ipdata = data) ;
				appd = (char *)tcp + tcp->th_off * 4	   ;	
				strcpy	   (pktinfo->proto, "tcp") ;
				FindEPoint ((char *)&ip->ip_src, ntohs (tcp->th_sport), &pktinfo->srce) ;
				FindEPoint ((char *)&ip->ip_dst, ntohs (tcp->th_dport), &pktinfo->dest) ;
				pktinfo->tcphash = TCPInfo (ip, tcp, dlen, pktinfo) ;
				break	;

			case IPPROTO_UDP  :
				udp  = (UDP_HDR *)(pktinfo->ipdata = data) ;
				appd = (char *)udp + sizeof(UDP_HDR)	   ;
				strcpy	   (pktinfo->proto, "udp") ;
				FindEPoint ((char *)&ip->ip_src, ntohs (udp->uh_sport), &pktinfo->srce) ;
				FindEPoint ((char *)&ip->ip_dst, ntohs (udp->uh_dport), &pktinfo->dest) ;
				break	;

			case IPPROTO_ICMP :
				pktinfo->ipdata = data ;
				strcpy	   (pktinfo->proto, "icmp") ;
				FindEPoint ((char *)&ip->ip_src, -1, &pktinfo->srce) ;
				FindEPoint ((char *)&ip->ip_dst, -1, &pktinfo->dest) ;
				break	;

			default :
				appd	= (char *)ip ;
				sprintf	(pktinfo->proto, "proto %d", ip->ip_p) ;
				break	;
		}
	}
	else	appd	= (char *)ip	;

	/* If either the fragments-follow bit 0x2000 is set, or the	*/
	/* flagment offset is non-zero then generate some information	*/
	/* about the flagment.						*/
	/* data). Make do with the IP address.				*/
	if ((off & 0x3fff) != 0)
	{
		sprintf	(pktinfo->info, "%d=%d@%d%s",
				ntohs (ip->ip_id),
				dlen,
				(off & 0x1fff) * 8,
				(off & IP_MF ) ? "+" : "") ;
		assert	(strlen(pktinfo->info) < sizeof(pktinfo->info)) ;
	}

	/* If the flagment offset is non-zero then we do not have the	*/
	/* next-level protocol header (ie., the header at the start of	*/
	/* the IP data) so make do with the addresses.			*/
	if ((off & 0x1fff) != 0)
	{
		strcpy	   (pktinfo->proto, "frag") ;
		FindEPoint ((char *)&ip->ip_src, -1, &pktinfo->srce) ;
		FindEPoint ((char *)&ip->ip_dst, -1, &pktinfo->dest) ;
	}

	/* If the user has view-only privileges and we have a pointer	*/
	/* at application data, then the application data is zapped to	*/
	/* all-ones.							*/
	if (vonly && (appd != NULL))
		memset	(appd, 0xff, sizeof(pktinfo->data) - (appd - pktinfo->data)) ;
}

/*  pkt_arp	: Decode in (R)ARP packet				*/
/*  iface	: cchar *	: Interface name			*/
/*  pktinfo	: PktInfo *	: Results structure			*/
/*  _p		: char *	: Packet data				*/
/*  (returns)	: void		:					*/

void	pkt_arp
	(	cchar	*iface,
		PktInfo	*pktinfo,
		char	*_p
	)
{
#define	REVARP_REQUEST	(3)
#define	REVARP_REPLY	(4)

	ETH_ARP	*earp	= (ETH_ARP *)_p ;
	u_short	pro	;
	u_short	hrd	;
	u_short	op	;

	strcpy	(pktinfo->proto, "(r)arp") ;

	/* Extract:							*/
	/*	pro	- Protocol requesting resolution		*/
	/*	hrd	- Hardware type					*/
	/*	op	- Operation code				*/
	pro	= (u_short)ntohs(*(u_short *)&earp->arp_pro) ;
	hrd	= (u_short)ntohs(*(u_short *)&earp->arp_hrd) ;
	op	= (u_short)ntohs(*(u_short *)&earp->arp_op ) ;

	/* Check that we know about the requesting protocol, and that	*/
	/* the hardware and protocol address lengths are as expected	*/
	/* for an ethernet.						*/
	if ( ((pro != ETHERTYPE_IP) && (pro != ETHERTYPE_TRAIL)) ||
	     (earp->arp_hln != sizeof(earp->arp_sha)) ||
	     (earp->arp_pln != sizeof(earp->arp_spa)) )
	{
		return	;
	}

	pktinfo->arp	= earp ;

	switch (op)
	{
		case ARPOP_REQUEST  :
			/* ARP Request: put the requestor in the source	*/
			/* field and the reuested target in the		*/
			/* destination field.				*/
			strcpy	   (pktinfo->proto, "arp who-has") ;
			FindEPoint ((char *)&earp->arp_spa, -1, &pktinfo->srce) ;
			FindEPoint ((char *)&earp->arp_tpa, -1, &pktinfo->dest) ;
			break	;

		case ARPOP_REPLY    :
			/* ARP Reply: put the sender in the source	*/
			/* field.					*/
			strcpy	   (pktinfo->proto, "arp is-at") ;
			FindEPoint ((char *)&earp->arp_spa, -1, &pktinfo->srce) ;
			FindEPoint ((char *)&earp->arp_tpa, -1, &pktinfo->dest) ;
			break	;

		case REVARP_REQUEST :
			strcpy	   (pktinfo->proto, "rarp who-is") ;
			break	;

		case REVARP_REPLY   :
			strcpy	   (pktinfo->proto, "rarp reply") ;
			break	;

		default	:
			sprintf	   (pktinfo->proto, "(r)arp %d", op) ;
			break	;
	}
}

