/***************************************************************************
                          dec_ipx.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<stdlib.h>
#include	<unistd.h>
#include	<fcntl.h>

#include	"ksnuffle.h"
#include	"netpcap.h"
#include	"capture.h"
#include	"nethdrs.h"

/*  Ethernet (MAC) -> IP address mapping				*/
/*  ------------------------------------				*/

/*  The code below is used to map 6-byte ethernet addresses to the	*/
/*  corresponding IP name or address. The hash table is loaded with	*/
/*  information for local interfaces from within FindAllIface, and	*/
/*  about interfaces on other machines from the ARP cache. Note that	*/
/*  this code will most likely need modifying for other interfaces.	*/

typedef	struct	_etaddr	EthAddr	;

struct	_etaddr
{	EthAddr	*next	  ;		/* Next on hash queue		*/
	uchar	etaddr[6] ;		/* Ethernet MAC address		*/
	char	ipaddr[4] ;		/* IP address			*/
}	;

#define	EHSIZE	256
static	EthAddr	*ehashtab[EHSIZE] ;

/*  EthHash	: Compute hash value for Ethernet MAC address		*/
/*  etaddr	: uchar *	: Ethernet MAC address			*/
/*  (returns)	: int		: Hash value				*/

inline	int	EthHash
	(	uchar	*etaddr
	)
{
	return	( etaddr[0] ^ etaddr[1] ^ etaddr[2] ^
		  etaddr[3] ^ etaddr[4] ^ etaddr[5] ) & (EHSIZE - 1) ;
}

/*  EthToIP	: Convert Ethernet MAC address to IP name or address	*/
/*  etaddr	: uchar *	: Ethernet MAC address			*/
/*  ipaddr	: char *	: Numerical IP address if known or NULL	*/
/*  (returns)	: char  *	: IP address				*/

char	*EthToIP
	(	uchar	*etaddr,
		char	*ipaddr
	)
{
	/* NOTE: This routine is used both to look up values and to	*/
	/*	 load values into the hash table, the latter where the	*/
	/*	 second argument is non-NULL.				*/
	int	hval	= EthHash (etaddr) ;
	EthAddr	*eap	;

	for (eap = ehashtab[hval] ; eap != NULL ; eap = eap->next)
		if (memcmp (etaddr, eap->etaddr, 6) == 0)
		{	if (ipaddr != NULL) memcpy (eap->ipaddr, ipaddr, 4) ;
			return	eap->ipaddr ;
		}

	eap		= new EthAddr	;
	eap->next	= ehashtab[hval] ;
	ehashtab[hval]	= eap		 ;

	memcpy	(eap->etaddr, etaddr, 6) ;

	if (ipaddr == NULL)
		memset	(eap->ipaddr, 0,      4) ;
	else	memcpy	(eap->ipaddr, ipaddr, 4) ;

	return	eap->ipaddr ;
}

/*  LoadArpCache: Load information from ARP cache			*/
/*  (returns)	: void		:					*/

void	LoadArpCache ()
{
	char	buff[256] ;
	FILE	*fd	  ;

	/* Open the ARP cache pseudo-file. Note that this only works	*/
	/* under Linux with the /proc file system loaded.		*/
	if ((fd = fopen ("/proc/net/arp", "r")) == NULL)
		return	;

	/* Skip the first line, which just contains column headings,	*/
	/* then process the real stuff.					*/
	fgets	(buff, sizeof(buff), fd) ;

	while (fgets (buff, sizeof(buff), fd) != NULL)
	{
		int	ip0, ip1, ip2, ip3 ;
		int	et0, et1, et2, et3, et4, et5 ;

		if (sscanf (buff, "%d.%d.%d.%d 0x%*d 0x%*d %x:%x:%x:%x:%x:%x",
				  &ip0, &ip1, &ip2, &ip3,
				  &et0, &et1, &et2, &et3, &et4, &et5) != 10)
			continue  ;

		char	ipaddr	[4] ;
		uchar	etaddr	[6] ;

		ipaddr[0] = ip0 ; ipaddr[1] = ip1 ;
		ipaddr[2] = ip2 ; ipaddr[3] = ip3 ;
		etaddr[0] = et0 ; etaddr[1] = et1 ;
		etaddr[2] = et2 ; etaddr[3] = et3 ;
		etaddr[4] = et4 ; etaddr[5] = et5 ;

		EthToIP (etaddr, ipaddr) ;
	}

	fclose	(fd) ;
}



/*  IPX_HDR								*/
/*  Structure to define an IPX header. Basically copied from the	*/
/*  tcpdump code.							*/

typedef	struct
{	ushort	cksum	  ;		/* Checksum			*/
	ushort	length	  ;		/* Length including header	*/
	uchar	tCtl	  ;		/* Transport Control		*/
	uchar	pType	  ;		/* Packet Type			*/
	ushort	dstNet [2];		/* destination net		*/
	uchar	dstNode[6];		/* destination node		*/
	ushort	dstSkt	  ;		/* destination socket		*/
	ushort	srcNet [2];		/* source net			*/
	uchar	srcNode[6];		/* source node 			*/
	ushort	srcSkt	  ;		/* source socket		*/
}	IPX_HDR		  ;

typedef	struct	ether_header	ETH_HDR	;

#if	0
static	char	*IpxNetStr
	(	ushort	*net
	)
{
	static	char	buff[10]  ;
	sprintf	(buff, "%08x.", ntohl (*(ulong *)net)) ;
	return	buff	  ;
}
#endif
#if	0
static	char	*IpxSktStr
	(	ushort	socket
	)
{
	static	char	buff[10]  ;
	sprintf	(buff, ".%04x", ntohs (socket)) ;
	return	buff	  ;
}
#endif

/*  pkt_ipx	: Decode an IPX packet					*/
/*  iface	: cchar *	: Interface name			*/
/*  pktinfo	: PktInfo *	: Results structure			*/
/*  _p		: char *	: Packet data				*/
/*  (returns)	: void		:					*/

void	pkt_ipx
	(	cchar	*iface,
		PktInfo	*pktinfo,
		char	*_p
	)
{
	ETH_HDR	*eth	= (ETH_HDR *)(&pktinfo->data[0]) ;

	strcpy	(pktinfo->proto, "ipx")   ;

	FindEPoint (EthToIP ((uchar *)&eth->ether_shost), -1, &pktinfo->srce) ;
	FindEPoint (EthToIP ((uchar *)&eth->ether_dhost), -1, &pktinfo->srce) ;

	pktinfo->srce.p = "Socket"	  ;
	pktinfo->srce.s = pktinfo->srce.p ;
	pktinfo->dest.p = "Socket"	  ;
	pktinfo->dest.s = pktinfo->dest.p ;
	pktinfo->datlen = pktinfo->piHdr.pktlen - sizeof(IPX_HDR) ;

	sprintf	(pktinfo->size, "%4d,%4d", pktinfo->piHdr.pktlen, pktinfo->datlen) ;
}

