/***************************************************************************
                          dec_net.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<stdlib.h>
#include	<unistd.h>
#include	<fcntl.h>

#include	"knd_pcap.h"
#include	"nethdrs.h"


/*  External routines for packet decoding according to packet type.	*/
/*									*/
/*  NOTE: The PktInfo structure contains the packet data 'off the wire'	*/
/*	  while the third argument points at the start of the real data	*/
/*	  _after_ any encapsulation has been stripped. It _must_ point	*/
/*	  into the data copied into the PktInfo structure, since the	*/
/*	  decoders will set protocol pointers based on the third	*/
/*	  argument.							*/

extern	void	pkt_ipx	(cchar *, PktInfo *, char *) ;
extern	void	pkt_ip	(cchar *, PktInfo *, char *) ;
extern	void	pkt_arp	(cchar *, PktInfo *, char *) ;


typedef	struct
{	uchar	dsap	;
	uchar	ssap	;
	union
	{	uchar	u_ctl	;
		ushort	is_ctl	;
		struct
		{	uchar	snap_ui	   ;
			uchar	snap_pi [5];
		}	snap	;
		struct
		{	uchar	snap_ui	   ;
			uchar	snap_org[3];
			uchar	snap_ety[2];
		}	snap_e	;
	}	;
}	LLC	;	

#define	LLCSAP_GLOBAL	0xff


/*  llc_decode	: Decode logical link control layer			*/
/*  iface	: cchar *	: Interface name			*/
/*  pktinfo	: PktInfo *	: Results structure			*/
/*  _p		: char *	: Packet data				*/
/*  (returns)	: bool		: True if decoded			*/

static	bool	llc_decode
	(	cchar	*iface,
		PktInfo	*pktinfo,
		char	*_p
	)
{
	LLC	*llc	= (LLC *)_p ;

	if ((llc->ssap == LLCSAP_GLOBAL) && (llc->dsap == LLCSAP_GLOBAL))
	{	pkt_ipx	(iface, pktinfo, _p) ;
		return	true ;
	}

	return	false	;
}

/*  SetTime	: Set time as string					*/
/*  pktinfo	: PktInfo *	: Packet information structure		*/
/*  (returns)	: void		:					*/

static	void	SetTime
	(	PktInfo	*pktinfo
	)
{
	int	hours	;
	int	mins	;
	int	secs	;

	/* Slave the last decoded second. If stuff is coming thick and	*/
	/* fast, this might save a bit. Note that when using threads	*/
	/* this is the main thread, so there is no problem with static	*/
	/* data.							*/
	static	int	lasts	  ;
	static	char	lastt[20] ;

	if (pktinfo->piHdr.tval.tv_usec != lasts)
	{
		lasts	= pktinfo->piHdr.tval.tv_sec	;
		secs	= pktinfo->piHdr.tval.tv_sec	;
		secs   %= 3600  *   24 	  	;
		hours	= secs  / 3600		;
		secs   -= hours * 3600		;
		mins	= secs	/   60		;
		secs   -= mins  *   60  	;

		sprintf	(lastt,  "%02d:%02d:%02d",  hours, mins, secs) ;
	}

	sprintf	(pktinfo->time, "%s.%06ld", lastt, pktinfo->piHdr.tval.tv_usec) ;
}

/*  EthDecode	: Ethernet packet decode main routine			*/
/*  iface	: cchar *	: Interface name			*/
/*  pktinfo	: PktInfo *	: Results structure			*/
/*  (returns)	: void		:					*/

static	void	EthDecode
	(	cchar	*iface,
		PktInfo	*pktinfo
	)
{
	ETH_HDR	*ehdr	;
	char	*data	;
	ushort	etype	;

	if (pktinfo->decoded) return ;

	SetTime	(pktinfo) ;

	ehdr	= (ETH_HDR *)(&pktinfo->data[0]) ;
	data	= (char    *)(&pktinfo->data[sizeof(ETH_HDR)]) ;
	etype	= ntohs (ehdr->ether_type) ;

	if (etype <= ETHERMTU)
	{
		if (llc_decode (iface, pktinfo, data))
		{	pktinfo->decoded = true ;
			return ;
		}
	}

	switch (etype)
	{
		case ETHERTYPE_IP 	:
			pkt_ip	(iface, pktinfo, data) ;
			break	;

		case ETHERTYPE_ARP	:
		case ETHERTYPE_REVARP	:
			pkt_arp	(iface, pktinfo, data) ;
			break	;

		default	:
			sprintf	(pktinfo->proto, "et %d", ntohs(ehdr->ether_type)) ;
			break	;
	}

	pktinfo->decoded = true ;
}

/*  RawDecode	: Raw socket packet decode main routine			*/
/*  iface	: cchar *	: Interface name			*/
/*  pktinfo	: PktInfo *	: Results structure			*/
/*  (returns)	: void		:					*/

static	void	RawDecode
	(	cchar	*iface,
		PktInfo	*pktinfo
	)
{
	if (!pktinfo->decoded)
	{
		SetTime	(pktinfo) ;
		pkt_ip	(iface,   pktinfo, (char *)&pktinfo->data[0]) ;
		pktinfo->decoded = true ;
	}
}

/*  NullDecode	: Null device packet decode main routine		*/
/*  iface	: cchar *	: Interface name			*/
/*  pktinfo	: PktInfo *	: Results structure			*/
/*  (returns)	: void		:					*/

static	void	NullDecode
	(	cchar	*iface,
		PktInfo	*pktinfo
	)
{
	if (!pktinfo->decoded)
	{
		SetTime	(pktinfo) ;
		pkt_ip	(iface,   pktinfo, (char *)(&pktinfo->data[4])) ;
		pktinfo->decoded = true ;
	}
}

/*  getDecoder	: Get packet decoder for interface			*/
/*  ltype	: int		: Interface type			*/
/*  (returns)	: DECODER *	: Decoder routine			*/

DECODER	*getDecoder
	(	int	ltype
	)
{
	char	*name	;

#if	1
	switch (ltype)
	{
		case DLT_EN10MB		:
		case DLT_IEEE802	: name	= "EthDecode"	; break ;
		case DLT_NULL		: name	= "NullDecode"	; break ;
		case DLT_RAW		: name	= "RawDecode"	; break ;
		default			: name	= "UnknownIF"	; break ;
	}
	fprintf	(stderr, "ksnuffle: getDecoder: %d->%s\n", ltype, name) ;
#endif

	switch (ltype)
	{
		case DLT_EN10MB		:
		case DLT_IEEE802	: return EthDecode	;
		case DLT_NULL		: return NullDecode	;
		case DLT_RAW		: return RawDecode	;
#ifdef	_NOTYET
		case DLT_SLIP		: return SlipDecode	;
		case DLT_SLIP_BSDOS	: return SlipBsdosDecode;
		case DLT_PPP_BSDOS	: return PPPBsdosDecode	;
		case DLT_FDDI		: return FDDIDecode	;
		case DLT_ATM_RFC1483	: return ATMDecode	;
#endif	/* _NOTYET */
		default			: break	 ;
	}

	return	NULL	;
}
