/***************************************************************************
                          knd_config.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<qstrlist.h>
#include	<qtooltip.h>

#include	"knd_config.h"
#include	"knd_view.h"


/*  KNDConf								*/
/*  KNDConf	: Constructor for configuration and control object	*/
/*  _parent	: KNDView *	: Parent object				*/
/*  trigger	: KNDTrigger *	: Logging trigger object		*/
/*  graphic	: KNDGraphic *	: Associated graphic display object	*/
/*  packets	: KNDpacket *	: Associated packet display object	*/
/*  multi	: KNDMulti *	: Multiple graph object			*/
/*  slot	: int		: Unique slot number			*/
/*  name	: char *	: Initial monitor name			*/
/*  (returns)	: KNDConf	:					*/

KNDConf::KNDConf
	(	KNDView		*_parent,
		KNDTrigger	*trigger,
		int		slot,
		const char	*name
	)
	:
	KNDPlugin	(_parent, "KND_CONFIG"),
	trigger		(trigger),
	slot		(slot),
	g_log		(this),	g_pkt	(this), g_gra	(this),
	b_log		(this),
	l_name		(this),	t_name	(this),
	c_time		(this),	c_proto	(this),
	c_srce		(this), c_dest	(this),
	c_info		(this), c_size	(this),
	c_multi		(this),
	l_load1		(this),	l_bar1	(this),	l_bar2	(this),
	s_load1		(this),	s_bar1	(this),	s_bar2	(this),
	g_plug		(this),
	b_aplug		(this),
	b_dplug		(this),
	gotplug		(true)
{
	l_name .setGeometry       (  5,  25,  40,  25) ;
	l_name .setText	          ("Name" ) ;
	t_name .setGeometry	  ( 75,  25, 100,  25) ;
	t_name .setText		  (name	  ) ;
	v_name	= name	    	  ;
	QToolTip::add (&t_name, i18n ("Monitor name")) ;

	g_pkt  .setGeometry	  (195, 120, 140, 115) ;
	g_pkt  .setTitle	  ("Packet monitor") ;

	c_time .setGeometry	  (200, 140,  50,  25) ;
	c_time .setText		  ("Time" ) ;
	c_time .setChecked	  (true   ) ;
	d_time 	= true	;
	QToolTip::add (&c_time, i18n ("Show time in packet display")) ;

	c_proto.setGeometry	  (275, 140,  50,  25) ;
	c_proto.setText		  ("Proto") ;
	c_proto.setChecked	  (true   ) ;
	d_proto	= true	;
	QToolTip::add (&c_proto, i18n ("Show protocol in packet display")) ;

	c_srce.setGeometry	  (200, 170,  50,  25) ;
	c_srce.setText		  ("Srce" ) ;
	c_srce.setChecked	  (true   ) ;
	d_srce	= true	;
	QToolTip::add (&c_srce, i18n ("Show source in packet display")) ;

	c_dest.setGeometry	  (275, 170,  50,  25) ;
	c_dest.setText		  ("Dest" ) ;
	c_dest.setChecked	  (true   ) ;
	d_dest	= true	;
	QToolTip::add (&c_dest, i18n ("Show destination in packet display")) ;

	c_info .setGeometry	  (200, 200,  50,  25) ;
	c_info .setText		  ("Info" ) ;
	c_info .setChecked	  (true   ) ;
	d_info = true	;
	QToolTip::add (&c_info, i18n ("Show other information in packet display")) ;

	c_size .setGeometry	  (275, 200,  50,  25) ;
	c_size .setText		  ("Size" ) ;
	c_size .setChecked	  (true   ) ;
	d_size	= true	;
	QToolTip::add (&c_size, i18n ("Show size in packet display")) ;

	g_gra  .setGeometry	  (340, 120, 130, 115) ;
	g_gra  .setTitle	  ("Graphic display") ;

	l_load1.setGeometry	  (345, 140,  60,  25) ;
	l_load1.setText		  ("Histogram") ;
	s_load1.setGeometry	  (415, 140,  50,  25) ;
	s_load1.setRange	  ( 1,   50) ;
	s_load1.setValue	  (genconf.v_load = 1) ;
	QToolTip::add (&s_load1, i18n ("Set update interval for graphic load histogram")) ;

	l_bar1 .setGeometry	  (345, 170,  60,  25) ;
	l_bar1 .setText		  ("Bar 1") ;
	s_bar1 .setGeometry	  (415, 170,  50,  25) ;
	s_bar1 .setRange	  ( 1,  250) ;
	s_bar1 .setValue	  (genconf.v_bar1 = 5) ;
	QToolTip::add (&s_bar1, i18n ("Set period for first graphic load-average display")) ;

	l_bar2 .setGeometry	  (345, 200,  60,  25) ;
	l_bar2 .setText		  ("Bar 2") ;
	s_bar2 .setGeometry	  (415, 200,  50,  25) ;
	s_bar1 .setRange	  ( 1,  250) ;
	s_bar2 .setValue	  (genconf.v_bar2 = 20) ;
	QToolTip::add (&s_bar2, i18n ("Set period for second graphic load-average display")) ;

	b_log  .setGeometry	  (395,  25,  65,  25) ;
	g_log  .setGeometry	  (195,   5, 275, 115) ;

	g_log  .setTitle	  ("Logging") ;
	b_log  .setText		  ("Browse" ) ;
	QToolTip::add (&b_log, i18n ("Browse filesystem to select log file")) ;

	connect	(&b_log, SIGNAL(clicked()),	 this, SLOT(clickLog())   ) ;

	c_multi.setGeometry       ( 20, 145, 140,  25) ;
	c_multi.setText		  ("Parallel display") ;
	c_multi.setChecked	  (true) ;
	QToolTip::add (&c_multi, i18n ("Display on parallel histogram page")) ;

	g_plug .setGeometry	  (480,   5,   80,115) ;
	g_plug .setTitle	  ("Plugins"	     ) ;

	b_aplug.setGeometry	  (485,  25,   65, 25) ;
	b_aplug.setText		  ("Select"	     ) ;
	QToolTip::add (&b_aplug, i18n ("Select additional plugins")) ;

	b_dplug.setGeometry	  (485,  60,   65, 25) ;
	b_dplug.setText		  ("Remove" 	     ) ;
	QToolTip::add (&b_dplug, i18n ("Remove additional plugins")) ;

	connect (&b_aplug, SIGNAL(clicked()), this, SLOT(addPlugin())) ;
	connect (&b_dplug, SIGNAL(clicked()), this, SLOT(remPlugin())) ;

	d_multi = true	;
	logfd	= NULL	;
}

/*  KNDConf								*/
/*  ~KNDConf	: Destructor for configuration object			*/
/*  (returns)	: void		:					*/

KNDConf::~KNDConf ()
{
	if (logfd != NULL)
	{	fclose	(logfd) ;
		logfd	= NULL	  ;
	}
}

/*  KNDConf								*/
/*  getOpts	: Get options flags				*/
/*  (returns)	: int		: Flags				*/

int	KNDConf::getOpts ()
{
	return	(d_time  ? PO_TIME  : 0) |
		(d_proto ? PO_PROTO : 0) |
		(d_srce  ? PO_SRC   : 0) |
		(d_dest  ? PO_DST   : 0) |
		(d_info  ? PO_INFO  : 0) |
		(d_size  ? PO_SIZE  : 0) ;
}

/*  KNDConf								*/
/*  halt	: Halt monitor if going					*/
/*  state	: MState	: Monitor state after halt		*/
/*  (returns)	: void		:					*/

void	KNDConf::halt
	(	MState	state
	)
{
	if (going)
	{
		parent->execute (false) ;

		if (logfd != NULL)
		{	fclose	(logfd) ;
			logfd	= NULL	;
		}

		frozen	= false	;
		going	= false	;
	}

	parent->setState (state) ;
}

/*  KNDConf								*/
/*  clickSet	: Handle a click of the set button			*/
/*  (returns)	: void		:					*/

void	KNDConf::clickSet ()
{
	d_time	= c_time .isChecked  () ;
	d_proto	= c_proto.isChecked  () ;
	d_srce	= c_srce .isChecked  () ;
	d_dest	= c_dest .isChecked  () ;
	d_info	= c_info .isChecked  () ;
	d_size	= c_size .isChecked  () ;
	d_multi = c_multi.isChecked  () ;
	v_name	= t_name .text	     () ;

	genconf.v_load	= s_load1.value	() ;
	genconf.v_bar1 	= s_bar1 .value	() ;
	genconf.v_bar2 	= s_bar2 .value	() ;

	parent->setName (v_name) ;
	halt	(MSStopped) ;
}

/*  KNDConf								*/
/*  clickFreeze	: Handle a click of the freeze button			*/
/*  (returns)	: void		:					*/

void	KNDConf::clickFreeze ()
{
	if (going)
		parent->freeze (frozen = !frozen) ;
}

/*  KNDConf								*/
/*  addPlugin	: Handle a click of the add plugin button		*/
/*  (returns)	: void		:					*/

void	KNDConf::addPlugin ()
{
	if (!going)
	{
		extern	 QStrList   *PluginsToLoad (QWidget *, QStrList *) ;
		QStrList	    *toload = PluginsToLoad (this, &gotplug) ;
		QListIterator<char> iter (*toload) ;
		char		    *name   ;

		while ((name = iter.current ()) != NULL)
		{
			parent->AddPlugin (name) ;
			gotplug.append    (name) ;
			iter += 1 ;
		}

		/* Don't need the loading list any more, the "gotplug"	*/
		/* list does deep copies.				*/
		delete	toload	;
	}
}

/*  KNDConf								*/
/*  remPlugin	: Handle a click of the remove plugin button		*/
/*  (returns)	: void		:					*/

void	KNDConf::remPlugin ()
{
	if (!going)
	{
		extern	 QStrList   *PluginsToRemove (QWidget *, QStrList *) ;
		extern	int	    StrInStrList     (cchar   *, QStrList *) ;
		QStrList	    *toremove = PluginsToRemove (this, &gotplug) ;
		QListIterator<char> iter (*toremove) ;
		char		    *name   ;
		int		    slot    ;

		while ((name = iter.current ()) != NULL)
		{
			parent->RemPlugin (name) ;
			if ((slot = StrInStrList (name, &gotplug)) >= 0)
				gotplug.remove (slot) ;
			iter += 1 ;
		}

		/* Don't need the loading list any more, the "gotplug"	*/
		/* list does deep copies.				*/
		delete	toremove ;
	}
}


/*  KNDConf								*/
/*  setProgRC	: Handle "setProg" result				*/
/*  rc		: bool		: Success				*/
/*  (returns)	: void		:					*/

void	KNDConf::setProgRC
	(	bool	rc
	)
{
	parent->setState  (rc ? MSSet : MSError) ;
}

/*  KNDConf								*/
/*  executeRC	: Handle "execute" result				*/
/*  rc		: bool		: Success				*/
/*  (returns)	: void		:					*/

void	KNDConf::executeRC
	(	bool	rc
	)
{
	b_aplug.setEnabled (!rc) ;
	b_dplug.setEnabled (!rc) ;

	if (going = rc) return   ;

	/* If we failed to start capture and there is an open logging	*/
	/* file then close it.						*/
	if (logfd != NULL)
	{	fclose	(logfd) ;
		logfd	= NULL	;
	}

	parent->setState (MSError) ;
}

