/***************************************************************************
                          knd_confmon.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<unistd.h>
#include	<errno.h>
#include	<qtooltip.h>
#include	<kfiledialog.h>

#include	"knd_confmon.h"
#include	"knd_view.h"
#include	"knd_pcap.h"
#include	"knd_pkt.h"
#include	"knd_gra.h"
#include	"knd_multi.h"
#include	"knd_trigger.h"


/*  WrToLog	: Write string header item to binary log file		*/
/*  logfd	: FILE *	: Log file				*/
/*  what	: LogItem	: Item type				*/
/*  value	: cchar *	: Item value				*/
/*  (returns)	: void		:					*/

void	WrToLog
	(	FILE	*logfd,
		LogItem	what,
		cchar	*value
	)
{
	int	w	= what ;
	int	len	;

	if (value == NULL) value = "" ;
	len	= strlen (value) + 1  ;

	fwrite	(&w,   sizeof (int),   1, logfd) ;
	fwrite	(&len, sizeof (int),   1, logfd) ;
	fwrite	(value,          1,  len, logfd) ;
}

/*  WrToLog	: Write integer header item to binary log file		*/
/*  logfd	: FILE *	: Log file				*/
/*  what	: LogItem	: Item type				*/
/*  value	: int		: Item value				*/
/*  (returns)	: void		:					*/

void	WrToLog
	(	FILE	*logfd,
		LogItem	what,
		int	value
	)
{
	WrToLog	(logfd, what, ItoA(value)) ;
}

/*  KNDConfMon								*/
/*  KNDConfMon	: Constructor for sniffer configuration object		*/
/*  view	: KNDView *	: Parent monitor object			*/
/*  trigger	: KNDTrigger *	: Trigger object			*/
/*  slot	: int		: Unique slot number			*/
/*  name	: cchar *	: Initial monitor name			*/
/*  config	: KConfig *	: Initial configuration or NULL if none	*/
/*  (returns)	: KNDConfMon	:					*/

KNDConfMon::KNDConfMon
	(	KNDView		*view,
		KNDTrigger	*trigger,
		int		slot,
		cchar		*name,
		KConfig		*config
	)
	:
	KNDConf	(view, trigger, slot, name),
	t_logt	(this),
	t_logb	(this),
	s_log	(this),
	l_if	(this),
	s_if	(this),
	c_save	(this),
	l_snap	(this),
	t_snap	(this)
{
	QToolTip::add (&t_name, i18n ("Monitor name")) ;

	l_if   .setGeometry	  (  5,  55,  60,  25) ;
	l_if   .setText		  ("Interface") ;
	s_if   .setGeometry	  ( 75,  55, 100,  25) ;
	v_if	= "" ;
	QToolTip::add (&s_if, i18n ("Network interface to be monitored")) ;

	s_log  .setGeometry	  (200,  25, 120,  25) ;
	t_logt .setGeometry	  (200,  55, 260,  25) ;
	t_logb .setGeometry	  (200,  85, 260,  25) ;

	g_log  .setTitle	  ("Logging") ;
	s_log  .insertItem 	  ("Off",    	    (int)LogNone      ) ;
	s_log  .insertItem 	  ("Record Text",   (int)LogRecordText) ;
	s_log  .insertItem 	  ("Record Binary", (int)LogRecordBin ) ;
	QToolTip::add (&s_log, i18n ("Set logging mode or disable logging")) ;

	t_logt .setEnabled	  (false)     ;
	t_logb .setEnabled	  (false)     ;
	b_log  .setEnabled	  (false)     ;
	t_logt .setText		  (v_logt = v_name + ".ksnt") ;
	t_logb .setText		  (v_logb = v_name + ".ksnb") ;
	m_log	= LogNone ;
	QToolTip::add (&t_logt, i18n ("Name of text logging file")) ;
	QToolTip::add (&t_logb, i18n ("Name of binary logging file")) ;

	connect	(&s_log, SIGNAL(activated(int)), this, SLOT(deltaLog(int))) ;

	c_save.setGeometry        ( 20, 175, 140,  25  ) ;
	c_save.setText		  ("Save configuration") ;
	c_save.setChecked	  (false)		 ;
	QToolTip::add (&c_save, i18n ("Configuration to be saved on Save/Save&Exit")) ;

	l_snap.setGeometry	  (  5, 115,  60,  25  ) ;
	l_snap.setText		  ("Snap len"          ) ;
	t_snap.setGeometry	  ( 75, 115,  60,  25  ) ;
	t_snap.setText		  (ItoA (v_snap = 68)  ) ;

	going	= false	;
	frozen	= false ;
	logfd	= NULL	;

	initPlug= ""	;

	if (config != NULL)
	{
		v_if		= config->readEntry	("InterFace",
							  s_if.count() <= 0 ? "" : s_if.text (0)) ;
		d_time	 	= config->readBoolEntry ("DisplayTime",		true) ;
		d_proto	 	= config->readBoolEntry ("DisplayProto",	true) ;
		d_srce   	= config->readBoolEntry ("DisplaySrce",		true) ;
		d_dest	 	= config->readBoolEntry ("DisplayDest",		true) ;
		d_info	 	= config->readBoolEntry ("DisplayInfo",		true) ;
		d_size	 	= config->readBoolEntry ("DisplaySize",		true) ;
		d_multi	 	= config->readBoolEntry ("ShowParallel",	true) ;
		genconf.v_load	= config->readNumEntry  ("LoadHistInterval",	1   ) ;
		genconf.v_bar1	= config->readNumEntry  ("LoadBar1Interval",	5   ) ;
		genconf.v_bar2	= config->readNumEntry  ("LoadBar2Interval",	20  ) ;
		v_snap		= config->readNumEntry  ("SnapLength",		68  ) ;
		initPlug	= config->readEntry	("Plugins",		""  ) ;
	}

	view->setState (MSUnset) ;
}

/*  KNDConfMon								*/
/*  ~KNDConfMon	: Destructor for configuration object			*/
/*  (returns)	: void		:					*/

KNDConfMon::~KNDConfMon ()
{
	halt (MSStopped) ;
}

/*  KNDConfMon								*/
/*  initPlugins	: Get initial plugins					*/
/*  (returns)	: void		:					*/

void	KNDConfMon::initPlugins ()
{
	char	*str	= strdup (initPlug) ;
	char	*ptr	;

	while ((ptr = strchr (str, '/')) != NULL)
	{
		*ptr	= 0 ;
		parent->AddPlugin (str) ;
		gotplug.append    (str) ;
		str	= ptr + 1 ;
	}

}

/*  KNDConfMon								*/
/*  clickSet	: Handle a click of the set button			*/
/*  (returns)	: void		:					*/

void	KNDConfMon::clickSet ()
{

	KNDConf::clickSet () ;
	trigger->clickSet () ;

	v_logt	= t_logt .text 	     () ;
	v_logb	= t_logb .text	     () ;
	v_if	= s_if   .currentText() ;
	v_snap	= atoi (t_snap.text ()) ;

	/* Pass the packet display options down to the packet object,	*/
	/* and the histogram and load bar time constants to the graphic	*/
	/* object.							*/
	genconf.p_opts = getOpts () ;
	parent->setOptions ((void *)&genconf) ;

	/* Tell the capture object which interface to use and the main	*/
	/* packet filter, plus the event trigger mode and event filters	*/
	parent->setProg    (v_if, v_snap) ;
}

/*  KNDConfMon								*/
/*  clickReset	: Handle a click of the reset button			*/
/*  (returns)	: void		:					*/

void	KNDConfMon::clickReset ()
{
	/* Note that this button simply restores the configuration to	*/
	/* the current values, so the running/frozen state is not	*/
	/* affected.							*/
	t_name .setText	(v_name ) ;

	s_if   .setCurrentItem(QCBFindItem(s_if, v_if, dflt_if)) ;

	c_time .setChecked	(d_time ) ;
	c_proto.setChecked	(d_proto) ;
	c_srce.setChecked	(d_srce ) ;
	c_dest.setChecked	(d_dest ) ;
	c_info .setChecked	(d_info ) ;
	c_size .setChecked	(d_size ) ;
	c_multi.setChecked	(d_multi) ;

	s_log  .setCurrentItem	(m_log  ) ;
	t_logt .setText		(v_logt ) ;
	t_logb .setText		(v_logb ) ;
	t_snap .setText		(ItoA(v_snap)) ;

	halt (MSReset) ;
}

/*  KNDConfMon								*/
/*  clickClear	: Handle a click of the clear button			*/
/*  (returns)	: void		:					*/

void	KNDConfMon::clickClear ()
{
	c_time .setChecked	(true)	  ;
	c_proto.setChecked	(true)	  ;
	c_srce .setChecked	(true)	  ;
	c_dest .setChecked	(true)	  ;
	c_info .setChecked	(true)	  ;
	c_size .setChecked	(true)	  ;
	c_multi.setChecked	(true)	  ;
	s_log  .setCurrentItem	(LogNone) ;
	t_logt .setEnabled	(false)	  ;
	t_logb .setEnabled	(false)	  ;
	b_log  .setEnabled	(false)	  ;

	s_load1.setValue	( 1) ;
	s_bar1 .setValue	( 5) ;
	s_bar2 .setValue	(20) ;
	t_snap .setText		(ItoA (68)) ;
	halt	(MSCleared) ;
}

/*  KNDConfMon								*/
/*  clickGo	: Handle a click of the go button			*/
/*  (returns)	: void		:					*/

void	KNDConfMon::clickGo ()
{
	int	ferr	;
	int	seuid	= geteuid () ;
	int	segid	= getegid () ;

	/* If we are currently running, then all we need to do is to	*/
	/* halt the monitor.						*/
	if (going)
	{
		halt (MSStopped) ;
		b_aplug.setEnabled (true) ;
		b_dplug.setEnabled (true) ;
		return	;
	}

	seteuid (getuid ()) ;
	setegid	(getgid ()) ;

	/* Open the logging file, or not as appropriate, and note the	*/
	/* error code in case this fails.				*/
	switch (s_log.currentItem())
	{
		case LogRecordText :
			logfd	= fopen (v_logt, "w") ;
			ferr	= errno	;
			break	;

		case LogRecordBin  :
			logfd	= fopen (v_logb, "w") ;
			ferr	= errno	;
			break	;

		default	:
			logfd	= NULL	;
			ferr	= 0	;
			break	;
	}

	seteuid (seuid) ;
	setegid	(segid) ;

	/* If the log file is not open and the error code is non-zero	*/
	/* then there was an error, which we duly report (but allow	*/
	/* monitoring to start anyway).					*/
	if ((logfd == NULL) && (ferr != 0))
		Error	(QMessageBox::Warning, "Ksnuffle file error",
			 			strerror(ferr)) ;

	/* If logging then write as much of the log file headers as we	*/
	/* can know about, that is the interface name and the trigger	*/
	/* details.							*/
	if (logfd != NULL)
	{
		TrigInf	*triginf = trigger->getInfo () ;

		/* If we are running as root but the real UID is not	*/
		/* root then change the logging file ownership to be	*/
		/* that of the invoking user.				*/
		if ((geteuid () == 0) && (getuid () != 0))
			fchown (fileno(logfd), getuid(), getgid()) ;

		if (s_log.currentItem() == (int)LogRecordBin)
		{	WrToLog	(logfd, ItemMagic,  MAGIC ) ;
			WrToLog	(logfd, ItemIface,  v_if  ) ;
			WrToLog	(logfd, ItemProg,   triginf->filter) ;
			WrToLog	(logfd, ItemStart,  triginf->start ) ;
			WrToLog	(logfd, ItemStop,   triginf->stop  ) ;
			WrToLog	(logfd, ItemMode,   triginf->mode  ) ;
		}
		else
		{	fprintf	(logfd, "; Interface %s\n", (cchar *)v_if  ) ;
			fprintf	(logfd, "; Filter    %s\n", triginf->filter) ;
			fprintf	(logfd, "; Start     %s\n", triginf->start ) ;
			fprintf	(logfd, "; Stop      %s\n", triginf->stop  ) ;
			fprintf	(logfd, "; Mode      %d\n", triginf->mode  ) ;
		}
	}

	/* Start the capture object executing ....			*/
	parent->execute (true,
			 logfd,
			 s_log.currentItem() == (int)LogRecordBin) ;
}

/*  KNDConfMon								*/
/*  clickLog	: Handle log browse button click			*/
/*  (returns)	: void		:					*/

void	KNDConfMon::clickLog ()
{
	char	*filter	= (LogMode)s_log.currentItem() == LogRecordBin ?
			  "*.ksnb" :
			  "*.ksnt" ;

	QString	name	= KFileDialog::getSaveFileName (NULL, filter, this) ;

	if (!name.isNull() && !name.isEmpty())
		if (s_log.currentItem() == (int)LogRecordText)
			t_logt.setText (name) ;
		else	t_logb.setText (name) ;
}


/*  KNDConfMon								*/
/*  deltaLog	: Handle change of log setting				*/
/*  index	: int		: Selected index			*/
/*  (returns)	: void		:					*/

void	KNDConfMon::deltaLog
	(	int	index
	)
{
	t_logt.setEnabled (index == (int)LogRecordText) ;
	t_logb.setEnabled (index == (int)LogRecordBin ) ;
	b_log .setEnabled (index != 0) ;
}


/*  KNDConfMon								*/
/*  writeConfig	: Write monitor configuration				*/
/*  config	: KConfig *	: Configuration object			*/
/*  (returns)	: void		:					*/

void	KNDConfMon::writeConfig
	(	KConfig	*config
	)
{
	config->writeEntry ("MonitorName",	v_name	) ;
	config->writeEntry ("InterFace",	v_if	) ;
	config->writeEntry ("SnapLength",	v_snap	) ;
	config->writeEntry ("DisplayTime",	d_time	) ;
	config->writeEntry ("DisplayProto",	d_proto	) ;
	config->writeEntry ("DisplaySrce",	d_srce	) ;
	config->writeEntry ("DisplayDest",	d_dest	) ;
	config->writeEntry ("DisplayInfo",	d_info	) ;
	config->writeEntry ("DisplaySize",	d_size	) ;
	config->writeEntry ("ShowParallel",	d_multi	) ;
	config->writeEntry ("LoadHistInterval",	genconf.v_load) ;
	config->writeEntry ("LoadBar1Interval",	genconf.v_bar1) ;
	config->writeEntry ("LoadBar2Interval",	genconf.v_bar2) ;

	QString		    plugins = "" ;
	QListIterator<char> iter (gotplug) ;
	cchar		    *entry	 ;

	while ((entry = iter.current ()) != NULL)
	{	plugins += entry ;
		plugins	+= "/"	 ;
		iter    += 1	 ;
	}

	config->writeEntry ("Plugins",		plugins ) ;
}

/*  KNDConfMon	: Get interface						*/
/*  (returns)	: cchar *	: Interface name			*/

cchar	*KNDConfMon::getIface ()
{
	return	v_if	;
}

/*  KNDConfMon								*/
/*  SaveMe	: See whether monitor should be saved			*/
/*  (returns)	: bool		:					*/

bool	KNDConfMon::SaveMe ()
{
	return	c_save.isChecked () ;
}


/*  KNDConfLoc								*/
/*  KNDConfLoc	: Constructor for local sniffer configuration		*/
/*  _parent	: KNDView *	: Parent monitor object			*/
/*  trigger	: KNDTrigger *	: Trigger object			*/
/*  slot	: int		: Unique slot number			*/
/*  name	: cchar *	: Initial monitor name			*/
/*  iflist	: QArray<char*>*: Array of interface names or NULL	*/
/*  config	: KConfig *	: Initial configuration or NULL if none	*/
/*  (returns)	: KNDConfMon	:					*/

KNDConfLoc::KNDConfLoc
	(	KNDView		*_parent,
		KNDTrigger	*trigger,
		int		slot,
		cchar		*name,
		QArray<char *>	*iflist,
		KConfig		*config
	)
	:
	KNDConfMon (_parent, trigger, slot, name, config)
{
	int	ifsel1	= -1 ;
	int	ifsel2	= -1 ;

	/* Insert the interfaces into the selection box. We try to pick	*/
	/* a reasonable one to start at; if we can find an "eth*" then	*/
	/* use the first such, otherwise the first that is not "lo". If	*/
	/* these checks yield nothing then the default applies.		*/
	for (unsigned int ifno = 0 ; ifno < iflist->size() ; ifno += 1)
	{
		char	*ifname	= (*iflist)[ifno] ;

		s_if.insertItem (ifname) ;

		if ((ifsel1 < 0) && (strncmp (ifname, "eth", 3) == 0))
			ifsel1 = ifno ;
		if ((ifsel2 < 0) && (strcmp  (ifname, "lo"    ) != 0))
			ifsel2 = ifno ;
	}

	if	(ifsel1 >= 0) dflt_if = ifsel1	;
	else if	(ifsel2 >= 0) dflt_if = ifsel2	;
	else		      dflt_if = 0	;

	clickReset () ;

	/* By default, local sniffer configurations are always saved.	*/
	c_save.setChecked (true) ;
}

/*  KNDConfLoc								*/
/*  ~KNDConfLoc	: Destructor for local sniffer configuration class	*/
/*  (returns)	:		:					*/

KNDConfLoc::~KNDConfLoc ()
{
}

/*  KNDConfLoc								*/
/*  AddIface	: Add interface from remote machine			*/
/*  spec	: cchar *	: Interface name or NULL to reset	*/
/*  (returns)	: void		:					*/

void	KNDConfLoc::AddIface
	(	cchar	*spec
	)
{
	/* This is a null operation and should never be called.		*/
	fprintf	(stderr, "KNDConfLoc::AddIface called\n") ;
}


/*  KNDConfRem								*/
/*  KNDConfRem	: Constructor for remote sniffer configuration		*/
/*  parent	: KNDView *	: Parent monitor object			*/
/*  trigger	: KNDTrigger *	: Trigger object			*/
/*  slot	: int		: Unique slot number			*/
/*  name	: cchar *	: Initial monitor name			*/
/*  host	: cchar *	: Host name				*/
/*  config	: KConfig *	: Initial configuration or NULL if none	*/
/*  (returns)	: KNDConfMon	:					*/

KNDConfRem::KNDConfRem
	(	KNDView		*_parent,
		KNDTrigger	*trigger,
		int		slot,
		cchar		*name,
		cchar		*host,
		KConfig		*config
	)
	:
	KNDConfMon (_parent, trigger, slot, name, config),
	l_remote   (this),
	t_remote   (this),
	sameif	   (32)
{
	l_remote.setGeometry (  5,  85,  60,  25) ;
	l_remote.setText     ("Host") ;
	t_remote.setGeometry ( 75,  85, 100,  25) ;
	t_remote.setText     ( name ) ; 

	t_remote.setFrameStyle	   (QFrame::Panel|QFrame::Sunken) ;
	t_remote.setLineWidth	   (2) ;
	t_remote.setBackgroundMode (PaletteBase) ;

	nsame	= 0 ;
	clickReset () ;

	/* By default, remote sniffer configurations are not saved,	*/
	/* except where loaded from the pervious configuration.		*/
	c_save.setChecked (config != NULL) ;

	dflt_if	= 0   ;
}

/*  KNDConfRem								*/
/*  ~KNDConfRem	: Destructor for remote sniffer configuration class	*/
/*  (returns)	:		:					*/

KNDConfRem::~KNDConfRem ()
{
	for (int idx = 0 ; idx < nsame ; idx += 1)
		free (sameif[idx]) ;
}

/*  KNDConfRem								*/
/*  AddIface	: Add interface from remote machine			*/
/*  spec	: cchar *	: Interface name or NULL to reset	*/
/*  (returns)	: void		:					*/

void	KNDConfRem::AddIface
	(	cchar	*spec
	)
{
	int	idx	;

	/* The interface name is passed as NULL in order to reset the	*/
	/* list. This is used when requerying the interface list on	*/
	/* the remote machine.						*/
	if (spec == NULL)
	{
		for (idx = 0 ; idx < nsame ; idx += 1)
			free (sameif[idx]) ;
		nsame	= 0 ;

		s_if.clear () ;
		return	;
	}

	/* If the interface name is null then this is the end of the	*/
	/* list, so see if the presently set interface is in the list.	*/
	if (spec[0] == 0)
	{
		if ((idx = QCBFindItem (s_if, v_if, -1)) >= 0)
		{	s_if.setCurrentItem (idx) ;
			return	;
		}

		s_if.setCurrentItem (0) ;
		v_if = s_if.text    (0) ;
		return	;	
	}

	/* The interface name is tagged with a minus if it is the same	*/
	/* interface as that used by the remote connection, in which	*/
	/* case sniffing it could be a bit dicey. Make a note of such	*/
	/* interfaces, and strip the marker.				*/
	if (spec[0] == '-')
	{	sameif[nsame] = strdup (&spec[1]) ;
		nsame += 1 ;
	}

	s_if.insertItem (&spec[1]) ;
}
