/***************************************************************************
                          knd_confrep.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<sys/types.h>
#include	<sys/stat.h>
#include	<errno.h>
#include	<unistd.h>
#include	<fcntl.h>

#include	<qtooltip.h>
#include	<kfiledialog.h>

#include	"knd_confrep.h"
#include	"knd_pcaprep.h"
#include	"knd_view.h"
#include	"knd_trigger.h"

extern	void	PrintProg	(cchar *, cchar *, BPFPROG &) ;

#define	GOT_LTYPE	(0x01)
#define	GOT_NMASK	(0x02)
#define	GOT_SNAPL	(0x04)
#define	GOT_MAGIC	(0x08)
#define	GOT_ALL		(GOT_LTYPE|GOT_NMASK|GOT_SNAPL|GOT_MAGIC)

/*  KNDSpeedBox								*/
/*  KNDSpeedBox	: Replay speed spin control constructor			*/
/*  minVal	: int		: Minimum value				*/
/*  maxVal	: int		: Maximum value				*/
/*  iniVal	: int		: Initial value				*/
/*  (returns)	: KNDSpeedBox	:					*/

KNDSpeedBox::KNDSpeedBox
	(	QWidget	*parent,
		int	minVal,
		int	maxVal,
		int	iniVal
	)
	:
	QSpinBox (parent)
{
	setRange (minVal, maxVal) ;
	setValue (speed = iniVal) ;

	connect	 (this, SIGNAL(valueChanged(int)), this, SLOT(speedChanged(int))) ;
}

/*  KNDSpeedBox								*/
/*  speedChanged: Handle change on speed box				*/
/*  newval	: int		: New value				*/
/*  (returns)	: void		:					*/

void	KNDSpeedBox::speedChanged
	(	int	newval
	)
{
	/* The purpose of this code is to arrange that we skip zero and	*/
	/* minus one, ie., going down from 1 goes to -2, and up from -2	*/
	/* goes to one. The code is arranged so that we don't fall foul	*/
	/* of nested "valueChanged" signals.				*/
	if (newval != speed)
		if ((newval == 0) || (newval == -1))
			if (newval > speed)
				setValue (speed =  1) ;
			else	setValue (speed = -2) ;
}

/*  KNDSpeedBox								*/
/*  mapValueToText: Map spin value to display text			*/
/*  value	  : int		: Current value				*/
/*  (returns)	  : QString	: Display text				*/

QString	KNDSpeedBox::mapValueToText
	(	int	value
	)
{
	QString	res	;

	if	(value == 1)
		res.sprintf ("Original speed") ;
	else if (value >  0)
		res.sprintf ("%d times speed-up",   value) ;
	else	res.sprintf ("%d times slow-down", -value) ;

	return	res	;
}

/*  KNDConfRep								*/
/*  KNDConfRep	: Constructor for configuration and control object	*/
/*  view	: KNDView *	: Parent object				*/
/*  capture	: KNDPCapRep *	: Capture object			*/
/*  trigger	: KNDTrigger *	: Trigger object			*/
/*  slot	: int		: Unique slot number			*/
/*  name	: char *	: Initial monitor name			*/
/*  (returns)	: KNDConfRep	:					*/

KNDConfRep::KNDConfRep
	(	KNDView		*view,
		KNDPCapRep	*capture,
		KNDTrigger	*trigger,
		int		slot,
		cchar		*name
	)
	:
	KNDConf	(view, trigger, slot, name),
	capture	(capture),
	t_logb	(this),
	l_if	(this),
	t_if	(this),
	s_speed	(this, -50, 50, 1),
	l_snap	(this),
	t_snap	(this)
{
	l_if   .setGeometry	  (  5,  55,  60,  25) ;
	l_if   .setText		  ("Interface") ;
	t_if   .setGeometry	  ( 75,  55, 100,  25) ;
	t_if   .setFrameStyle	  (QFrame::Panel|QFrame::Sunken) ;
	t_if   .setLineWidth	  (2) ;
	t_if   .setBackgroundMode (PaletteBase) ;

	QToolTip::add (&t_if,   i18n ("Network interface monitored")) ;
	QToolTip::add (&t_name, i18n ("Replay name")) ;

	l_snap.setGeometry	  (  5, 115,  60,  25  ) ;
	l_snap.setText		  ("Snap len"          ) ;
	t_snap.setGeometry	  ( 75, 115,  60,  25  ) ;
	t_snap.setFrameStyle	  (QFrame::Panel|QFrame::Sunken) ;
	t_snap.setLineWidth	  (2) ;
	t_snap.setBackgroundMode  (PaletteBase) ;

	t_logb .setGeometry	  (200,  85, 260,  25) ;
	t_logb .setText		  (v_logb = v_name + ".ksnb") ;
	QToolTip::add (&t_logb, i18n ("Name of binary logging file")) ;

	s_speed.setGeometry	  (295,  55, 165,  25) ;

	logfd	= -1		;
	going	= false		;
	frozen	= false		;

	view->setState (MSUnset);
}

/*  KNDConfRep								*/
/*  ~KNDConfRep	: Destructor for configuration object			*/
/*  (returns)	: void		:					*/

KNDConfRep::~KNDConfRep ()
{
	if (logfd >= 0) ::close (logfd) ;
}

/*  KNDConfRep								*/
/*  clickSet	: Handle a click of the set button			*/
/*  (returns)	: void		:					*/

void	KNDConfRep::clickSet ()
{
	int	len		;
	char	buff	[512]	;
	int	what		;
	BPFI32	netmask		;
	int	snaplen		;
	DECODER	*decoder	;
	bool	error		= false ;
	int	gotmask		= 0	;

	KNDConf::clickSet     () ;
	v_logb	= t_logb.text () ;

	filter	= ""	;
	start	= ""	;
	stop	= ""	;

	triginf.mode	= TR_ALL ;

	v_speed	= s_speed.value () ;
	
	/* Attempt to open the specified log file, reporting an error	*/
	/* if it does not exist or cannot be opened.			*/
	if ((logfd = open (v_logb, O_RDONLY)) < 0)
	{
		Error	("Ksnuffle file error", STRERR) ;
		parent->setState (MSError) ;
		return	;
	}

	/* Read the header section, which should give us the inteface	*/
	/* name and the filter program.					*/
	for (what = -1 ; (what != (int)ItemLast) && !error ; )
	{
		/* Read the next item marker. Check for an error,	*/
		/* otherwise see if it is the end-of-header marker.	*/
		if (read (logfd, &what, sizeof(int)) != sizeof(int))
		{	error	= true ;
			break	;
		}

		if ((LogItem)what == ItemLast) break ;

		/* Read in the item length followed by the item value,	*/
		/* again checking for errors. We set an arbitrary limit	*/
		/* on the item length.					*/
		if ( (read (logfd, &len,  sizeof(int)) != sizeof(int )) ||
		     ((unsigned int)len >= 		  sizeof(buff)) ||
		     (read (logfd, buff,  len) 	       != len	      ) )
		{
			error	= true ;
			break	;
		}

		switch ((LogItem)what)
		{
			case ItemMagic	:
				/* Magic marker. Either flag an error	*/
				/* if it is wrong, or flag that we have	*/
				/* found it if it is OK.		*/
				if (strcmp (buff, MAGIC) != 0)
					error	 = true	     ;
				else	gotmask |= GOT_MAGIC ;
				break	;

			case ItemIface	:
				/* Set the interface name ...		*/
				t_if.setText (buff) ;
				break	;

			case ItemProg	:
				/* Set the filter program ...		*/
				filter	= buff ;
				break	;

			case ItemStart	:
				/* ... the start trigger ...		*/
				start	= buff ;
				break	;

			case ItemStop	:
				/* ... and the stop trigger.		*/
				stop	= buff	;
				break	;

			case ItemMode	:
				/* Also set the trigger mode		*/
				triginf.mode	= atoi(buff) ;
				break	;

			case ItemLType	:
				/* Original link type. We need this in	*/
				/* order to determin the packet decoder	*/
				linktype = atoi(buff) ;
				gotmask	|= GOT_LTYPE  ;
				break	;

			case ItemNMask	:
				/* Ditto above ...			*/
				netmask	 = atoi(buff) ;
				gotmask	|= GOT_NMASK  ;
				break	;

			case ItemSnapl	:
				/* Ditto above ...			*/
				snaplen	 = atoi(buff) ;
				gotmask	|= GOT_SNAPL  ;
				break	;

			default	:
				/* Anything else is unrecognised and is	*/
				/* treated as an error.			*/
				error	= true ;
				break	;
		}
	}

	/* Report an error if either the error flag has been set	*/
	/* explicitely or if we failedto find the magic markder.	*/
	if (error || (gotmask != GOT_ALL))
	{
		Error	("Ksnuffle file error",
			 "File does not appear to be a binary logging file") ;
		::close	(logfd) ;
		parent->setState (MSError) ;
		return	;
	}

	if ((decoder = getDecoder (linktype)) == NULL)
	{	Error	("KSnuffle error", "Cannot find decoder for interface") ;
		::close (logfd) ;
		parent->setState (MSError) ;
		return	;
	}

	/* Note the offset of the first record, then check that the	*/
	/* file actually appears to contain some logging data.		*/
	frec	= lseek (logfd, 0, SEEK_CUR) ;

	if (lseek (logfd, 0, SEEK_END) <= frec)
	{
		Error	("Ksnuffle file error", "Logging file is empty") ;
		::close	(logfd) ;
		parent->setState (MSError) ;
		return	;
	}

	/* Pass the filter, the triggers and the trigger mode up to the	*/
	/* parent; this will pass them on to the trigger object to be	*/
	/* displayed. Also, pass the log file and speed to the capture	*/
	/* object.							*/
	triginf.filter = filter ;
	triginf.start  = start  ;
	triginf.stop   = stop   ;
	trigger->setTrigger (&triginf) ;
	capture->setReplay  (logfd, v_speed, decoder) ;

	/* Pass the packet display options down to the packet object,	*/
	/* and the histogram and load bar time constants to the graphic	*/
	/* object.							*/
	genconf.v_load	= s_load1.value () ;
	genconf.v_bar1	= s_bar1 .value () ;
	genconf.v_bar2	= s_bar2 .value () ;
	genconf.p_opts  = getOpts () ;
	parent ->setOptions ((void *)&genconf) ;
}

/*  KNDConfRep								*/
/*  initPlugins	: Get initial plugins					*/
/*  (returns)	: void		:					*/

void	KNDConfRep::initPlugins ()
{
	/* Dummay routine as we never save replayer state.		*/
}

/*  KNDConfRep								*/
/*  clickReset	: Handle a click of the reset button			*/
/*  (returns)	: void		:					*/

void	KNDConfRep::clickReset ()
{
}

/*  KNDConfRep								*/
/*  clickClear	: Handle a click of the clear button			*/
/*  (returns)	: void		:					*/

void	KNDConfRep::clickClear ()
{
	c_time .setChecked	(true)	  ;
	c_proto.setChecked	(true)	  ;
	c_srce .setChecked	(true)	  ;
	c_dest .setChecked	(true)	  ;
	c_info .setChecked	(true)	  ;
	c_size .setChecked	(true)	  ;
	c_multi.setChecked	(true)	  ;

	s_load1.setValue	( 1) ;
	s_bar1 .setValue	( 5) ;
	s_bar2 .setValue	(20) ;

	halt	(MSCleared) ;
}

/*  KNDConfRep								*/
/*  clickGo	: Handle a click of the go button			*/
/*  (returns)	: void		:					*/

void	KNDConfRep::clickGo ()
{
	if (logfd <= 0)
		return ;

	/* If we are currently running, then all we need to do is to	*/
	/* halt the reply.						*/
	if (going)
	{	halt	(MSStopped) ;
		b_aplug.setEnabled (true) ;
		b_dplug.setEnabled (true) ;
		return	;
	}

	/* In case we are restarting the log, seek back to the offset	*/
	/* of the first record.						*/
	if (lseek (logfd, frec, SEEK_SET) != frec)
	{	Error	("Ksnuffle file error", STRERR) ;
		parent->setState (MSError) ;
		return	;
	}

	/* Notify the packet and graphic display objects that we are	*/
	/* now running ...						*/
	parent->execute (true) ;
}

/*  KNDConfRep								*/
/*  clickLog	: Handle a click of the log browse button		*/
/*  (returns)	: void		:					*/

void	KNDConfRep::clickLog ()
{
	QString	name = KFileDialog::getSaveFileName (NULL, "*.ksnb", this) ;

	if (!name.isNull() && !name.isEmpty()) t_logb.setText (name) ;
}

/*  KNDConfRep								*/
/*  timerTick	: Handle a timer tick					*/
/*  second	: long		: Second				*/
/*  (returns)	: void		:					*/

void	KNDConfRep::timerTick
	(	long	second
	)
{
}

/*  KNDConfRep								*/
/*  getIface	: Get interface						*/
/*  (returns)	: cchar *	: Interface name			*/

cchar	*KNDConfRep::getIface ()
{
	return	t_if.text () ;
}

/*  KNDConfRep								*/
/*  writeConfig	: Write configuration					*/
/*  config	: KConfig *	: Application configuration object	*/
/*  (returns)	: void		:					*/

void	KNDConfRep::writeConfig
	(	KConfig	*config
	)
{
	/* This method is never actually called. It is only needed as	*/
	/* the KNDConf base class has a pure virtual method.		*/
}

/*  KNDConfRep								*/
/*  SaveMe	: See whether monitor should be saved			*/
/*  (returns)	: bool		:					*/

bool	KNDConfRep::SaveMe ()
{
	/* Reply monitor configurations are never saved. All the	*/
	/* information is in the binary log file.			*/
	return	false	;
}
