/***************************************************************************
                          knd_detail.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<stdarg.h>
#include	<qtooltip.h>

#include	<ktmainwindow.h>
#include	<kiconloader.h>

#include	"pcaphdr.h"
#include	"knd_plugin.h"
#include	"knd_view.h"
#include	"knd_pkt.h"
#include	"knd_detail.h"
#include	"knd_pcap.h"

#include	"nethdrs.h"
#include	"resource.h"

extern	char	*dmap	;

static	int	p_domain	= 53 ;

/*  _F		: Format						*/
/*  fmt		: const char *	: Format string				*/
/*  ...		: ...		: Arguments				*/	
/*  (returns)	: const char *	: Ascii string				*/

cchar	*_F
	(	cchar	*fmt,
		...
	)
{
	static	 char	buff[128] ;
	va_list	 aptr	;

	va_start (aptr, fmt)		;
	vsprintf (buff, fmt, aptr)	;
	va_end	 (aptr)			;

	return	 buff	;
}

/*  Range	: Calculate range for highlighting			*/
/*  pktinfo	: PktInfo *	: Packet in question			*/
/*  ptr		: char *	: Start of range			*/

QPoint	Range
	(	PktInfo	*pktinfo,
		char	*ptr
	)
{
	/* In this call, the range is implicitely to the end of the	*/
	/* packet, so just use a very large length.			*/
	return	QPoint	(ptr - pktinfo->data, 0x7fff) ;
}

/*  Range	: Calculate range for highlighting			*/
/*  pktinfo	: PktInfo *	: Packet in question			*/
/*  ptr		: char *	: Start of range			*/
/*  len		: int		: Length of range			*/

QPoint	Range
	(	PktInfo	*pktinfo,
		char	*ptr,
		int	len
	)
{
	return	QPoint	(ptr - pktinfo->data, len) ;
}


/*  GetShort	: Get an arbitrarily aligned short			*/
/*  p		: u_char *	: Pointer at short			*/
/*  (returns)	: uint		: Result				*/

uint	GetShort
	(	u_char	*p
	)
{
	uint	p0	= p[0]	;
	uint	p1	= p[1]	;
	return	(p0<<8)|p1	;
}

/*  GetLong	: Get an arbitrarily aligned long			*/
/*  p		: u_char *	: Pointer at long			*/
/*  (returns)	: ulong		: Result				*/

ulong	GetLong
	(	u_char	*p
	)
{
	ulong	p0	= p[0]	;
	ulong	p1	= p[1]	;
	ulong	p2	= p[2]	;
	ulong	p3	= p[3]	;
	return	(p0<<24)|(p1<<16)|(p2<<8)|p3 ;
}

/*  ------------------------------------------------------------------- */

/*  lastChild	: Get last child item below object			*/
/*  lv		: QListView *	: List view to scan			*/
/*  (returns)	: QListViewItem*: Last child or NULL if none		*/

static	QListViewItem	*lastChild
	(	QListView	*lv
	)
{
	QListViewItem	*child	= lv->firstChild () ;
	QListViewItem	*next	;

	if (child != NULL)
		while ((next = child->nextSibling()) != NULL)
			child	= next ;

	return	child	;
}

/*  lastChild	: Get last child item below object			*/
/*  lv		: QListViewItem	: List view item to scan		*/
/*  (returns)	: QListViewItem*: Last child or NULL if none		*/

static	QListViewItem	*lastChild
	(	QListViewItem	*lv
	)
{
	QListViewItem	*child	= lv->firstChild () ;
	QListViewItem	*next	;

	if (child != NULL)
		while ((next = child->nextSibling()) != NULL)
			child	= next ;

	return	child	;
}




/*  codeMapScan	: Scan code map for entry				*/
/*  cmap	: CodeMap *	: Code map to be scanned		*/
/*  code	: int		: Code to scan for			*/
/*  down	: CodeMap *&	: Return for next level map		*/
/*  (returns)	: cchar *	: Text desctiption			*/

cchar	*codeMapScan
	(	CodeMap	*cmap,
		int	code,
		CodeMap	*&down
	)
{
	/* Check in case no map is specified. This can occur if we run	*/
	/* out of maps while descending some series of codes.		*/
	if (cmap == NULL)
	{
		down	= NULL	;
		return	_F ("%d", code) ;
	}

	while (cmap->code != -1)
		if (cmap->code == code)
		{	down  = cmap->down	;
			return	cmap->text	;
		}
		else	cmap += 1 ;

	/* If the code is not found then the text for the end marker	*/
	/* entry should contain a format string.			*/
	down	= NULL	;
	return	_F (cmap->text, code) ;
}

/*  ICMP Maps								*/
/*  ---------								*/

static	CodeMap	icmpMapUnreach [] =
{
{	ICMP_NET_UNREACH,	"Network Unreachable",		NULL		},
{	ICMP_HOST_UNREACH,	"Host Unreachable",		NULL		},
{	ICMP_PROT_UNREACH,	"Protocol Unreachable",		NULL		},
{	ICMP_PORT_UNREACH,	"Port Unreachable",		NULL		},
{	ICMP_FRAG_NEEDED,	"Fragmentation Needed/DF set",	NULL		},
{	ICMP_SR_FAILED,		"Source Route failed",		NULL		},
{	ICMP_NET_UNKNOWN,	"Network unknown",		NULL		},
{	ICMP_HOST_UNKNOWN,	"Host unknown",			NULL		},
{	ICMP_HOST_ISOLATED,	"Host isolated",		NULL		},
{	ICMP_NET_ANO,		"ICMP_NET_ANO",			NULL		},
{	ICMP_HOST_ANO,		"ICMP_HOST_ANO",		NULL		},
{	ICMP_NET_UNR_TOS,	"ICMP_NET_UNR_TOS",		NULL		},
{	ICMP_HOST_UNR_TOS,	"ICMP_HOST_UNR_TOS",		NULL		},
{	ICMP_PKT_FILTERED,	"Packet filtered",		NULL		},
{	ICMP_PREC_VIOLATION,	"Precedence violation",		NULL		},
{	ICMP_PREC_CUTOFF,	"Precedence cut off",		NULL		},
{	-1,			"Unknown unreachable code %d",	NULL		}
}	;

static	CodeMap	icmpMapRedir[] =
{
{	ICMP_REDIR_NET,		"Redirect Net",			NULL		},
{	ICMP_REDIR_HOST,	"Redirect Host",		NULL		},
{	ICMP_REDIR_NETTOS,	"Redirect Net for TOS",		NULL		},
{	ICMP_REDIR_HOSTTOS,	"Redirect Host for TOS",	NULL		}
}	;

static	CodeMap	icmpMapTime[] =
{
{	ICMP_EXC_TTL,		"TTL count",			NULL		},
{	ICMP_EXC_FRAGTIME,	"Fragment reassemby",		NULL		}
}	;

static	CodeMap	icmpMap[] =
{
{	ICMP_ECHOREPLY,		"Echo Reply",			NULL		},
{	ICMP_DEST_UNREACH,	"Destination Unreachable",	icmpMapUnreach	},
{	ICMP_SOURCE_QUENCH,	"Source Quench",		NULL		},
{	ICMP_REDIRECT,		"Redirect (change route",	icmpMapRedir	},
{	ICMP_ECHO,		"Echo Request",			NULL		},
{	ICMP_TIME_EXCEEDED,	"Time Exceeded",		icmpMapTime	},
{	ICMP_PARAMETERPROB,	"Parameter Problem",		NULL		},
{	ICMP_TIMESTAMP,		"Timestamp Request",		NULL		},
{	ICMP_TIMESTAMPREPLY,	"Timestamp Reply",		NULL		},
{	ICMP_INFO_REQUEST,	"Information Request",		NULL		},
{	ICMP_INFO_REPLY,	"Information Reply",		NULL		},
{	ICMP_ADDRESS,		"Address Mask Request",		NULL		},
{	ICMP_ADDRESSREPLY,	"Address Mask Reply",		NULL		},
{	-1,			"Unknown ICMP type %02x",	NULL		}
}	;

/*  -------------------------------------------------------------------	*/

/*  showIPHeader							*/
/*  parent	: KNDDetailItem*: Parent display item			*/
/*  detpkt	: PktInfo *	: Packet information			*/
/*  ip		: IP *		: IP header pointer			*/
/*  (returns)	: void		:					*/

static	void	showIPHeader
	(	KNDDetailItem	*parent,
		PktInfo		*detpkt,
		IP		*ip
	)
{
	EPoint	eps	;
	EPoint	epd	;

	FindEPoint ((char *)&ip->ip_src, -1, &eps) ;
	FindEPoint ((char *)&ip->ip_dst, -1, &epd) ;

	NX(Leaf,    "Version",	       R3 (ip, 1),	    _F ("%d",	       ip->ip_v   ))  ;
	NX(Leaf,    "Header Length",   R3 (ip, 1),	    _F ("%d",	       ip->ip_hl*4))  ;
	NX(IPTOS,   "Type of Service", R3 (&ip->ip_tos, 1), _F ("0x%02x",      ip->ip_tos ))  ;
	NX(Leaf,    "Identification",  R3 (&ip->ip_id,  2), F2 ("%d (0x%04x)", ip->ip_id  )) ;
	NX(IPFlags, "Flags",	       R3 (&ip->ip_off, 2), F2 ("%d (0x%01x)", ip->ip_off >> 13   )) ;
	NX(Leaf,    "Fragment",	       R3 (&ip->ip_off, 2), _F ("%d", 	       ip->ip_off & 0x1fff)) ;
	NX(Leaf,    "Time to Live",    R3 (&ip->ip_ttl, 1), _F ("%d",	       ip->ip_ttl )) ;
	NX(Leaf,    "Protocol",        R3 (&ip->ip_p,   1), ip->ip_p == IPPROTO_TCP  ? "TCP"  :
				  			    ip->ip_p == IPPROTO_UDP  ? "UDP"  :
							    ip->ip_p == IPPROTO_ICMP ? "ICMP" :
							    _F ("0x%02x",      ip->ip_p   ))  ;
	NX(Leaf,    "Checksum",	       R3 (&ip->ip_sum, 2), _F ("0x%04x",ntohs(ip->ip_sum)))  ;
	NX(Leaf,    "Source",	       R3 (&ip->ip_src, 4), _F ("%s %s",      eps.a, epd.n)) ;
	NX(Leaf,    "Destination",     R3 (&ip->ip_dst, 4), _F ("%s %s",      eps.a, epd.n)) ;
}


/*  KNDDetailItem							*/
/*  KNDDetailItem: Constuctor for detail list view item base class	*/
/*  parent	 : QListView *	: Parent list view			*/
/*  text	 : const char *	: Item text				*/
/*  range	 : QPoint	: Byte range				*/
/*  detpkt	 : PktInfo *	: Packet to be detailed			*/
/*  bytes	 : KNDPktBytes *: Bytes display object			*/
/*  (returns)	 : KNDDetailItem:					*/

KNDDetailItem::KNDDetailItem
	(	QListView	*parent,
		cchar		*text,
		QPoint		range,
		PktInfo		*detpkt,
		KNDPktBytes	*bytes
	)
	:
	QListViewItem	(parent, ::lastChild (parent), text),
	range		(range ),
	detpkt		(detpkt),
	arg		(NULL)
{
}

/*  KNDDetailItem							*/
/*  KNDDetailItem: Constuctor for detail list view item base class	*/
/*  parent	 : QListView *	: Parent list view			*/
/*  text	 : const char *	: Item text				*/
/*  range	 : QPoint	: Byte range				*/
/*  details	 : const char *	: Details text				*/
/*  (returns)	 : KNDDetailItem:					*/

KNDDetailItem::KNDDetailItem
	(	KNDDetailItem	*parent,
		cchar		*text,
		QPoint		range,
		cchar		*details,
		u_char		*arg
	)
	:
	QListViewItem	(parent, ::lastChild (parent), text, details),
	range		(range),
	detpkt		(parent->getPktInfo  ()),
	bytes		(parent->getPktBytes ()),
	arg		(arg)
{
}

/*  KNDDetailItem							*/
/*  expand	 : Expand tree						*/
/*  (returns)	 : void		:					*/

void	KNDDetailItem::expand ()
{

	setOpen  (true)  ;

	KNDDetailItem *item = (KNDDetailItem *)firstChild () ;

	while (item != NULL)
	{	item->expand   () ;
		item = (KNDDetailItem *)item->nextSibling (); 
	}
}

/*  KNDDetailItem							*/
/*  expand	 : Expand tree						*/
/*  (returns)	 : void		:					*/

void	KNDDetailItem::collapse ()
{
	setOpen  (false) ;

	KNDDetailItem *item = (KNDDetailItem *)firstChild () ;

	while (item != NULL)
	{	item->collapse () ;
		item = (KNDDetailItem *)item->nextSibling (); 
	}
}

/*  KNDDetFrame	:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetFrame::addItem ()
{
	NI(Leaf, "Time",	     R0, detpkt->time) ;
	NI(Leaf, "Packet length",    R0, _F ("%d", detpkt->piHdr.pktlen)) ;
	NI(Leaf, "Data length",      R0, _F ("%d", detpkt->datlen      )) ;
}


/*  KNDDetNetwork:							*/
/*  addItem	 : Add item details					*/
/*  (returns)	 : void		:					*/

void	KNDDetNetwork::addItem ()
{
	NI(Leaf, "MAC Source",       R0, "") ;
	NI(Leaf, "MAC Destination",  R0, "") ;
}

/*  ARPHWType	: Get ARP hardware type	as string			*/
/*  type	: int		: Type code				*/
/*  (returns)	: cchar *	: Type as string			*/

static	cchar	*ARPHWType
	(	int	type
	)
{
	switch	(type)
	{
		case ARPHRD_NETROM   : return "KA9Q: NET/ROM pseudo"		;
		case ARPHRD_ETHER    : return "Ethernet 10Mbps"			;
		case ARPHRD_EETHER   : return "Experimental Ethernet"		;
		case ARPHRD_AX25     : return "AX.25 Level 2"			;
		case ARPHRD_PRONET   : return "PROnet token ring"		;
		case ARPHRD_CHAOS    : return "Chaosnet"			;
		case ARPHRD_IEEE802  : return "IEEE 802.2 Ethernet/TR/TB"	;
		case ARPHRD_ARCNET   : return "ARCnet"				;
		case ARPHRD_APPLETLK : return "APPLEtalk"			;
		case ARPHRD_DLCI     : return "Frame Relay DLCI"		;
		case ARPHRD_METRICOM : return "Metricom STRIP (new IANA id)"	;
		default		     : break  ;
	}

	return	_F ("Unknown type 0x%04x", type) ;
}

/*  ARPHWType	: Get ARP operation type as string			*/
/*  type	: int		: Operation code			*/
/*  (returns)	: cchar *	: Operation as string			*/

static	cchar	*ARPOPType
	(	int	type
	)
{
#define	REVARP_REQUEST	(3)
#define	REVARP_REPLY	(4)

	switch (type)
	{
		case ARPOP_REQUEST   : return "ARP who-has"	;
		case ARPOP_REPLY     : return "ARP is-at"	;
		case REVARP_REQUEST  : return "RARP who-is"	;
		case REVARP_REPLY    : return "RARP reply"	;
		default		     : break  ;
	}

	return	_F ("Unknown operation 0x%04x", type) ;
}

/*  HWAddress	: Convert MAC address to string				*/
/*  addr	: uchar *	: MAC address pointer			*/
/*  alen	: int		: MAC address length			*/
/*  (returns)	: cchar *	: MAC address as string			*/

static	cchar	*HWAddress
	(	uchar	*addr,
		int	alen
	)
{
	static	char	b[80]	;
	char	*bp	= b	;

	for (int idx = 0 ; idx < alen ; idx += 1)
	{
		if (idx > 0) *bp++ = ':'	;
		sprintf	(bp, "%02x", addr[idx]) ;
		bp += 2 ;
	}

	return	b ;
}

/*  KNDDetARP	:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetARP::addItem ()
{
	ETH_ARP	*arp	= detpkt->arp	;
	if (arp == NULL) return		;

	NI(Leaf, "Hardware type",    R3 (&arp->arp_hrd, 2), ARPHWType (ntohs(arp->arp_hrd))) ;

	switch (ntohs(arp->arp_op))
	{
		case ARPOP_REQUEST  :
		case ARPOP_REPLY    :
		case REVARP_REQUEST :
		case REVARP_REPLY   :
			break	;

		default	:
			return	;
	}

	NI(Leaf, "Requesting proto", R3 (&arp->arp_pro, 2), _F ("0x%04x", ntohs(arp->arp_pro))) ;
	NI(Leaf, "Address length",   R3 (&arp->arp_hln, 1), _F ("%d",     arp->arp_hln       )) ;
	NI(Leaf, "Protocol length",  R3 (&arp->arp_pln, 1), _F ("%d",     arp->arp_pln       )) ;
	NI(Leaf, "Operation",	     R3 (&arp->arp_op,  2), ARPOPType (   ntohs(arp->arp_op ))) ;
	NI(Leaf, "Sender MAC addr",  R3 (&arp->arp_sha, 6), HWAddress (arp->arp_sha,  6      )) ;
	NI(Leaf, "Sender IP addr",   R3 (&arp->arp_spa, 4), _F ("%s %s",  detpkt->srce.a, detpkt->srce.n)) ;
	NI(Leaf, "Target MAC addr",  R3 (&arp->arp_tha, 6), HWAddress (arp->arp_tha,  6      )) ;
	NI(Leaf, "Target IP addr",   R3 (&arp->arp_tpa, 4), _F ("%s %s",  detpkt->dest.a, detpkt->dest.n)) ;
}

/*  KNDDetIP	:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetIP::addItem ()
{
	showIPHeader (this, detpkt, (IP *)detpkt->ip) ;
}


/*  KNDDetIPTOS	:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetIPTOS::addItem ()
{
	int	tos	= ((IP *)detpkt->ip)->ip_tos ;
	QPoint	r	= R3 (&((IP *)detpkt->ip)->ip_tos, 1) ;

	NI(Leaf, "Precedence",  r, _F ("%d", (tos >> 5) & 0x07)) ;
	NI(Leaf, "Delay",	r, (tos & 0x10) ? "Low"  : "Normal") ;
	NI(Leaf, "Throughput",  r, (tos & 0x40) ? "High" : "Normal") ;
	NI(Leaf, "Reliability", r, (tos & 0x20) ? "High" : "Normal") ;
}

/*  KNDDetIPFlags:							*/
/*  addItem	 : Add item details					*/
/*  (returns)	 : void		:					*/

void	KNDDetIPFlags::addItem ()
{
	int	flags	= ((IP *)detpkt->ip)->ip_off ;
	QPoint	r	= R3 (&((IP *)detpkt->ip)->ip_off, 1) ;

	NI(Leaf, "Do not fragment", r, (flags & 0x4000) ? "Set" : "Clear") ;
	NI(Leaf, "More fragments",  r, (flags & 0x2000) ? "Set" : "Clear") ;
}

/*  KNDDetTCP	:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetTCP::addItem ()
{
	TCP_HDR	*tcp	= (TCP_HDR *)(detpkt->ipdata) ;

	NI(Leaf,     "Source port",	 R3 (&tcp->th_sport, 2), _F ("%s %s", detpkt->srce.p, detpkt->srce.s)) ;
	NI(Leaf,     "Destination port", R3 (&tcp->th_dport, 2), _F ("%s %s", detpkt->dest.p, detpkt->dest.s)) ;
	NI(Leaf,     "Sequence",	 R3 (&tcp->th_seq,   4), _F ("%d",	    ntohl(tcp->th_seq  ))) ; 
	NI(Leaf,     "Acknowledge",	 R3 (&tcp->th_ack,   4), _F ("%d",	    ntohl(tcp->th_ack  ))) ;
	NI(Leaf,     "Data offset",	 R3 ((u_char *)&tcp->th_ack + sizeof(long),
							     1), _F ("%d",	    tcp->th_off	* 4     )) ;
	NI(TCPFlags, "Flags",		 R3 (&tcp->th_flags, 1), _F ("%d",	    tcp->th_flags	)) ;
	NI(Leaf,     "Window",		 R3 (&tcp->th_win,   2), F2 ("%d (0x%04x)", ntohs(tcp->th_win  ))) ;
	NI(Leaf,     "Checksum",	 R3 (&tcp->th_sum,   2), F2 ("%d (0x%04x)", ntohs(tcp->th_sum  ))) ;
	NI(Leaf,     "Urgent pointer",	 R3 (&tcp->th_urp,   2), _F ("%d",	    ntohs(tcp->th_urp  ))) ;

	NI(Leaf,     "TCP Data",	 R2 ((u_char *)tcp + tcp->th_off * 4), "") ;
}

/*  KNDDetTCPFlags:							*/
/*  addItem	  : Add item details					*/
/*  (returns)	  : void	:					*/

void	KNDDetTCPFlags::addItem ()
{
	TCP_HDR	*tcp	= (TCP_HDR *)(detpkt->ipdata) ;
	int	flags	= tcp->th_flags 	      ;
	QPoint	r	= R3 (&tcp->th_flags, 1)      ;

	NI(Leaf, "FIN",  r, (flags & TH_FIN ) ? "Set" : "Clear") ;
	NI(Leaf, "SYN",  r, (flags & TH_SYN ) ? "Set" : "Clear") ;
	NI(Leaf, "RST",  r, (flags & TH_RST ) ? "Set" : "Clear") ;
	NI(Leaf, "PUSH", r, (flags & TH_PUSH) ? "Set" : "Clear") ;
	NI(Leaf, "ACK",  r, (flags & TH_ACK ) ? "Set" : "Clear") ;
	NI(Leaf, "URG",  r, (flags & TH_URG ) ? "Set" : "Clear") ;
}

/*  KNDDetUDP	:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetUDP::addItem ()
{
	UDP_HDR	*udp	= (UDP_HDR *)(detpkt->ipdata) ;

	NI(Leaf, "Source port",	     R3 (&udp->uh_sport, 2), _F ("%s %s", detpkt->srce.p, detpkt->srce.s)) ;
	NI(Leaf, "Destination port", R3 (&udp->uh_dport, 2), _F ("%s %s", detpkt->dest.p, detpkt->dest.s)) ;
	NI(Leaf, "Length", 	     R3 (&udp->uh_ulen,  2), _F ("%d",		ntohs(udp->uh_ulen ))) ;
	NI(Leaf, "Checksum", 	     R3 (&udp->uh_sum,   2), F2 ("%d (0x%04x)", ntohs(udp->uh_sum  ))) ;
	NI(Leaf, "UDP Data",	     R2 ((u_char *)&udp->uh_sum + 2), "") ;
}

/*  KNDDetICMPEcho:							*/
/*  addItem	  : Add item details					*/
/*  (returns)	  : void	:					*/

void	KNDDetICMPEcho::addItem ()
{
	ICMP	*icmp	= (ICMP *)detpkt->ipdata ;

	NI(Leaf, "Identifier", R3 (&icmp->un.echo.id, 	    2), _F ("%d", icmp->un.echo.id	)) ;
	NI(Leaf, "Sequence",   R3 (&icmp->un.echo.sequence, 2), _F ("%d", icmp->un.echo.sequence)) ;
	NI(Leaf, "Data",       R2 ((u_char *)icmp + sizeof(ICMP)), NULL) ;
}

/*  KNDDetICMPUnR:							*/
/*  addItem	 : Add item details					*/
/*  (returns)	 : void		:					*/

void	KNDDetICMPError::addItem ()
{
	ICMP	*icmp	= (ICMP *)detpkt->ipdata ;
	IP	*ip	= (IP   *)((u_char *)icmp + sizeof(ICMP)) ;

	showIPHeader (this, detpkt, ip) ;
}


/*  KNDDetICMP	:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetICMP::addItem ()
{
	ICMP	*icmp	= (ICMP *)detpkt->ipdata ;
	QPoint	rd	= R2 (&icmp->un)	 ;
	CodeMap	*cMap	;

	NI(Leaf, "Type",	R3 (&icmp->type,     1), codeMapScan (icmpMap, icmp->type, cMap)) ;
	NI(Leaf, "Code",	R3 (&icmp->code,     1), codeMapScan (cMap,    icmp->code, cMap)) ;
	NI(Leaf, "Checksum",	R3 (&icmp->checksum, 2), _F ("%d", icmp->checksum)) ;

	switch (icmp->type)
	{
		case ICMP_ECHO		:
			NI(ICMPEcho,  "Echo Request",		 rd, NULL) ;
			break	;
		case ICMP_ECHOREPLY	:
			NI(ICMPEcho,  "Echo Reply",		 rd, NULL) ;
			break	;
		case ICMP_DEST_UNREACH	:
			NI(ICMPError, "Destination unreachable", rd, NULL) ;
			break	;
		case ICMP_TIME_EXCEEDED	:
			NI(ICMPError, "Time exceeded",		 rd, NULL) ;
			break	;
		case ICMP_PARAMETERPROB	:
			NI(ICMPError, "Parameter problem",	 rd, NULL) ;
			break	;
		case ICMP_SOURCE_QUENCH	:
			NI(ICMPError, "Source quench",		 rd, NULL) ;
			break	;
		case ICMP_REDIRECT	:
		case ICMP_TIMESTAMP	:
		case ICMP_TIMESTAMPREPLY:
		case ICMP_INFO_REQUEST	:
		case ICMP_INFO_REPLY	:
		case ICMP_ADDRESS	:
		case ICMP_ADDRESSREPLY	:
		default			:
			NI(Leaf, "ICMP Data", rd, NULL) ;
			break	;
	}
}

/*  KNDDetailItem							*/
/*  setOpen	 : Handle user open/close of branch			*/
/*  open	 : bool	 	: True on open				*/
/*  (returns)	 : void		:					*/

void	KNDDetailItem::setOpen
	(	bool	open
	)
{
	/* If opening and there are no children, ie., the item has not	*/
	/* been opened before, then add the child items before actually	*/
	/* opening the item itself.					*/
	if (open && (childCount() == 0)) addItem () ;
	QListViewItem::setOpen (open) ;
}



/*  KNDPktBytes								*/
/*  KNDPktBytes	: Constructor for packet bye display object		*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KNDPktBytes	:					*/

KNDPktBytes::KNDPktBytes
	(	QWidget	*parent
	)
	:
	QScrollView (parent)
{
	minsize	= QSize (0, 0) ;
	detpkt	= NULL	;
	offset	= 0	;
	len	= 0	;
}

/*  KNDPktBytes								*/
/*  resizeEvent	: Handle a resize					*/
/*  e		: QResizeEvent *: Resize event information		*/
/*  (returns)	: void		:					*/

void	KNDPktBytes::resizeEvent
	(	QResizeEvent	*e
	)
{
	/* We need to make sure that the contents widget is (a) not	*/
	/* smalled than the size needed to display the packet and (b)	*/
	/* not smaller than the scroll display (so that the background	*/
	/* is uniform).							*/
	QSize	s	= size () ;
	if (s.width  () < minsize.width  ()) s.setWidth  (minsize.width ()) ;
	if (s.height () < minsize.height ()) s.setHeight (minsize.height()) ;
	resizeContents (s.width(), s.height()) ;
}

/*  KNDPktBytes								*/
/*  setPktInfo	: Set new packet to be detailed				*/
/*  _detpkt	: PktInfo *	: Packet				*/
/*  (returns)	: void		:					*/

void	KNDPktBytes::setPktInfo
	(	PktInfo	*_detpkt
	)
{
	detpkt	= _detpkt ;	/* Note the new packet			*/
	offset	= 0	  ;	/* Highlight range ....			*/
	len	= 0	  ;	/* ... is initially empty		*/

	/* We resize the bytes display to be just deep enough to show	*/
	/* the entire packet, although some of this may be hidden by	*/
	/* virtue of the scroller.					*/
	QFontMetrics m  (QFont ("fixed", 10)) ;

	minsize	= QSize (m.maxWidth    () * (6 + (16 * 3) + 2 + 16) + 5,
			 m.lineSpacing () * ((detpkt->piHdr.pktlen + 15) / 16) + 5) ;
	setMaximumHeight(minsize.height()) ;

	QSize	s	= size () ;
	if (s.width  () < minsize.width  ()) s.setWidth  (minsize.width ()) ;
	if (s.height () < minsize.height ()) s.setHeight (minsize.height()) ;
	resizeContents (s.width(), s.height()) ;
	setContentsPos (0,0) ;
	viewport()->repaint (false) ;
}

/*  KNDPktBytes								*/
/*  setRange	: Set highlight range					*/
/*  range	: QPoint	: Range					*/
/*  (returns)	: void		:					*/

void	KNDPktBytes::setRange
	(	QPoint	range
	)
{
	offset	= range.x () ;
	len	= range.y () ;
	viewport()->repaint (false) ;
}

/*  KNDPktBytes								*/
/*  drawContentsOffset							*/
/*		: Draw contents						*/
/*  p		: QPainter *	: Painter object			*/
/*  offsetx	: int		: X scroll offset			*/
/*  offsety	: int		: Y scroll offset			*/
/*  clipx	: int		: X clip value				*/
/*  clipy	: int		: Y clip value				*/
/*  clipw	: int		: Clip width				*/
/*  cliph	: int		: Clip height				*/
/*  (returns)	: void		:					*/

void	KNDPktBytes::drawContentsOffset
	(	QPainter *p,
		int	 offsetx,
		int	 offsety,
		int	 clipx,
		int	 clipy,
		int	 clipw,
		int	 cliph
	)
{
	/* Get the font metrics, and clear the entire display area. It	*/
	/* isn't worth trying to optimise anything.			*/
	p->setFont	(QFont ("fixed", 10)) ;
	QFontMetrics	metrics	= p->fontMetrics () ;

	p->fillRect (QRect (0, 0, contentsWidth(), contentsHeight()), white) ;
	if (detpkt == NULL) return ;


	int	y	= metrics.lineSpacing () - offsety ;
	int	w	= metrics.maxWidth    () ;
	QColor	pen	;

	p->setPen (pen = black) ;

	/* This is pretty non-optimal, but it doesn't seem worth-while	*/
	/* to do anything clever .... The effect is to colour text in	*/
	/* the (offset,len) range red; anything beyond the capture	*/
	/* length is blue and appears as "xx".				*/
	for (int pos = 0 ; pos < detpkt->piHdr.pktlen ; pos += 16)
	{
		char	b[256]	;
		uchar	*dp	= (uchar *)&detpkt->data[pos] ;
		QColor	p2	;

		int	x1	= 0 - offsetx + 5 ;
		int	x2	= 0 - offsetx + 5 ;
		char	*ch	;

		if (pen != black) p->setPen (pen = black) ;
		sprintf	(b, "%04x", pos) ;
		p->drawText (x1, y, b)	 ;

		x1	+= 6 * w	 ;
		x2	+= 6 * w + (16 * 3 * w) + 2 * w	  ;

		for (int off = 0 ; off < 16 ; off += 1)
			if (pos + off < detpkt->piHdr.pktlen)
			{
				sprintf	(b, "%02x", *dp) ;
				ch	= &dmap[*dp]	 ;

				if	(pos + off >= detpkt->piHdr.caplen)
				{	strcpy	(b, "xx") ;
					ch	= "."	  ;
					p2	= blue	  ;
				}
				else if (((pos + off) >= offset) && ((pos + off) < (offset + len)))
					p2	= red	  ;
				else	p2	= black	  ;

				if (p2 != pen) p->setPen (pen = p2) ;

				p->drawText (x1, y, b) ;
				p->drawText (x2, y, &dmap[*dp], 1) ;
				x1	+= 3 * w ;
				x2	+= w ;
				dp	+= 1 ;
			}

		y += metrics.lineSpacing () ;
	}
}



/*  KNDDetails								*/
/*  KNDDetails	: Constructor for packet details objects		*/
/*  parent	: KNDPacket	: Parent object				*/
/*  name	: cchar *	: Sniffer name				*/
/*  iface	: cchar *	: Interface				*/
/*  (returns)	: KNDDetails	:					*/

KNDDetails::KNDDetails
	(	KNDPacket	*parent,
		cchar		*name,
		cchar		*iface
	)
	:
	KTMainWindow	(NULL),
	name		(name),
	iface		(iface),
	parent		(parent)
{
	split	= NULL	;
	details	= NULL	;
	bytes	= NULL	;
	detpkt	= NULL	;

	setCaption (name) ;
	resize	   (540, 400) ;

	toolBar()->insertButton (Icon("ksn_expand.xpm"  ), ID_DETAILS_EXPAND,   true, "Expand all"  ) ;
	toolBar()->insertButton (Icon("ksn_collapse.xpm"), ID_DETAILS_COLLAPSE, true, "Collapse all") ;
	toolBar()->insertButton (Icon("ksn_first.xpm"),    ID_DETAILS_FIRST, 	true, "First TCP/IP packet in stream") ;
	toolBar()->insertButton (Icon("ksn_prev.xpm"),     ID_DETAILS_PREVIOUS, true, "Previous TCP/IP packet in stream") ;
	toolBar()->insertButton (Icon("ksn_next.xpm"),     ID_DETAILS_NEXT,     true, "Next TCP/IP packet in stream"    ) ;
	toolBar()->insertButton (Icon("ksn_last.xpm"),     ID_DETAILS_LAST,     true, "Last TCP/IP packet in stream"    ) ;
	connect(toolBar(), SIGNAL(clicked(int)), SLOT(command(int))) ;

	split   = new QSplitter    (QSplitter::Vertical, this) ;
	details	= new QListView    (split) ;
	bytes	= new KNDPktBytes  (split) ;
	KTMainWindow::setView (split)	   ;

	details->addColumn ("Item",    64) ;
	details->addColumn ("Details", -1) ;
	details->setRootIsDecorated (true) ;
	details->setSorting	(-1)	   ;
	details->show		()	   ;
	bytes  ->show		()	   ;

	split  ->setGeometry (QRect (QPoint (0,0), size())) ;
	show	() ;

	connect	(details, SIGNAL(selectionChanged(QListViewItem *)),
		 this,    SLOT  (selection       (QListViewItem *))) ;

	QSize	s = bytes->size () ;
	bytes->resizeContents (s.width(), s.height()) ;
}

/*  KNDDetails								*/
/*  ~KNDDetails	: Destructor for packet details objects			*/
/*  (returns)	:		:					*/

KNDDetails::~KNDDetails ()
{
	if (split  != NULL) delete split   	 ;
	if (detpkt != NULL) freePktInfo (detpkt) ;
}

/*  KNDDetails								*/
/*  command	: Handle command from toolbar				*/
/*  id		: int		: Command				*/
/*  (returns)	: void		:					*/

void	KNDDetails::command
	(	int	id
	)
{
	KNDDetailItem	*item	;

	switch (id)
	{
		case ID_DETAILS_EXPAND   :
			/* Request to expand the entire tree. Apply	*/
			/* this to each detail item, whence we recurse	*/
			/* down.					*/
			item = (KNDDetailItem *)details->firstChild () ;
			while (item != NULL)
			{	item->expand () ;
				item = (KNDDetailItem *)item->nextSibling () ;
			}
			break	;

		case ID_DETAILS_COLLAPSE :
			/* Request to collapse the entire tree. Apply	*/
			/* this to each detail item, whence we recurse	*/
			/* down.					*/
			item = (KNDDetailItem *)details->firstChild () ;
			while (item != NULL)
			{	item->collapse () ;
				item = (KNDDetailItem *)item->nextSibling (); 
			}
			break	;

		case ID_DETAILS_PREVIOUS :
			/* Move to the previous packet. This may fail	*/
			/* if we are at the first (recorded) packet.	*/
			if (!parent->moveAlong (this, detpkt, id))
				Error	("No previous packet found", "") ;
			break	;

		case ID_DETAILS_NEXT	 :
			/* Move to the next packet. This may fail	*/
			/* if we are at the last (recorded) packet.	*/
			if (!parent->moveAlong (this, detpkt, id))
				Error	("No next packet found", "") ;
			break	;

		case ID_DETAILS_FIRST	 :
		case ID_DETAILS_LAST	 :
			/* Move to first or last ... might fail.	*/
			if (!parent->moveAlong (this, detpkt, id))
				Error	("Unable to find end packet", "") ;
			break	;

		default	:
			break	;
	}
}
		

/*  KNDDetails								*/
/*  resizeEvent	: Handle a resize					*/
/*  e		: QResizeEvent *: Resize event information		*/
/*  (returns)	: void		:					*/

void	KNDDetails::resizeEvent
	(	QResizeEvent	*e
	)
{
	/* Simply adjust the splitter to the new size if this widget.	*/
	/* The splitter will do the rest.				*/
	if (split != NULL)
		split->setGeometry (QRect (QPoint (0,0), size())) ;

}

/*  KNDDetails								*/
/*  closeEvent	: Handle a close					*/
/*  e		: QCloseEvent *	: Close event information		*/
/*  (returns)	: void		:					*/

void	KNDDetails::closeEvent
	(	QCloseEvent	*e
	)
{
	/* This event is picked up so that we can inform our parent	*/
	/* (who is not a parent from Qt's point of view) of our demise.	*/
	parent->endDetails  (this) ;
	QWidget::closeEvent (e)	   ;
}

/*  KNDDetails								*/
/*  setPktInfo	: Set packet to be detailed				*/
/*  _detpkt	: PktInfo *	: packet in question			*/
/*  (returns)	: void		:					*/

void	KNDDetails::setPktInfo
	(	PktInfo	*_detpkt
	)
{
	char	caption[128] ;
	UDP_HDR	*udp	     ;

	sprintf	   (caption, "%s on %s at %s",
			     (cchar *)name, (cchar *)iface, _detpkt->time) ;
	setCaption (caption) ;

	details->clear () ;
	if (detpkt != NULL) freePktInfo (detpkt) ;
	usePktInfo (detpkt = _detpkt) ;

	toolBar()->setItemEnabled (ID_DETAILS_FIRST,    detpkt->tcphash != NULL) ;
	toolBar()->setItemEnabled (ID_DETAILS_LAST,     detpkt->tcphash != NULL) ;
	toolBar()->setItemEnabled (ID_DETAILS_PREVIOUS, detpkt->tcphash != NULL) ;
	toolBar()->setItemEnabled (ID_DETAILS_NEXT,     detpkt->tcphash != NULL) ;

	/* All packets have an initial frame and network entries; what	*/
	/* follows then depends on the contents of the packet.		*/
	new KNDDetFrame   (details, "Frame",   QPoint(0,0), detpkt, bytes) ;
	new KNDDetNetwork (details, "Network", QPoint(0,0), detpkt, bytes) ;


	if (detpkt->ip != NULL)
	{
		/* First case is IP packets. Add an IP header item,	*/
		/* followed by an entry appropriate to the next-level	*/
		/* protocol.						*/
		new KNDDetIP (details, "IP", Range (detpkt, (char *)detpkt->ip), detpkt, bytes) ;

		switch (detpkt->ip->ip_p)
		{
			case IPPROTO_TCP  :
				new KNDDetTCP  (details,
						"TCP",
						Range (detpkt, detpkt->ipdata),
						detpkt,
						bytes) ;
				break	;

			case IPPROTO_UDP  :
				new KNDDetUDP  (details,
						"UDP",
						Range (detpkt, detpkt->ipdata),
						detpkt,
						bytes) ;


				udp	= (UDP_HDR *)(detpkt->ipdata) ;

				if (ntohs (udp->uh_dport) == p_domain)
				{	new KNDDetDNS (details,
						       "DNS Request",
						       Range (detpkt, (char *)&udp->uh_sum + 2),
						       detpkt,
						       bytes) ;
					break	;
				}
				if (ntohs (udp->uh_sport) == p_domain)
				{	new KNDDetDNS (details,
						       "DNS Reply",
						       Range (detpkt, (char *)&udp->uh_sum + 2),
						       detpkt,
						       bytes) ;
					break	;
				}
				break	;

			case IPPROTO_ICMP :
				new KNDDetICMP (details,
						"ICMP",
						Range (detpkt, detpkt->ipdata),
						detpkt,
						bytes) ;
				break	;

			default	:
				break	;
		}
	}
	else if (detpkt->arp != NULL)
	{
		/* Second case is ARP, which also includes RARP ...	*/
		new KNDDetARP (details, "(R)ARP", Range (detpkt, (char *)detpkt->arp), detpkt, bytes) ;
	}

	bytes->setPktInfo (detpkt) ;
}

/*  KNDDetails								*/
/*  selection	: Handle list view selection change			*/
/*  lvi		: QListViewItem *: Newly selected item			*/
/*  (returns)	: void		:					*/

void	KNDDetails::selection
	(	QListViewItem	*lvi
	)
{
	/* The argument is really a KNDDetailItem. Get its range, and	*/
	/* pass this to the byte-by-byte display.			*/
	bytes->setRange (((KNDDetailItem *)lvi)->getRange ()) ;
}
