/***************************************************************************
                          knd_detail.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef	KND_DETAIL_H
#define	KND_DETAIL_H

#include	<qscrollview.h>
#include	<qlistview.h>
#include	<qsplitter.h>

#include	"ksnuffle.h"

/*  KNDPktBytes								*/
/*  This class is used to display the contents of a packet on a byte-	*/
/*  by-byte basis.							*/

class	KNDPktBytes : public QScrollView
{
	PktInfo	*detpkt	;		/* Currently displayed packet	*/
	int	offset	;		/* Highlight offset		*/
	int	len	;		/* Highlight length		*/
	QSize	minsize	;		/* Minimum size			*/

protected :

	void	drawContentsOffset  (QPainter *, int, int, int, int, int, int) ;
	virtual	void	resizeEvent (QResizeEvent *) ;

public	:
	KNDPktBytes (QWidget *) ;

	void	setPktInfo	(PktInfo *) ;
	void	setRange	(QPoint   ) ;
}	;

/*  KNDDetailItem							*/
/*  This is the base class for detailed packet display. It is really	*/
/*  just QListViewItem, but with a few common things added for the	*/
/*  benefit of the actual detail classes.				*/

class	KNDDetailItem : public QListViewItem
{
protected :

	QPoint		range	;	/* Range covered by item	*/
	PktInfo		*detpkt	;	/* Underlying packet		*/
	KNDPktBytes	*bytes	;	/* Parent display object	*/
	u_char		*arg	;	/* Item-specific pointer	*/

	KNDDetailItem	*lastChild () ;

public	:

	/* There are three constructors					*/
	/*	(a) for the top-level items				*/
	/*	(b) for lower-level items				*/
	KNDDetailItem (QListView     *, cchar *, QPoint, PktInfo *, KNDPktBytes *) ;
	KNDDetailItem (KNDDetailItem *, cchar *, QPoint, cchar   *, u_char      *) ;

	virtual	void		setOpen	    (bool)  ;
	virtual	void		addItem	    ()	{}  ;

	inline	PktInfo		*getPktInfo ()  { return detpkt ; } ;
	inline	KNDPktBytes	*getPktBytes()  { return bytes  ; } ;
	inline	QPoint		getRange    ()  { return range  ; } ;

	void	expand	 () ;
	void	collapse () ;
}	;

class	KNDDetailRoot : public KNDDetailItem
{
public	:

	inline	KNDDetailRoot
		(	QListView	*parent,
			cchar		*label,
			QPoint		range,
			PktInfo		*pktinfo,
			KNDPktBytes	*bytes
		)
		:
		KNDDetailItem (parent, label, range, pktinfo, bytes)
	{
		setExpandable (true) ;
	}	;

	virtual	void	addItem	    ()	{}  ;
}	;

class	KNDDetailNode : public KNDDetailItem
{
public	:

	inline	KNDDetailNode
		(	KNDDetailItem	*parent,
			cchar		*label,
			QPoint		range,
			cchar		*detail	= NULL,
			u_char		*arg    = NULL
		)
		:
		KNDDetailItem (parent, label, range, detail, arg)
	{
		setExpandable (true) ;
	}	;

	virtual	void	addItem	    ()	{}  ;
}	;

class	KNDDetLeaf : public KNDDetailItem
{
public	:

	inline	KNDDetLeaf
		(	KNDDetailItem	*parent,
			cchar		*label,
			QPoint		range,
			cchar		*detail,
			u_char		*arg	= NULL
		)
		:
		KNDDetailItem (parent, label, range, detail, arg)
	{
		setExpandable (false) ;
	}	;
}	;



/*  For convenience, define three macros:				*/
/*	DECLROOT: Used to define classes for the top-level detail items	*/
/*	DECLNODE: Used to define classes for intermediate detail items	*/

#define	DECLROOT(c)	\
	class	c : public KNDDetailItem				\
	{								\
	public	:							\
		inline	c 						\
			(	QListView	*parent,		\
				cchar		*label,			\
				QPoint		range,			\
				PktInfo		*detpkt,		\
				KNDPktBytes	*bytes			\
			)						\
			:						\
			KNDDetailItem (parent, label,  range, 		\
					       detpkt, bytes)		\
		{							\
			setExpandable (true) ;				\
		}							\
									\
		virtual	void	addItem () ; 				\
	}	;

#define	DECLNODE(c)	\
	class	c : public KNDDetailItem 				\
	{								\
	public	:							\
		inline	c 						\
			(	KNDDetailItem	*parent,		\
				cchar		*label,			\
				QPoint		range,			\
				cchar	  	*detail,		\
				u_char		*arg	= NULL		\
			)						\
			:						\
			KNDDetailItem (parent, label, range,		\
						      detail, arg)	\
		{							\
			setExpandable (true) ;				\
		}							\
									\
		virtual	void	addItem () ;	 			\
	}	;

DECLROOT(KNDDetFrame		)
DECLROOT(KNDDetNetwork		)
DECLROOT(KNDDetARP		)
DECLROOT(KNDDetIP		)
DECLNODE(KNDDetIPTOS		)
DECLNODE(KNDDetIPFlags		)
DECLROOT(KNDDetTCP		)
DECLNODE(KNDDetTCPFlags		)
DECLROOT(KNDDetUDP		)
DECLROOT(KNDDetDNS		)
DECLNODE(KNDDetDNSHdr		)
DECLNODE(KNDDetDNSQuery		)
DECLNODE(KNDDetDNSAnswer	)
DECLROOT(KNDDetICMP		)
DECLNODE(KNDDetICMPEcho		)
DECLNODE(KNDDetICMPError	)



/*  KNDDetails								*/
/*  This class is used to implement the display of packet details; it	*/
/*  is split into a structural list view, and a byte-by-byte display.	*/

class	KNDDetails : public KTMainWindow
{
	Q_OBJECT

	QString		name		;
	QString		iface		;

	KNDPacket	*parent		;
	QSplitter	*split		;
	QListView	*details	;
	KNDPktBytes	*bytes		;
	PktInfo		*detpkt		;

protected :

	virtual	void	closeEvent  (QCloseEvent  *) ;
	virtual	void	resizeEvent (QResizeEvent *) ;

public	:

	KNDDetails (KNDPacket *, cchar *, cchar *) ;
       ~KNDDetails () ;

	void	setPktInfo (PktInfo *) ;

public slots :

	void	selection  (QListViewItem *) ;
	void	command	   (int) ;
}	;



/*  -------------------------------------------------------------------	*/
/*  The CodeMap structure is used to map codes to text descriptions. In	*/
/*  addition, entries can refer to next-level maps for sub-codes (eg.,	*/
/*  for ICMP "type" and "code" paramaters.				*/

struct	CodeMap
{	int	code	;		/* Code value			*/
	char	*text	;		/* Descriptive text		*/
	CodeMap	*down	;		/* Possible sub-map		*/
}	;


extern	cchar	*_F		(cchar *, ...) ;
extern	QPoint	Range		(PktInfo *, char *) ;
extern	QPoint	Range		(PktInfo *, char *, int) ;
extern	uint	GetShort	(u_char *) ;
extern	ulong	GetLong		(u_char *) ;
extern	cchar	*codeMapScan	(CodeMap *, int, CodeMap *&) ;

/*  ------------------------------------------------------------------- */
/*  Now follow various macros, used to reduce the amount of text used	*/
/*  expand various items ...						*/

#define	F2(f,v)		_F((f),(v),(v))
#define	R0		(QPoint(0,0))
#define	R2(v)		(Range (detpkt, (char *)(v)))
#define	R3(v,l)		(Range (detpkt, (char *)(v), l))
#define	NI(c,l,r,d)	new KNDDet##c (this,   (l), (r), (d))
#define	NA(c,l,r,d,a)	new KNDDet##c (this,   (l), (r), (d), (a))
#define	NX(c,l,r,d)	new KNDDet##c (parent, (l), (r), (d))

#endif	/* KND_DETAIL_H */
