/***************************************************************************
                          knd_dns.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	"pcaphdr.h"
#include	"knd_plugin.h"
#include	"knd_view.h"
#include	"knd_pkt.h"
#include	"knd_detail.h"
#include	"knd_pcap.h"

#include	"nethdrs.h"
#include	<arpa/nameser.h>

/*  DNS Maps								*/
/*  --------								*/

static	CodeMap	dnsOpcodeMap[] =
{
{	QUERY,			"Query",			NULL		},
{	IQUERY,			"Inverse query",		NULL		},
{	STATUS,			"Status request",		NULL		},
{	-1,			"Unknown query type %d",	NULL		}
}	;

static	CodeMap	dnsRCodeMap[] =
{
{	NOERROR,		"No error",			NULL		},
{	FORMERR,		"Format error",			NULL		},
{	SERVFAIL,		"Server failure",		NULL		},
{	NXDOMAIN,		"Name error",			NULL		},
{	NOTIMP,			"Not implemented",		NULL		},
{	REFUSED,		"Refused",			NULL		},
{	-1,			"Unknown code %d",		NULL		}
}	;

static	CodeMap	dnsQTypeMap[] =
{
{	T_A,			"Host address",				NULL		},
{	T_NS,			"Authoritative server",			NULL		},
{	T_MD,			"Mail destination",			NULL		},
{	T_MF,			"Mail forwarder",			NULL		},
{	T_CNAME,		"Canonical name",			NULL		},
{	T_SOA,			"Start of authority zone",		NULL		},
{	T_MB,			"Mailbox domain name",			NULL		},
{	T_MG,			"Mail group member",			NULL		},
{	T_MR,			"Mail rename name",			NULL		},
{	T_NULL,			"Null resource record",			NULL		},
{	T_WKS,			"Well known service",			NULL		},
{	T_PTR,			"Domain name pointer",			NULL		},
{	T_HINFO,		"Host information",			NULL		},
{	T_MINFO,		"Mailbox information",			NULL		},
{	T_MX,			"Mail routing information",		NULL		},
{	T_TXT,			"Text strings",				NULL		},
{	T_RP,			"Responsible person",			NULL		},
{	T_AFSDB,		"Afs cell database",			NULL		},
{	T_X25,			"X_25 calling address",			NULL		},
{	T_ISDN,			"Isdn calling address",			NULL		},
{	T_RT,			"Router",				NULL		},
{	T_NSAP,			"Nsap address",				NULL		},
{	T_NSAP_PTR,		"Reverse NSAP lookup (deprecated)",	NULL		},
{	T_SIG,			"Security signature",			NULL		},
{	T_KEY,			"Security key",				NULL		},
{	T_PX,			"X.400 mail mapping",			NULL		},
{	T_GPOS,			"Geographical position (withdrawn)",	NULL		},
{	T_AAAA,			"Ip6 Address",				NULL		},
{	T_LOC,			"Location Information",			NULL		},
{	T_NXT,			"Next Valid Name in Zone",		NULL		},
{	T_EID,			"Endpoint identifier",			NULL		},
{	T_NIMLOC,		"Nimrod locator",			NULL		},
{	T_SRV,			"Server selection",			NULL		},
{	T_ATMA,			"Atm Address",				NULL		},
{	T_NAPTR,		"Naming Authority PoinTeR",		NULL		},
{	T_UINFO,		"User (finger) information",		NULL		},
{	T_UID,			"User ID",				NULL		},
{	T_GID,			"Group ID",				NULL		},
{	T_UNSPEC,		"Unspecified format (binary data)",	NULL		},
{	T_IXFR,			"Incremental zone transfer",		NULL		},
{	T_AXFR,			"Transfer zone of authority",		NULL		},
{	T_MAILB,		"Transfer mailbox records",		NULL		},
{	T_MAILA,		"Transfer mail agent records",		NULL		},
{	T_ANY,			"Wildcard match",			NULL		},
{	-1,			"Unknown type %d",			NULL		}
}	;

static	CodeMap	dnsQClassMap[] =
{
{	C_IN,			"Internet",				NULL		},
{	C_CHAOS,		"Chaos",				NULL		},
{	C_HS,			"Hesiod",				NULL		},
{	C_ANY,			"Wildcard match",			NULL		},
{	-1,			"Unknown class %d",			NULL		}
}	;

#define	DNSHDR	HEADER

/*  DNSQLen	: Calculate name length					*/
/*  qptr	: u_char *	: Pointer at name			*/
/*  (returns)	: int		: Length in bytes			*/

static	int	DNSNameLen
	(	u_char	*nptr
	)
{
	u_char	*ptr2	= nptr	;

	while (*ptr2 != 0)
	{	if ((*ptr2 & 0xc0) == 0xc0) return (ptr2 - nptr) + 2 ;
		ptr2	+= 1 + *ptr2 ;
	}

	return	(ptr2 - nptr) + 1 ;
}

/*  DNSQueryLen	: Calculate query block length				*/
/*  qptr	: u_char *	: Pointer at query block		*/
/*  (returns)	: int		: Length in bytes			*/

int	DNSQueryLen
	(	u_char	*qptr
	)
{
	return	DNSNameLen (qptr) + 4 ;
}

/*  DNSQResourceLen: Calculate resource block length			*/
/*  qptr	   : u_char *	: Pointer at query block		*/
/*  (returns)	   : int	: Length in bytes			*/

int	DNSResourceLen
	(	u_char	*rptr
	)
{
	u_char	*lptr	= rptr + DNSNameLen (rptr) + 8 ;
	int	rdlen	= GetShort (lptr) ;

	return	(lptr + 2 + rdlen) - rptr ;
}

/*  DNSBuildName: Construct textual name				*/
/*  name	: QString &	: Name result string			*/
/*  hdr		: u_char *	: Pointer at DNS header			*/
/*  ptr		: u_char *	: Name pointer				*/
/*  (returns)	: u_char *	: Pointer just past name		*/

u_char	*DNSBuildName
	(	QString	&name,
		u_char	*hdr,
		u_char	*ptr
	)
{
	u_char	*ptr2	= ptr ;

	while (*ptr2 != 0)
	{
		if (ptr2 != ptr) name += (char)'.' ;

		if ((*ptr2 & 0xc0) == 0xc0)
		{	DNSBuildName (name, hdr, hdr + (GetShort (ptr2) & 0x3fff)) ;
			return	ptr2 + 2 ;
		}

		for (int idx = 1 ; idx <= *ptr2 ; idx += 1)
			name += (char)ptr2[idx] ;

		ptr2 += *ptr2 + 1 ;
	}

	return	ptr2 + 1 ;
}

static	u_char	*DNSaddItem
	(	KNDDetailItem	*parent,
		PktInfo		*detpkt,
		u_char		*arg
	)
{
	QString	aName	= ""	;
	DNSHDR	*hdr	= (DNSHDR  *)(detpkt->ipdata + sizeof(UDP_HDR)) ;
	u_char	*ptr	= DNSBuildName (aName, (u_char *)hdr, arg) ;
	CodeMap	*dMap	;

	NX(Leaf, "Name",	R3(arg, ptr - arg), aName) ;
	NX(Leaf, "Type",	R3(&ptr[0], 2),     codeMapScan (dnsQTypeMap,  GetShort(&ptr[0]), dMap)) ;
	NX(Leaf, "Class",	R3(&ptr[2], 2),     codeMapScan (dnsQClassMap, GetShort(&ptr[2]), dMap)) ;

	return	ptr + 4 ;
}

/*  KNDDetDNSQuery:							*/
/*  addItem	  : Add item details					*/
/*  (returns)	  : void	:					*/

void	KNDDetDNSQuery::addItem ()
{
	DNSaddItem (this, detpkt, arg) ;
}

/*  KNDDetDNSAnswer:							*/
/*  addItem	   : Add item details					*/
/*  (returns)	   : void	:					*/

void	KNDDetDNSAnswer::addItem ()
{
	DNSHDR	*hdr	= (DNSHDR  *)(detpkt->ipdata + sizeof(UDP_HDR)) ;
	u_char	*ptr	= DNSaddItem (this, detpkt, arg) ;
	uint	type	= GetShort   (ptr - 4)		 ;
	u_char	*ptr2	;
	QString	aName	;
	CodeMap	*dMap	;

	NI(Leaf, "TimeToLive", R3 (&ptr[0], 4), _F ("%ld", GetLong (&ptr[0]))) ;
	NI(Leaf, "Length",     R3 (&ptr[4], 2), _F ("%d",  GetShort(&ptr[4]))) ;
	ptr += 6 ;

	/* The remained of the answer block depends on the type code.	*/
	/* Currently, not all record types are decoded.			*/
	switch (type)
	{
		case T_A	:
			NI(Leaf, "Address",
				 R3 (ptr, 4),
				 _F ("%d.%d.%d.%d", ptr[0], ptr[1], ptr[2], ptr[3])) ;
			break	;

		case T_NS	:
		case T_PTR	:
		case T_CNAME	:
			aName	= "" ;
			ptr2	= DNSBuildName (aName, (u_char *)hdr, ptr) ;
			NI(Leaf, "Name",    R3 (ptr, ptr2 - ptr), aName) ;
			break	;

		case T_SOA	:
			aName	= "" 	;
			ptr2	= DNSBuildName (aName, (u_char *)hdr, ptr) ;
			NI(Leaf, "MName",   R3 (ptr, ptr2 - ptr), aName) ;
			ptr	= ptr2	;
			aName	= "" 	;
			ptr2	= DNSBuildName (aName, (u_char *)hdr, ptr) ;
			NI(Leaf, "RName",   R3 (ptr, ptr2 - ptr), aName) ;
			ptr	= ptr2	;
			NI(Leaf, "Serial",  R3 (&ptr[ 0], 4), _F ("%ld", GetLong(&ptr[ 0]))) ; 
			NI(Leaf, "Refresh", R3 (&ptr[ 4], 4), _F ("%ld", GetLong(&ptr[ 4]))) ; 
			NI(Leaf, "Retry",   R3 (&ptr[ 8], 4), _F ("%ld", GetLong(&ptr[ 8]))) ; 
			NI(Leaf, "Expire",  R3 (&ptr[12], 4), _F ("%ld", GetLong(&ptr[12]))) ; 
			NI(Leaf, "Min TTL", R3 (&ptr[16], 4), _F ("%ld", GetLong(&ptr[16]))) ; 
			break	;

		default	:
			NI(Leaf, "RData",
				 R3 (ptr, 0),
				 codeMapScan (dnsQTypeMap, type, dMap)) ;
			break	;
	}
	
}

/*  KNDDetDNSHdr:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetDNSHdr::addItem ()
{
	DNSHDR	*hdr	= (DNSHDR  *)(detpkt->ipdata +  sizeof(UDP_HDR)) ;
	QPoint	rf	= R3 ((u_char *)(&hdr->id) + sizeof(hdr->id), 2) ;
//	int	word2	= ntohs(hdr->word2) ;
	CodeMap	*dMap	;

	NI(Leaf, "Ident",	R3(&hdr->id, 2), _F ("%d", ntohs(hdr->id))) ;
	NI(Leaf, "QR Flag",	rf, hdr->qr ? "Response" : "Query") ;
	NI(Leaf, "Opcode",	rf, codeMapScan (dnsOpcodeMap, hdr->opcode,   dMap)) ;
	NI(Leaf, "AA Flag",	rf, hdr->aa ? "Authoritative" : "Non-Authoritative") ;
	NI(Leaf, "TC Flag",	rf, hdr->tc ? "Truncated"     : "OK"	           ) ;
	NI(Leaf, "RD Flag",	rf, hdr->rd ? "Recursive"     : "Non-Recursive"    ) ;
	NI(Leaf, "RA Flag",	rf, hdr->ra ? "Recursive"     : "Non-Recursive"    ) ;
	NI(Leaf, "AD Flag",	rf, hdr->ad ? "Authentic"     : "Non-Authentic"    ) ;
	NI(Leaf, "CD Flag",	rf, hdr->ad ? "Disabled"      : "Enabled"          ) ;
	NI(Leaf, "RCode",	rf, codeMapScan (dnsRCodeMap,  hdr->rcode,     dMap)) ;
	NI(Leaf, "Query Count",	R3(&hdr->qdcount, 2), _F ("%d", ntohs(hdr->qdcount))) ;
	NI(Leaf, "Answer Count",R3(&hdr->ancount, 2), _F ("%d", ntohs(hdr->ancount))) ;
	NI(Leaf, "Auth Count",	R3(&hdr->nscount, 2), _F ("%d", ntohs(hdr->nscount))) ;
	NI(Leaf, "Extra Count",	R3(&hdr->arcount, 2), _F ("%d", ntohs(hdr->arcount))) ;
}

/*  KNDDetDNS	:							*/
/*  addItem	: Add item details					*/
/*  (returns)	: void		:					*/

void	KNDDetDNS::addItem ()
{
	DNSHDR	*hdr	= (DNSHDR  *)(detpkt->ipdata + sizeof(UDP_HDR)) ;
	u_char	*dptr	= (u_char  *)(detpkt->ipdata + sizeof(UDP_HDR) + sizeof(DNSHDR)) ;

	NI(DNSHdr, "Header",	R3(hdr, sizeof(DNSHDR)), "") ;

	for (int qd = 0 ; qd < ntohs(hdr->qdcount) ; qd += 1)
	{	int qlen = DNSQueryLen (dptr) ;
		NA(DNSQuery,  _F("Query %d",  qd + 1), R3(dptr, qlen), "", dptr) ;
		dptr += qlen ;
	}

	for (int an = 0 ; an < ntohs(hdr->ancount) ; an += 1)
	{	int alen = DNSResourceLen (dptr) ;
		NA(DNSAnswer, _F("Answer %d", an + 1), R3(dptr, alen), "", dptr) ;
		dptr += alen ;
	}

	for (int ns = 0 ; ns < ntohs(hdr->nscount) ; ns += 1)
	{	int alen = DNSResourceLen (dptr) ;
		NA(DNSAnswer, _F("Auth %d",   ns + 1), R3(dptr, alen), "", dptr) ;
		dptr += alen ;
	}

	for (int ar = 0 ; ar < ntohs(hdr->arcount) ; ar += 1)
	{	int alen = DNSResourceLen (dptr) ;
		NA(DNSAnswer, _F("Extra %d",  ar + 1), R3(dptr, alen), "", dptr) ;
		dptr += alen ;
	}
}

/*  DNSGetQType	: Look up query type					*/
/*  type	: int		: Type code				*/
/*  (returns)	: cchar *	: Query type as string			*/

cchar	*DNSGetQType
	(	int	type
	)
{
	CodeMap	*dmap	;
	return	codeMapScan (dnsQTypeMap, type, dmap) ;
}
