/***************************************************************************
                          knd_event.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<qtooltip.h>

#include	"ksnuffle.h"
#include	"knd_event.h"
#include	"knd_view.h"
#include	"knd_filter.h"

/*  KNDEvent								*/
/*  KNDEvent	: Constructor for event settings object			*/
/*  parent	: KNDView *	: Parent				*/
/*  (returns)	: KNDEvent	:					*/

KNDEvent::KNDEvent
	(	KNDView		*_parent,
		KConfig		*config
	)
	:
	KNDPlugin  (_parent, "KND_EVENTS"),
	k_tabctl   (this)
{
	k_tabctl.setGeometry (135, 5, 415, 210) ;

	for (int idx = 0 ; idx < NEVENT ; idx += 1)
	{
		char	prefix	[ 8] ;
		char	label	[16] ;

		sprintf	(prefix, "e%d",      idx + 1) ;
		sprintf	(label,  "Event %d", idx + 1) ;

		events[idx] = new KNDFilter (&k_tabctl, prefix, config, true) ;
		k_tabctl.addTab (events[idx], label ) ;
	}
}

/*  KNDEvent								*/
/*  ~KNDEvent	: Destructor for packet display object			*/
/*  (returns)	:		:					*/

KNDEvent::~KNDEvent ()
{
	for (int idx = 0 ; idx < NEVENT ; idx += 1)
		delete	events[idx] ;
}

/*  KNDEvent								*/
/*  clickSet	: Handle set event					*/
/*  (returns)	: void		:					*/

void	KNDEvent::clickSet ()
{
	for (int idx = 0 ; idx < NEVENT ; idx += 1)
		events[idx]->clickSet () ;
}

/*  KNDEvent								*/
/*  clickClear	: Handle clear button					*/
/*  (returns)	: void		:					*/

void	KNDEvent::clickClear ()
{
	for (int idx = 0 ; idx < NEVENT ; idx += 1)
		events[idx]->clickClear () ;
}

/*  KNDEvent								*/
/*  clickReset	: Handle reset button					*/
/*  (returns)	: void		:					*/

void	KNDEvent::clickReset ()
{
	for (int idx = 0 ; idx < NEVENT ; idx += 1)
		events[idx]->clickReset () ;
}

/*  KNDEvent								*/
/*  writeConfig	: Write configuration					*/
/*  config	: KConfig *	: Configuration object			*/
/*  (returns)	: void		:					*/

void	KNDEvent::writeConfig
	(	KConfig	*config
	)
{
	for (int idx = 0 ; idx < NEVENT ; idx += 1)
		events[idx]->writeConfig (config) ;
}

/*  KNDEvent								*/
/*  getInfo	: Get event settings					*/
/*  (returns)	: EventInf *	: Information				*/

EventInf *KNDEvent::getInfo ()
{
	/* Get all set events. If there is no filter or no command to	*/
	/* execute then the event is ignored.				*/
	for (int idx = 0 ; idx < NEVENT ; idx += 1)
	{
		cchar	*event	= events[idx]->getProgram () ;
		cchar	*exec	= events[idx]->getExec    () ;

		if (!StrIsSet(event) || !StrIsSet(exec))
			event = exec = NULL ;

		eventinf.event[idx].event = event ;
		eventinf.event[idx].exec  = exec  ;
	}

	return	&eventinf ;
}
