/***************************************************************************
                          knd_filter.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<qtooltip.h>

#include	"knd_filter.h"
#include	"pcaphdr.h"
#include	"capture.h"


/*  VerifyPCap	: Verify packet capture program				*/
/*  prog	: QString	: Program in question			*/
/*  (returns)	: bool		: True if verified OK			*/

bool	VerifyPCap
	(	QString	prog
	)
{
	char	ebuf	[PCAP_ERRBUF_SIZE] ;
	BPFI32	locnet	;
	BPFI32	netmask	;
	BPFPROG	pr	;
	pcap_t	*pd	;

	char	*iface	= "eth0" ;
	int	snaplen = 68	 ;

	if ((pd = pcap_open_live ((char *)iface, snaplen, 1, 1000, ebuf)) == NULL)
	{	Error	("Ksnuffle error",   ebuf) ;
		return	false ;
	}

	if (pcap_lookupnet ((char *)iface, &locnet, &netmask, ebuf))
	{	Error	(QMessageBox::Information, "Ksnuffle warning", ebuf) ;
		pcap_close (pd) ;
	}	

	memset	(&pr, 0, sizeof(BPFPROG)) ;

	if (!CapCompile (pd, &pr, prog, 0, netmask))
	{	Error	("Ksnuffle filter error",  pcap_geterr (pd)) ;
		pcap_close (pd) ;
		return	false	;
	}

	pcap_close  (pd) ;
	return	true	 ;
}

/*  KNDFilter								*/
/*  KNDFilter	: Constructor for filter program object			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  prefix	: const char *	: Configuration prefix			*/
/*  config	: KConfig *	: Initial configuration or NULL if none	*/
/*  event	: bool		: This is an event object		*/
/*  (returns)	: KNDFilter	:					*/

KNDFilter::KNDFilter
	(	QWidget	*parent,
		cchar	*prefix,
		KConfig	*config,
		bool	event
	)
	:
	QWidget	(parent),
	c_prog	(this),
	l_proto	(this),	s_epro	(this),	s_ipro	(this),
	s_host1	(this),	t_host1	(this),	s_port1	(true, this),
	s_host2	(this),	t_host2	(this),	s_port2	(true, this),
	s_andor	(this),
	t_prog	(this),
	b_verify(this),
	l_exec	(this),
	t_exec	(this),
	prefix	(prefix),
	event	(event)
{
	replay	= false	;

	c_prog .setGeometry	  (315,  20,  90,  25) ;
	c_prog .setText		  ("Use program") ;
	c_prog .setChecked	  (false ) ;
	b_prog	= false	;

	connect(&c_prog, SIGNAL(toggled(bool)), this, SLOT(clickProg(bool))) ;
	QToolTip::add (&c_prog, i18n ("Monitor with user-supplied filter program")) ;

	l_proto.setGeometry	  (  5,  20,  60,  25) ;
	l_proto.setText		  ("Protocols") ;
	s_epro .setGeometry	  ( 75,  20,  80,  25) ;
	s_ipro .setGeometry	  (165,  20,  80,  25) ;
	v_epro	= "" ;
	v_ipro	= "" ;

	for (unsigned int epno = 0 ; epno < eplist->size() ; epno += 1)
		s_epro.insertItem ((*eplist)[epno]) ;
	for (unsigned int ipno = 0 ; ipno < iplist->size() ; ipno += 1)
		s_ipro.insertItem ((*iplist)[ipno]) ;

	QToolTip::add (&s_epro, i18n ("Select network protocol" )) ;
	QToolTip::add (&s_ipro, i18n ("Select internet protocol")) ;

	connect	(&s_epro, SIGNAL(activated(int)), this, SLOT(deltaEProto(int))) ;

	s_host1.setGeometry	  (  5,  50,  60,  25) ;
	s_host1.insertItem	  ("Host", (int)Host ) ;
	s_host1.insertItem	  ("Srce", (int)Srce ) ;
	s_host1.insertItem	  ("Dest", (int)Dest ) ;
	t_host1.setGeometry	  ( 75,  50, 260,  25) ;
	s_port1.setGeometry	  (345,  50,  60,  25) ;
	v_host1	= "" 	;
	m_host1	= Host	;
	QToolTip::add (&s_host1, i18n ("Endpoint is source, destination, or either")) ;
	QToolTip::add (&t_host1, i18n ("Name or address of endpoint")) ;
	QToolTip::add (&s_port1, i18n ("Service name or port number of endpoint")) ;

	s_host2.setGeometry	  (  5,  80,  60,  25) ;
	s_host2.insertItem	  ("Host", (int)Host ) ;
	s_host2.insertItem	  ("Srce", (int)Srce ) ;
	s_host2.insertItem	  ("Dest", (int)Dest ) ;
	t_host2.setGeometry	  ( 75,  80, 260,  25) ;
	s_port2.setGeometry	  (345,  80,  60,  25) ;
	v_host2	= ""	;
	m_host2	= Host	;
	QToolTip::add (&s_host2, i18n ("Endpoint is source, destination, or either")) ;
	QToolTip::add (&t_host2, i18n ("Name or address of endpoint")) ;
	QToolTip::add (&s_port2, i18n ("Service name or port number of endpoint")) ;

	for (unsigned int svno = 0 ; svno < svlist->size() ; svno += 1)
	{	s_port1.insertItem ((*svlist)[svno]) ;
		s_port2.insertItem ((*svlist)[svno]) ;
	}

	s_port1.setInsertionPolicy (QComboBox::AtTop) ;
	s_port2.setInsertionPolicy (QComboBox::AtTop) ;
	QToolTip::add (&s_andor, i18n ("Logical combination of endpoints")) ;

	s_andor.setGeometry	  (255,  20,  50,  25) ;
	s_andor.insertItem	  ("And", (int)And)    ;
	s_andor.insertItem	  ("Or",  (int)Or )    ;
	m_andor	= And	;

	t_prog .setGeometry	  ( 75, 115, 330,  25) ;
	t_prog .setEnabled	  (false  ) ;
	v_prog	= "" ;
	QToolTip::add (&t_prog, i18n ("Explicitely entered filter program")) ;

	b_verify.setGeometry	  (  5, 115,  60,  25) ;
	b_verify.setText	  ("Verify") ;
	QToolTip::add (&b_verify, i18n("Verify filter program")) ;
	connect(&b_verify, SIGNAL(clicked()), this, SLOT(verify())) ;

	if (event)
	{	l_exec.setGeometry ( 5, 145,  60, 25) ;
		l_exec.setText	   ("Command")	      ;
		t_exec.setGeometry (75, 145, 330, 25) ;
		v_exec = "" ;
		QToolTip::add (&t_exec, i18n("Command to execute on event")) ;
	}
	else
	{	l_exec.hide () ;
		t_exec.hide () ;
	}

	if (config != NULL)
	{
		v_epro	 = config->readEntry	 (Pfx ("EtherProto"   ), (*eplist)[0]) ;
		v_ipro	 = config->readEntry	 (Pfx ("InternetProto"), (*iplist)[0]) ;
		v_host1	 = config->readEntry	 (Pfx ("Host1_Name"   ), ""  ) ;
		v_host2	 = config->readEntry	 (Pfx ("Host2_Name"   ), ""  ) ;
		v_port1	 = config->readEntry	 (Pfx ("Host1_Port"   ), (*svlist)[0]) ;
		v_port2	 = config->readEntry	 (Pfx ("Host2_Port"   ), (*svlist)[0]) ;
		v_prog	 = config->readEntry	 (Pfx ("ProgramText"  ), ""  ) ;
		v_exec	 = config->readEntry	 (Pfx ("EventExec"    ), ""  ) ;

		m_host1	 = (HostMode)config->readNumEntry (Pfx ("Host1_Mode"),  (int)Host) ;
		m_host2	 = (HostMode)config->readNumEntry (Pfx ("Host2_Mode"),  (int)Host) ;
		m_andor	 = (JoinMode)config->readNumEntry (Pfx ("Hosts_AndOr"), (int)And ) ;

		clickReset () ;
	}
}

/*  KNDFilter								*/
/*  KNDFilter	: Constructor for filter program object for replay	*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KNDFilter	:					*/

KNDFilter::KNDFilter
	(	QWidget		*parent
	)
	:
	QWidget	(parent),
	c_prog	(this),
	l_proto	(this),	s_epro	(this),	s_ipro	(this),
	s_host1	(this),	t_host1	(this),	s_port1	(true, this),
	s_host2	(this),	t_host2	(this),	s_port2	(true, this),
	s_andor	(this),
	t_prog	(this),
	prefix	(NULL)
{
	replay	= true	;

	c_prog .hide () ;
	l_proto.hide () ;
	s_epro .hide () ;
	s_ipro .hide () ;
	s_host1.hide () ;
	s_host2.hide () ;
	t_host1.hide () ;
	t_host2.hide () ;
	s_port1.hide () ;
	s_port2.hide () ;
	s_andor.hide () ;

	t_prog .setGeometry	  (  5, 115, 400,  25) ;
	t_prog .setEnabled	  (true  ) ;
	t_prog .setFocusPolicy	  (QWidget::NoFocus) ;
}

/*  KNDFilter								*/
/*  Pfx		: Prefix identifier					*/
/*  ident	: char *	: Identifier				*/
/*  (returns)	: QString &	: Static prefixed identifier		*/

QString	&KNDFilter::Pfx
	(	char	*ident
	)
{
	static	QString	string	;

	if (prefix != NULL)
	{	string	 = prefix ;
		string	+= "_"	  ;
		string	+= ident  ;
	}
	else	string	 = ident  ;

	return	string	; 
}

/*  KNDFilter								*/
/*  clickReset	: Handle a click of the reset button			*/
/*  (returns)	: void		:					*/

void	KNDFilter::clickReset ()
{
	if (!replay)
	{
		/* Note that this button simply restores the		*/
		/* configuration to the current values, so the		*/
		/* runningfrozen state is not affected.			*/
		t_host1.setText	(v_host1) ;
		t_host2.setText	(v_host2) ;
		t_exec .setText (v_exec ) ;
		t_prog .setText (v_prog ) ;

		s_host1.setCurrentItem((int)m_host1) ;
		s_host2.setCurrentItem((int)m_host2) ;
		s_andor.setCurrentItem((int)m_andor) ;
		s_port1.setCurrentItem(QCBFindItem(s_port1, v_port1, 0)) ;
		s_port2.setCurrentItem(QCBFindItem(s_port2, v_port2, 0)) ;
		s_epro .setCurrentItem(QCBFindItem(s_epro,  v_epro,  0)) ;
		s_ipro .setCurrentItem(QCBFindItem(s_ipro,  v_ipro,  0)) ;

		deltaEProto(s_epro.currentItem()) ;
	}
}

/*  KNDFilter								*/
/*  addHost	: Add a host to the monitor program			*/
/*  prog	: QString &	: Program under construction		*/
/*  mode	: HostMode	: Host mode (host/srce/dest)		*/
/*  host	: QString &	: Host name				*/
/*  port	: QString &	: Port name				*/
/*  (returns)	: void		:					*/

void	KNDFilter::addHost
	(	QString	 &prog,
		HostMode mode,
		QString	 &host,
		QString	 &port
	)
{
	bool	both	= !host.isEmpty() && !port.isEmpty () ;

	if (both) prog += "( " ;

	if (!host.isEmpty()) 
	{
		switch (mode)
		{	case Host : prog += "host " ; break ;
			case Srce : prog += "src "  ; break ;
			case Dest : prog += "dst "  ; break ;
			default	  :		      break ;
		}

		prog	+= host ;
		prog	+= " "	;
	}

	if (both) prog += "and " ;

	if (!port.isEmpty())
	{
		switch (mode)
		{	case Host :		      break ;
			case Srce : prog += "src "  ; break ;
			case Dest : prog += "dst "  ; break ;
			default	  :		      break ;
		}

		prog	+= "port " ;
		prog	+= port	   ;
		prog	+= " "	   ;
	}

	if (both) prog += ") " ;
}

/*  KNDFilter								*/
/*  buildProg	: Build program string					*/
/*  (returns)	: QString	: Program string			*/

QString	KNDFilter::buildProg ()
{
	QString	 program	= ""	;
	bool	 any		= false	;

	QString	 _v_epro	= s_epro .currentText() ;
	QString	 _v_ipro	= s_ipro .currentText() ;
	QString	 _v_host1	= t_host1.text	     () ;
	QString	 _v_host2	= t_host2.text	     () ;
	QString	 _v_port1	= s_port1.currentText() ;
	QString	 _v_port2	= s_port2.currentText() ;
	QString	 _v_prog	= t_prog .text	     () ;

	HostMode _m_host1	= (HostMode)s_host1.currentItem() ;
	HostMode _m_host2	= (HostMode)s_host2.currentItem() ;
	JoinMode _m_andor	= (JoinMode)s_andor.currentItem() ;

	if ((strcmp (_v_epro, "") != 0) && (strcmp (_v_epro, "ip") != 0))
	{	program += _v_epro	;
		program += " "		;
		any	 = true		;
	}
	else if (strcmp (_v_ipro, "") != 0)
	{	program += "proto \\"	;
		program += _v_ipro	;
		program += " "		;
		any	 = true		;
	}

	bool	end1	= !_v_host1.isEmpty() || !_v_port1.isEmpty() ;
	bool	end2	= !_v_host2.isEmpty() || !_v_port2.isEmpty() ;

	if	(end1 && end2)
	{
		if (any) program += "and " ;

		program	+= "( " ;
		addHost (program, _m_host1, _v_host1, _v_port1) ;

		switch (_m_andor)
		{	case And : program += "and " ; break ;
			case Or	 : program += "or "  ; break ;
			default	 :		       break ;
		}

		addHost	(program, _m_host2, _v_host2, _v_port2) ;
		program	+= ") " ;
	}
	else if (end1)
	{
		if (any) program += "and " ;
		addHost	(program, _m_host1, _v_host1, _v_port1) ;
	}
	else if (end2)
	{
		if (any) program += "and " ;
		addHost	(program, _m_host2, _v_host2, _v_port2) ;
	}

	return	program ;
}

/*  KNDFilter								*/
/*  clickSet	: Handle a click of the set button			*/
/*  (returns)	: void		:					*/

void	KNDFilter::clickSet ()
{
	v_epro	= s_epro .currentText() ;
	v_ipro	= s_ipro .currentText() ;
	v_host1	= t_host1.text	     () ;
	v_host2	= t_host2.text	     () ;
	v_port1	= s_port1.currentText() ;
	v_port2	= s_port2.currentText() ;
	v_prog	= t_prog .text	     () ;
	v_exec	= t_exec .text	     () ;

	m_host1	= (HostMode)s_host1.currentItem() ;
	m_host2	= (HostMode)s_host2.currentItem() ;
	m_andor	= (JoinMode)s_andor.currentItem() ;

	if (!b_prog) t_prog.setText (v_prog = buildProg()) ;
}

/*  KNDFilter								*/
/*  clickClear	: Handle a click of the clear button			*/
/*  (returns)	: void		:					*/

void	KNDFilter::clickClear ()
{
	t_host1.setText	  	("")	  ;
	t_host2.setText	  	("")	  ;
	t_exec .setText		("")	  ;
	s_host1.setCurrentItem	((int)Host) ;
	s_host2.setCurrentItem	((int)Host) ;
	s_andor.setCurrentItem	((int)And ) ;
	s_port1.setCurrentItem	(0)	  ;
	s_port2.setCurrentItem	(0)	  ;
	s_ipro .setCurrentItem  (0)	  ;
	s_epro .setCurrentItem  (0)	  ;
	s_epro .setEnabled	(true)	  ;
}

/*  KNDFilter								*/
/*  clickProg	: Handle change in program checkbox			*/
/*  on		: bool		: TRUE if checkbox is checked		*/
/*  (returns)	: void		:					*/

void	KNDFilter::clickProg
	(	bool	on
	)
{
	if (!replay)
	{
		b_prog = on ;
		t_prog .setEnabled ( on) ;
		s_epro .setEnabled (!on) ;
		s_ipro .setEnabled (!on) ;
		t_host1.setEnabled (!on) ;
		t_host2.setEnabled (!on) ;
		s_host1.setEnabled (!on) ;
		s_host2.setEnabled (!on) ;
		s_port1.setEnabled (!on) ;
		s_port2.setEnabled (!on) ;
		s_andor.setEnabled (!on) ;
	}
}

/*  KNDFilter								*/
/*  deltaEProto	: Handle change of ethernet protocol setting		*/
/*  index	: int		: Selected index			*/
/*  (returns)	: void		:					*/

void	KNDFilter::deltaEProto
	(	int	index
	)
{
	QString	eproto	= s_epro.currentText () ;
	s_ipro.setEnabled ( (strcmp (eproto, ""  ) == 0) ||
			    (strcmp (eproto, "ip") == 0) ) ;
}

/*  KNDFilter								*/
/*  writeConfig	: Write monitor configuration				*/
/*  config	: KConfig *	: Configuration object			*/
/*  (returns)	: void		:					*/

void	KNDFilter::writeConfig
	(	KConfig	*config
	)
{
	if (!replay)
	{
		config->writeEntry (Pfx ("ProgramText"),	v_prog  ) ;
		config->writeEntry (Pfx ("EtherProto"),		v_epro	) ;
		config->writeEntry (Pfx ("InternetProto"),	v_ipro	) ;
		config->writeEntry (Pfx ("Host1_Name"),		v_host1 ) ;
		config->writeEntry (Pfx ("Host2_Name"),		v_host2	) ;
		config->writeEntry (Pfx ("Host1_Port"),		v_port1 ) ;
		config->writeEntry (Pfx ("Host2_Port"),  	v_port2 ) ;
		config->writeEntry (Pfx ("EventExec"), 		v_exec  ) ;

		config->writeEntry (Pfx ("Host1_Mode"),		(int)m_host1) ;
		config->writeEntry (Pfx ("Host2_Mode"), 	(int)m_host2) ;
		config->writeEntry (Pfx ("Hosts_AndOr"),  	(int)m_andor) ;

		config->writeEntry (Pfx ("UseProgramText"),	b_prog	) ;
	}
}

/*  KNDFilter								*/
/*  verify	: Verify program					*/
/*  (returns)	: void		:					*/

void	KNDFilter::verify ()
{
	if (VerifyPCap (c_prog.isChecked() ? (QString)t_prog.text() : buildProg()))
	{
		QMessageBox mb (i18n("KSnuffle verify filter"),
				i18n("Filter program OK"),
				QMessageBox::Information,
				QMessageBox::Ok, 0, 0) ;
		mb.exec () ;
	}
}
