/***************************************************************************
                          knd_gra.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	"pcaphdr.h"
#include	"knd_gra.h"
#include	"knd_view.h"
#include	"knd_load.h"

/*  intToExp	: Convert value to xEy equivalent			*/
/*  value	: int		: Value in question			*/
/*  buff	: char *	: Text buffer				*/
/*  (returns)	: char *	: Text buffer				*/

static	char	*intToExp
	(	int	value,
		char	*buff
	)
{
	/* Note that this routine assumes that the value can be		*/
	/* expressed in the form xEy where x is an integer in the range	*/
	/* 1...9. If not, then it will not work properly, on account of	*/
	/* the integer truncation in the final division.		*/
	int	p10	;
	int	mul	;
	int	exp	;

	if (value > 10000)
	{	for (p10 = 1, exp = 0 ; p10 * 10 <= value ; p10 *= 10, exp += 1) ;
		mul	= value / p10 ;
		sprintf	(buff, "%dE%d", mul, exp) ;
	}
	else	sprintf	(buff, "%d", value) ;

	return	buff ;

}


/*  KNDGraGrp								*/
/*  KNDGraGrp	: Constructor for graphics object group			*/
/*  parent	: QFrame *	: Parent frame				*/
/*  title	: char *	: Group title format			*/
/*  (returns)	: KNDGraGrp	:					*/

KNDGraGrp::KNDGraGrp
	(	QFrame	*parent,
		char	*title
	)
	:
	QWidget	(parent, NULL),
	g_box	(this),
	l_hist	(this),
	l_bar1	(this),
	l_bar2	(this),
	title	(title)
{
	g_box	.setTitle     (" ")	    ;
	l_hist	.setAlignment (AlignCenter) ;
	l_bar1	.setAlignment (AlignCenter) ;
	l_bar2	.setAlignment (AlignCenter) ;
	hide	()	;
	hist	= NULL	;
	bar1	= NULL	;
	bar2	= NULL	;
}

/*  KNDGraGrp								*/
/*  resize	: Resize group						*/
/*  size	: QSize		: New size				*/
/*  (returns)	: void		:					*/

void	KNDGraGrp::resize
	(	QSize	size
	)
{
	int	hwid	= size.width ()
			  - 10			/* Outside group box	*/
			  - 10			/* Right-hand edge	*/
			  - 2 * (25 + 10) ;	/* Space for bars	*/

	int	vdep	= size.height()
			  - 5			/* Space at top		*/
			  - 10 ;		/* Space at bottom	*/

	int	over	;
	char	text[32];

	sprintf	(text, "%d secs", hwid * v_hist) ;
	l_hist	.setText (text) ;
	l_bar1	.setText (ItoA(v_bar1)) ;
	l_bar2	.setText (ItoA(v_bar2)) ;

	over	 = 5 ;
	g_box.setGeometry    (over,  5, size.width() - 10, vdep) ;

	over	+= 5	     ;
	hist  ->setGeometry  (over, 25, hwid, vdep - 45) ;
	l_hist .setGeometry  (over, size.height () - 35, hwid, 20) ; 

	over	+= hwid + 10 ;
	bar1  ->setGeometry  (over, 25, 25,   vdep - 45) ;
	l_bar1 .setGeometry  (over, size.height () - 35, 25,   20) ;

	over	+= 25   + 10 ;
	bar2  ->setGeometry  (over, 25, 25,   vdep - 45) ;
	l_bar2 .setGeometry  (over, size.height () - 35, 25,   20) ;
}

/*  KNDGraGrp								*/
/*  setup	: Set up with controls and values			*/
/*  size	: QSize		: Initial size				*/
/*  _v_hist	: int		: Histogram step period			*/
/*  _v_bar1	: int		: Time constant for first bar		*/
/*  _v_bar2	: int		: Time constant for second bar		*/
/*  _hist	: QFrame *	: Histogram				*/
/*  _bar1	: QFrame *	: First load bar			*/
/*  _bar2	: QFrame *	: Second load bar			*/
/*  (returns)	: void		:					*/

void	KNDGraGrp::setup
	(	QSize	size,
		int	_v_hist,
		int	_v_bar1,
		int	_v_bar2,
		QFrame	*_hist,
		QFrame	*_bar1,
		QFrame	*_bar2
	)
{
	hist	= _hist		;
	bar1	= _bar1		;
	bar2	= _bar2		;
	v_hist	= _v_hist	;
	v_bar1	= _v_bar1	;
	v_bar2	= _v_bar2	;

	resize	(size)	;
	show	()	;
	hist->show ()   ;
	bar1->show ()   ;
	bar2->show ()   ;
}

/*  KNDGraGrp								*/
/*  setMax	: Set scale maximum					*/
/*  maxv	: char *	: Maximum value text			*/
/*  (returns)	: void		:					*/

void	KNDGraGrp::setMax
	(	char	*maxv
	)
{
	char	text[64] ;
	sprintf	(text, title, maxv) ;
	g_box.setTitle (text) ;
}

/*  KNDGraphic								*/
/*  KNDGraphic	: Constructor for graphic display			*/
/*  parent	: KNDView *	: Parent object				*/
/*  (returns)	: KNDGraphic	:					*/

KNDGraphic::KNDGraphic
	(	KNDView	*parent
	)
	:
	KNDPlugin	(parent, "KND_GRAPHIC"),
	load		(this, "Load: %s bytes/sec"),
	pkts		(this, "Load: %s packets/sec")
{
	going	  = false ;
	frozen	  = false ;

	loadHist  = NULL  ;
	loadBar1  = NULL  ;
	loadBar2  = NULL  ;
	pktHist	  = NULL  ;
	pktBar1	  = NULL  ;
	pktBar2	  = NULL  ;
}

/*  KNDGraphic								*/
/*  ~KNDGraphic	: Destructor for graphic display			*/
/*  (returns)	:		:					*/

KNDGraphic::~KNDGraphic ()
{
	if (loadHist != NULL) delete loadHist ;
	if (loadBar1 != NULL) delete loadBar1 ;
	if (loadBar2 != NULL) delete loadBar2 ;
	if (pktHist  != NULL) delete pktHist  ;
	if (pktBar1  != NULL) delete pktBar1  ;
	if (pktBar2  != NULL) delete pktBar2  ;
}
/*  KNDGraphic								*/
/*  setOptions	: Set display options					*/
/*  _lvopts	: void *	: Options				*/
/*  (returns)	: void *	:					*/

void	*KNDGraphic::setOptions
	(	void	*_lvopts
	)
{
	/* Destroy the existing load histograms and load bars if such	*/
	/* exist ....							*/
	if (loadHist != NULL) delete loadHist ;
	if (loadBar1 != NULL) delete loadBar1 ;
	if (loadBar2 != NULL) delete loadBar2 ;
	if (pktHist  != NULL) delete pktHist  ;
	if (pktBar1  != NULL) delete pktBar1  ;
	if (pktBar2  != NULL) delete pktBar2  ;

	v_hist	  = ((GenConfig *)_lvopts)->v_load ;
	v_bar1	  = ((GenConfig *)_lvopts)->v_bar1 ;
	v_bar2	  = ((GenConfig *)_lvopts)->v_bar2 ;

	loadScale = MINLOAD - 1 ;
	pktScale  = MINPCNT - 1 ;


	loadHist  = new KNDLoadHist (&load, v_hist, 400) ;
	loadBar1  = new KNDLoadBar  (&load, v_bar1) ;
	loadBar2  = new KNDLoadBar  (&load, v_bar2) ;
	pktHist   = new KNDPktHist  (&pkts, v_hist, 400) ;
	pktBar1   = new KNDPktBar   (&pkts, v_bar1) ;
	pktBar2   = new KNDPktBar   (&pkts, v_bar2) ;

	int	w	= (width() - 15) / 2 ;

	load.setGeometry (5,	  5, w, height() - 10) ;
	pkts.setGeometry (w + 15, 5, w, height() - 10) ;

	load.setup (QSize (w, height()), v_hist,   v_bar1,   v_bar2,
					 loadHist, loadBar1, loadBar2) ;
	pkts.setup (QSize (w, height()), v_hist,   v_bar1,   v_bar2,
					 pktHist,  pktBar1,  pktBar2 ) ;

	/* Maybe needed ...						*/
	repaint    	  () ;
	/* ... maybe needed end.					*/

	going	= false ;
	frozen	= false ;

	return	NULL	;
}

/*  KNDGraphic								*/
/*  execute	: Set executing flag					*/
/*  _going	: bool		: Executing				*/
/*  (returns)	: void		:					*/

void	KNDGraphic::execute
	(	bool	_going
	)
{
	going	= _going ;
	frozen	= false	 ;
}

/*  KNDGraphic								*/
/*  timerTick	: Handle a one-second timer tick			*/
/*  second	: long		: Second				*/
/*  (returns)	: void		:					*/

void	KNDGraphic::timerTick
	(	long	second
	)
{
	if (going && (loadHist != NULL))
	{
		/* First stage is to pass the timer tick down to each	*/
		/* of the load objects, so that they can perform their	*/
		/* per-tick processing.					*/
		int	lScale	= loadHist->timerTick (second) ;
		int	pScale	= pktHist ->timerTick (second) ;
		char	text[32];

		if (lScale != loadScale)
			load.setMax (intToExp (loadScale = lScale, text)) ;
		if (pScale != pktScale )
			pkts.setMax (intToExp (pktScale  = pScale, text)) ;

		loadBar1->timerTick (second) ;
		loadBar2->timerTick (second) ;
		pktBar1 ->timerTick (second) ;
		pktBar2 ->timerTick (second) ;

		/* If this graphic display object is on view and we are	*/
		/* not frozen, then repaint the displays. If not, then	*/
		/* there is no point in doing anything.			*/
		if (onview && !frozen) paintAll () ;

		parent->setLoad (loadBar1->value(), v_bar1,
				 loadBar2->value(), v_bar2) ;
	}
}

/*  KNDGraphic								*/
/*  paintAll	: Paint everything					*/
/*  (returns)	: void		:					*/

void	KNDGraphic::paintAll ()
{
	/* The scale determined by the load histogram is applied to	*/
	/* the bar displays. Since the former is scaled according to	*/
	/* the peak load within its total period, and since its total	*/
	/* period is greater than the time constant for the bar		*/
	/* displays, their values cannot exceed that of the former.	*/
	loadBar1 ->setScale (loadScale) ;
	loadBar2 ->setScale (loadScale) ;
	pktBar1  ->setScale (pktScale ) ;
	pktBar2  ->setScale (pktScale ) ;


	/* All ready to go, now redisplay all of the load displays.	*/
	loadHist->repaint () ;
	loadBar1->repaint () ;
	loadBar2->repaint () ;
	pktHist ->repaint () ;
	pktBar1 ->repaint () ;
	pktBar2 ->repaint () ;
}

/*  KNDGraphic								*/
/*  addPacket	: Add a new packet to the display			*/
/*  pktinfo	: PktInfo *	  : Packet information			*/
/*  (returns)	: void		  :					*/

void	KNDGraphic::addPacket
	(	PktInfo		*pktinfo
	)
{
	loadHist->addPacket (pktinfo) ;
	loadBar1->addPacket (pktinfo) ;
	loadBar2->addPacket (pktinfo) ;
	pktHist ->addPacket (pktinfo) ;
	pktBar1 ->addPacket (pktinfo) ;
	pktBar2 ->addPacket (pktinfo) ;
}

/*  KNDGraphic								*/
/*  setOnview	: Set whether on view or not				*/
/*  _onview	: bool		: New state				*/
/*  (returns)	: void		:					*/

void	KNDGraphic::setOnview
	(	bool	_onview
	)
{
	/* Note the state, and repaint if we are coming into view ...	*/
	if ((onview = _onview) && going) paintAll () ;
}

/*  KNDGraphic								*/
/*  freeze	: Set frozen or unfrozen state				*/
/*  _frozen	: bool		: New state				*/
/*  (returns)	: void		:					*/

void	KNDGraphic::freeze
	(	bool	_frozen
	)
{
	frozen	= _frozen ;
	if (loadHist != NULL) loadHist->freeze (frozen) ;
	if (loadBar1 != NULL) loadBar1->freeze (frozen) ;
	if (loadBar2 != NULL) loadBar2->freeze (frozen) ;
}

/*  KNDGraphic								*/
/*  resizeEvent	: Handle resize						*/
/*  e		: QResizeEvent *: Resize event information		*/
/*  (returns)	: void		:					*/

void	KNDGraphic::resizeEvent
	(	QResizeEvent *e
	)
{
	if (loadHist != NULL)
	{
		int w = (width() - 15) / 2 ;

		load.setGeometry (5,	  5, w, height() - 10) ;
		pkts.setGeometry (w + 15, 5, w, height() - 10) ;
		load.resize 	 (QSize (w, height())) ;
		pkts.resize	 (QSize (w, height())) ;

		if (onview) paintAll () ;
	}
}
