/***************************************************************************
                          knd_load.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef 	KSNUFFLELOAD_H 
#define 	KSNUFFLELOAD_H 

#include 	"ksnuffle.h"

#define		LOADWID	350

/*  MINLOAD specifies the minimum scale for the load display, in bytes	*/
/*  Note that it _must_ be a power of 10 for the scale adjustment code	*/
/*  to work!								*/
#define		MINLOAD	100

/*  MINPCNT similarly specifies the minimum scale for the packet count	*/
/*  display. Again, it _must_ be a power of 10.				*/
#define		MINPCNT	10


typedef	struct
{	int	pktlen	;
	int	datlen	;
}	LoadPair ;

/*  KNDLoadHist								*/
/*  -----------								*/

/*  This class is responsible for displaying a sliding histogram which	*/
/*  shows the traffic load, marking actual data and actual network	*/
/*  traffic. Note that the parent object is responsible for displaying	*/
/*  any labels around it.						*/

class	KNDLoadHist : public QFrame
{
	Q_OBJECT

	LoadPair	*pktcnt	;	/* Packet data counts		*/
	LoadPair	*dispval;	/* Display values		*/

	int		offsetp	;	/* Packet data offset		*/
	int		offsetd	;	/* Packet display offset	*/

	int		nsecs	;	/* Shift interval in seconds	*/
	int		maxwid	;	/* Maximum width		*/
	int		tickno	;	/* Tick number for shifting	*/
	bool		frozen	;	/* Display is frozen		*/
	long		second	;	/* Current time in seconds	*/
	int		insert	;	/* Insertion offset		*/

	bool		adjust	;	/* Automatic scale adjustment	*/
	int		scpref	;	/* Current preferred scale	*/
	int		sccurr	;	/* Scale currently being used	*/
	int		scdown	;	/* Scale reduction point	*/
	int		maxval	;

	inline	int	boundary
		(	int	v
		)
	{
		return	height() - (v * height()) / sccurr ;
	}

	void	reScale	()	;

public	:
	KNDLoadHist	  (QWidget *, int, int, bool = true) ;
       ~KNDLoadHist	  () ;

	void	addPacket (PktInfo *) ;
	int	timerTick (long) ;
	void	freeze	  (bool) ;
	void	setScale  (int)	 ;

protected :

	virtual	void	drawContents		(QPainter     *) ;
	virtual	void	mouseDoubleClickEvent	(QMouseEvent  *) ;
	virtual	void	resizeEvent		(QResizeEvent *) ;

signals :

	void	doubleClick () ;
}	;


/*  KNDLoadBar								*/
/*  ----------								*/

/*  This class draws a load bar showing network traffic over the	*/
/*  previous N seconds, where N is configurable.			*/

class	KNDLoadBar : public QFrame
{
	Q_OBJECT

	LoadPair	*pktcnt	;	/* Packet data counts		*/
	LoadPair	dispval	;	/* Display values		*/
	int		nsecs	;
	
	int		offset	;
	int		scale	;
	bool		scdiff	;
	bool		frozen	;	/* Display is frozen		*/
	long		second	;	/* Current time in seconds	*/
	int		insert	;	/* Insertion offset		*/
	int		pktlen	;	/* Total packet bytes		*/
	int		datlen	;	/* Total data bytes		*/

	inline	int	boundary
		(	int	v
		)
	{
		return	height() - (v * height()) / scale ;
	}

	void	reScale	()	;

public	:
	KNDLoadBar	  (QWidget *, int) ;
       ~KNDLoadBar	  () ;

	void	addPacket (PktInfo *) ;
	void	timerTick (long) ;
	void	freeze	  (bool) ;
	void	setScale  (int ) ;

	inline	int	value	()
	{
		return	pktlen	;
	}

protected :

	virtual	void	drawContents		(QPainter    *) ;
	virtual	void	mouseDoubleClickEvent	(QMouseEvent *) ;

signals:

	void	doubleClick () ;
}	;



/*  KNDPktHist								*/
/*  ----------								*/

/*  This class is responsible for displaying a sliding histogram which	*/
/*  shows the packet count. Note that the parent object is responsible	*/
/*  for displaying any labels around it.				*/

class	KNDPktHist : public QFrame
{
	Q_OBJECT

	int		*pktcnt	;	/* Packet data counts		*/
	int		*dispval;	/* Display values		*/

	int		offsetp	;
	int		offsetd	;

	int		nsecs	;	/* Shift interval in seconds	*/
	int		maxwid	;	/* Maximum width		*/
	int		tickno	;	/* Tick number of shifting	*/
	bool		frozen	;	/* Display is frozen		*/
	long		second	;	/* Current time in seconds	*/
	int		insert	;	/* Insertion offset		*/

	bool		adjust	;	/* Automatic scale adjustment	*/
	int		scpref	;	/* Current preferred scale	*/
	int		sccurr	;	/* Scale currently being used	*/
	int		scdown	;	/* Scale reduction point	*/
	int		maxval	;

	inline	int	boundary
		(	int	v
		)
	{
		return	height() - (v * height()) / sccurr ;
	}

	void	reScale	()	;

public	:
	KNDPktHist	  (QWidget *, int, int, bool = true) ;
       ~KNDPktHist	  () ;

	void	addPacket (PktInfo *) ;
	int	timerTick (long) ;
	void	freeze	  (bool) ;
	void	setScale  (int)	 ;

protected :

	virtual	void	drawContents		(QPainter     *) ;
	virtual	void	mouseDoubleClickEvent	(QMouseEvent  *) ;
	virtual	void	resizeEvent		(QResizeEvent *) ;

signals :

	void	doubleClick () ;
}	;

/*  KNDPktBar								*/
/*  ---------								*/

/*  This class draws a load bar showing network packets over the	*/
/*  previous N seconds, where N is configurable.			*/

class	KNDPktBar : public QFrame
{
	Q_OBJECT

	int		*pktcnt	;	/* Packet counts		*/
	int		dispval	;	/* Display values		*/
	int		nsecs	;
	
	int		offset	;
	int		scale	;
	bool		frozen	;	/* Display is frozen		*/
	long		second	;	/* Current time in seconds	*/
	int		insert	;	/* Insertion offset		*/

	inline	int	boundary
		(	int	v
		)
	{
		return	height() - (v * height()) / scale ;
	}

	void	reScale	()	;

public	:
	KNDPktBar	  (QWidget *, int) ;
       ~KNDPktBar	  () ;

	void	addPacket (PktInfo *) ;
	void	timerTick (long) ;
	void	freeze	  (bool) ;
	void	setScale  (int ) ;

protected :

	virtual	void	drawContents		(QPainter    *) ;
	virtual	void	mouseDoubleClickEvent	(QMouseEvent *) ;

signals:

	void	doubleClick () ;
}	;


#endif // KSNUFFLELOAD_H 
