/***************************************************************************
                          knd_multi.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef	KND_MULTI_H
#define	KND_MULTI_H

#include	<qlistbox.h>

#include	"ksnuffle.h"


/*  QDragListBox							*/
/*  This is a QListBox extended so that entries can be reordered by	*/
/*  dragging.								*/

class	QDragListBox : public QListBox
{
	Q_OBJECT	;

	KNDMulti  *parent	;
	int	  mouseDownAt	;
	int	  mouseUpAt	;

public	:
	QDragListBox (KNDMulti *) ;

protected :

	virtual	void	mousePressEvent   (QMouseEvent   *) ;
	virtual	void	mouseReleaseEvent (QMouseEvent   *) ;

signals :
	void	moved	(int, int) ;
}	;

/*  KNDClient								*/
/*  This class is used to handle the display of a histogram for	a	*/
/*  particular monitor in the parallel graph display object.		*/

class	KNDClient : public QObject
{
	Q_OBJECT	;

	KNDMulti	*parent		;
	KNDLoadHist	*hist		;
	QLabel		label		;
	int		slot		;
	int	 	posn	  	;
	QString		name		;

public:
	KNDClient	(KNDMulti *, int, const char *, int) ;
       ~KNDClient	() ;

	int	setOrder  (QRect, int)	;
	void	setScale  (int)	;
	void	addPacket (PktInfo  *)	;
	int	timerTick (long);
	void	repaint	  ()	;

	inline	const	char	*getName ()
	{
		return	name ;
	}	;

private slots :

	void	doubleClick () ;
}	;

/*  KNDMulti							*/
/*  This is the class that displays serveral load histograms	*/
/*  at once on the same display page.				*/

class	KNDMulti : public QWidget
{
	Q_OBJECT

	KNDApp			*parent	;
	QLabel			period	;
	QLabel			scale	;

	QArray<KNDClient *>	clients	;
	QList <KNDClient  >	ondisp	;
	QDragListBox		order	;

	bool			onview	;
	int			sccurr	;
	int			scdown	;

	void	rePlace	  () ;

public:

	KNDMulti (KNDApp *parent) ;
       ~KNDMulti () ;

	void	Register  (int, const char *) ;
	void	unRegister(int) ;
	void	showClient(int)	;

	void	addPacket (PktInfo *, int) ;

	void	timerTick (long) ;
	void	setOnview (bool) ;
	void	paint	  ()	 ;

protected	:

	virtual	void	resizeEvent (QResizeEvent  *) ;

protected slots :

	void	selected  (int)	 ;
	void	reOrder	  (int, int) ;
}	;

#endif	// KND_MULTI_H
