/***************************************************************************
                          knd_pcap.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef		KND_PCAP_H 
#define		KND_PCAP_H 

#include	"ksnuffle.h"
#include	"pcaphdr.h"
#include	"capture.h"

typedef	struct	_tcpaddr	TCPADDR	;
typedef	struct	_tcphash	TCPHASH	;

/*  TCP/IP deconding (with due acknowledgement to the tcpdump(8)	*/
/*  authors) attempts to identifify ACK and SEQ offsets in TCP/IP	*/
/*  streams. A hash table (hashed on ordered source/destination		*/
/*  addresses) holds that latest ACK and SEQ values for each stream.	*/
/*  The hash table size is straight from the tcpdump(8) code.		*/

struct	_tcpaddr
{	ulong	src	;	/* Source address			*/
	ulong	dst	;	/* Destination address			*/
	ulong	portp	;	/* Source and destination ports		*/
}	;

struct	_tcphash
{	TCPHASH	*next	;	/* Next on hash queue			*/
	TCPADDR	tcpaddr	;	/* TCP/IP addresses for entry		*/
	ulong	seq	;	/* Last sequence			*/
	ulong	ack	;	/* Last ack				*/
}	;

/*  KNDPCap	: Ksnuffle packet capture				*/
/*  Instance of this class handle a particular capture configuration.	*/
/*  There is one instance for each monitor created by the user.		*/

class	KNDPCap : public QObject
{
	Q_OBJECT

protected :

	KNDView	*view	;		/* Associated view object	*/
	char	*iface	;		/* Interface name		*/

public	:
	KNDPCap 	 	 (KNDView *) ;
	virtual	~KNDPCap 	 () ;

	virtual	void	setProg	 (cchar *, TrigInf *, EventInf *, int) = 0 ;
	virtual	void	execute	 (bool, FILE * = NULL, bool = false) = 0 ;
	virtual	void	timerTick(long) = 0 ;

	virtual	PCapInf	*getInfo   () = 0 ;
	virtual	void	clickQuery () = 0 ;

protected :

		void	clear	   () ;

	bool	execing	;		/* Monitor is executing		*/
}	;

typedef	struct	ether_header	ETH_HDR	;
typedef	struct	ip		IP	;
typedef	struct	ether_arp	ETH_ARP	;
typedef	struct	tcphdr		TCP_HDR	;
typedef	struct	udphdr		UDP_HDR	;

#endif	// KND_PCAP_H 
