/***************************************************************************
                          knd_pcaploc.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<errno.h>
#include	<unistd.h>
#include	<fcntl.h>

#include	<ksock.h>

#include	"knd_pcaploc.h"
#include	"knd_mon.h"
#include	"knd_pkt.h"
#include	"knd_gra.h"
#include	"knd_multi.h"

#include	"nethdrs.h"
#include	"netpcap.h"

/*  KNDPCapLoc								*/
/*  KNDPCapLoc	: Constructor for monitor module			*/
/*  monitor	: KNDMonitor *	: Associated monitor object		*/
/*  prefilter	: cchar *	: User packet pre-filter		*/
/*  (returns)	: KNDPCapLoc	:					*/

KNDPCapLoc::KNDPCapLoc
	(	KNDMonitor	*monitor,
		cchar		*prefilter
	)
	:
	KNDPCapMon	(monitor),
	capture		(this),
	prefilter	(prefilter)
{
	ksock	= NULL		;
	execing	= false 	;
}

/*  KNDPCapLoc								*/
/*  ~KNDPCapLoc	: Destructor for monitor module				*/
/*  (returns)	:		:					*/

KNDPCapLoc::~KNDPCapLoc ()
{
	clear	() ;
}

/*  KNDPCapLoc	:							*/
/*  execute	: Start or stop execution of the monitor		*/
/*  _execing	: bool		: TRUE to start				*/
/*  _logfd	: FILE *	: Log file if any			*/
/*  _binary	: bool		: Binary logging			*/
/*  (returns)	: void		:					*/

void	KNDPCapLoc::execute
	(	bool	_execing,
		FILE	*_logfd,
		bool	_binary
	)
{
	/* If we are asked to start but are not already doing so then	*/
	/* create a new process to handle the monitor.			*/
	if (_execing && !execing)
	{
		if (!capture.Execute ())
		{	Error	("KSnuffle error", capture.getError()) ;
			view->executeRC (false) ;
			clear	() ;
			return	;
		}

		/* If we do not have a KDE level socket then create it	*/
		/* now using the underlying libpcap socket. Note that	*/
		/* we must not delete an extant KSocket at this point	*/
		/* as this would close the system socket and terminate	*/
		/* packet capture; this would break Set/Go/Halt/Go...	*/
		/* sequences.						*/
		if (ksock == NULL)
		{
			netpcap	= capture.getNetPCap () ;
			ksock	= new KSocket (netpcap->getPCap()->fd) ;
			connect	(ksock, SIGNAL(readEvent(KSocket *)),
				 this,  SLOT  (sockRead (KSocket *))) ;
		}

		ksock->enableRead (true) ;
		setLog (_logfd, _binary) ;

		view->setState  (capture.getState (), false) ;
		view->executeRC (true) ;

		execing	= true	;
		return	;
	}

	/* If we are asked to stop and are currently executing the	*/
	/* monitor then simplay cancel the process/thread.		*/
	if (!_execing && execing)
	{
		ksock->enableRead (false) ;
		capture.Finish () ;
		execing	= false	  ;
		return	;
	}
}

/*  KNDPCapLoc	:							*/
/*  clear	: Clear up current monitor				*/
/*  (returns)	: void		:					*/

void	KNDPCapLoc::clear ()
{
	if (ksock != NULL)
	{	delete 	ksock	;
		ksock	= NULL	;
	}

	capture.Finish  () ;
	KNDPCap::clear  () ;
}

/*  KNDPCapLoc	:							*/
/*  setProg	: Set the monitoring program				*/
/*  _iface	: cchar *	: Interface (device) name		*/
/*  triginf	: TrigInf *	: Trigger information			*/
/*  eventinf	: EventInf *	: Event information			*/
/*  snaplen	: int		: Snap length				*/
/*  (returns)	: void		:					*/

void	KNDPCapLoc::setProg
	(	cchar	 *_iface,
		TrigInf	 *triginf,
		EventInf *eventinf,
		int	_snaplen
	)
{
	/* Clean up anything left over from a previous run of this	*/
	/* particular monitor, and note the possible new interface.	*/
	clear	() ;
	iface	= strdup (_iface) ;
	snaplen	= _snaplen	  ;

	/* Now build the filter and event triggers ...			*/
	if (!capture.Build (iface, snaplen, triginf, eventinf, prefilter))
	{	Error	("KSnuffle error", capture.getError()) ;
		clear	() ;
		view->setProgRC (false) ;
		return	;
	}

	/* Get the decoder appropriate to the type of inteface.		*/
	if ((decoder = getDecoder (capture.getLType())) == NULL)
	{	Error	("KSnuffle error", "Cannot find decoder for interface") ;
		clear	()	;
		view->setProgRC (false) ;
		return	;
	}

	/* OK, program set correctly, so note any event commands and	*/
	/* then pass back a success notifigication.			*/
	setEvents   (eventinf) ;
	view->setProgRC (true) ;
}

/*  KNDPCapLoc								*/
/*  setState	: Set monitor state					*/
/*  state	: MState	: State					*/
/*  (returns)	: void		:					*/

void	KNDPCapLoc::gotState
	(	MState	state
	)
{
	view->setState (state) ;
}

/*  KNDPCapLoc								*/
/*  gotPacket	: Handle captured packet				*/
/*  pktinfo	: PktInfo *	: Packet data				*/
/*  (returns)	: void		:					*/

void	KNDPCapLoc::gotPacket
	(	PktInfo	*pktinfo
	)
{
	/* This may be either a captured packet for display, or a	*/
	/* packwt that has triggered an event.				*/
	if (pktinfo->tag == RS_PACKET)
		KNDPCapMon::gotPacket (pktinfo) ;
	else	KNDPCapMon::gotEvent  (pktinfo) ;
}

/*  KNDPCapLoc								*/
/*  getInfo	: Get capture information				*/
/*  (returns)	: PCapInf *	:					*/

PCapInf	*KNDPCapLoc::getInfo ()
{
	pcapinf.ltype	= capture.getLType () ;
	pcapinf.nmask	= capture.getNMask () ;
	pcapinf.snapl	= snaplen ;

	return	&pcapinf  ;
}

/*  KNDPCapLoc								*/
/*  timerTick	: Handle timer tick					*/
/*  seconds	: long		: Time in seconds			*/
/*  (returns)	: void		:					*/

void	KNDPCapLoc::timerTick
	(	long	seconds
	)
{
	/* This is a null operation for local monitors. Timing is	*/
	/* derived from cpatured packets.				*/
}

/*  KNDPcapLoc								*/
/*  clickQuery	: Handle a click of the query button			*/
/*  (returns)	: void		:					*/

void	KNDPCapLoc::clickQuery ()
{
	/* This is a dummy method here and should never be called.	*/
}


void	KNDPCapLoc::sockRead
	(	KSocket	*ksock
	)
{
	netpcap->sockRead (ksock->socket()) ;
}
