/***************************************************************************
                          knd_pcapmon.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<errno.h>
#include	<unistd.h>
#include	<fcntl.h>

#include	<ksock.h>

#include	"knd_pcapmon.h"
#include	"knd_mon.h"
#include	"knd_pkt.h"
#include	"knd_gra.h"
#include	"knd_multi.h"

#include	"nethdrs.h"

#define		MAXCMD	(16)		/* Maximum commands outstanding	*/
static	pid_t	cmdpids[MAXCMD]	;	/* Outstanding command pids	*/

/*  GetCmdSlot	: Find slot for PID for new command			*/
/*  (returns)	: pid_t *	: Slot or NULL if none free		*/

static	pid_t	*GetCmdSlot ()
{
	for (int idx = 0 ; idx < MAXCMD ; idx += 1)
		if (cmdpids[idx] == 0)
			return	&cmdpids[idx] ;

	return	NULL ;
}

/*  FreeCmdSlot	: Free slot when a child dies				*/
/*  pid		: pid_t		: Process ID of deceased child		*/
/*  (returns)	: void		:					*/

void	FreeCmdSlot
	(	pid_t	pid
	)
{
	for (int idx = 0 ; idx < MAXCMD ; idx += 1)
		if (cmdpids[idx] == pid)
		{	cmdpids[idx] = 0 ;
			return ;
		}

	/* Not found. Probably the deceased child was a capture process	*/
	/* so don't complain.						*/
}

/*  KNDPCapMon								*/
/*  KNDPCapMon	: Constructor for monitor module			*/
/*  view	: KNDView *	: Associated monitor object		*/
/*  (returns)	: KNDPCapMon	:					*/

KNDPCapMon::KNDPCapMon
	(	KNDView	*view
	)
	:
	KNDPCap	(view)
{
	for (int eidx = 0 ; eidx < NEVENT ; eidx += 1)
		events[eidx] = NULL ;

	logfd	= NULL	;
	binary	= false	;
}

/*  KNDPCapMon								*/
/*  ~KNDPCapMon	: Destructor for monitor module				*/
/*  (returns)	:		:					*/

KNDPCapMon::~KNDPCapMon ()
{
	DelEvents () ;
}

/*  KNDPCapMon								*/
/*  DelEvents	: Delete all events					*/
/*  (returns)	: void		:					*/

void	KNDPCapMon::DelEvents ()
{
	for (int eidx = 0 ; eidx < NEVENT ; eidx += 1)
		if (events[eidx] != NULL)
		{	free ((void *)events[eidx]) ;
			events[eidx] = NULL ;
		}
}

/*  KNDPCapMon								*/
/*  setLog	: Setup logging						*/
/*  _logfd	: FILE *	: Log file if any			*/
/*  _binary	: bool		: Binary logging			*/
/*  (returns)	: void		:					*/

void	KNDPCapMon::setLog
	(	FILE	*_logfd,
		bool	_binary
	)
{
	logfd	= _logfd   ;
	binary	= _binary  ;
	if (logfd == NULL) return ;

	getInfo () ;	/* Make sure pcapinfo is filled in.		*/

	/* Write the last of the logging header, including the end-of-	*/
	/* header marker.						*/
	if (binary = _binary)
	{	int l = ItemLast ;
		WrToLog	(logfd, ItemLType,	       pcapinf.ltype) ;
		WrToLog	(logfd, ItemNMask,	       pcapinf.nmask) ;
		WrToLog (logfd, ItemSnapl,	       pcapinf.snapl) ;
		fwrite	(&l, sizeof(int), 1, logfd) ;
	}
	else
	{	fprintf	(logfd, "; LinkType  %d\n",    pcapinf.ltype) ;
		fprintf	(logfd, "; NetMask   %08lx\n", pcapinf.nmask) ;
		fprintf	(logfd, "; SnapLen   %d\n",    pcapinf.snapl) ;
		fprintf	(logfd, ";\n") ;
	}
}

/*  KNDPCapMon								*/
/*  gotPacket	: Handle incoming packet				*/
/*  pktinfo	: PktInfo *	: Packet data				*/
/*  (returns)	: void		:					*/

void	KNDPCapMon::gotPacket
	(	PktInfo	*pktinfo
	)
{
	long	timenow	 ;

	(*decoder) (iface, pktinfo) ;

	/* Before passing the packet to the display objects, ensure	*/
	/* that we have an up-to-date view of the time, in case a	*/
	/* timer tick as got behind a packet.				*/
	timenow	     = upToTick (pktinfo->piHdr.tval.tv_sec) ;
	pktinfo->ago = timenow - pktinfo->piHdr.tval.tv_sec  ;

	/* If the log file is open then log the packet. This is either	*/
	/* a binary dump of the data we got from the monitor		*/
	/* processthread, or a text version suitably formatted.		*/
	if (logfd != NULL)
		if (binary)
			fwrite	(&pktinfo->piHdr, PKDATSZ, 1, logfd) ;
		else	fprintf	(logfd, "%s: %4d: %s %s.%s -> %s.%s : %s\n",
					pktinfo->time,
					pktinfo->piHdr.pktlen,
					pktinfo->proto,
					pktinfo->srce.n,
					pktinfo->srce.s,
					pktinfo->dest.n,
					pktinfo->dest.s,
					pktinfo->info) ;

	/* Worst case is the packet appears to have arrived in the	*/
	/* future. This should never happen!				*/
	if (pktinfo->ago >= 0)
		view->addPacket (pktinfo) ;
	else	fprintf	(stderr, "RSnuffle: packet arrived in future by %d\n",
				 -pktinfo->ago) ;
}

/*  KNDPCapMon								*/
/*  setEvents	: Note event commands					*/
/*  eventinf	: EventInf *	: Event information			*/
/*  (returns)	: void		:					*/

void	KNDPCapMon::setEvents
	(	EventInf *eventinf
	)
{
	DelEvents () ;

	for (int eidx = 0 ; eidx < NEVENT ; eidx += 1)
		if (StrIsSet (eventinf->event[eidx].exec))
			events[eidx] = strdup (eventinf->event[eidx].exec) ;
}

/*  KNDPCapMon								*/
/*  SubsEPoint	: Substitute endpoint information			*/
/*  event	: cchar *	: Pointer at %... in raw command	*/
/*  subcmd	: SString &	: Substituted command buffer		*/
/*  epoint	: EPoint &	: Endpoint information			*/
/*  (returns)	: cchar *	: Next character in raw command		*/

cchar	*KNDPCapMon::SubsEPoint
	(	cchar	*event,
		SString	&subcmd,
		EPoint	&epoint
	)
{
	switch (event[2])
	{
		case 'a' : subcmd.append (epoint.a) ; break	;
		case 'n' : subcmd.append (epoint.n) ; break	;
		case 'p' : subcmd.append (epoint.p) ; break	;
		case 's' : subcmd.append (epoint.s) ; break	;
		default	 :			      break	;
	}

	return	&event[3] ;
}

/*  KNDPCapMon								*/
/*  SubsTime	: Substitute packet time				*/
/*  event	: cchar *	: Pointer at %... in raw command	*/
/*  subcmd	: SString &	: Substituted command buffer		*/
/*  tval	: time_t	: Packet time				*/
/*  (returns)	: cchar *	: Next character in raw command		*/

cchar	*KNDPCapMon::SubsTime
	(	cchar	*event,
		SString	&subcmd,
		time_t	tval
	)
{
	char	*ep	;
	char	tbuf[99];

	event	+= 2	;
	if ((ep = strchr (event, ']')) == NULL) return event ;

	*ep	= 0 	;
	strftime (tbuf, sizeof(tbuf), event, localtime (&tval)) ;
	*ep	= ']'	;

	subcmd.append (tbuf) ;

	return	&ep[1]	;
}

/*  KNDPCapMon								*/
/*  Substitute	: Perform command substitution				*/
/*  event	: cchar   *	: Raw command				*/
/*  subcmd	: SString &	: Substituted command buffer		*/
/*  pktinfo	: Pktinfo *	: Packet data				*/
/*  (returns)	: cchar   *	: Substituted command			*/

cchar	*KNDPCapMon::Substitute
	(	cchar	*event,
		SString	&subcmd,
		PktInfo	*pktinfo
	)
{
	char	t[32]	;

	/* Decode the packet. It may not have been decoded yet ...	*/ 
	(*decoder) (iface, pktinfo) ;

	/* Substitutions are:						*/
	/*	%Sa	Source IP address				*/
	/*	%Sh	Source host name				*/
	/*	%Sp	Source port number				*/
	/*	%Ss	Source service name				*/
	/*	%Da	Destination IP address				*/
	/*	%Dh	Destination host name				*/
	/*	%Dp	Destination port number				*/
	/*	%Ds	Destination service name			*/
	/*	%i	Decoded information				*/
	/*	%s	Decoded size values				*/
	/*	%[...]	... is strftime format on packet time		*/
	/*	%t	Packet microsecond time				*/
	/*	%%	Insert %age character				*/
	while (*event != 0)
	{
		if ((*event == '%') && (event[1] != 0))
		{
			switch (event[1])
			{
				case '%' :
					subcmd.Append ((char)'%') ;
					event	+= 2 ;
					break	;

				case 'S' :
					event	= SubsEPoint (event, subcmd, pktinfo->srce) ;
					break	;

				case 'D' :
					event	= SubsEPoint (event, subcmd, pktinfo->dest) ;
					break	;

				case '[' :
					event	= SubsTime   (event, subcmd, pktinfo->piHdr.tval.tv_sec) ;
					break	;

				case 'i' :
					subcmd.append (pktinfo->info) ;
					event	+= 2 ;
					break	;

				case 's' :
					subcmd.append (pktinfo->size) ;
					event	+= 2 ;
					break	;

				case 't' :
					sprintf	      (t, "%06ld", pktinfo->piHdr.tval.tv_usec) ;
					subcmd.append (t) ;
					event	+= 2 ;
					break	;

				default	 :
					event	+= 1 ;
					break	;
			}

			continue   ;
		}

		subcmd.Append (*event) ;
		event  += 1 ;
	}

	return	subcmd	;
}


/*  KNDPCapMon								*/
/*  gotEvents	: Handle possible event					*/
/*  pktinfo	: PktInfo *	: Event and packer data			*/
/*  (returns)	: bool		: True if this is an event		*/

bool	KNDPCapMon::gotEvent
	(	PktInfo	*pktinfo
	)
{
	cchar	*event	;
	int	eidx	;
	SString	subcmd	;
	pid_t	pid	;
	pid_t	*slot	;

	/* See if the tag is within the range of event identifiers and	*/
	/* that there is an associated command to execute. Silently	*/
	/* return if either is not the case.				*/
	if ((pktinfo->tag < RS_EVENT_0) || (pktinfo->tag >= RS_EVENT_0 + NEVENT))
		return	false ;
	if ((event = events[eidx = pktinfo->tag - RS_EVENT_0]) == NULL)
		return	true  ;

	/* We need a slot in which to store the PID of the child	*/
	/* command. This also limits the number of commands that can be	*/
	/* outstanding.							*/
	if ((slot = GetCmdSlot ()) == NULL)
	{	fprintf	(stderr, "ksnuffle: too many event commands\n") ;
		return	true ;
	}

	/* If the command contains any substitutions then process them	*/
	/* and use the result as the command to execute.		*/
	if (strchr (event, '%') != NULL)
		event = Substitute (event, subcmd, pktinfo) ;

	switch (pid = fork ())
	{
		case -1 :
			/* Fork failed. Delete all events so that we	*/
			/* don't get this error again, then display a	*/
			/* warning.					*/
			DelEvents () ;
			Error	  (QMessageBox::Warning,
				   "KSnuffle fork error",
				   "Error forking for event command") ;
			break	  ;

		case  0 :
			/* This is the child. Close all but the standar	*/
			/* I/O streams, then execute the command and	*/
			/* exit.					*/
			fprintf	(stderr, "ksnuffle: event child %d\n", getpid()) ;
			for (int fd = 3 ; fd < 20 ; fd += 1)
				close	(fd) ;

			/* Make sure that we are back to the real UID	*/
			/* and GID. We set both the real and effective	*/
			/* values so that the child cannot revert.	*/
			setreuid (getuid (), getuid ()) ;
			setregid (getgid (), getgid ()) ;

			system	 (event) ;
			exit	 (0)	 ;

		default	:
			/* Parent ... just continue.			*/
			break	;
	}

	*slot	= pid ;
	return	true  ;
}
