/***************************************************************************
                          knd_pcaprem.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<stdlib.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<errno.h>

#include	<ksock.h>

#include	"knd_pcap.h"
#include	"knd_pcaprem.h"
#include	"knd_remote.h"
#include	"knd_pkt.h"
#include	"knd_gra.h"
#include	"knd_multi.h"

#include	"nethdrs.h"

#undef		_DEBUG
#undef		_SHOWADJ

/*  KNDPCapRem								*/
/*  KNDPCapRem	: Constructor for monitor module			*/
/*  remote	: KNDRemote *	: Associated remote monitor object	*/
/*  host	: cchar *	: Remote host to sniff			*/
/*  port	: cchar *	: Remote sniffer port			*/
/*  pass	: cchar *	: Password on remote machine		*/
/*  prefilter	: cchar *	: User packet pre-filter		*/
/*  (returns)	: KNDPCapRem	:					*/

KNDPCapRem::KNDPCapRem
	(	KNDRemote	*remote,
		cchar		*host,
		cchar		*port,
		cchar		*pass,
		cchar		*prefilter
	)
	:
	KNDPCapMon (remote),
	remote	   (remote),
	pass	   (pass),
	prefilter  (prefilter)
{
	ksock		= new KSocket (host, atoi(port)) ;
	pcapinf.snapl	= 68	 	;
	execing		= false 	;
	tnext		= -1		;
	recsize		= 0		;

	ksock->enableRead (true)	;
	connect	(ksock, SIGNAL(readEvent(KSocket *)), this, SLOT(sockRead(KSocket *))) ;
}

/*  KNDPCapRem								*/
/*  ~KNDPCapRem	: Destructor for monitor module				*/
/*  (returns)	:		:					*/

KNDPCapRem::~KNDPCapRem ()
{
	if (ksock != NULL) delete ksock ;
}

/*  KNDPCapRem								*/
/*  SendRemCmd	: Send command to remote end				*/
/*  cmd		: int		: Command				*/
/*  arg		: cchar *	: Argument string or NULL		*/
/*  (returns)	: void		:					*/

void	KNDPCapRem::SendRemCmd
	(	int	cmd,
		cchar	*arg
	)
{
	PktInfo	pktinfo	;

	pktinfo.tag = cmd ;
	strcpy	((char *)pktinfo.data, arg == NULL ? "" : arg) ;
	write	(ksock->socket (), &pktinfo, sizeof(PktInfo) ) ;
}

/*  KNDPCapRem	:							*/
/*  execute	: Start or stop execution of the monitor		*/
/*  _execing	: bool		: TRUE to start				*/
/*  _logfd	: FILE *	: Log file if any			*/
/*  _binary	: bool		: Binary logging			*/
/*  (returns)	: void		:					*/

void	KNDPCapRem::execute
	(	bool	_execing,
		FILE	*_logfd,
		bool	_binary
	)
{
	/* If we are asked to start but are not already doing so then	*/
	/* create a new process to handle the monitor.			*/
	if (_execing && !execing)
	{
		setLog (_logfd, _binary) ;

		if (ksock == NULL)
		{	view->executeRC (false) ;
			return	;
		}

		SendRemCmd (RS_GO, NULL) ;
		return	;
	}

	/* If we are asked to stop and are currently executing the	*/
	/* monitor then simplay cancel the process/thread.		*/
	if (!_execing && execing)
	{
		SendRemCmd (RS_STOP, NULL) ;
		execing	= false ;
		return	;
	}
}

/*  KNDPCapRem	:							*/
/*  setProg	: Set the monitoring program				*/
/*  iface	: cchar *	: Interface (device) name		*/
/*  triginf	: TrigInf *	: Trigger information			*/
/*  eventinf	: EventInf *	: Event information			*/
/*  snaplen	: int		: Snap length				*/
/*  (returns)	: void		:					*/

void	KNDPCapRem::setProg
	(	cchar	 *iface,
		TrigInf	 *triginf,
		EventInf *eventinf,
		int	snaplen
	)
{
	SendRemCmd (RS_IFACE,	  iface) ;
	SendRemCmd (RS_FILTER,	  triginf->filter) ;
	SendRemCmd (RS_BEGIN,	  triginf->start ) ;
	SendRemCmd (RS_END,	  triginf->stop  ) ;
	SendRemCmd (RS_MODE,	  ItoA (triginf->mode)) ;
	SendRemCmd (RS_SNAP,	  ItoA (snaplen      )) ;
	SendRemCmd (RS_PREFILTER, prefilter == NULL ? "" : prefilter) ;
	SendRemCmd (RS_SET,       NULL) ;

	setEvents  (eventinf) ;
}

/*  KNDPCapRem								*/
/*  AdjTime	: Adjust time difference				*/
/*  buf		: cchar *	: Remote time				*/
/*  (returns)	: void		:					*/

void	KNDPCapRem::AdjTime
	(	cchar	*buf
	)
{
	time_t	tnow	;
	TVAL	lval	;
	TVAL	rval	;
	TVAL	tsum	;
	int	idx	;
	int	fixup	;

	/* Decode the remote time. Anything that appears invalid is	*/
	/* silently ignored.						*/
	if (sscanf (buf, "%ld:%ld", &rval.tv_sec, &rval.tv_usec) != 2)
		return	;

	/* We want the time as (seconds-from-epoch, microseconds). The	*/
	/* loop handles the situation where the second increments	*/
	/* inconveniently.						*/
	do
	{	tnow	    = time  (NULL) ;
		gettimeofday (&lval, NULL) ;
	}
	while	(tnow != time (NULL))	;

#ifdef	_SHOWADJ
	fprintf	(stderr, "r=(%ld:%ld) l=(%ld:%ld) ...",
			 rval.tv_sec, rval.tv_usec,
			 lval.tv_sec, lval.tv_usec) ;
#endif
	/* Calculate how far ahead of the remote machine we are. Note	*/
	/* that the uSec part of the time is kept positive. Hence, 0.5	*/
	/* seconds in the past is (-1,+500000).				*/
	lval.tv_sec	-= rval.tv_sec	;
	lval.tv_usec	-= rval.tv_usec	;
	if (lval.tv_usec < 0)
	{	lval.tv_usec	+= 1000000 ;
		lval.tv_sec	-= 1	   ;
	}

	/* Store the new difference in the circular buffer; if this is	*/
	/* the first then all entries are initialised.			*/
	if (tnext < 0)
		for (idx = 0 ; idx < NDSAMP ; idx += 1)
			tdiff[idx] = lval ;
	else	tdiff[tnext] = lval ;

	if ((tnext += 1) >= NDSAMP) tnext = 0 ;

	/* We now need to calculate the average time difference over	*/
	/* the samples .... 						*/
	tsum.tv_sec	= 0 ;
	tsum.tv_usec	= 0 ;
	for (idx = 0 ; idx < NDSAMP ; idx += 1)
	{	tsum.tv_sec	+= tdiff[idx].tv_sec  ;
		tsum.tv_usec	+= tdiff[idx].tv_usec ;
	}

	/* To make the division easy, apply a fixup which makes the	*/
	/* number of seconds non-negative and which is a multiple of	*/
	/* the number of samples.					*/
	fixup	      = tsum.tv_sec > 0 ? 0 : -tsum.tv_sec ;
	tsum.tv_sec  += fixup * NDSAMP ;

	tsum.tv_usec += (tsum.tv_sec % NDSAMP) * 1000000 ;
	tsum.tv_sec  /= NDSAMP ;
	tsum.tv_usec /= NDSAMP ;
	while (tsum.tv_usec >= 1000000)
	{	tsum.tv_usec -= 1000000 ;
		tsum.tv_sec  += 1	;
	}

	tsum.tv_sec -= fixup ;

#ifdef	_SHOWADJ
	fprintf	(stderr, "(%ld:%ld)\n", tsum.tv_sec, tsum.tv_usec) ;
#endif

	toff	= tsum	;
}

/*  KNDPCapRem								*/
/*  FixTime	: Fix packet time to local time				*/
/*  pktinfo	: PktInfo *	:					*/
/*  (returns)	: void		:					*/

void	KNDPCapRem::FixTime
	(	PktInfo	*pktinfo
	)
{
	/* Add the time difference to the nominal packet time, then	*/
	/* normalise the result so that the microsecond value is in the	*/
	/* range (0 ... 999999).					*/
	pktinfo->piHdr.tval.tv_sec  += toff.tv_sec  ;
	pktinfo->piHdr.tval.tv_usec += toff.tv_usec ;

	while (pktinfo->piHdr.tval.tv_usec >= 100000)
	{	pktinfo->piHdr.tval.tv_sec  += 1       ;
		pktinfo->piHdr.tval.tv_usec -= 1000000 ;
	}
	while (pktinfo->piHdr.tval.tv_usec <       0)
	{	pktinfo->piHdr.tval.tv_sec  -= 1       ;
		pktinfo->piHdr.tval.tv_usec += 1000000 ;
	}
}

/*  KNDPCapRem								*/
/*  sockRead	: Socket read slot					*/
/*  sock	: KSocket *	: Socket				*/
/*  (returns)	: void		:					*/

void	KNDPCapRem::sockRead
	(	KSocket	*sock
	)
{
	PktInfo	*pktinfo ;
	int	got	 ;
	char	*sp	 ;

	/* The loop executes so long as there is sufficient data to	*/
	/* constitute an incoming message ....				*/
	while ((got = read (ksock->socket(), &recbuff[recsize], sizeof(PktInfo) - recsize)) > 0)
	{
		/* If we have not got enough for a message then exit	*/
		/* routine; if we have then allocate a packet info	*/
		/* structure and copy in the data.			*/
		if ((recsize += got) < sizeof(PktInfo)) return ;

		recsize	= 0 ;
		pktinfo	= getPktInfo () ;
		memcpy	(pktinfo, recbuff, sizeof(PktInfo)) ;
#ifdef	_DEBUG
		fprintf	(stderr, "Block [%c] [%s]\n", pktinfo->tag, pktinfo->data) ;
#endif
		switch (pktinfo->tag)
		{
			case RS_HELLO	:
				/* Remote end is running. Send query	*/
				/* command to get list of interfaces;	*/
				/* the password is piggy-backed on this	*/
				SendRemCmd (RS_QUERY, pass) ;
				break	;

			case RS_IFACE	:
				/* This supplies an interface name at	*/
				/* the remote end. It is added to the	*/
				/* displayed list.			*/
				remote->AddIface ((cchar *)pktinfo->data) ;
				break	;

			case RS_ERROR	:
			case RS_SETERR	:
			case RS_GOERR	:
				/* Error messages. The sniffer is	*/
				/* stopped if running, and the error	*/
				/* displayed.				*/
				view->setState (MSError) ;
				if ((sp = strchr ((cchar *)pktinfo->data, '|')) != NULL)
					*sp++	= 0 ;
				else	sp	= "Unknown error" ;
				Error	((cchar *)pktinfo->data, sp) ;

				if (pktinfo->tag == RS_SETERR)
					view->setProgRC (false) ;
				if (pktinfo->tag == RS_GOERR )
					view->executeRC (false) ;
				break	;

			case RS_FATAL	:
				/* Display an error, then terminate	*/
				/* the sniffer.				*/
				view->setState (MSError) ;
				if ((sp = strchr ((cchar *)pktinfo->data, '|')) != NULL)
					*sp++	= 0 ;
				else	sp	= "Unknown error" ;
				Error	((cchar *)pktinfo->data, sp) ;

				ksock ->enableRead (false) ;
				remote->Finish     () ;
				return	;

			case RS_SET	:
				/* This is a positive acknowledgement	*/
				/* after a set request. The buffer	*/
				/* contains various bits of information	*/
				/* that we need.			*/
				if (sscanf ((cchar *)pktinfo->data,
					    "%d|%d|%lu",
					    &pcapinf.ltype,
					    &pcapinf.snapl,
					    &pcapinf.nmask) != 3)
				{
					view->setProgRC (false  ) ;
					view->setState  (MSError) ;
					Error	("RSnuffle error", "Bad reply to set request") ;
					break	;
				}

				/* Get the decoder function for the	*/
				/* type of link; if there is none then	*/
				/* we cannot display anything so report	*/
				/* an error.				*/
				if ((decoder = getDecoder (pcapinf.ltype)) == NULL)
				{
					view->setProgRC (false  ) ;
					view->setState  (MSError) ;
					Error	("KSnuffle error", "Cannot find decoder for interface") ;
					break	;
				}

				view->setProgRC (true ) ;
				view->setState  (MSSet) ;
				break	;

			case RS_GO	:
				/* Positive acknowledgement to a go	*/
				/* request.				*/
				view->executeRC (true) ;
				break	;

			case RS_STATE	:
				/* This message indicates the remote	*/
				/* monitor state.			*/
				view->setState  ((MState)atoi((cchar *)pktinfo->data)) ;
				break	;

			case RS_TIME	:
				/* This is a time synchronisation	*/
				/* message.				*/
				AdjTime	((cchar *)pktinfo->data) ;
				break	;

			case RS_PACKET	:
				/* Actual data packet! Provided that we	*/
				/* have a valid decoder then we can	*/
				/* deal with it.			*/
				if (decoder != NULL)
				{	FixTime	  (pktinfo) ;
					gotPacket (pktinfo) ;
				}
				break	;

			default	:
				/* Check for events ...			*/
				if (gotEvent (pktinfo)) break ;

				/* Anything else is silently ignored.	*/
				break ;
		}

		/* We may not be able to re-use the packet information	*/
		/* structure, since it may be in use, hence we have to	*/
		/* call freePktInfo (in case it is not in use).		*/
		freePktInfo (pktinfo)   ;
	}

	if (got < 0)
	{	if (errno != EAGAIN)
		{	view ->setState   (MSError) ;
			ksock->enableRead (false  ) ;
			Error	("Ksnuffle socket read error", STRERR) ;
		}
		return	;
	}

	view ->setState   (MSStopped) ;
	ksock->enableRead (false) ;
}

/*  KNDPCapRem								*/
/*  getInfo	: Get capture information				*/
/*  (returns)	: PCapInf *	:					*/

PCapInf	*KNDPCapRem::getInfo ()
{
	/* Just return a pointer at the information structure. It	*/
	/* should already have been filled in.				*/
	return	&pcapinf  ;
}

/*  KNDPCapRem								*/
/*  timerTick	: Handle timer tick					*/
/*  seconds	: long		: Time in seconds			*/
/*  (returns)	: void		:					*/

void	KNDPCapRem::timerTick
	(	long	seconds
	)
{
	/* This is a null operation for remote monitors. Timing is	*/
	/* derived from cpatured packets.				*/
}

/*  KNDPCapRem								*/
/*  clickQuery	: Handle a click of the query button			*/
/*  (returns)	: void		:					*/

void	KNDPCapRem::clickQuery ()
{
	/* Clear the interface list, then send a request to the remote	*/
	/* sniffer to get the list again.				*/
	remote->AddIface ((cchar *)NULL ) ;
	SendRemCmd	 (RS_QUERY, pass) ;
}
