/***************************************************************************
                          knd_pcaprep.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<errno.h>
#include	<unistd.h>
#include	<fcntl.h>

#include	<ksock.h>

#include	"knd_pcap.h"
#include	"knd_pcaprep.h"
#include	"knd_replay.h"
#include	"knd_pkt.h"
#include	"knd_gra.h"
#include	"knd_multi.h"

#include	"nethdrs.h"


/*  AddTime	: Add times together					*/
/*  tval	: struct timeval *	: Initial and result values	*/
/*  incr	: struct timeval *	: Increment			*/
/*  (returns)	: void			:				*/

static	void	AddTime
	(	TVAL	&tval,
		TVAL	&incr
	)
{
	tval.tv_sec	+= incr.tv_sec	;
	tval.tv_usec	+= incr.tv_usec	;

	if (tval.tv_usec >= 1000000)
	{	tval.tv_sec  += 1	;
		tval.tv_usec -= 1000000 ;
	}
}

/*  KNDPCapRep								*/
/*  KNDPCapRep	: Constructor for monitor module			*/
/*  replay	: KNDReplay *	: Associated replay object		*/
/*  (returns)	: KNDPCapRep	:					*/

KNDPCapRep::KNDPCapRep
	(	KNDReplay	*replay
	)
	:
	KNDPCap	(replay)
{
	execing	= false ;
	pktinfo	= getPktInfo () ;
}

/*  KNDPCapRep								*/
/*  ~KNDPCapRep	: Destructor for monitor module				*/
/*  (returns)	:		:					*/

KNDPCapRep::~KNDPCapRep ()
{
	if (pktinfo != NULL) freePktInfo (pktinfo) ;
}

/*  KNDPCapRep	:							*/
/*  execute	: Start or stop execution of the monitor		*/
/*  _execing	: bool		: TRUE to start				*/
/*  _logfd	: FILE *	: Ignored				*/
/*  _binary	: bool		: Ignored				*/
/*  (returns)	: void		:					*/

void	KNDPCapRep::execute
	(	bool	_execing,
		FILE	*_logfd,
		bool	_binary
	)
{
	/* If we are asked to start but are not already doing so then	*/
	/* create a new process to handle the monitor.			*/
	if (_execing && !execing)
	{
		/* Preload the first logged packet, then note the time	*/
		/* of the first packet for use in timing the replay.	*/
		if (read (logfd, &pktinfo->piHdr, PKDATSZ) != PKDATSZ)
		{	Error	("Ksnuffle file error", "Log file is truncated") ;
			view->setState (MSError) ;
			return	;
		}

		reptime	= pktinfo->piHdr.tval ;

		/* Set the time increment to be used on each clock	*/
		/* tick depending on the speed-up or slow-down.		*/
		if (v_speed > 0)
			inctime.tv_sec = v_speed, inctime.tv_usec = 0 ;
		else	inctime.tv_sec = 0,	  inctime.tv_usec = 1000000/(-v_speed) ;

		view->setState  (MSRunning, false) ;
		view->executeRC (true) ;
		execing	= true	;
		return	;
	}

	/* If we are asked to stop and are currently executing the	*/
	/* replay then all we need do is to clear the flag.		*/
	if (!_execing && execing)
	{
		execing	= false   ;
		return	;
	}
}

/*  KNDPCapRep	:							*/
/*  setProg	: Set the monitoring program				*/
/*  iface	: cchar *	: Interface (device) name		*/
/*  triginf	: TrigInf *	: Trigger information			*/
/*  eventinf	: EventInf *	: Event information			*/
/*  snaplen	: int		: Snap length				*/
/*  (returns)	: void		:					*/

void	KNDPCapRep::setProg
	(	cchar	 *iface,
		TrigInf	 *triginf,
		EventInf *eventinf,
		int	snaplen
	)
{
	/* This is a null operation, other than to report success back	*/
	/* to the viewer object.					*/
	view->setProgRC (true) ;
}

/*  KNDPCapRep								*/
/*  getInfo	: Get capture information				*/
/*  (returns)	: PCapInf *	:					*/

PCapInf	*KNDPCapRep::getInfo ()
{
	/* This method should never be called, and exists only as the	*/
	/* base class has a pure virtual method.			*/
	return	NULL ;
}

/*  KNDPCapRep								*/
/*  timerTick	: Handle timer tick					*/
/*  seconds	: long		: Time in seconds			*/
/*  (returns)	: void		:					*/

void	KNDPCapRep::timerTick
	(	long	seconds
	)
{
	/* Rather then have an additional replay object, equivalent to	*/
	/* the capture object, to spit out packets from the log, it is	*/
	/* easier to do it here.					*/
	if (execing)
	{
		/* Increment the replay time by the increment. For	*/
		/* speed ups, this will be greater than one second,	*/
		/* while for slow-downs it will be less.		*/
		AddTime (reptime, inctime) ;

		/* The loop now displays packets until we either reach	*/
		/* the end of the logging file, or we get to a packet	*/
		/* which was logged beyond the replay time.		*/
		while (timercmp (&pktinfo->piHdr.tval, &reptime, <))
		{
			(*decoder) (iface, pktinfo) ;
			pktinfo->ago = 0 ;

			view->addPacket (pktinfo) ;
			freePktInfo     (pktinfo) ;
			pktinfo = getPktInfo () ;

			/* Preload the next packet, checking for the	*/
			/* end of the recording.			*/
			if (read (logfd, &pktinfo->piHdr, PKDATSZ) != PKDATSZ)
			{	execing	= false ;
				view->setState (MSStopped) ;
				break	;
			}
		}
	}
}

/*  KNDPCapRep								*/
/*  setReplay	: Set replay information				*/
/*  _logfd	: int		: Log file				*/
/*  _v_speed	: int		: Replay speed-up/slow-down		*/
/*  _decoder	: DECODER *	: Packet decoder			*/
/*  (returns)	: void		:					*/

void	KNDPCapRep::setReplay
	(	int	_logfd,
		int	_v_speed,
		DECODER	*_decoder
	)
{
	logfd	= _logfd   ;
	v_speed	= _v_speed ;
	decoder	= _decoder ;
}

/*  KNDPCapRep								*/
/*  clickQuery	: Handle a click of the query button			*/
/*  (returns)	: void		:					*/

void	KNDPCapRep::clickQuery ()
{
	/* This is a dummy method here and should never be called.	*/
}
