/***************************************************************************
                          knd_pkt.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	"netinet/ip.h"

#include	<qtooltip.h>

#include	"pcaphdr.h"
#include	"knd_plugin.h"
#include	"knd_view.h"
#include	"knd_pkt.h"
#include	"knd_detail.h"
#include	"knd_tcpdata.h"

#include	"resource.h"

#undef		_DEBUG


extern	int	g_pauseat  ;	/* Packet load to pause at		*/
extern	int	g_pausefor ;	/* Pause time in seconds		*/
extern	bool	d_addr	   ;	/* Decode IP addresses to host names	*/
extern	bool	d_port	   ;	/* Decode IP ports to service names	*/
extern	bool	multiPkt   ;	/* Show multiple packet detail windows	*/
extern	QFont	*pktfont   ;	/* Font to be used for packet list	*/

/*  KNDPacket								*/
/*  KNDPacket	: Constructor for packet display object			*/
/*  parent	: KNDView *	: Parent				*/
/*  config	: KCOnfig *	: Configuration				*/
/*  replay	: bool		: Showing replay			*/
/*  (returns)	: KNDPacket	:					*/

KNDPacket::KNDPacket
	(	KNDView	*_parent,
		KConfig	*config,
		bool	replay
	)
	:
	KNDPlugin  (_parent, "KND_PACKETS"),
	l_accept   (this),
	l_dropped  (this),
	l_missed   (this),
	l_pktbytes (this),
	l_datbytes (this),
	v_accept   (this),
	v_dropped  (this),
	v_missed   (this),
	v_pktbytes (this),
	v_datbytes (this),
	replay	   (replay)
{
	plist	= NULL	;	/* No packet display object yet ...	*/
	lastp	= NULL	;	/* ... nor any items inserted		*/
	going	= false	;	/* Not at the moment ...		*/
	frozen	= false	;	/* ... nor this				*/
	pausefor= 0	;	/* ... not nor this neither		*/
	pitem	= NULL	;

	p_this	= 0	;
	p_last	= 0	;

	packets	= NULL	;

	l_accept  .hide () ;
	l_dropped .hide () ;
	l_missed  .hide () ;
	l_pktbytes.hide () ;
	l_datbytes.hide () ;
	v_dropped .hide () ;
	v_missed  .hide () ;
	v_pktbytes.hide () ;
	v_datbytes.hide () ;

	if (config != NULL)
	{
		w_Time	= config->readNumEntry ("w_Time",  100) ;
		w_Proto	= config->readNumEntry ("w_Proto",  80) ;
		w_Src	= config->readNumEntry ("w_Srce",  200) ;
		w_Dst	= config->readNumEntry ("w_Dest",  200) ;
		w_Info	= config->readNumEntry ("w_Info",  110) ;
		w_Size	= config->readNumEntry ("w_Size",   70) ;
	}
	else
	{
		w_Time	= 100 ;
		w_Proto	=  80 ;
		w_Src	= 200 ;
		w_Dst	= 200 ;
		w_Info	= 110 ;
		w_Size	=  70 ;
	}
}

/*  KNDPacket								*/
/*  ~KNDPacket	: Destructor for packet display object			*/
/*  (returns)	:		:					*/

KNDPacket::~KNDPacket ()
{
	/* Destroy the packettcp data displays object if any, and	*/
	/* delete any stored packets. Note that in the former, we	*/
	/* remove entries from the lisr _before_ deletion, as		*/
	/* destroying the detail window might cause an "endDetails"	*/
	/* callback.							*/
	while (details.count () > 0)
	{	KNDDetails *detail = details.first () ;
		details.remove (detail) ;
		delete	detail ;
	}
	while (tcpdata.count () > 0)
	{	KNDTCPData *tcpdat = tcpdata.first () ;
		tcpdata.remove (tcpdat) ;
		delete	tcpdat ;
	}
	if (plist != NULL)
		delete plist ;

	if (packets != NULL)
	{
		for (int i = 0 ; i < numpack ; i += 1)
			if (packets[i] != NULL)
				freePktInfo (packets[i]) ;

		delete	packets	;
	}
}

/*  KNDPacket								*/
/*  setPositions: Set positions of count fields				*/
/*  (returns)	: void		:					*/

void	KNDPacket::setPositions ()
{
	QRect	r = geometry () ;

	l_accept  .setGeometry   ( 10, r.height() - 25, 70, 20) ;
	v_accept  .setGeometry   ( 90, r.height() - 25, 60, 20) ;
	l_pktbytes.setGeometry   (160, r.height() - 25, 70, 20) ;
	v_pktbytes.setGeometry   (240, r.height() - 25, 80, 20) ;
	l_datbytes.setGeometry   (310, r.height() - 25, 50, 20) ;
	v_datbytes.setGeometry   (390, r.height() - 25, 80, 20) ;

	l_dropped .setGeometry   (460, r.height() - 25, 70, 20) ;
	v_dropped .setGeometry   (540, r.height() - 25, 60, 20) ;
	l_missed  .setGeometry   (610, r.height() - 25, 70, 20) ;
	v_missed  .setGeometry   (690, r.height() - 25, 60, 20) ;
}

/*  KNDPacket								*/
/*  setOptions	: Set display options					*/
/*  _lvopt	: void *	: New options				*/
/*  (returns)	: void *	:					*/

void	*KNDPacket::setOptions
	(	void	*_lvopt
	)
{

	int	_options = ((GenConfig *)_lvopt)->p_opts ;
	QRect	r	 = geometry () 	  ;

	/* If there is a current packet display list then destroy it,	*/
	/* as we will be building a new one. Also drop any packets that	*/
	/* we have stored.						*/
	while (details.count () > 0)
	{	KNDDetails *detail = details.first () ;
		details.remove (detail) ;
		delete	detail ;
	}
	while (tcpdata.count () > 0)
	{	KNDTCPData *tcpdat = tcpdata.first () ;
		tcpdata.remove (tcpdat) ;
		delete	tcpdat ;
	}
	if (plist != NULL)
	{	delete	plist   ;
		plist	= NULL	;
	}

	lastp	= NULL ;

	if (packets != NULL)
	{	for (int i = 0 ; i < numpack ; i += 1)
			if (packets[i] != NULL)
				freePktInfo (packets[i]) ;
		delete	packets	;
		packets	= NULL	;
	}

	packets	= new PktInfo *[numpack = maxpack] ;
	memset	(packets, 0, sizeof(PktInfo *) * maxpack) ;
	offset	= 0 ;

	/* Note the specified options; if there are none then there is	*/
	/* no packet display at all.					*/
	if ((options = _options) == 0) return NULL ;

	/* Create a new packet display object, and set it to just about	*/
	/* fill the window within which it is displayed.		*/
	plist	 = new QListView    (this) ;

	connect	(plist, SIGNAL(doubleClicked     (QListViewItem *)),
		 this,  SLOT  (showPacket        (QListViewItem *))) ;
	connect (plist, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)),
			SLOT  (rightClicked	 (QListViewItem *, const QPoint &, int))) ;
	if (pktfont != NULL) plist->setFont (*pktfont) ;

	plist->setGeometry (10, 10, r.width() - 20, r.height() - 40) ;
	plist->setSorting  (-1)	   ;
	plist->show	   ()	   ;

	/* Now add columns to the packet display object corresponding	*/
	/* to each of the selected options. The order is predefined,	*/
	/* the user has no control over this.				*/
	if (options & PO_TIME ) plist->addColumn ("Time",  w_Time ) ;
	if (options & PO_PROTO) plist->addColumn ("Proto", w_Proto) ;
	if (options & PO_SRC  ) plist->addColumn ("Srce",  w_Src  ) ;
	if (options & PO_DST  ) plist->addColumn ("Dest",  w_Dst  ) ;
	if (options & PO_INFO ) plist->addColumn ("Info",  w_Info ) ;
	if (options & PO_SIZE ) plist->addColumn ("Size",  w_Size ) ;

	setPositions () ;

	l_accept  .setAlignment  (AlignRight) ;
	v_accept  .setAlignment  (AlignRight) ;
	l_dropped .setAlignment  (AlignRight) ;
	v_dropped .setAlignment  (AlignRight) ;
	l_missed  .setAlignment  (AlignRight) ;
	v_missed  .setAlignment  (AlignRight) ;
	l_pktbytes.setAlignment  (AlignRight) ;
	v_pktbytes.setAlignment  (AlignRight) ;
	l_datbytes.setAlignment  (AlignRight) ;
	v_datbytes.setAlignment  (AlignRight) ;

	v_accept  .setFrameStyle (QFrame::Panel|QFrame::Sunken) ;
	v_dropped .setFrameStyle (QFrame::Panel|QFrame::Sunken) ;
	v_missed  .setFrameStyle (QFrame::Panel|QFrame::Sunken) ;
	v_pktbytes.setFrameStyle (QFrame::Panel|QFrame::Sunken) ;
	v_datbytes.setFrameStyle (QFrame::Panel|QFrame::Sunken) ;

	l_accept  .setText   (i18n("Accepted")) ;
	l_dropped .setText   (i18n("Dropped" )) ;
	l_missed  .setText   (i18n("Missed"  )) ;
	l_pktbytes.setText   (i18n("Traffic" )) ;
	l_datbytes.setText   (i18n("Data"    )) ;

	QToolTip::add (&v_accept,   i18n("Packets accepted at interface" )) ;
	QToolTip::add (&v_dropped,  i18n("Packets dropped at interface"  )) ;
	QToolTip::add (&v_missed,   i18n("Packets missed at interface"   )) ;
	QToolTip::add (&v_pktbytes, i18n("Total accepted network traffic")) ;
	QToolTip::add (&v_datbytes, i18n("Total accepted data"		 )) ;

	v_accept  .setText ("0") ;
	v_dropped .setText ("0") ;
	v_missed  .setText ("0") ;
	v_pktbytes.setText ("0") ;
	v_datbytes.setText ("0") ;

	l_accept  .show () ;
	v_accept  .show () ;
#ifdef	_DROPPED
	l_dropped .show () ;
	v_dropped .show () ;
#endif	/* _DROPPED */
#ifdef	_MISSED
	l_missed  .show () ;
	v_missed  .show () ;
#endif	/* _MISSED */
	l_pktbytes.show () ;
	v_pktbytes.show () ;
	l_datbytes.show () ;
	v_datbytes.show () ;

	s_pktbytes = 0 ;
	s_datbytes = 0 ;

	p_last	   = 0 ;
	p_this	   = 0 ;

	going	   = false ;
	frozen	   = false ;
	pausefor   = 0	   ;

	return	NULL ;
}

/*  KNDPacket								*/
/*  execute	: Set executing flag					*/
/*  _going	: bool		: Executing				*/
/*  (returns)	: void		:					*/

void	KNDPacket::execute
	(	bool	_going
	)
{
	going	= _going ;	/* Not the requsted execution state	*/
	frozen	= false	 ;	/* Changes always reset frozen state	*/
}

/*  KNDPacket								*/
/*  addPacket	: Add a new packet to the display			*/
/*  pktinfo	: PktInfo *	: Packet information			*/
/*  (returns)	: void		:					*/

void	KNDPacket::addPacket
	(	PktInfo	*pktinfo
	)
{
	/* Ignore the packet if there is no display ...			*/
	if (plist == NULL) return ;

	/* If we currently have the full complement of packets stored	*/
	/* then drop the oldest one; if on view, and not frozen or	*/
	/* paused then also drop if from the display.			*/
	if (packets[offset] != NULL)
	{
		freePktInfo (packets[offset]) ;
		packets[offset]  = NULL	      ;

		if (onview && !frozen && (pausefor == 0))
			delete	plist->firstChild() ;
	}

	/* Add a new packet to the list, then save the packet		*/
	/* information. Note that the use count is incremented so that	*/
	/* originator will not free the packet.				*/
	packets[offset] = pktinfo ;
	usePktInfo (pktinfo)	  ;
	if ((offset += 1) >= numpack) offset = 0 ;

	p_this		+= 1	  ;
	s_pktbytes	+= pktinfo->piHdr.pktlen ;
	s_datbytes	+= pktinfo->datlen	 ;

	/* If the number of packets received so far this second exceeds	*/
	/* the pause threshhold, then set the pause counter, and insert	*/
	/* a pause marker (but not if replaying) ....			*/
	if (!replay && (p_this >= g_pauseat) && (pausefor == 0))
	{
		if (onview && !frozen) putDots () ;
		pausefor = g_pausefor	 ;
		parent->setPaused (true) ;
		return	 ;
	}

	/* Get the packet put into the list, again unless we are not	*/
	/* on view, or are frozen.					*/
	if (onview && !frozen && (pausefor == 0))
		putPacket (pktinfo, true) ;
}

/*  KNDPacket								*/
/*  putPacket	: Put packet into display				*/
/*  pktinfo	: PktInfo *	: New packet				*/
/*  makevis	: bool		: Make packet visible			*/
/*  (returns)	: void		:					*/

void	KNDPacket::putPacket
	(	PktInfo	*pktinfo,
		bool	makevis
	)
{
	char	tidx		;
	char	*tptr	[  6]	;
	char	s	[100]	;
	char	d	[100]	;

	/* The scheme is to construct an array of up to size pointers,	*/
	/* corresponding to the selected options. We can then create a	*/
	/* new QListViewItem object with the requested values appearing	*/
	/* in the correct columns.					*/
	memset	(tptr, 0, sizeof(tptr)) ;
	tidx	= 0 ;

	if (options & PO_TIME )
		tptr[tidx++] = pktinfo->time  ;

	if (options & PO_PROTO)
		tptr[tidx++] = pktinfo->proto ;

	if (options & PO_SRC  )
		tptr[tidx++] = JoinStr (s, d_addr ? pktinfo->srce.n : pktinfo->srce.a,
					   ".",
					   d_port ? pktinfo->srce.s : pktinfo->srce.p,
					   NULL
				       ) ;

	if (options & PO_DST  )
		tptr[tidx++] = JoinStr (d, d_addr ? pktinfo->dest.n : pktinfo->dest.a,
					   ".",
					   d_port ? pktinfo->dest.s : pktinfo->dest.p,
				  	   NULL
				       ) ;

	if (options & PO_INFO )
		tptr[tidx++] = pktinfo->info  ;

	if (options & PO_SIZE )
		tptr[tidx++] = pktinfo->size  ;

	lastp	= new KNDPacketItem (plist,
				     lastp,
				     pktinfo,
				     tptr[0], tptr[1], tptr[2],
				     tptr[3], tptr[4], tptr[5]) ;

	/* If the make-visible flag is set then ensure that the just-	*/
	/* added list item is visible, and update the various counter	*/
	/* fields.							*/
	if (makevis)
	{
		plist->ensureItemVisible (lastp) ;

		v_accept  .setText (ItoA (pktinfo->piHdr.accept )) ;
		v_dropped .setText (ItoA (pktinfo->piHdr.dropped)) ;
		v_missed  .setText (ItoA (pktinfo->piHdr.missed )) ;
		v_pktbytes.setText (ItoA (s_pktbytes)) ;
		v_datbytes.setText (ItoA (s_datbytes)) ;
	}
}

/*  KNDPacket								*/
/*  putDots	: Add a marked packet of dots				*/
/*  (returns)	: void		:					*/

void	KNDPacket::putDots ()
{
	static	PktInfo	*dots	;
	static	bool	first	= true	   ;
	static	char	*dstr	= "......" ;

	if (first)
	{
		/* First call. Allocate a packet information structure	*/
		/* and fill it in. Note that we up the use count so	*/
		/* that it can never be freed.				*/
		usePktInfo (dots = getPktInfo ()) ;

		strcpy	(dots->time,  dstr) ;
		strcpy	(dots->proto, dstr) ;
		strcpy	(dots->info,  dstr) ;
		strcpy	(dots->size,  dstr) ;

		dots->srce.a = dstr ;
		dots->srce.n = dstr ;
		dots->srce.p = dstr ;
		dots->srce.s = dstr ;
		dots->dest.a = dstr ;
		dots->dest.n = dstr ;
		dots->dest.p = dstr ;
		dots->dest.s = dstr ;
	}

	putPacket (dots, true) ;
}

/*  KNDPacket								*/
/*  redisplay	: Rebuild display					*/
/*  (returns)	: void		:					*/

void	KNDPacket::redisplay ()
{
	int	idx	;
	int	slot	;

	plist->clear ()	;
	lastp	= NULL	;

	/* Put all recorded packets back into the display. We scan	*/
	/* forward from the current insertion offset, since it will	*/
	/* contain the oldest packet.					*/
	for (idx = 0, slot = offset ; idx < numpack ; idx += 1)
	{
		if (packets[slot] != NULL)
			putPacket (packets[slot], false) ;

		if ((slot += 1) >= numpack)
			slot = 0 ;
	}

	/* If paused then add the line of dots that indicates this to	*/
	/* the viewer.							*/
	if (pausefor > 0) putDots () ;

	/* Make sure that the last entry is visible. If we put in some	*/
	/* dots then this will be the case already, but no neccessarily	*/
	/* otherwise.							*/
	plist->ensureItemVisible (lastp) ;

	/* Also repaint the counters so that they immediately come up	*/
	/* to date.							*/
	v_accept  .repaint () ;
	v_dropped .repaint () ;
	v_missed  .repaint () ;
	v_pktbytes.repaint () ;
	v_datbytes.repaint () ;
}

/*  KNDPacket								*/
/*  setOnview	: Set whether we are on view or not			*/
/*  _onview	: bool		: TRUE if on view			*/
/*  (returns)	: void		:					*/

void	KNDPacket::setOnview
	(	bool	_onview
	)
{
	if ((onview = _onview) && (plist != NULL) && !frozen)
		redisplay () ;
}

/*  KNDPacket								*/
/*  freeze	: Freeze or unfreeze the display			*/
/*  _frozen	: bool		: New state				*/

void	KNDPacket::freeze
	(	bool	_frozen
	)
{
	if (!(frozen = _frozen) && (plist != NULL) && onview && (pausefor == 0))
		redisplay () ;
}

/*  KNDPacket								*/
/*  timerTick	: Handle timer tick					*/
/*  second	: long		: Second number				*/
/*  (returns)	: void		:					*/

void	KNDPacket::timerTick
	(	long	second
	)
{
	p_last	= p_this ;
	p_this	= 0	 ;

	/* If the last second's packet count was below the pause	*/
	/* threshhold, and we are currently paused, then decrement the	*/
	/* pause. Once this falls to zero, redisplay everything,	*/
	/* provided that we are on view and are now frozen.		*/
	if ((p_last < g_pauseat) && (pausefor > 0))
		if ((pausefor -= 1) == 0)
		{	if (onview && !frozen) redisplay () ;
			parent->setPaused (false) ;
		}

	/* If we are still paused, then at least update the count	*/
	/* fields (again, provided that we are on view and not paused).	*/
	if ((pausefor > 0) && onview && !frozen)
	{
		int	slot	;
		PktInfo	*lastpkt;

		if ((slot = offset + numpack - 1) >= numpack) slot -= numpack ;
		lastpkt	= packets[slot] ;

		v_accept  .setText (ItoA (lastpkt->piHdr.accept )) ;
		v_dropped .setText (ItoA (lastpkt->piHdr.dropped)) ;
		v_missed  .setText (ItoA (lastpkt->piHdr.missed )) ;
		v_pktbytes.setText (ItoA (s_pktbytes)) ;
		v_datbytes.setText (ItoA (s_datbytes)) ;
	}
}

/*  KNDPacket								*/
/*  resizeEvent	: Handle resize						*/
/*  e		: QResizeEvent *: Resize event information		*/
/*  (returns)	: void		:					*/

void	KNDPacket::resizeEvent
	(	QResizeEvent *e
	)
{
	if (plist != NULL)
		plist->resize (e->size().width() - 20, e->size().height() - 40) ;

	setPositions () ;
}


/*  KNDPacket								*/
/*  writeConfig	: Write packet display configuration			*/
/*  _config	: KConfig *	: Configuration object			*/
/*  (returns)	: void		:					*/

void	KNDPacket::writeConfig
	(	KConfig	*_config
	)
{
	if (plist != NULL)
	{
		int	slot	= 0	;

		if (options & PO_TIME )
		{	_config->writeEntry ("w_Time",  plist->columnWidth (slot)) ;
			slot	+= 1 ;
		}
		if (options & PO_PROTO)
		{	_config->writeEntry ("w_Proto", plist->columnWidth (slot)) ;
			slot	+= 1 ;
		}
		if (options & PO_SRC  )
		{	_config->writeEntry ("w_Srce",  plist->columnWidth (slot)) ;
			slot	+= 1 ;
		}
		if (options & PO_DST  )
		{	_config->writeEntry ("w_Dest",  plist->columnWidth (slot)) ;
			slot	+= 1 ;
		}
		if (options & PO_INFO )
		{	_config->writeEntry ("w_Info",  plist->columnWidth (slot)) ;
			slot	+= 1 ;
		}
		if (options & PO_SIZE )
		{	_config->writeEntry ("w_Size",  plist->columnWidth (slot)) ;
			slot	+= 1 ;
		}
	}
}

/*  KNDPacket	: 							*/
/*  showPacket	: Show details for packet				*/
/*  lvi		: QListViewItem *: List view item for packet		*/
/*  (returns)	: void		 :					*/

void	KNDPacket::showPacket
	(	QListViewItem	*lvi
	)
{
	KNDDetails *detail ;

	/* If either there are no packet detail windows, or we are set	*/
	/* to show multiple ones, then create a new details object;	*/
	/* otherwise, use the extant one.				*/
	if (multiPkt || (details.count() == 0))
	{
		detail	= new KNDDetails (this,
					  parent->getName  (),
					  parent->getIface ()) ;
		details.append    (detail) ;
	}
	else	detail	= details.first () ;

	/* Raise the window, in case it was hidden somwhere, then give	*/
	/* it the packet.						*/
	detail->raise () ;
	detail->setPktInfo (((KNDPacketItem *)lvi)->getPktInfo ()) ;
}

/*  KNDPacket								*/
/*  showPacket	: Show packet details					*/
/*  (returns)	: void		:					*/

void	KNDPacket::showPacket ()
{
	/* This slot is invoked from the right-click menu, in which	*/
	/* case the list view item should be set in "pitem".		*/
	if (pitem != NULL) showPacket (pitem) ;
}

/*  KNDPacket								*/
/*  showTCPData	: Show TCP stream data					*/
/*  (returns)	: void		:					*/

void	KNDPacket::showTCPData ()
{
	KNDTCPData *tcpdat ;

	/* This slot is invoked from the right-click menu, in which	*/
	/* case the list view item should be set in "pitem".		*/
	if (pitem == NULL) return ;

	/* If either there are no TCP data windows, or we are set	*/
	/* to show multiple ones, then create a new TCP data object;	*/
	/* otherwise, use the extant one.				*/
	if (multiPkt || (tcpdata.count() == 0))
	{
		tcpdat	= new KNDTCPData (this,
					  parent->getName  (),
					  parent->getIface ()) ;
		tcpdata.append    (tcpdat) ;
	}
	else	tcpdat	= tcpdata.first () ;

	/* Raise the window, in case it was hidden somwhere, then give	*/
	/* it the packet.						*/
	tcpdat->raise () ;
	refreshTCP (tcpdat, ((KNDPacketItem *)pitem)->getPktInfo()->tcphash) ;
}

/*  KNDPacket								*/
/*  refreshTCP	: Refresh TCP data stream display			*/
/*  tcpdat	: KNDTCPData *	: Data stream display			*/
/*  tcph	: TCPHASH *	: TCP stream identity			*/

void	KNDPacket::refreshTCP
	(	KNDTCPData	*tcpdat,
		TCPHASH		*tcph
	)
{
	int	idx	;
	int	slot	;

	tcpdat->clear () ;

	for (idx = 0, slot = offset ; idx < numpack ; idx += 1)
	{
		if ((packets[slot] != NULL) && (packets[slot]->tcphash == tcph))
		{
			tcpdat->addData (packets[slot]) ;
#ifdef	_DEBUG
			fprintf	(stderr, "tcpdata: add %3d: 0x%08lx\n",
					 slot,
					 (long)packets[slot]->tcphash) ;
#endif	/* _DEBUG */
		}

		if ((slot += 1) >= numpack)
			slot = 0 ;
	}

	tcpdat->loaded (tcph) ;
}

/*  KNDPacket								*/
/*  moveAlong	: Move along TCP/IP data stream				*/
/*  display	: KNDDetails *	: Details display object		*/
/*  pktinfo	: PktInfo *	: Current packet			*/
/*  id		: int		: Operation requested			*/
/*  (returns)	: bool		: TRUE if packet found			*/

bool	KNDPacket::moveAlong
	(	KNDDetails	*display,
		PktInfo		*pktinfo,
		int		id
	)
{
	int	idx	;
	int	slot	;
	PktInfo	*ppkt	;
	PktInfo	*move	= NULL ;
	TCPHASH	*tcph	= pktinfo->tcphash ;

	switch (id)
	{
		case ID_DETAILS_NEXT	:
			for (idx = 0, slot = offset ; idx < numpack ; idx += 1)
			{
				if ((slot -= 1) < 0) slot = numpack - 1 ;

				/* If this slot contains the currently	*/
				/* detailed packet then exit from the	*/
				/* loop.				*/
				if ((ppkt = packets[slot]) == pktinfo)
					break ;

				/* If this slot defines a packet and	*/
				/* it is for the same TCPIP stream	*/
				/* then note it as the next packet.	*/
				if ((ppkt != NULL) && (ppkt->tcphash == tcph))
					move	= ppkt ;
			}
			break	;

		case ID_DETAILS_PREVIOUS:
			for (idx = 0, slot = offset ; idx < numpack ; idx += 1)
			{
				/* If this slot contains the currently	*/
				/* detailed packet then exit from the	*/
				/* loop.				*/
				if ((ppkt = packets[slot]) == pktinfo)
					break ;

				/* If this slot defines a packet and	*/
				/* it is for the same TCPIP stream	*/
				/* then note it as the previous		*/
				/* packet.				*/
				if ((ppkt != NULL) && (ppkt->tcphash == tcph))
					move	= ppkt ;

				if ((slot += 1) >= numpack) slot = 0 ;
			}
			break	;

		case ID_DETAILS_FIRST	:
			/* Find first (recorded) packet. Work forward	*/
			/* through packets starting with the one that	*/
			/* would next be zapped.			*/
			for (idx = 0, slot = offset ; idx < numpack ; idx += 1)
			{
				if ((ppkt = packets[slot]) != NULL)
					if (ppkt->tcphash == tcph)
					{	move	= ppkt ;
						break	;
					}

				if ((slot += 1) >= numpack) slot = 0 ;
			}
			break	;

		case ID_DETAILS_LAST	:
			/* Find last (recorded) packet. Work backward	*/
			/* through packets starting with the last one	*/
			/* added.					*/
			for (idx = 0, slot = offset ; idx < numpack ; idx += 1)
			{
				if ((slot -= 1) < 0) slot = numpack - 1 ;

				if ((ppkt = packets[slot]) != NULL)
					if (ppkt->tcphash == tcph)
					{	move	= ppkt ;
						break	;
					}
			}
			break	;

		default	:
			break	;
			
	}

	if (move != NULL)
	{	display->setPktInfo (move) ;
		return	true ;
	}

	return	false ;
}


/*  KNDPacket	: 							*/
/*  rightClicked: Activate popup menu					*/
/*  lvi		: QListViewItem *: List view item for packet		*/
/*  pos		: const QPoint & : Position				*/
/*  col		: int		 : Column number			*/
/*  (returns)	: void		 :					*/

void	KNDPacket::rightClicked
	(	QListViewItem	*lvi,
		const QPoint	&pos,
		int		col
	)
{
	QPopupMenu	popup	;
	PktInfo		*detpkt	= ((KNDPacketItem *)lvi)->getPktInfo () ;

	pitem	= lvi	;
	popup.insertItem ("Show details", this, SLOT(showPacket())) ;

	if ((detpkt->ip != NULL) && (detpkt->ip->ip_p == IPPROTO_TCP))
		popup.insertItem ("Show TCP data", this, SLOT(showTCPData())) ;

	popup.exec(QCursor::pos()) ;
}

/*  KNDPacket	: 							*/
/*  endDetails	: Packet details object reports termination		*/
/*  detail	: KNDDetails *	: Specific object			*/
/*  (returns)	: void		:					*/

void	KNDPacket::endDetails
	(	KNDDetails *detail
	)
{
	details.remove (detail) ;
}

/*  KNDPacket	: 							*/
/*  endTCPData	: TCP data object reports termination		*/
/*  detail	: KNDTCPData *	: Specific object			*/
/*  (returns)	: void		:					*/

void	KNDPacket::endTCPData
	(	KNDTCPData *detail
	)
{
	tcpdata.remove (detail) ;
}
