/***************************************************************************
                          knd_pkt.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef	KND_PKT_H
#define KND_PKT_H 

#include	<qsplitter.h>
#include	<qscrollview.h>
#include	<qlistview.h>

#include	"ksnuffle.h"
#include	"knd_plugin.h"


#define	DEFMAXPACK	(100)
#define	DEFMINPACK	(DEFMAXPACK - 20)



/*  KNDPacketItem							*/
/*  This class is a slightly extended QListViewItem, mainly to keep	*/
/*  a packet information structure pointer.				*/

class	KNDPacketItem : public QListViewItem
{
	PktInfo	*info	;

public	:
	inline	KNDPacketItem
		(	QListView	*qlv,
			KNDPacketItem	*lastp,
			PktInfo		*info,
			const char	*a1,
			const char	*a2,
			const char	*a3,
			const char	*a4,
			const char	*a5,
			const char	*a6
		)
		:
		QListViewItem	(qlv, lastp, a1, a2, a3, a4, a5, a6),
		info		(info)
	{
		usePktInfo 	(info) ;
	}	;

	inline	~KNDPacketItem ()
	{
		freePktInfo	(info) ;
	}	;

	inline	PktInfo	*getPktInfo ()
	{
		return	info ;
	}	;
}	;


/*  KNDPacket	:							*/
/*  Instances of this class handle packet-level display of data coming	*/
/*  from the associated monitor.					*/

class	KNDPacket : public KNDPlugin
{
	Q_OBJECT
						/* Labels ...		*/
	QLabel		l_accept	 ;	/* Packets accepted	*/
	QLabel		l_dropped	 ;	/* Packets dropped	*/
	QLabel		l_missed	 ;	/* Packets missed	*/
	QLabel		l_pktbytes	 ;	/* Packet byte count	*/
	QLabel		l_datbytes	 ;	/* Packet data count	*/
						/* Values ...		*/
	QLabel		v_accept	 ;	/* Packets accepted	*/
	QLabel		v_dropped	 ;	/* Packets dropped	*/
	QLabel		v_missed	 ;	/* Packets missed	*/
	QLabel		v_pktbytes	 ;	/* Packet byte count	*/
	QLabel		v_datbytes	 ;	/* Packet data count	*/

	int		s_pktbytes	 ;	/* Total packet bytes	*/
	int		s_datbytes	 ;	/* Total packet data	*/

	bool		replay		 ;	/* This is a replayer	*/
	int		p_this		 ;	/* Packets this second	*/
	int		p_last		 ;	/* Packets last second	*/
	int		pausefor	 ;	/* Display paused time	*/

	int		w_Time		 ;
	int		w_Proto		 ;
	int		w_Src		 ;
	int		w_Dst		 ;
	int		w_Info		 ;
	int		w_Size		 ;

	QListView	*plist		 ;	/* Packet display	*/
	KNDPacketItem	*lastp		 ;	/* Last item added	*/
	QListViewItem	*pitem		 ;	/* Popup menu item	*/
	int		options		 ;	/* Option flags		*/

	QList<KNDDetails> details 	 ;	/* Details windows	*/
	QList<KNDTCPData> tcpdata	 ;	/* TCP data windows	*/

	PktInfo		**packets	 ;	/* Current packet list	*/
	int		numpack		 ;	/* Number of packets	*/
	int		offset		 ;	/* Current list offset	*/

	bool		onview		 ;	/* Display on view	*/
	bool		going		 ;	/* Display going	*/
	bool		frozen		 ;

	void	putPacket	(PktInfo *, bool) ;
	void	putDots		()	 ;
	void	redisplay	()	 ;
	void	setPositions	()	 ;

virtual	void	resizeEvent	(QResizeEvent  *) ;

public	:
	KNDPacket	(KNDView *, KConfig *, bool);

	virtual	~KNDPacket	() ;

	virtual	void	addPacket	(PktInfo *) ;
	virtual	void	*setOptions	(void    *) ;
	virtual	void	setOnview	(bool)	    ;
	virtual	void	execute		(bool)	    ;
	virtual	void	freeze		(bool)	    ;
	virtual	void	timerTick	(long)	    ;
	virtual	void	writeConfig	(KConfig *) ;

	virtual	cchar	*getLabel   () { return	"Packets" ; } ;
	virtual	bool	isConfig    () { return	false	  ; } ;


	void	endDetails	(KNDDetails *) ;
	void	endTCPData	(KNDTCPData *) ;
	void	refreshTCP	(KNDTCPData *, TCPHASH *) ;
	bool	moveAlong	(KNDDetails *, PktInfo *, int) ;

public	slots :

	void	showPacket   ()		       ;
	void	showPacket   (QListViewItem *) ;
	void	rightClicked (QListViewItem *, const QPoint &, int) ;
	void	showTCPData  ()		       ;
}	;


#endif	// KND_PKT_H
