/***************************************************************************
                        knd_plugdlg.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<stdio.h>
#include	<string.h>
#include	<unistd.h>
#include	<dlfcn.h>
#include	<sys/types.h>
#include	<dirent.h>
#include	<errno.h>

#include	<qstrlist.h>
#include	<qlist.h>

#include	"knd_view.h"
#include	"knd_plugdlg.h"
#include	"knd_plugin.h"


typedef	struct	dirent	DIRENT	;

/*  Lists of plugins, identifier by name and with their corresponding	*/
/*  files, are updated each time the "addPluginDir" routine is called.	*/
/*  They are then used when the plugin selection dialog box is		*/
/*  displayed.								*/
class	PlugSpec
{
public	:
	QString	name	;
	QString	desc	;
	QString	file	;

	inline	PlugSpec
		(	cchar	*name,
			cchar	*desc,
			cchar	*file
		)
		:
		name	(name),
		desc	(desc),
		file	(file)
	{
	}	;
}	;


static	QList<PlugSpec>	plugspecs ;

/*  GetPluginFile: Get file for named plugin				*/
/*  name	 : cchar *	: Plugin name				*/
/*  file	 : QString &	: Plugin file				*/
/*  (returns)	 : bool		: True if found				*/

bool	GetPluginFile
	(	cchar	*name,
		QString	&file
	)
{
	QListIterator<PlugSpec> plugit (plugspecs) ;
	PlugSpec		*spec		   ;

	while ((spec = plugit.current ()) != NULL)
	{
		if (spec->name == name)
		{	file	= spec->file + ".so" ;
			return	true ;
		}

		plugit += 1 ;
	}

	return	false	;
}

/*  StrInStrList: See if string is in string list			*/
/*  string	: cchar		: String to search for			*/
/*  list	: QStrList *	: String list				*/
/*  (returns)	: int		: Index or negative if not found	*/

int	StrInStrList
	(	cchar	 *string,
		QStrList *list
	)
{
	for (uint idx = 0 ; idx < list->count() ; idx += 1)
		if (strcmp (list->at(idx), string) == 0) return idx ; 

	return	-1 ;
}

/*  KNDDlgList								*/
/*  KNDDlgList	: Constructor for list box with greyed items		*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KNDDlgList	:					*/

KNDDlgList::KNDDlgList
	(	QWidget	*parent
	)
	:
	QListView (parent)
{
	/* We want to know when the selection changes so that we can	*/
	/* check for selected greyed items, and unselect them.		*/
	connect	 (this, SIGNAL(selectionChanged()), this, SLOT(select())) ;
}

/*  KNDDlgList								*/
/*  select	: Handle selection change				*/
/*  (returns)	: void		:					*/

void	KNDDlgList::select ()
{
	/* Since the user may drag-select, we need to check all the	*/
	/* items.							*/
	for (QListViewItem *child = firstChild () ; child != NULL ; child = child->nextSibling())
		if (((KNDDlgItem *)child)->greyed)
			((KNDDlgItem *)child)->setSelected (false) ;
}


/*  KNDDlgItem								*/
/*  KNDDlgItem	: Constructor for greyable plugin list box item		*/
/*  parent	: KNDDlgList *	: Parent widget				*/
/*  _name	: cchar *	: Plugin name				*/
/*  _summary	: cchar *	: Plugin summary			*/
/*  _file	: cchar *	: Base file name			*/
/*  _greyed	: bool		: Greyed text flag			*/
/*  (returns)	: KNDDlgItem	:					*/

KNDDlgItem::KNDDlgItem
	(	KNDDlgList	*parent,
		cchar		*_name,
		cchar		*_summary,
		cchar		*_file,
		bool		_greyed
	)
	:
	QListViewItem	(parent, _name, _summary),
	name		(_name),
	file		(_file),
	greyed		(_greyed)
{
}

/*  KNDDlgItem								*/
/*  paint	: Paint an item						*/
/*  p		: QPainter *	: Paint context				*/
/*  (returns)	: void		:					*/

void	KNDDlgItem::paintCell
	(	QPainter 		*p,
		const QColorGroup	&cg,
		int			column,
		int			width,
		int			align
	)
{
	QColorGroup cg2 (cg.foreground(),
			 cg.background(),
			 cg.light     (),
			 cg.dark      (),
			 cg.mid	      (),
			 greyed ? gray : cg.text (),
			 cg.base      ()) ;
	QListViewItem::paintCell (p, cg2, column, width, align) ;
}


/*  KNDPluginDialog							*/
/*  KNDPluginDialog: Constructor for plugin selection dialog		*/
/*  parent	   : QWidget *	: Parent widget				*/
/*  res		   : QStrList *	: Results list				*/
/*  ignore	   : QStrList *	: Plugins to ignore			*/
/*  add		   : bool	: True for plugin add			*/
/*  (returns)	   : Dialog ..	:					*/

KNDPluginDialog::KNDPluginDialog
	(	QWidget	 *parent,
		QStrList *res,
		QStrList *ignore,
		bool	 add
	)
	:
	QDialog	(parent, "Plugin selection", true),
	label	(this),
	plugins	(this),
	info	(this),
	bOK	(this),
	bCancel	(this),
	res	(res)
{
	label  .setGeometry (  5,   5, 350,  25) ;
	plugins.setGeometry (  5,  35, 350, 150) ;
	info   .setGeometry (  5, 190, 350, 150) ;
	bOK    .setGeometry (225, 350,  60,  25) ; 
	bCancel.setGeometry (295, 350,  60,  25) ;
	bOK    .setText	    ("OK"    ) ;
	bCancel.setText	    ("Cancel") ;
	label  .setText	    ("Select plugins to load into sniffer") ;

	plugins.addColumn   ("Plugin", 	    100) ;
	plugins.addColumn   ("Description", 250) ;

	connect	(&bOK,     SIGNAL(clicked()), this, SLOT(accept())) ;
	connect	(&bCancel, SIGNAL(clicked()), this, SLOT(reject())) ;

	connect	(&plugins, SIGNAL(currentChanged(QListViewItem *)),
		 this,     SLOT  (current(	 QListViewItem *))) ;


	QListIterator<PlugSpec> plugit (plugspecs) ;
	PlugSpec		*spec		   ;

	/* Scan the plugins lists getting entries for the list box. We	*/
	/* skip any that that are specified in the ignore list (note,	*/
	/* these will be full path names, so we need to get the		*/
	/* corresponding entry from the file list and append the .so	*/
	/* extension).							*/
	while ((spec = plugit.current ()) != NULL)
	{
		bool	inl	= StrInStrList (spec->name, ignore) >= 0 ;

		new KNDDlgItem (&plugins,
				spec->name,
				spec->desc,
				spec->file, add ? inl : !inl) ;

		plugit += 1 ;
	}

	plugins.setMultiSelection (true) ;
	info   .setReadOnly	  (true) ;
}

/*  KNDPluginDialog							*/
/*  ~KNDPluginDialog: Constructor for plugin selection dialog		*/
/*  (returns)	    :		:					*/

KNDPluginDialog::~KNDPluginDialog ()
{
}


/*  KNDPluginDialog							*/
/*  select	   : Handle new selection				*/
/*  (returns)	   : void	:					*/

void	KNDPluginDialog::current
	(	QListViewItem	*lvi
	)
{
	FILE		*fd	;
	char		buff	[512]	;

	info.clear ()	;

	/* Check that we can get a corresponding entry from the list of	*/
	/* plugin files, and that the specification file still exists.	*/
	/* Provided that it does, copy the text into the information	*/
	/* window.							*/
	sprintf	(buff, "%s.spec", ((KNDDlgItem *)lvi)->getFile()) ;
	if ((fd = fopen (buff, "r")) == NULL)
	{	info.setText ("ERROR: lost specification file") ;
		return	;
	}

	fgets	(buff, sizeof(buff), fd) ;

	while (fgets (buff, sizeof(buff), fd) != NULL)
	{
		buff[strlen(buff)-1] = 0 ;
		info.append (buff) ;
	}

	fclose	(fd) ;
}


/*  KNDPluginDialog							*/
/*  done	   : Handle dialog completion				*/
/*  ok		   : int	: Non zero on OK			*/
/*  (returns)	   : void	:					*/

void	KNDPluginDialog::done
	(	int	ok
	)
{
	if (ok)
		for (QListViewItem *child = plugins.firstChild () ; child != NULL ; child = child->nextSibling())
			if (child->isSelected ())
				res->append (((KNDDlgItem *)child)->getName ()) ;

	QDialog::done (ok) ;
}


/*  addPluginDir: Add plugins from a directory				*/
/*  dirname	: char *	: Directory name			*/
/*  (returns)	: void		:					*/

void	addPluginDir
	(	cchar	*dirname
	)
{
	DIR	*dir	;
	DIRENT	*dirent	;
	char	path	[512]	;

	/* Open the directory. If this fails then we have been grossly	*/
	/* mislead.							*/
	if ((dir = opendir (dirname)) == NULL)
	{	fprintf	(stderr, "ksnuffle: %s: %s\n", dirname, STRERR) ;
		return	;
	}

	/* Scan the directory. The plugin criteria is that the plugin	*/
	/* file must have a name like "plugin.so", and there must be	*/
	/* a specification file called "plugin.spec".			*/
	while ((dirent = readdir (dir)) != NULL)
	{
		char	*name	= dirent->d_name ;
		int	nlen	= strlen (name)	 ;
		FILE	*fd	;
		char	l1[80]	;
		char	*lp	;

		if ((nlen < 3) || (strcmp (&name[nlen-3], ".so") != 0))
			continue ;

		name[nlen - 3] = 0 ;

		sprintf	(path, "%s/%s.spec", dirname, name) ;
		if ((fd = fopen (path, "r")) == NULL) continue ;
		fgets	(l1, sizeof(l1), fd) ;
		fclose	(fd) ;
		if ((lp = strchr (l1, '\n')) != NULL) *lp = 0 ;

		/* Found a plugin. Add the path name (less the		*/
		/* extension) to the path list, and the name to the	*/
		/* name list.						*/
		sprintf	(path, "%s/%s", dirname, name) ;
		plugspecs.append (new PlugSpec (name, l1, path)) ;

	}

	closedir (dir)	;
}

/*  PluginsToLoad: Get a list of plugins to load			*/
/*  parent	 : QWidget *	: Parent widget				*/
/*  ignore	 : QStrList *	: List of plugins to ignore		*/
/*  (returns)	 : QStrList *	: List of plugins (may be empty)	*/

QStrList *PluginsToLoad
	 (	QWidget	 *parent,
		QStrList *ignore
	 )
{
	QStrList	*res	= new QStrList  (true) ;
	KNDPluginDialog pd (parent, res, ignore, true) ;

	pd.exec	()	;
	return	res	;
}

/*  PluginsToRemove: Get a list of plugins to remove			*/
/*  parent	   : QWidget *	: Parent widget				*/
/*  loaded	   : QStrList *	: List of plugins loaded		*/
/*  (returns)	   : QStrList *	: List of plugins (may be empty)	*/

QStrList *PluginsToRemove
	 (	QWidget	 *parent,
		QStrList *loaded
	 )
{
	QStrList	*res	= new QStrList   (true) ;
	KNDPluginDialog pd (parent, res, loaded, false)  ;

	pd.exec	()	;
	return	res	;
}


