/***************************************************************************
                        knd_plugin.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<stdio.h>
#include	<string.h>
#include	<unistd.h>
#include	<dlfcn.h>
#include	<errno.h>

#include	<qstrlist.h>
#include	<qlist.h>

#include	"knd_view.h"
#include	"knd_plugin.h"

/*  The dlopen(3) keeps track of dynamically loaded modules, so that a	*/
/*  given module is loaded at most once. If this is not the case for	*/
/*  some system, then un'define _DLSHARED				*/
#define	_DLSHARED

#ifndef	_DLSHARED

/*  PluginMap								*/
/*  An array of PluginMap structures is used to maintain information	*/
/*  about all plugin sets which have been loaded since the program	*/
/*  started.								*/
struct	PluginMap
{
	void		*lib	  ;	/* Loaded library module	*/
	GETPLUGIN	*func	  ;	/* getPlugin function pointer	*/
	char		*name	  ;	/* Plugin name			*/
}	;

static	QArray<PluginMap> plugins ;

#endif	/* _DLSHARED */

extern	bool	GetPluginFile (cchar *, QString &) ;

/*  KNDPlugin								*/
/*  KNDPlugin	: Constructor for plugin object				*/
/*  _parent	: KNDView *	: Parent view object			*/
/*  name	: char *	: Plugin name				*/
/*  (returns)	: KNDPlugin	:					*/

KNDPlugin::KNDPlugin
	(	KNDView	*_parent,
		char	*name
	)
	:
	QFrame	(_parent, name),
	parent	(_parent)
{
}

/*  KNDPlugin								*/
/*  ~KNDPlugin	: Destructor for plugin object				*/
/*  (returns)	:		:					*/

KNDPlugin::~KNDPlugin ()
{
}



/*  LoadPlugin	: Load a named plugin					*/
/*  name	: cchar *	: Plugin name				*/
/*  (returns)	: GETPLUGIN *	: Routine to get plugin or NULL		*/

GETPLUGIN *LoadPlugin
	  (	cchar	*name
	  )
{
	void		*lib	;
	GETPLUGIN	*func	;
	QString		file	;

	if (!GetPluginFile (name, file))
	{	Error	("Lost plugin name->file mapping", name) ;
		return	NULL ;
	}

#ifndef	_DLSHARED
	/* See if the plugin is already loaded; if so then we can use	*/
	/* it (all plugins are assumed re-entrant).			*/
	for (uint idx = 0 ; idx < plugins.size() ; idx += 1)
		if (strcmp (plugins[idx].name, name) == 0)
			return	plugins[idx].func ;
#endif	/* _DLSHARED */

	/* Load the library and locate the "getPlugin" routine therein.	*/
	/* If either fails return failure, unloading the library if	*/
	/* necessary.							*/
	if ((lib  = dlopen (file, RTLD_NOW)) == NULL)
	{
		fprintf	(stderr, "LoadPlugin: dlopen(%s): %s\n", (cchar *)file, dlerror()) ;
		return	NULL	;
	}

	if ((func = (GETPLUGIN *)dlsym(lib, "getPlugin")) == NULL)
	{
		fprintf	(stderr, "LoadPlugin: dlsym(getPlugin): %s\n", dlerror()) ;
		dlclose	(lib)	;
		return	NULL	;
	}

#ifndef	_DLSHARED
	/* Increase the size of the plugins array by one, and save	*/
	/* the informatio about this plugin.				*/
	uint slot = plugins.size () ;
	plugins.resize (slot + 1)   ;

	plugins[slot].lib	= lib	;
	plugins[slot].func	= func	;
	plugins[slot].name	= strdup (name)	;
#endif	_DLSHARED

	return	func ;
}


/*  plugConfig	: Pass configuration information within a plugin set	*/
/*  set		: PluginSet *	: Plugin set in question		*/
/*  (returns)	: void		:					*/

void	plugConfig
	(	PluginSet *set
	)
{
	int	idx	;
	void	*conf	;

	/* Work through eack plugin in the set in order; start with a	*/
	/* null pointer, and chain this value through each plugin. If	*/
	/* this is insufficient for some plugin set, then the plugins	*/
	/* can always "wire" themselves together, and the call to	*/
	/* "setOptions" on the first plugin can be used to trigger	*/
	/* whatever configuration needs to be done.			*/
	for (conf = NULL, idx = 0 ; idx < set->nPlugins ; idx += 1)
		conf = set->plugin[idx]->setOptions (conf) ;
}
