/***************************************************************************
                          knd_plugin.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef	KND_PLUGIN_H
#define KND_PLUGIN_H 

#include	<qlistbox.h>
#include	<qpushbutton.h>
#include	<qmultilinedit.h>

#include	"ksnuffle.h"


/*  The PluginSet is used to pass back a set of plugins from a loaded	*/
/*  plugin module. The "getPlugin" call into the module should return	*/
/*  a malloc'ed PluginSet structure, which can be sized to hold as many	*/
/*  plugins as needed.							*/

struct	PluginSet
{
	char		*name	   ;	/* Plugin name			*/
	int		major	   ;	/* Major release		*/
	int		minor	   ;	/* Major release		*/
	int		nPlugins   ;	/* Number of plugins		*/
	KNDPlugin	*plugin[1] ;	/* Array of plugins		*/
}	;



/*  KNDPlugin	:							*/
/*  Instances of this provide a common base for internal and external	*/
/*  plugin modules.							*/

class	KNDPlugin : public QFrame
{
	Q_OBJECT

protected :

	KNDView		*parent	;

public	:

	KNDPlugin (KNDView *, char *) ;
	virtual	~KNDPlugin () ;

	/* The first set of methods are implemented by plugins that	*/
	/* provide configuration information.				*/
	virtual	void	clickSet	()	    {} ;
	virtual	void	clickReset	()	    {} ;
	virtual	void	clickClear	()	    {} ;

	/* The second set of methods are implemented by display plugins	*/
	/* to display packet or related data.				*/
	virtual	void	addPacket	(PktInfo *) {} ;
	virtual	void	setOnview	(bool)	    {} ;
	virtual	void	execute		(bool)	    {} ;
	virtual	void	freeze		(bool)	    {} ;
	virtual	void	timerTick	(long)	    {} ;
	virtual	void	writeConfig	(KConfig *) {} ;

	/* Thirdly, method to be implemented by both configuration and	*/
	/* display plugins.						*/
	virtual	void	*setOptions	(void    *) { return NULL ; } ;

	/* Lastly, there are a pair of method which all plugins _must_	*/
	/* implement (hence, they are pure virtual methods), to provide	*/
	/* a label for the tab, and to indicate whether it is a		*/
	/* configuration plugin (strictly, whether it is active all the	*/
	/* time, rather than only when the sniffer is active). Note	*/
	/* that the pointer returned by "getLabel" must be valid for	*/
	/* the life of the plugin.					*/
	virtual	cchar	*getLabel	() = 0 ;
	virtual	bool	isConfig 	() = 0 ;
}	;



/*  For convenience, GETPLUGIN is a typedef for the result returned	*/
/*  by the "LoadPlugin" routine.					*/
typedef	PluginSet *GETPLUGIN  (KNDView *, KConfig *) ;
extern	GETPLUGIN *LoadPlugin (cchar   *) ;


/*  setVersion is effectively a macro, and is used to get the major and	*/
/*  minor version number.						*/

inline	void	getVersion
	(	int	&major,
		int	&minor
	)
{
	sscanf	(VERSION, "%d.%d", &major, &minor) ;
}
#endif	// KND_PLUGIN_H
