/*************************************************************************
                          knd_remote.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	"knd_remote.h"
#include	"knd_confmon.h"
#include	"knd_pcaprem.h"
#include	"knd_pkt.h"
#include	"knd_gra.h"
#include	"knd_trigger.h"
#include	"knd_multi.h"


/*  remName	: Get remote name for specified host			*/
/*  host	: cchar *	: Host name				*/
/*  (returns)	: cchar *	: Remote sniffer name			*/

static	cchar	*remName
	(	cchar	*host
	)
{
	cchar	*dp	;

	if ((dp = strrchr (host, '.')) != NULL) return &dp[1] ;

	return	host	;
}

/*  KNDRemote								*/
/*  KNDRemote	: Constructor for view object				*/
/*  parent	: QWidget *	: Parent widget				*/
/*  app		: KNDApp  *	: Application object			*/
/*  slot	: int		: Slot in parent view list		*/
/*  host	: cchar *	: Remote host name			*/
/*  port	: cchar *	: Remote sniffer port			*/
/*  pass	: cchar *	: Passord on remote machine		*/
/*  multi	: KNDMulti *	: Multiple graph object			*/
/*  prefilter	: ccahr *	: User packet pre-filter		*/
/*  appconf	: KConfig *	: Configuration information		*/
/*  (returns)	: KNDRemote	:					*/

KNDRemote::KNDRemote
	(	QWidget		*parent,
		KNDApp		*app,
		int		slot,
		cchar		*host,
		cchar		*port,
		cchar		*pass,
		KNDMulti	*multi,
		cchar		*prefilter,
		KConfig		*appconf
	)
	:
	KNDView	(parent, app, slot, remName(host), false, multi, appconf),
	remhost	(host),
	remport	(port),
	rempass	(pass)
{
	capture	= new KNDPCapRem (this, host, port, pass, prefilter) ;
	remconf = new KNDConfRem (this, trigger, slot, remName(host), host, appconf) ;
	config	= remconf ;

	addTabs	() ;
}

/*  KNDRemote								*/
/*  ~KNDRemote	: Destructor for view object				*/
/*  (returns)	:		:					*/

KNDRemote::~KNDRemote ()
{
}

/*  KNDRemote								*/
/*  AddIface	: Add an interface					*/
/*  iface	: cchar *	: Interface name			*/
/*  (returns)	: void		:					*/

void	KNDRemote::AddIface
	(	cchar	*iface
	)
{
	remconf->AddIface (iface) ;
}

/*  KNDRemote								*/
/*  getMType	: Get monitor type					*/
/*  (returns)	: MType		:					*/

MType	KNDRemote::getMType ()
{
	return	Remote	;
}

/*  KNDRemote								*/
/*  Finish	: Terminate remote sniffer				*/
/*  (returns)	: void		:					*/

void	KNDRemote::Finish ()
{
	app->Finish (this) ;
}

/*  KNDRemote								*/
/*  writeConfig	: Write monitor configuration				*/
/*  config	: KConfig *	: Configuration object			*/
/*  (returns)	: void		:					*/

void	KNDRemote::writeConfig
	(	KConfig	*config
	)
{
	config->writeEntry   ("RemHost", remhost) ;
	config->writeEntry   ("RemPort", remport) ;
	config->writeEntry   ("RemPass", rempass) ;
	KNDView::writeConfig (config) ;
}
