/***************************************************************************
                          knd_setup.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<errno.h>

#include	<qtooltip.h>
#include	<kfontdialog.h>

#include	"knd_setup.h"
#include	"knd_view.h"
#include	"knd_pkt.h"
#include	"pcaphdr.h"
#include	"capture.h"



bool	d_addr	   ;		/* Decode IP addresses to names		*/
bool	d_port	   ;		/* Decode IP ports to service names	*/
bool	j_priv	   ;		/* Just list privileged ports/services	*/
int	g_pauseat  ;
int	g_pausefor ;

bool	dropDom	   = true	;
bool	multiPkt   = false	;
int	maxpack	   = DEFMAXPACK ;


/*  KNDSetup								*/
/*  KNDSetup	: Constructor for general selup object			*/
/*  parent	: KNDApp *	: Parent widget				*/
/*  (returns)	: KNDSetup	:					*/

KNDSetup::KNDSetup
	(	KNDApp	*parent
	)
	:
	QWidget	 (parent),
	g_opts	 (this),
	c_addr	 (this),
	c_port	 (this),
	c_privp	 (this),
	c_dropd	 (this),
	c_ppkt	 (this),
	l_pat	 (this),	s_pat	(this),
	l_pfor	 (this),	s_pfor	(this),
	l_maxp	 (this),	s_maxp	(this),
	b_font	 (this),
	b_savec	 (this)
{
	extern	QFont	*pktfont;
 	KConfig	*gconf = kapp->getConfig () ;

	hide	() ;

	f_isset	= false ;

	g_opts	.setGeometry ( 10, 10,  535, 250) ;
	g_opts	.setTitle    (i18n ("Options")) ;

	c_addr	.setGeometry ( 20,  30, 160,  25) ;
	c_addr	.setText     (i18n ("Decode IP addresses"    )) ;
	c_port	.setGeometry ( 20,  60, 160,  25) ;
	c_port	.setText     (i18n ("Decode port numbers"    )) ;
	c_privp	.setGeometry (200,  30, 160,  25) ;
	c_privp	.setText     (i18n ("List priv ports only"   )) ;
	c_dropd	.setGeometry (200,  60, 160,  25) ;
	c_dropd	.setText     (i18n ("Drop local domain name" )) ;
	c_ppkt  .setGeometry (380,  30, 160,  25) ;
	c_ppkt  .setText     (i18n ("Multiple packet details")) ;

	QToolTip::add (&c_addr,  i18n ("Decode IP addresses to names"        )) ;
	QToolTip::add (&c_port,  i18n ("Decode port numbers to services"     )) ;
	QToolTip::add (&c_privp, i18n ("List only privileged ports"	     )) ;
	QToolTip::add (&c_dropd, i18n ("Drop local domain name from hosts"   )) ;
	QToolTip::add (&c_ppkt,  i18n ("Show multiple packet details windows")) ;

	l_pat	.setGeometry (20, 120, 60, 25) ;
	l_pat	.setText     (i18n ("Pause at" )) ;
	s_pat	.setGeometry (90, 120, 60, 25) ;
	s_pat	.setRange    (1, 200) ;
	l_pfor	.setGeometry (20, 150, 60, 25) ;
	l_pfor	.setText     (i18n ("Pause for")) ;
	s_pfor	.setGeometry (90, 150, 60, 25) ;
	s_pfor	.setRange    (1,  10) ;
	l_maxp	.setGeometry (20, 180, 60, 25) ;
	l_maxp	.setText     (i18n ("Max pkts" )) ;
	s_maxp	.setGeometry (90, 180, 60, 25) ;
	s_maxp	.setRange    (50, 10000) ;

	QToolTip::add (&s_pat,   i18n ("Pause packet display load (pkts/sec)")) ;
	QToolTip::add (&s_pfor,  i18n ("Pause packet display time"	     )) ;
	QToolTip::add (&s_maxp,  i18n ("Maximum number of packets to display")) ; 

	gconf->setGroup (i18n ("Global")) ;
	c_addr	.setChecked  (d_addr     = gconf->readBoolEntry ("DecodeIP",      true)) ;
	c_port	.setChecked  (d_port     = gconf->readBoolEntry ("DecodePorts",   true)) ;
	c_privp	.setChecked  (j_priv     = gconf->readBoolEntry ("JustPrivPorts", true)) ;
	c_dropd	.setChecked  (dropDom    = gconf->readBoolEntry ("DropDomain",    true)) ;
	c_ppkt	.setChecked  (dropDom    = gconf->readBoolEntry ("MultiDetails", false)) ;
	s_pat   .setValue    (g_pauseat  = gconf->readNumEntry  ("PauseAt",	  15  )) ;
	s_pfor  .setValue    (g_pausefor = gconf->readNumEntry  ("PauseFor",	  2   )) ;
	s_maxp  .setValue    (maxpack	 = gconf->readNumEntry  ("MaxPackets",	  DEFMAXPACK)) ;

	f_family	= gconf->readEntry    ("font_family") ;
	f_psize		= gconf->readNumEntry ("font_size"  ) ;

	if ((f_family.length() > 0) && (f_psize >= 0))
		pktfont	= new QFont (f_family, f_psize) ;

	b_font  .setGeometry ( 20, 215, 70,  25) ;
	b_font  .setText     (i18n ("Font")) ;
	b_savec .setGeometry (100, 215, 70,  25) ;
	b_savec .setText     (i18n ("Save")) ;

	connect	(&b_font,  SIGNAL(clicked()), this, SLOT(clickPickFont ())) ;
	QToolTip::add (&b_font,  i18n ("Select packet list font")) ;
	connect	(&b_savec, SIGNAL(clicked()), this, SLOT(clickSaveSetup())) ;
	QToolTip::add (&b_savec, i18n ("Save global options")) ;
}

/*  KNDUser								*/
/*  ~KNDUser	: Destructor for global setup object			*/
/*  (returns)	:		:					*/

KNDSetup::~KNDSetup ()
{
}

/*  KNDSetup								*/
/*  clickSaveSetup: Save the global setup options			*/
/*  (returns)	  : void	:					*/

void	KNDSetup::clickSaveSetup ()
{
 	KConfig	*gconf = kapp->getConfig () ;

	gconf->setGroup   ("Global") ;

	gconf->writeEntry ("DecodeIP",	    d_addr     = c_addr .isChecked()) ;
	gconf->writeEntry ("DecodePorts",   d_port     = c_port .isChecked()) ;
	gconf->writeEntry ("JustPrivPorts", j_priv     = c_privp.isChecked()) ;
	gconf->writeEntry ("DropDomain",    dropDom    = c_dropd.isChecked()) ;
	gconf->writeEntry ("MultiDetails",  multiPkt   = c_ppkt .isChecked()) ;
	gconf->writeEntry ("PauseAt", 	    g_pauseat  = s_pat  .value()) ;
	gconf->writeEntry ("PauseFor", 	    g_pausefor = s_pfor .value()) ;
	gconf->writeEntry ("MaxPackets",    maxpack    = s_maxp .value()) ;

	if (f_isset)
	{	gconf->writeEntry ("font_family", f_family) ;
		gconf->writeEntry ("font_size",   f_psize ) ;
	}

	gconf->sync () ;
}

/*  KNDSetup								*/
/*  clickPickFont : Select packet display font				*/
/*  (returns)	  : void	:					*/

void	KNDSetup::clickPickFont ()
{
	extern	QFont	*pktfont ;

	KFontDialog	fDialog (this, NULL, TRUE) ;
	QFont		theFont	 ;

	if (pktfont != NULL) fDialog.setFont (*pktfont) ;

	if (!fDialog.getFont (theFont)) return  ;

	if (pktfont != NULL) delete	pktfont	;

	pktfont		= new QFont (theFont)   ;
	f_isset		= true	;
	f_family	= pktfont->family    () ;
	f_psize		= pktfont->pointSize ()	;
}
