/***************************************************************************
                          knd_tcpdata.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<stdarg.h>
#include	<ctype.h>
#include	<qtooltip.h>

#include	<ktmainwindow.h>
#include	<kiconloader.h>

#include	"pcaphdr.h"
#include	"knd_plugin.h"
#include	"knd_view.h"
#include	"knd_pkt.h"
#include	"knd_tcpdata.h"
#include	"knd_pcap.h"

#include	"nethdrs.h"
#include	"resource.h"


extern	char	*dmap	;	/* Character display map		*/

/*  KNDTCPData								*/
/*  KNDTCPData	: Constructor for TCP stream data objects		*/
/*  parent	: KNDPacket	: Parent object				*/
/*  name	: cchar *	: Sniffer name				*/
/*  iface	: cchar *	: Interface				*/
/*  (returns)	: KNDTCPData	:					*/

KNDTCPData::KNDTCPData
	(	KNDPacket	*parent,
		cchar		*name,
		cchar		*iface
	)
	:
	KTMainWindow	(NULL),
	name		(name),
	iface		(iface),
	parent		(parent)
{
	setCaption (name) ;
	resize	   (540, 400) ;

	toolBar()->insertButton (Icon("ksn_refresh.xpm"), ID_TCPDATA_REFRESH, true, "Refresh") ;
	connect	(toolBar(), SIGNAL(clicked(int)), SLOT(command(int))) ;

	tcphash	= NULL ;
	bytes	= new KNDTCPBytes (this) ;
	bytes->setGeometry (QRect (QPoint (0,0), size())) ;
	KTMainWindow::setView (bytes) ;

	show	() ;
}

/*  KNDTCPData								*/
/*  ~KNDTCPData	: Destructor for packet details objects			*/
/*  (returns)	:		:					*/

KNDTCPData::~KNDTCPData ()
{
}

/*  KNDTCPData								*/
/*  command	: Handle command from toolbar				*/
/*  id		: int		: Command				*/
/*  (returns)	: void		:					*/

void	KNDTCPData::command
	(	int	id
	)
{
	switch (id)
	{
		case ID_TCPDATA_REFRESH :
			if (tcphash != NULL)
				parent->refreshTCP (this, tcphash) ;
			return	;

		default	:
			break	;
	}
}

/*  KNDTCPData								*/
/*  closeEvent	: Handle a close					*/
/*  e		: QCloseEvent *	: Close event information		*/
/*  (returns)	: void		:					*/

void	KNDTCPData::closeEvent
	(	QCloseEvent	*e
	)
{
	/* This event is picked up so that we can inform our parent	*/
	/* (who is not a parent from Qt's point of view) of our demise.	*/
	parent->endTCPData    (this) ;
	KTMainWindow::closeEvent (e) ;
}


/*  KNDTCPBytes								*/
/*  KNDTCPBytes	: Constructor for packet bye display object		*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KNDTCPBytes	:					*/

KNDTCPBytes::KNDTCPBytes
	(	QWidget	*parent
	)
	:
	QScrollView (parent)
{
}

/*  KNDTCPBytes								*/
/*  doResize	: Handle resize						*/
/*  (returns)	: void		:					*/

void	KNDTCPBytes::doResize ()
{
	/* We need to make sure that the contents widget is (a) not	*/
	/* smalled than the size needed to display the packet and (b)	*/
	/* not smaller than the scroll display (so that the background	*/
	/* is uniform).							*/
	int	width	= size().width () ;
	int	height	= size().height() ;

	if (width  < minwidth ) width  = minwidth  ;
	if (height < minheight) height = minheight ;

	resizeContents (width, height) ;
}

/*  KNDTCPBytes								*/
/*  resizeEvent	: Handle a resize					*/
/*  e		: QResizeEvent *: Resize event information		*/
/*  (returns)	: void		:					*/

void	KNDTCPBytes::resizeEvent
	(	QResizeEvent	*e
	)
{
	doResize () ;
}

/*  KNDTCPBytes								*/
/*  clear	: Clear display preparatory to reload			*/
/*  (returns)	: void		:					*/

void	KNDTCPBytes::clear ()
{
	text.clear  ()	;
	text.append (new QString ("")) ;
	maxcols	  = 0	;
	minwidth  = 0	;
	minheight = 0	;
}

/*  KNDTCPBytes								*/
/*  addData	: Add data to window					*/
/*  pktinfo	: PktInfo *	: Packet				*/
/*  (returns)	: void		:					*/

void	KNDTCPBytes::addData
	(	PktInfo	*pktinfo
	)
{
	TCP_HDR	*tcp	= (TCP_HDR *)(pktinfo->ipdata)  ;
	char	*app	= (char *)tcp + tcp->th_off * 4 ;
	int	len	= pktinfo->piHdr.pktlen - (app - pktinfo->data) ;
	char	mask	= pktinfo->rev ? 0x80 : 0x00	;
	QString	*l	= text.current () ;

	while (len > 0)
	{
		/* Lines are forcibly split at 256 characters. Apart	*/
		/* from this, split on the newline character. Since we	*/
		/* will only display printable ASCII characters, the	*/
		/* top bit can be used to flag the direction.		*/
		if (l->length() < 256)
		{
			if (*app == '\n')
				text.append (l = new QString ("")) ;
			else	*l += (dmap[*app] & 0x7f) | mask   ;

			app += 1 ;
			len -= 1 ;
		}
		else	text.append (l = new QString ("")) ;

		if (l->length() > maxcols) maxcols = l->length() ;
	}
}

/*  KNDTCPBytes								*/
/*  loaded	: Handle data load completion				*/
/*  (returns)	: void		:					*/

void	KNDTCPBytes::loaded ()
{
	QFontMetrics m  (QFont ("fixed", 10)) ;

	if (text.current()->length() > maxcols)
		maxcols	= text.current()->length()  ;

	minwidth  = m.maxWidth   () * maxcols + 10  ;
	minheight = m.lineSpacing() * text.count () ;

	doResize	    ()      ;
	setContentsPos 	    (0,0)   ;
	viewport()->repaint (false) ;
}

/*  KNDTCPBytes								*/
/*  drawContentsOffset							*/
/*		: Draw contents						*/
/*  p		: QPainter *	: Painter object			*/
/*  offsetx	: int		: X scroll offset			*/
/*  offsety	: int		: Y scroll offset			*/
/*  clipx	: int		: X clip value				*/
/*  clipy	: int		: Y clip value				*/
/*  clipw	: int		: Clip width				*/
/*  cliph	: int		: Clip height				*/
/*  (returns)	: void		:					*/

void	KNDTCPBytes::drawContentsOffset
	(	QPainter *p,
		int	 offsetx,
		int	 offsety,
		int	 clipx,
		int	 clipy,
		int	 clipw,
		int	 cliph
	)
{
	/* Get the font metrics, and clear the entire display area. It	*/
	/* isn't worth trying to optimise anything.			*/
	p->setFont	(QFont ("fixed", 10)) ;
	QFontMetrics	m = p->fontMetrics () ;

	/* Sort out clipping and stuff. We could make this a bit	*/
	/* quicker since we start with x1=y1=0, but the code is general	*/
	/* and follows the QScrollView documentation example.		*/
	int	x1	= 0 ;
	int	y1	= 0 ;
	int	x2	= contentsWidth  () ;
	int	y2	= contentsHeight () ;

	if (x1 < clipx) x1 = clipx ;
	if (y1 < clipy) y1 = clipy ;
	if (x2 > clipx + clipw) x2 = clipx + clipw ;
	if (y2 > clipy + cliph) y2 = clipy + cliph ;

	x1 -= offsetx ;
	y1 -= offsety ;
	x2 -= offsetx ;
	y2 -= offsety ;

	if ((x2 > x1) && (y2 > y1))
		p->fillRect (x1, y1, x2 - x1, y2 - y1, white) ;

	p->setPen   (black) ;

	/* Just redraw the lines that appear in the window. However,	*/
	/* we don't bother to clip left-right; since lines are chopped	*/
	/* at 256 characters, the line length will never exceed the	*/
	/* available coordinate system, not would it save much time, so	*/
	/* its not worth it.						*/
	uint l1 = (offsety 		    ) / m.lineSpacing () ;
	uint l2 = (offsety + size().height()) / m.lineSpacing () ;

	if (l2 > text.count()) l2 = text.count () ;

	for (uint idx = l1 ; idx < l2 ; idx += 1)
	{
		char	b[300]	;
		char	*bp	;
		int	off0	= 0 ;
		int	off1	= 0 ;
		int	mask	= 0 ;
		cchar	*line	= *text.at(idx) ;
		int	llen	=  text.at(idx)->length() ;

		/* Scan the line into chunks with the same direction	*/
		/* flag; each such chunk can be written to the display	*/
		/* window, colour coded for direction.			*/
		while (off0 < llen)
		{
			bp	= b ;

			while ((off1 < llen) && ((line[off1] & 0x80) == mask))
			{	*bp++	= line[off1] & 0x7f ;
				off1   += 1 ;
			}

			p->setPen   (mask == 0 ? black : red) ;
			p->drawText (m.maxWidth   () * off0      - offsetx + 5,
				     m.lineSpacing() * (idx + 1) - offsety,
				     b,
				     off1 - off0) ;

			off0	= off1	;
			mask	= line[off0] & 0x80 ;
		}
	}
}
