/***************************************************************************
                          knd_trigger.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<qtooltip.h>

#include	"knd_trigger.h"
#include	"knd_view.h"
#include	"knd_filter.h"

/*  KNDTrigger								*/
/*  KNDTrigger	: Constructor for filter/trigger settings object	*/
/*  parent	: KNDView *	: Parent				*/
/*  (returns)	: KNDTrigger	:					*/

KNDTrigger::KNDTrigger
	(	KNDView		*_parent,
		KConfig		*config
	)
	:
	KNDPlugin  (_parent, "KND_TRIGGER"),
	k_tabctl   (this),
	g_mode	   (this),
	b_all	   (this),
	b_tostop   (this),
	b_fromstart(this),
	b_oneshot  (this),
	b_repeat   (this)
{
	filter	   = new KNDFilter (&k_tabctl, NULL, config) ;
	start	   = new KNDFilter (&k_tabctl,  "b", config) ;
	stop	   = new KNDFilter (&k_tabctl,  "e", config) ;

	k_tabctl.setGeometry (135, 5, 415, 210) ;

	k_tabctl.addTab (filter, "Packet filter") ;
	k_tabctl.addTab (start,  "Begin trigger") ;
	k_tabctl.addTab (stop,   "End trigger"  ) ;

	g_mode     .setGeometry (  5,   5, 120, 210) ;
	g_mode     .setTitle    ("Trigger mode" ) ;

	b_all      .setGeometry ( 10,  25, 100,  25) ;
	b_all	   .setText     ("All packets" ) ;
	b_all	   .setChecked  (true ) ;
	QToolTip::add (&b_all,       i18n("Display all packets")) ;

	b_tostop   .setGeometry ( 10,  55, 100,  25) ;
	b_tostop   .setText     ("Until stop"  ) ;
	b_tostop   .setChecked  (false) ;
	QToolTip::add (&b_tostop,    i18n("Display packets until stop event")) ;

	b_fromstart.setGeometry ( 10,  85, 100,  25) ;
	b_fromstart.setText     ("From start"  ) ;
	b_fromstart.setChecked  (false) ;
	QToolTip::add (&b_fromstart, i18n("Shaw packets after start event")) ;

	b_oneshot  .setGeometry ( 10, 115, 100,  25) ;
	b_oneshot  .setText     ("One shot"    ) ;
	b_oneshot  .setChecked  (false) ;
	QToolTip::add (&b_oneshot,   i18n("Display packets between start and stop event")) ;

	b_repeat   .setGeometry ( 10, 145, 100,  25) ;
	b_repeat   .setText     ("Repeat"      ) ;
	b_repeat   .setChecked  (false) ;
	QToolTip::add (&b_repeat,    i18n("Repeated display between start and stop events")) ;

	g_mode     .insert      (&b_all,       TR_ALL      ) ;
	g_mode     .insert      (&b_tostop,    TR_TOSTOP   ) ;
	g_mode     .insert      (&b_fromstart, TR_FROMSTART) ;
	g_mode     .insert      (&b_oneshot,   TR_ONESHOT  ) ;
	g_mode     .insert      (&b_repeat,    TR_REPEAT   ) ;

	connect	(&g_mode,  SIGNAL(clicked(int)), this, SLOT(selectMode(int))) ;

	if (config != NULL)
		setMode	(config->readNumEntry ("triggerMode", TR_ALL)) ;
	else	setMode	(TR_ALL) ;
}

/*  KNDTrigger								*/
/*  ~KNDTrigger	: Destructor for packet display object			*/
/*  (returns)	:		:					*/

KNDTrigger::~KNDTrigger ()
{
	delete	filter	;
	delete	start	;
	delete	stop	;
}

/*  KNDTrigger								*/
/*  setMode	: Set trigger mode					*/
/*  _mode	: int		: Mode to set				*/
/*  (returns)	: void		:					*/

void	KNDTrigger::setMode
	(	int	_mode
	)
{
	switch (triginf.mode = _mode) 
	{	case TR_ALL 	 : b_all      .setChecked (true) ; break ;
		case TR_TOSTOP	 : b_tostop   .setChecked (true) ; break ;
		case TR_FROMSTART: b_fromstart.setChecked (true) ; break ;
		case TR_ONESHOT  : b_oneshot  .setChecked (true) ; break ;
		case TR_REPEAT	 : b_repeat   .setChecked (true) ; break ;
		default		 : b_all      .setChecked (true) ;
				   triginf.mode = TR_ALL	 ; break ;
				   
	}
}

/*  KNDTrigger								*/
/*  selectMode	: Trigger mode select event				*/
/*  _mode	: int		: Mode selected				*/
/*  (returns)	: void		:					*/

void	KNDTrigger::selectMode
	(	int	_mode
	)
{
	triginf.mode = _mode ;
}

/*  KNDTrigger								*/
/*  clickSet	: Handle set event					*/
/*  (returns)	: void		:					*/

void	KNDTrigger::clickSet ()
{
	filter->clickSet () ;
	start ->clickSet () ;
	stop  ->clickSet () ;
}

/*  KNDTrigger								*/
/*  clickClear	: Handle clear button					*/
/*  (returns)	: void		:					*/

void	KNDTrigger::clickClear ()
{
	filter->clickClear () ;
	start ->clickClear () ;
	stop  ->clickClear () ;
}

/*  KNDTrigger								*/
/*  clickReset	: Handle reset button					*/
/*  (returns)	: void		:					*/

void	KNDTrigger::clickReset ()
{
	filter->clickReset () ;
	start ->clickReset () ;
	stop  ->clickReset () ;
}

/*  KNDTrigger								*/
/*  getInfo	: Get trigger information				*/
/*  (returns)	: TrigInf *	: Information structure			*/

TrigInf	*KNDTrigger::getInfo ()
{
	triginf.filter	= filter->getProgram () ;
	triginf.start	= triginf.mode & TR_START ? (cchar *)start ->getProgram () : NULL ;
	triginf.stop	= triginf.mode & TR_STOP  ? (cchar *)stop  ->getProgram () : NULL ;
	return	&triginf;
}

/*  KNDTrigger								*/
/*  writeConfig	: Write configuration					*/
/*  config	: KConfig *	: Configuration object			*/
/*  (returns)	: void		:					*/

void	KNDTrigger::writeConfig
	(	KConfig	*config
	)
{
	config->writeEntry  ("triggerMode", triginf.mode) ;
	filter->writeConfig (config) ;
	start ->writeConfig (config) ;
	stop  ->writeConfig (config) ;
}

/*  KNDTrigger								*/
/*  setTrigger	: Set trigger configuration				*/
/*  _filter	: cchar *	: Main filter				*/
/*  _start	: cchar *	: Start event				*/
/*  _stop	: cchar *	: Stop event				*/
/*  _mode	: int		: Trigger mode				*/
/*  (returns)	: void		:					*/

void	KNDTrigger::setTrigger
	(	TrigInf	*triginf
	)
{
	filter->setProgram (triginf->filter) ;
	start ->setProgram (triginf->start ) ;
	stop  ->setProgram (triginf->stop  ) ;
	setMode		   (triginf->mode  ) ;
}
