/***************************************************************************
                          knd_user.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<pwd.h>
#include	<errno.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<unistd.h>
#include	<shadow.h>

#include	<qtooltip.h>
#include	<kfontdialog.h>

#include	"knd_user.h"
#include	"knd_view.h"
#include	"knd_pkt.h"
#include	"pcaphdr.h"
#include	"capture.h"


/*  KNDPreFilter							*/
/*  KNDPreFilter: Constructor for pre-filter dialog			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  title	: cchar *	: Dialog box title			*/
/*  filter	: cchar *	: Current pre-filter text		*/
/*  (returns)	: KNDPreFilter	:					*/

KNDPreFilter::KNDPreFilter
	(	QWidget	*parent,
		cchar	*title,
		cchar	*filter
	)
	:
	QDialog	(parent, title, true),
	lFilter	(this),
	eFilter	(this),
	bOK	(this),
	bCancel	(this)
{
	lFilter.setGeometry ( 10, 10,  50, 25) ;
	lFilter.setText	    ("Filter") ;
	eFilter.setGeometry ( 60, 10, 350, 25) ;
	eFilter.setText	    (filter) ;
	bOK    .setGeometry (300, 40,  50, 25) ;
	bOK    .setText	    ("OK") ;
	bCancel.setGeometry (360, 40,  50, 25) ;
	bCancel.setText	    ("Cancel") ;

	connect	(&bOK,     SIGNAL(clicked()), SLOT(accept())) ;
	connect	(&bCancel, SIGNAL(clicked()), SLOT(reject())) ;
}

/*  KNDPreFilter							*/
/*  ~KNDPreFilter: Destructor for pre-filter dialog			*/
/*  (returns)	 :		:					*/

KNDPreFilter::~KNDPreFilter ()
{
}

/*  KNDPreFilter							*/
/*  done	: Handle user done (OK or Cancel)			*/
/*  ok		: int		: Non-zero for OK			*/
/*  (returns)	: void		:					*/

void	KNDPreFilter::done
	(	int	ok
	)
{
	/* Actually, we don't do anything at the moment ...		*/
	QDialog::done (ok) ;
}

/*  KNDUser								*/
/*  KNDUser	: Constructor for general selup object			*/
/*  parent	: KNDApp *	: Parent widget				*/
/*  uconf	: KConfig *	: Privileged user configuration		*/
/*  (returns)	: KNDUser	:					*/

KNDUser::KNDUser
	(	KNDApp	*parent,
		KConfig	*uconf
	)
	:
	QWidget	 (parent),
	g_users	 (this),
	u_all	 (this),
	u_full	 (this),
	u_view	 (this),
	l_priv	 (this),
	l_view	 (this),
	b_padd	 (this),
	b_prem	 (this),
	b_premall(this),
	b_vadd	 (this),
	b_vrem	 (this),
	b_vremall(this),
	b_saveu	 (this),
	b_setuid (this),
	uconf	 (uconf)
{
	struct	passwd	*ppw	;
	struct	spwd	*spw	;

	hide	() ;

	g_users	.setGeometry (200, 10, 440, 250) ;
	g_users	.setTitle    (i18n ("Privileged users")) ;

	struct	stat	statb	;
	QString	cmdpath		;

	u_all	 .setGeometry (210,  60,  80, 190) ;
	u_full	 .setGeometry (380,  60,  80, 190) ;
	u_view	 .setGeometry (550,  60,  80, 190) ;
	l_priv	 .setGeometry (380,  30,  80,  25) ;
	l_view	 .setGeometry (550,  30,  80,  25) ;
	l_priv	 .setText     (i18n ("Full access")) ;
	l_view	 .setText     (i18n ("Data hidden")) ;
	b_padd	 .setGeometry (300,  60,  70,  25) ;
	b_padd	 .setText     (i18n ("Add >")    ) ;
	b_prem	 .setGeometry (300,  90,  70,  25) ;
	b_prem	 .setText     (i18n ("< Remove") ) ;
	b_premall.setGeometry (300, 120,  70,  25) ;
	b_premall.setText     (i18n ("<< RemAll")) ;
	b_vadd	 .setGeometry (470,  60,  70,  25) ;
	b_vadd	 .setText     (i18n ("Add >")    ) ;
	b_vrem	 .setGeometry (470,  90,  70,  25) ;
	b_vrem	 .setText     (i18n ("< Remove") ) ;
	b_vremall.setGeometry (470, 120,  70,  25) ;
	b_vremall.setText     (i18n ("<< RemAll")) ;

	b_setuid.setGeometry  (300, 160, 70,  50) ;
	b_setuid.hide 	      () ;

	QToolTip::add (&u_full, i18n ("Users with full snuffling privileges" )) ;
	QToolTip::add (&u_view, i18n ("Users who cannot see application data")) ;

	/* See about showing the setuid button. Only do this if the	*/
	/* command exists as expected, it has UID and GID root, and	*/
	/* cannot be written by anyone else. These checks minimize	*/
	/* hacking ....							*/
	cmdpath	= KApplication::kde_bindir().data() +
		  QString((const char *)"/ksnuffle") ;

	if ( (stat (cmdpath, &statb)    == 0) &&
	     (statb.st_uid              == 0) &&
	     (statb.st_gid              == 0) &&
	     ((statb.st_mode & S_IWOTH) == 0) )
	{
		setuid	= (statb.st_mode & (S_ISUID|S_ISGID)) == (S_ISUID|S_ISGID) ;
		b_setuid.show    () ;
		b_setuid.setText (setuid ? i18n ("Disable\nSetUID") :
					   i18n ("Enable \nSetUID" )) ;
		connect	(&b_setuid, SIGNAL(clicked()), this, SLOT(clickSetUID()))  ;
		QToolTip::add (&b_setuid, i18n ("Set/Unset setUid-root")) ;
	}

	QToolTip::add (&b_padd,    i18n ("Add user to privileged list")) ;
	QToolTip::add (&b_prem,    i18n ("Remove user from privileged list")) ;
	QToolTip::add (&b_premall, i18n ("Remove all users from privileged list")) ;
	QToolTip::add (&b_vadd,    i18n ("Add user to view-only list")) ;
	QToolTip::add (&b_vrem,    i18n ("Remove user from view-only list")) ;
	QToolTip::add (&b_vremall, i18n ("Remove all users from view-only list")) ;

	uconf->setGroup   ("Privilege") ;
	QString	priv	= uconf->readEntry ("full") ;
	QString	view	= uconf->readEntry ("view") ;
	char	*_priv	= QStrToStr (priv) ;
	char	*_view	= QStrToStr (view) ;
	char	*pptr	;

	if (_priv[0] != 0)
		for (pptr = strtok (_priv, ",\n") ; pptr != NULL ; pptr = strtok (NULL, ",\n"))
			u_full.insertItem (pptr) ;
	free (_priv) ;
	if (_view[0] != 0)
		for (pptr = strtok (_view, ",\n") ; pptr != NULL ; pptr = strtok (NULL, ",\n"))
			u_view.insertItem (pptr) ;
	free (_view) ;

	/* Scan the password file, looking for non-sytem users who	*/
	/* have passwords. If these are not in the full or view lists	*/
	/* then add them to the main list.				*/
	while ((ppw = getpwent ()) != NULL)
	{
		if (strcmp (ppw->pw_passwd, "*") == 0) continue ;
		if (ppw->pw_uid                  == 0) continue ;

		if ((spw = getspnam (ppw->pw_name)) != NULL)
		{
			if (strcmp (spw->sp_pwdp,  "*") == 0) continue ;
			if (strcmp (spw->sp_pwdp, "!!") == 0) continue ;
		}

		if (QLBFindItem (u_full, ppw->pw_name, -1) >= 0) continue ;
		if (QLBFindItem (u_view, ppw->pw_name, -1) >= 0) continue ;

		u_all .insertItem (ppw->pw_name) ;
	}

	endpwent () ;

	b_saveu	.setGeometry (300, 215,  70,  25) ;
	b_saveu .setText     ("Save") ;
	QToolTip::add (&b_saveu, i18n ("Save privileged users")) ;

	connect	(&b_saveu,   SIGNAL(clicked()),     this, SLOT(clickSaveUsers())) ;

	connect	(&b_padd,    SIGNAL(clicked()),     this, SLOT(clickPAddUser ())) ;
	connect	(&b_prem,    SIGNAL(clicked()),     this, SLOT(clickPRemUser ())) ;
	connect	(&b_premall, SIGNAL(clicked()),     this, SLOT(clickPRemAll  ())) ;
	connect	(&b_vadd,    SIGNAL(clicked()),     this, SLOT(clickVAddUser ())) ;
	connect	(&b_vrem,    SIGNAL(clicked()),     this, SLOT(clickVRemUser ())) ;
	connect	(&b_vremall, SIGNAL(clicked()),     this, SLOT(clickVRemAll  ())) ;

	connect	(&u_all,     SIGNAL(selected(int)), this, SLOT(clickPAddUser ())) ;
	connect	(&u_full,    SIGNAL(selected(int)), this, SLOT(selectedP  (int))) ;
	connect	(&u_view,    SIGNAL(selected(int)), this, SLOT(selectedV  (int))) ;
}

/*  KNDUser								*/
/*  ~KNDUser	: Destructor for user configuration object		*/
/*  (returns)	:		:					*/

KNDUser::~KNDUser ()
{
}


/*  KNDUser								*/
/*  clickPAddUser: Add selected user					*/
/*  (returns)	 : void		:					*/

void	KNDUser::clickPAddUser ()
{
	int	item	;
	QString	name	;

	if ((item = u_all.currentItem()) >= 0)
	{	name = u_all.text  (item) ;
		u_all .removeItem  (item) ;
		u_full.insertItem  (name) ;
	}
}

/*  KNDUser								*/
/*  clickPRemUser: Remove selected user					*/
/*  (returns)	 : void		:					*/

void	KNDUser::clickPRemUser ()
{
	int	item	;
	QString	name	;

	if ((item = u_full.currentItem()) >= 0)
	{	name = u_full.text (item) ;
		u_full.removeItem  (item) ;
		u_all .insertItem  (name) ;
	}
}

/*  KNDUser								*/
/*  clickPRemAll: Remove all privileged users				*/
/*  (returns)	: void		:					*/

void	KNDUser::clickPRemAll ()
{
	while (u_full.count () > 0)
	{	QString	name = u_full.text (0) ;
		u_full.removeItem (0) ;
		u_all .insertItem (name) ;
	}
}

/*  KNDUser								*/
/*  clickVAddUser: Add selected user					*/
/*  (returns)	 : void		:					*/

void	KNDUser::clickVAddUser ()
{
	int	item	;
	QString	name	;

	if ((item = u_all.currentItem()) >= 0)
	{	name = u_all.text  (item) ;
		u_all .removeItem  (item) ;
		u_view.insertItem  (name) ;
	}
}

/*  KNDUser								*/
/*  clickVRemUser: Remove selected user					*/
/*  (returns)	 : void		:					*/

void	KNDUser::clickVRemUser ()
{
	int	item	;
	QString	name	;

	if ((item = u_view.currentItem()) >= 0)
	{	name = u_view.text (item) ;
		u_view.removeItem  (item) ;
		u_all .insertItem  (name) ;
	}
}

/*  KNDUser								*/
/*  clickVRemAll: Remove all privileged users				*/
/*  (returns)	: void		:					*/

void	KNDUser::clickVRemAll ()
{
	while (u_view.count () > 0)
	{	QString	name = u_view.text (0) ;
		u_view.removeItem (0) ;
		u_all .insertItem (name) ;
	}
}

/*  KNDUser								*/
/*  clickSaveUsers: Save the privileged users setup			*/
/*  (returns)	  : void	:					*/

void	KNDUser::clickSaveUsers ()
{
	unsigned int slot  ;
	QString	     full  ;
	QString	     view  ;

	for (full = "", slot = 0 ; slot < u_full.count() ; slot += 1)
	{	if (slot > 0) full  += "," ;
		full += u_full.text (slot) ;
	}
	for (view  = "", slot = 0 ; slot < u_view.count() ; slot += 1)
	{	if (slot > 0) view  += "," ;
		view += u_view.text (slot) ;
	}

	uconf->setGroup   ("Privilege")    ;
	uconf->writeEntry ("full", full)   ;
	uconf->writeEntry ("view", view)   ;
	uconf->sync	  ()		   ;
}


/*  KNDUser								*/
/*  clickSetUID	: Toggle setUID state					*/
/*  (returns)	: void		:					*/

void	KNDUser::clickSetUID ()
{
	struct	stat	statb	;
	int	mode		;
	QString	cmdpath	= KApplication::kde_bindir().data() +
			  QString((const char *)"/ksnuffle") ;

	if ( (stat (cmdpath, &statb)    == 0) &&
	     (statb.st_uid              == 0) &&
	     (statb.st_gid              == 0) &&
	     ((statb.st_mode & S_IWOTH) == 0) )
	{
		if (setuid)
			mode	= statb.st_mode & ~(S_ISUID|S_ISGID) ;
		else	mode	= statb.st_mode |  (S_ISUID|S_ISGID) ;

		if (chmod (cmdpath, mode) != 0)
		{	Error	("KSnuffle setUid error", STRERR) ;
			return	;
		}

		setuid	= !setuid ;
		b_setuid.setText (setuid ? i18n ("Disable\nSetUID") :
					   i18n ("Enable \nSetUID")) ;
	}
}
/*  KNDUser								*/
/*  selectedP	: Request to set pre-filter for specified user		*/
/*  user	: cchar *	: User name				*/
/*  (returns)	: void		:					*/

void	KNDUser::selected
	(	cchar	*user
	)
{
	extern	bool	VerifyPCap (QString) ;
	char	lbl[64]	;
	char	cfe[64] ;

	sprintf	(lbl, "Pre-filter for user %s", user) ;
	sprintf	(cfe, "pre_filter_%s",		user) ;

	uconf->setGroup   ("Privilege") ;
	QString	filter	= uconf->readEntry (cfe,  "") ;

	KNDPreFilter	pfd (this, lbl, filter) ;

	if (!pfd.exec ()) return ;

	if (VerifyPCap (filter = pfd.getFilter ()))
		uconf->writeEntry (cfe, filter) ;
}

/*  KNDUser								*/
/*  selectedP	: Request to set pre-filter for privileged user		*/
/*  idx		: int		: User-list index			*/
/*  (returns)	: void		:					*/

void	KNDUser::selectedP
	(	int	idx
	)
{
	selected (u_full.text (idx)) ;
}

/*  KNDUser								*/
/*  selectedV	: Request to set pre-filter for view-only user		*/
/*  idx		: int		: User-list index			*/
/*  (returns)	: void		:					*/

void	KNDUser::selectedV
	(	int	idx
	)
{
	selected (u_view.text (idx)) ;
}

