/***************************************************************************
                          knd_view.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<qtooltip.h>

#include	"ctrlset.h"
#include	"knd_view.h"
#include	"knd_trigger.h"
#include	"knd_event.h"
#include	"knd_pcap.h"
#include	"knd_pkt.h"
#include	"knd_gra.h"
#include	"knd_multi.h"
#include	"knd_config.h"


/*  KNDDisplay								*/
/*  KNDDisplay	: Constructor for display splitter			*/
/*  parent	: QWidget *	: Parent				*/
/*  (returns)	: KNDDisplay	:					*/

KNDDisplay::KNDDisplay
	(	QWidget	*parent
	)
	:
	QWidget	(parent)
{
	butbar	= NULL	;
	view	= NULL	;
}

/*  KNDDisplay								*/
/*  ~KNDDisplay	: Destructor for display splitter			*/
/*  (returns)	:		:					*/

KNDDisplay::~KNDDisplay ()
{
}

/*  KNDDisplay								*/
/*  setButBar	: Set the button bar					*/
/*  _butbar	: QWidget *	: Button bar				*/
/*  (returns)	: void		:					*/

void	KNDDisplay::setButBar
	(	KNDButBar *_butbar
	)
{
	butbar	= _butbar ;
}

/*  KNDDisplay								*/
/*  setView	: Set the monitor to be displayed			*/
/*  _view	: KNDView *	: Monitor to display			*/
/*  (returns)	: void		:					*/

void	KNDDisplay::setView
	(	KNDView	*_view
	)
{
	/* If there is already a monitor on display then hide it as it	*/
	/* will be superceeded by this one, before noting the one that	*/
	/* is not to be shown.						*/
	if (view != NULL) view->hide () ;
	view = _view ;

	/* Display the new monitor, which should occupy the area not	*/
	/* used by the button bar. Also show the button bar in case	*/
	/* the user has switched from, say, the global setup.		*/
	view  ->show   ()  ;
	butbar->show   ()  ;

	/* Note resize the components. On the first call this will	*/
	/* size the button bar, which will not change thereafter.	*/
	sizeComponents ()  ;
}

/*  KNDDisplay								*/
/*  resizeEvent	: Handle resize event					*/
/*  e		: QResizeEvent * : Resize event details			*/
/*  (returns)	: void		 :					*/

void	KNDDisplay::resizeEvent
	(	QResizeEvent *e
	)
{
	sizeComponents () ;
}

/*  KNDDisplay								*/
/*  sizeComponents: Size button bar and monitor				*/
/*  (returns)	  : void	:					*/

void	KNDDisplay::sizeComponents ()
{
	/* Only do thos once there is a monitor object. We could always	*/
	/* do the button bar, but a monitor will appear soon after the	*/
	/* program is started.						*/
	if (view != NULL)
	{
		QRect	r = geometry () ;
		butbar ->setGeometry ( 0, 2,             80, r.height() - 3) ;
		view   ->setGeometry (82, 0, r.width() - 82, r.height()    ) ;
		view   ->setSizes    () ;
	}
}


/*  KNDButBar								*/
/*  KNDButBar	: Constructor for common button bar			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  app		: KNDApp  *	: Application object			*/
/*  (returns)	: KNDButBar	:					*/

KNDButBar::KNDButBar
	(	QWidget	*parent,
		KNDApp	*app
	)
	:
	QFrame	(parent),
	b_set	(this),
	b_res	(this),
	b_clr	(this),
	b_go	(this),
	b_freeze(this),
	b_query	(this)
{
	setFrameStyle (QFrame::Panel|QFrame::Raised) ;
	setLineWidth  (2) ;

	b_set  .setGeometry	  ( 10,  25,  60,  25) ;
	b_set  .setText		  ("Set"  ) ;
	connect(&b_set,    SIGNAL(clicked()), app, SLOT(clickSet   ())) ;
	QToolTip::add (&b_set, i18n ("Set and check displayed configuration")) ;

	b_res  .setGeometry	  ( 10,  55,  60,  25) ;
	b_res  .setText		  ("Reset") ;
	connect(&b_res,    SIGNAL(clicked()), app, SLOT(clickReset ())) ;
	QToolTip::add (&b_res, i18n ("Reset to previously set configuration")) ;

	b_clr  .setGeometry	  ( 10,  85,  60,  25) ;
	b_clr  .setText		  ("Clear") ;
	connect(&b_clr,    SIGNAL(clicked()), app, SLOT(clickClear ())) ;
	QToolTip::add (&b_clr, i18n ("Clear to default configuration")) ;

	b_go   .setGeometry	  ( 10, 115,  60,  25) ;
	b_go   .setText		  ("Go"   ) ;
	b_go   .setEnabled	  (false  ) ;
	connect(&b_go,     SIGNAL(clicked()), app, SLOT(clickGo    ())) ;
	QToolTip::add (&b_go, i18n ("Start/stop monitoring/replay")) ;
	
	b_freeze.setGeometry	  ( 10, 145,  60,  25) ;
	b_freeze.setText	  ("Freeze") ;
	b_freeze.setEnabled	  (false   ) ;
	connect(&b_freeze, SIGNAL(clicked()), app, SLOT(clickFreeze())) ;
	QToolTip::add (&b_freeze, i18n ("Freeze/resume graphic and packet displays")) ;

	b_query .setGeometry	  ( 10, 175,  60,  25) ;
	b_query .setText	  ("Query" ) ;
	b_query .setEnabled	  (false   ) ;
	b_query .hide		  ()	     ;
	connect(&b_query,  SIGNAL(clicked()), app, SLOT(clickQuery ())) ;
	QToolTip::add (&b_query,  i18n ("Requery remote interfaces")) ;
}


/*  KNDButBar								*/
/*  ~KNDButBar	: Destructor for common button bar			*/
/*  (returns)	:							*/

KNDButBar::~KNDButBar ()
{
}

/*  KNDButBar								*/
/*  setState	: Set buttons according to monitor state		*/
/*  state	: MState	: Monitor state				*/
/*  frozen	: bool		: Display is frozen			*/
/*  mtype	: MType		: Monitor type				*/
/*  (returns)	: void		:					*/

void	KNDButBar::setState
	(	MState	state,
		bool	frozen,
		MType	mtype
	)
{
	bool	eGo	 = false 	;
	bool	eFreeze	 = false 	;
	bool	eQuery	 = true		;
	char	*tGo	 = "Go"		;
	char	*tFreeze = "Freeze"	;

	switch (state)
	{
		case MSUnset	:
		case MSCleared	:
		case MSReset	:
			break	;

		case MSSet	:
			eGo	= true	;
			break	; 

			
		case MSWaiting	:
		case MSRunning	:
			eGo	= true	;
			eFreeze	= true	;
			eQuery	= false ;
			tGo	= "Halt";
			tFreeze	= frozen ? "Resume" : "Freeze" ;
			break	;

		case MSHalted	:
		case MSStopped	:
			eQuery	= false ;
			eGo	= true	;
			break	;

		case MSError	:
			break	;

		default	:
			break	;
	}

	b_go	.setEnabled (eGo    ) ;
	b_freeze.setEnabled (eFreeze) ;
	b_query .setEnabled (eQuery ) ;
	b_go	.setText    (tGo    ) ;
	b_freeze.setText    (tFreeze) ;

	if (mtype == Remote)
		b_query.show () ;
	else	b_query.hide () ;
}


/*  KNDView								*/
/*  KNDView	: Constructor for view object				*/
/*  parent	: QWidget *	: Parent widget				*/
/*  app		: KNDApp  *	: Application object			*/
/*  slot	: int		: Slot in parent view list		*/
/*  base	: char *	: Base for new name			*/
/*  replay	: bool		: This is a replayer			*/
/*  multi	: KNDMulti *	: Parallel display object		*/
/*  appconf	: KConfig *	: Configuration or null			*/
/*  (returns)	: KNDView	:					*/

KNDView::KNDView
	(	QWidget		*parent,
		KNDApp		*app,
		int		slot,
		cchar		*base,
		bool		replay,
		KNDMulti	*multi,
		KConfig		*appconf
	)
	:
	QWidget	(parent),
	replay	(replay),
	app	(app),
	multi	(multi),
	slot	(slot)
{
	if (appconf == NULL)
		name.sprintf ("%s %d", base, slot) ;
	else	name = appconf->readEntry ("MonitorName", "NoName") ;

	nPlugin		= 0 ;
	nPluginSet	= 0 ;
	memset	(plugSet, 0, sizeof(plugSet)) ;
	memset	(plugins, 0, sizeof(plugins)) ;


	trigger	= new KNDTrigger (this, appconf) ;
	event	= new KNDEvent   (this, appconf) ;
	graphic = new KNDGraphic (this) ;
	packets	= new KNDPacket  (this, appconf, replay) ;

	tabBar	= NULL	 ;
	curtab	= 0	 ;
	onview	= true	 ;
	state	= MSUnset;
	going	= false	 ;
	frozen	= false  ;
	paused	= false  ;
}

/*  KNDView								*/
/*  ~KNDView	: Destructor for view object				*/
/*  (returns)	:		:					*/

KNDView::~KNDView ()
{
	/* Unregister from the parallel display. If we were not		*/
	/* registered then this call will be silently failed.		*/
	multi->unRegister (slot) ;

	delete	config	;
	delete	capture	;
	delete	trigger ;
	delete	event	;
	delete	graphic ;

	/* Destroy all dynamically created plugins, and release the	*/
	/* space returned for their descriptions.			*/
	for (int set = 0 ; set < nPluginSet ; set += 1)
	{	for (int idx = 0 ; idx < plugSet[set]->nPlugins ; idx += 1)
			delete	plugSet[set]->plugin[idx] ;
		free	(plugSet[set]) ;
	}

	delete	tabBar	;
}

/*  KNDView								*/
/*  makeTabBar	: Build tab bar						*/
/*  (returns)	: void		:					*/

void	KNDView::makeTabBar ()
{
	if (tabBar != NULL) delete tabBar ;
	tabBar = new KTabBar (this) 	  ;

	for (int idx = 0 ; idx < nPlugin ; idx += 1)
	{
		PlugData &pd  = plugins[idx] ;
		pd.tab	      = new KNDViewTab (pd.plugin) ;
		
		tabBar->addTab        (pd.tab) ;
		tabBar->setTabEnabled (pd.tab->id, pd.plugin->isConfig ()) ;
	}

	tabBar->show () ;
	setSizes     () ;

	connect	(tabBar, SIGNAL(selected(int)), this, SLOT(tabSelect(int))) ;
}

/*  KNDView								*/
/*  setSizes	: Set tab bar and plugin sizes				*/
/*  (returns)	: void		:					*/

void	KNDView::setSizes ()
{
	if (tabBar != NULL)
	{
		QRect	r  = geometry () ;
		int	th = tabBar->geometry().height() ;

		tabBar->setGeometry  (0, 0, r.width(), th) ;
		for (int idx = 0 ; idx < nPlugin ; idx += 1)
			plugins[idx].plugin->setGeometry (0, th, r.width(), r.height() - th) ;
	}
}

/*  KNDView								*/
/*  addTabs	: Add tab-accessed widgets				*/
/*  (returns)	: void		:					*/

void	KNDView::addTabs ()
{
	plugins[0].plugin = config  ;
	plugins[1].plugin = trigger ;
	plugins[2].plugin = event   ;
	plugins[3].plugin = graphic ;
	plugins[4].plugin = packets ;
	nPlugin	  	  = 5	    ;

	config->initPlugins () ;
	makeTabBar () ;
}


/*  KNDView								*/
/*  setOptions	: Pass on options from configuration object		*/
/*  lvopts	: void *	: Options				*/
/*  (returns)	: void		:					*/

void	KNDView::setOptions
	(	void	*lvopts
	)
{
	packets->setOptions (lvopts) ;
	graphic->setOptions (lvopts) ;
}

/*  KNDView								*/
/*  setProg	: Pass on filter and triggers from configuration object	*/
/*  iface	: cchar *	: Network interface name		*/
/*  snaplen	: int		: Snap length				*/
/*  (returns)	: void		:					*/

void	KNDView::setProg
	(	cchar	 *iface,
		int	snaplen
	)
{
	/* Pass the details on to the capture object. The success or	*/
	/* otherwise of this is returned via the "setProgRC" method.	*/
	capture->setProg (iface, trigger->getInfo(), event->getInfo(), snaplen) ;
}

/*  KNDView								*/
/*  execute	: Start or stop executing packet capture		*/
/*  exec	: bool		: True to start executing		*/
/*  logfd	: FILE *	: Logging file descriptor or NULL	*/
/*  binary	: bool		: True for binary logging		*/
/*  (returns)	: void		:					*/

void	KNDView::execute
	(	bool	exec,
		FILE	*logfd,
		bool	binary
	)

{
	/* If stopping executing then inform the capture object 	*/
	/* followed by the display objects. Note that stopping cannot	*/
	/* fail!							*/
	if (!exec)
	{	capture->execute    (false) ;
		packets->execute    (false) ;
		graphic->execute    (false) ;

		/* Halt all dynamic plugins ...				*/
		for (int idx = 5 ; idx < nPlugin ; idx += 1)
			plugins[idx].plugin->execute (false) ;

		multi  ->unRegister (slot ) ;
		going	= false ;
		return	;
	}

	/* Pass the details on to the capture object. The success or	*/
	/* otherwise of this is returned via the "executeRC" method.	*/
	capture->execute  (true, logfd, binary) ;
}

/*  KNDView								*/
/*  setProgRC	: Pass back result from capture->setProg call		*/
/*  rc		: bool		: Success				*/
/*  (returns)	: void		:					*/

void	KNDView::setProgRC
	(	bool	rc
	)
{
	config->setProgRC (rc) ;
}

/*  KNDView								*/
/*  setProgRC	: Pass back result from capture->execute call		*/
/*  rc		: bool		: Success				*/
/*  (returns)	: void		:					*/

void	KNDView::executeRC
	(	bool	rc
	)
{
	/* If the execute call succeeded then start the display objects	*/
	/* before passing the result up to the configuration object.	*/
	if (going = rc)
	{	if (config->useMulti ()) multi->Register (slot, name) ;
		packets->execute    (true) ;
		graphic->execute    (true) ;

		/* Start all dynamic plugins ...			*/
		for (int idx = 5 ; idx < nPlugin ; idx += 1)
			plugins[idx].plugin->execute (true) ;

		setState (MSRunning) ;
	}
	else	setState (MSError  ) ;

	/* Pass the result back to the configuration object ...		*/
	config->executeRC (rc) ;
}

/*  KNDView								*/
/*  addPacket	: Add packet to display objects				*/
/*  pktinfo	: PktInfo *	: Decoded packet information		*/
/*  (returns)	: void		:					*/

void	KNDView::addPacket
	(	PktInfo	*pktinfo
	)
{
	/* Pass the packet to all dynamic plugins ....			*/
	for (int idx = 5 ; idx < nPlugin ; idx += 1)
		plugins[idx].plugin->addPacket (pktinfo) ;

	packets->addPacket (pktinfo) ;
	graphic->addPacket (pktinfo) ;
	multi  ->addPacket (pktinfo, slot) ;
}

/*  KNDView								*/
/*  freeze	: Set or unset frozen state				*/
/*  _frozen	: bool		: True to freeze			*/
/*  (returns)	: void		:					*/

void	KNDView::freeze
	(	bool	_frozen
	)
{
	frozen	= _frozen ;

	/* Pass the frozen state down to all dynamic plugins ...	*/
	for (int idx = 5 ; idx < nPlugin ; idx += 1)
		plugins[idx].plugin->freeze (frozen) ;

	packets->freeze (frozen) ;
	graphic->freeze (frozen) ;
	app    ->setState (this) ;
}

/*  KNDView								*/
/*  setName	: Set monitor name					*/
/*  newname	: const char *	: New monitor name			*/
/*  (returns)	: void		:					*/

void	KNDView::setName
	(	const	char	*newname
	)
{
	if (strcmp (name, newname) != 0)
	{	app->rename (name, newname) ;
		name = newname ;
	}
}

/*  KNDView								*/
/*  tabSelect	: Handle tab selection					*/
/*  tab		: int		: Selected tab				*/
/*  (returns)	: void		:					*/

void	KNDView::tabSelect
	(	int	tab
	)
{
	tabBar->setCurrentTab (curtab = tab) ;

	for (int idx = 0 ; idx < nPlugin ; idx += 1)
	{
		KNDPlugin *pl = plugins[idx].plugin ;

		if (((KNDViewTab *)tabBar->tab (tab))->plugin == pl)
		{	pl->show      () ;
			pl->setOnview (true ) ;
		}
		else
		{	pl->hide      () ;
			pl->setOnview (false) ;
		}
	}
}

/*  KNDView								*/
/*  setOnview	: Called to inform object whether it is on view		*/
/*  _onview	: bool		: TRUE if on view			*/
/*  (returns)	: void		:					*/

void	KNDView::setOnview
	(	bool	_onview
	)
{
	/* If this view is no longer on view then tell the graphic and	*/
	/* packet objects the same, so that thay can cease display	*/
	/* updates. However, if we are on view then call "tabSelect" to	*/
	/* inform which of them is on view.				*/
	if (!(onview = _onview))
	{
		graphic->setOnview (false) ;
		packets->setOnview (false) ;

		for (int idx = 5 ; idx < nPlugin ; idx += 1)
			plugins[idx].plugin->setOnview (false) ;
	}
	else	tabSelect (curtab) ;
	
}

/*  KNDView								*/
/*  viewStart	: Start monitor						*/
/*  (returns)	: void		:					*/

void	KNDView::viewStart ()
{
	if (!going)
	{	config->clickSet () ;
		config->clickGo  () ;
	}
}

/*  KNDView								*/
/*  viewStop	: Stop monitor						*/
/*  (returns)	: void		:					*/

void	KNDView::viewStop ()
{
	if ( going) config->clickGo () ;
}

/*  KNDView								*/
/*  viewFreeze	: Freeze monitor					*/
/*  (returns)	: void		:					*/

void	KNDView::viewFreeze ()
{
	if (!frozen) config->clickFreeze () ;
}

/*  KNDView								*/
/*  viewResume	: Resume monitor					*/
/*  (returns)	: void		:					*/

void	KNDView::viewResume ()
{
	if ( frozen) config->clickFreeze () ;
}

/*  KNDView								*/
/*  setState	: Set sniffer or replayer state				*/
/*  _state	: MState	: New state				*/
/*  reset	: bool		: Reset paused and frozen		*/
/*  (returns)	: void		:					*/

void	KNDView::setState
	(	MState	_state,
		bool	reset
	)
{
	state	= _state ;
	if (reset) frozen = paused = false ;

	/* Call the application object to tell it that something has	*/
	/* changed so that the status bar can be updated, _except_ if	*/
	/* the monitor is being closed (in which case the status bar	*/
	/* will be set for the next monitor).				*/
	if (state != MSClosed) app->setState (this) ;

	bool	eDisp	= (state == MSWaiting) || (state == MSRunning) ||
			  (state == MSStopped) || (state == MSHalted ) ;

	for (int idx = 3 ; idx < nPlugin ; idx += 1)
		if (!plugins[idx].plugin->isConfig ())
			tabBar->setTabEnabled (plugins[idx].tab->id, eDisp) ;
}

/*  setPaused	: Set sniffer or replayer paused			*/
/*  _frozen	: bool		: Frozen or not				*/
/*  (returns)	: void		:					*/

void	KNDView::setPaused
	(	bool	_paused
	)
{
	paused	= _paused ;
	app->setState (this) ;
}

/*  KNDView								*/
/*  timerTick	: Pass down a timer tick				*/
/*  second	: long		: Second				*/
/*  (returns)	: void		:					*/

void	KNDView::timerTick
	(	long	second
	)
{
	capture->timerTick (second) ;
	graphic->timerTick (second) ;
	packets->timerTick (second) ;

	for (int idx = 5 ; idx < nPlugin ; idx += 1)
		if (!plugins[idx].plugin->isConfig ())
			plugins[idx].plugin->timerTick (second) ;
}

/*  KNDView								*/
/*  writeConfig	: Write monitor configuration				*/
/*  _config	: KConfig *	: Configuration object			*/
/*  (returns)	: void		:					*/

void	KNDView::writeConfig
	(	KConfig	*_config
	)
{
	config ->writeConfig (_config) ;
	event  ->writeConfig (_config) ;
	trigger->writeConfig (_config) ;
	packets->writeConfig (_config) ;

	for (int idx = 5 ; idx < nPlugin ; idx += 1)
		plugins[idx].plugin->writeConfig (_config) ;
}

/*  KNDView								*/
/*  clickSet	: Pass Set button click through				*/
/*  (returns)	: void		:					*/

void	KNDView::clickSet ()
{
	for (int idx = 0 ; idx < nPlugin ; idx += 1)
		plugins[idx].plugin->clickSet () ;
}

/*  KNDView								*/
/*  clickReset	: Pass Reset button click through				*/
/*  (returns)	: void		:					*/

void	KNDView::clickReset ()
{
	for (int idx = 0 ; idx < nPlugin ; idx += 1)
		plugins[idx].plugin->clickReset () ;

	tabSelect (0) ;
}

/*  KNDView								*/
/*  clickClear	: Pass Clear button click through				*/
/*  (returns)	: void		:					*/

void	KNDView::clickClear ()
{
	for (int idx = 0 ; idx < nPlugin ; idx += 1)
		plugins[idx].plugin->clickClear () ;

	tabSelect (0) ;
}

/*  KNDView								*/
/*  clickGo	: Pass Go button click through				*/
/*  (returns)	: void		:					*/

void	KNDView::clickGo ()
{
	extern	void	plugConfig (PluginSet *) ;

	/* Pass configuration information within each plugin set if we	*/
	/* are not going (ie., this is not a halt.			*/
	if (!going)
		for (int idx = 0 ; idx < nPluginSet ; idx += 1)
			plugConfig (plugSet[idx]) ;

	/* .... then start running. Pass the click down to the main	*/
	/* configuration object; this will call back to pass its own	*/
	/* configuration about, and to start execution.			*/
	config->clickGo () ;

}

/*  KNDView								*/
/*  clickFreeze	: Pass Freeze button click through				*/
/*  (returns)	: void		:					*/

void	KNDView::clickFreeze ()
{
	config->clickFreeze () ;
}

/*  KNDView								*/
/*  clickQuery	: Pass Query button click through				*/
/*  (returns)	: void		:					*/

void	KNDView::clickQuery ()
{
	capture->clickQuery () ;
}

/*  KNDView								*/
/*  AddPlugin	: Add a new plugin					*/
/*  name	: cchar *	: Plugin name				*/
/*  (returns)	: void		:					*/

void	KNDView::AddPlugin
	(	cchar	*name
	)
{
	GETPLUGIN *fn	;
	PluginSet *ps	;
	KConfig	  *conf	= kapp->getConfig () ;
	QString	  gnam	;
	int	  ctab	= curtab ;

	gnam.sprintf   ("MONITOR_%d", slot) ;
	conf->setGroup (gnam) ;

	/* Load the plugin and retrieve its plugin set. If either of	*/
	/* these fails then giv up.					*/
	if ((fn	= LoadPlugin  (name)) == NULL)
	{	Error	("Failed to load plugin",   name) ;
		return	;
	}
	if ((ps = (*fn) (this, conf)) == NULL)
	{	Error	("Failed to access plugin", name) ;
		return	;
	}


	/* Get the major and minor versions of this release, and check	*/
	/* them against those for the plugin. If they do not match then	*/
	/* grumble, and do not use the plugin.				*/
	int	major	;
	int	minor	;

	getVersion (major, minor) ;

	if ((ps->major != major) || (ps->minor != minor))
	{
		Error	("Plugin load error",
			 "Plugin is for a version %d.%d",
			 ps->major,
			 ps->minor
			) ;
		return	;
	}
	if (strcmp (ps->name, name) != 0)
	{
		Error	("Plugin load error",
			 "Internal name \"%s\" and external name \"%s\" differ", 
			 name,
			 ps->name
			) ;
		return	;
	}

	plugSet[nPluginSet++] = ps ;

	/* Add the plugins from this new set. We preserve the order	*/
	/* specified by the plugin module; all plugins which are not	*/
	/* marked for configuration are disabled.			*/
	for (int idx = 0 ; idx < ps->nPlugins ; idx += 1)
	{	plugins[nPlugin].plugset = ps ;
		plugins[nPlugin].plugin  = ps->plugin[idx] ;
		nPlugin += 1 ;
	}


	makeTabBar ()     ;
	tabSelect  (ctab) ;
}

/*  KNDView								*/
/*  RemPlugin	: Remove a plugin					*/
/*  name	: cchar *	: Plugin name				*/
/*  (returns)	: void		:					*/

void	KNDView::RemPlugin
	(	cchar	*name
	)
{
	int	idx1	;
	int	idx2	;

	for (idx1 = 5, idx2 = 5 ; idx1 < nPlugin ; idx1 += 1)
		if (strcmp (plugins[idx1].plugset->name, name) != 0)
		{	plugins[idx2] = plugins[idx1] ;
			idx2 += 1; 
		}
		else	delete	plugins[idx1].plugin ;

	nPlugin = idx2 ;

	for (idx1 = 0, idx2 = 0 ; idx1 < nPluginSet ; idx1 += 1)
		if (strcmp (plugSet[idx1]->name, name) != 0)
		{	plugSet[idx2] = plugSet[idx1] ;
			idx2 += 1; 
		}
		else	free	(plugSet[idx1]) ;

	nPluginSet = idx2 ;

	makeTabBar () ;
}
