/***************************************************************************
                          knd_view.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef	KND_VIEW_H 
#define KND_VIEW_H 

#include	<qevent.h>
#include	<qpushbutton.h>
#include	<ktabbar.h>

#include	"ksnuffle.h"
#include	"knd_config.h"
#include	"capture.h"

/*  KNDButBar								*/
/*  This class contains the common Set, Reset, ... buttons which apply	*/
/*  to all monitors.							*/

class	KNDButBar : public QFrame
{
	Q_OBJECT

	QPushButton	b_set		;
	QPushButton	b_res		;
	QPushButton	b_clr		;
	QPushButton	b_go		;
	QPushButton	b_freeze	;
	QPushButton	b_query		;

public:
	KNDButBar	(QWidget *, KNDApp *) ;
       ~KNDButBar	() ;

	void	setState(MState, bool, MType)	;
}	;

/*  KNDDisplay								*/
/*  The KNDDisplay class splits its area between a button bar on the	*/
/*  left (with fixed width) and a monitor display on the right.		*/

class	KNDDisplay : public QWidget
{
	Q_OBJECT

	KNDButBar *butbar ;		/* Button bar			*/
	KNDView	  *view   ;		/* Currently displayed monitor	*/

	void	sizeComponents () ;

public	:

	KNDDisplay (QWidget *)	;
       ~KNDDisplay ()	;

	void	setButBar (KNDButBar *) ;
	void	setView	  (KNDView   *) ;

protected :

	virtual	void	resizeEvent (QResizeEvent *) ;
}	;


class	KNDViewTab : public QTab
{
public	:

	KNDPlugin  *plugin ;

	inline	KNDViewTab
		(	KNDPlugin *plugin
		)
		:
		plugin	(plugin)
	{
		label	= plugin->getLabel () ;
	}
}	;


struct	PlugData
{
	PluginSet	*plugset;	/* Underlying plugin set	*/
	KNDPlugin	*plugin	;	/* Actual plugin object		*/
	KNDViewTab	*tab	;	/* Tab bar tab item		*/
}	;


/*  KNDView								*/
/*  The KNDView class is the base for all sniffers and replayers, and	*/
/*  coordinates the separate tabbed sections.				*/

class	KNDView : public QWidget
{
	Q_OBJECT

	bool		onview		;
	int		curtab		;
	bool		replay		;

	/* We keep track of all accessed plugin sets via the "plugSet"	*/
	/* array; "nPluginSet" is the number of entries therein.		*/
	int		nPluginSet	;
	PluginSet	*plugSet[8]	;

	/* Individual plugins are referenced from the "plugins" array	*/
	/* with each entry containing a pointer at the actual plugin	*/
	/* object, and a pointer at the information structure for it.	*/
	int		nPlugin		;
	PlugData	plugins [32]	;

	KTabBar		*tabBar		;

protected :

	KNDApp		*app	   ;
	KNDConf		*config	   ;
	KNDTrigger	*trigger   ;
	KNDEvent	*event	   ;
	KNDGraphic	*graphic   ;
	KNDPacket	*packets   ;
	KNDMulti	*multi	   ;
	KNDPCap		*capture   ;

	int		slot	   ;	/* Slot number at top level	*/
	QString		name	   ;	/* Monitor or whatever name	*/
	MState		state	   ;	/* Monitor execution state	*/
	bool		going	   ;	/* Capturing packets		*/
	bool		frozen	   ;	/* Display frozen		*/
	bool		paused	   ;	/* Packet display has paused	*/

	void	makeTabBar ()	   ;
	void	addTabs	   () 	   ;

public:
	KNDView (QWidget *, KNDApp *, int, cchar *, bool, KNDMulti *, KConfig * = NULL) ;

	virtual	~KNDView () ;

	inline	const char *getName  ()	{ return name	; }
	inline	int	   getSlot   ()	{ return slot	; }

	inline	const char *getIface ()
	{
		return	config->getIface () ;
	}

	void	setState	(MState, bool = true) ;
	void	setPaused	(bool  ) ;

	inline	MState	getState ()
	{	return	state	;
	}
	inline	bool	getFrozen ()
	{	return	frozen ;
	}
	inline	bool	getPaused ()
	{	return	paused ;
	}
	inline	void	setLoad
		(	int	bar1,
			int	period1,
			int	bar2,
			int	period2
		)
	{
		app->setLoad (this, bar1, period1, bar2, period2) ;
	}
	inline	bool	SaveMe ()
	{
		return	config->SaveMe () ;
	}

	void	setName		(const char *)	;
	void	setOnview	(bool)		;
	void	timerTick 	(long)		;

	void	viewStop	() ;
	void	viewStart	() ;
	void	viewFreeze	() ;
	void	viewResume	() ;

	void	clickSet    	() ;
	void	clickReset  	() ;
	void	clickClear  	() ;
	void	clickGo     	() ;
	void	clickFreeze 	() ;
	void	clickQuery 	() ;


public  slots :

		void	tabSelect   (int) ;

public	:

	void	setSizes   () ;
	void	setOptions (void *) ;
	void	addPacket  (PktInfo *) ;

	void	setProg	   (cchar *, int) ;
	void	execute	   (bool, FILE * = NULL, bool = false) ;
	void	setProgRC  (bool) ;
	void	executeRC  (bool) ;
	void	freeze	   (bool) ;

	virtual	MType	   getMType () = 0 ;
	virtual	void	   writeConfig (KConfig *) ;

	void	AddPlugin  (cchar *) ;
	void	RemPlugin  (cchar *) ;
}	;

#define	TAB_CONFIG	(0)
#define	TAB_TRIGGERS	(1)
#define	TAB_EVENTS	(2)
#define	TAB_GRAPHIC	(3)
#define	TAB_PACKETS	(4)

#define	LAB_CONFIG	"Config"
#define	LAB_TRIGGER	"Filter/trigger"
#define	LAB_EVENTS	"Events"
#define	LAB_GRAPHIC	"Graphic"
#define	LAB_PACKETS	"Monitor"

#endif // KND_VIEW_H 
