/***************************************************************************
                          knetinet.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<stdio.h>
#include	<stdarg.h>
#include	<errno.h>
#include	<unistd.h>
#include	<signal.h>
#include	<sys/ioctl.h>
#include	<sys/types.h>
#include	<sys/wait.h>

#include	<qarray.h>
#include	<ksock.h>

#include	"ksnuffle.h"
#include	"nethdrs.h"

typedef	struct	ifconf	IFCONF	;	/* Interface configuration	*/
typedef	struct	ifreq	IFREQ	;	/* Interface request		*/
typedef	struct	hostent	HOSTENT	;	/* Host entry			*/
typedef	struct	servent	SERVENT	;	/* Service entry		*/

#define	AINCR	32

extern	bool	j_priv	;	/* Just list privileged ports/services	*/
static	pid_t	nslpid	;	/* Name lookup process PID		*/
extern	bool	dropDom	;	/* Drop local domain names		*/
static	char	*domName;	/* Local domain name			*/

static	SArray	*ourips	;

/*  BubbleSort2	: Sort all but first element in list			*/
/*  list	: SArray *	: Array of string pointers		*/
/*  (returns)	: SArray *	: The array				*/

static	SArray	*BubbleSort2
	(	SArray	*list
	)
{
	for (unsigned int idx1 = 2 ; idx1 < list->size() ; idx1 += 1)
		for (unsigned int idx2 = idx1 ; idx2 > 1 ; idx2 -= 1)
			if (strcmp ((*list)[idx2], (*list)[idx2-1]) < 0)
			{	char	*tproto	= (*list)[idx2-1] ;
				(*list)[idx2-1]	= (*list)[idx2  ] ;
				(*list)[idx2  ]	= tproto ;
			}
			else	break	;
	return	list	;
}

/*  memdup	: Duplicate block of memory				*/
/*  text	: char *	: Memory to duplicate			*/
/*  length	: int		: Length thereof			*/
/*  (returns(	: char *	: Duplicate				*/

static	char	*memdup
	(	char	*text,
		int	length
	)
{
	char	*res	= (char *)malloc (length) ;
	memcpy	(res, text, length) ;
	return	res	;
}

/*  INTERFACES								*/
/*  ----------								*/

/*  FindAllIface: Find all interfaces					*/
/*  (returns)	: SArray *		: List of interfaces		*/

SArray	*FindAllIface ()
{
	IFCONF	ifconf		;
	IFREQ	ifreq1		;
	IFREQ	ifreq2		;
	IFREQ	ifreq3		;
	IFREQ	*ifr		;
	char	buff[BUFSIZ]	;
	char	*cp		;
	char	*name		;
	int	sockfd		;
	uint	ifncnt		= 0 ;
	bool	gothwaddr	;
	bool	gotipaddr	;
	SArray	*iflist		= new SArray (AINCR) ;
		ourips		= new SArray (AINCR) ;

	if ((sockfd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
		ErrExit	("can't create rx socket: %s", STRERR) ;

	ifconf.ifc_len	= sizeof(buff)	;
	ifconf.ifc_buf	= buff		;
	if (ioctl (sockfd, SIOCGIFCONF, (char *)&ifconf) < 0)
		ErrExit	("ioctl SIOCGIFCONF failed: %s", STRERR) ;

	/* The loop scans all interfaces, rejecting any that are not up	*/
	/* For each that is OK, try to garner and store information	*/
	/* about it.							*/
	for (cp = buff ; cp < buff + ifconf.ifc_len ;
			 cp += sizeof(ifr->ifr_name) + sizeof(ifr->ifr_addr))
	{
		ifr	= (IFREQ *)cp	;
		ifreq1	= *ifr		;

		if (ioctl (sockfd, SIOCGIFFLAGS,  (char *)&ifreq1)  < 0)
			ErrExit	("ioctl SIOCGIFFLAGS failed: %s", STRERR) ;
		if ((ifreq1.ifr_flags & IFF_UP) == 0)
			continue ;

		name = strdup (ifr->ifr_name) ;

		if (ifncnt >= iflist->size()) iflist->resize (ifncnt + AINCR) ;
		if (ifncnt >= ourips->size()) ourips->resize (ifncnt + AINCR) ;
		(*iflist)[ifncnt] = name	;
		(*ourips)[ifncnt] = NULL	;		

		ifreq2	  = *ifr ;
		ifreq3	  = *ifr ;
		gothwaddr = ioctl (sockfd, SIOCGIFHWADDR, (char *)&ifreq2) >= 0 ;
		gotipaddr = ioctl (sockfd, SIOCGIFADDR,   (char *)&ifreq3) >= 0 ;

		/* If we have an IP address, then add it to the list of	*/
		/* our IP addresses, to be used when checking whether a	*/
		/* packet is incoming or outgoing.			*/
		if (gotipaddr)
			(*ourips)[ifncnt] = memdup ((char *)&ifreq3.ifr_addr.sa_data[2], 4) ;
		
		/* Provided we have both an IP address and a MAC	*/
		/* address, then add a MAC->IP mapping (execpt that we	*/
		/* skip aliased interfaces, since we cannot distinguish	*/
		/* them at the MAC level).				*/
		if (strchr (ifr->ifr_name, ':') != NULL)
			if (gotipaddr && gothwaddr)
				EthToIP ((uchar *)&ifreq2.ifr_hwaddr.sa_data[0],
					 (char  *)&ifreq3.ifr_addr  .sa_data[2]) ;

		ifncnt	+= 1 ;
	}

	close	(sockfd) ;
	iflist->resize (ifncnt) ;
	ourips->resize (ifncnt) ;
	return	iflist   ;
}

/*  ETHERNET PROTOCOLS							*/
/*  ------------------							*/

/*  FindAllEProto: Load all ethernet protocols				*/
/*  (returns)	 : SArray *	: List of protocols			*/

SArray	*FindAllEProto ()
{
	SArray	*list	= new QArray<char *> (AINCR) ;
	uint	cnt	;

	/* The list of ethernet protocols recognised by libpcap is held	*/
	/* in an array of (char *, u_short) structures, named eproto_db	*/
	/* Scan this to get our list.					*/
	struct	EProto
	{	char	*name	;
		u_short	proto	;
	}	;

	extern	EProto	eproto_db[] ;

	(*list)[0] = "" ;
	cnt	   = 1  ;

	for (EProto *ep = &eproto_db[0] ; ep->name != NULL ; ep += 1)
	{	if (cnt >= list->size()) list->resize (cnt + AINCR) ;
		(*list)[cnt++] = ep->name	;
	}

	list->resize (cnt) ;

	/* Sort the protocols into alphabetic order, except that we	*/
	/* leave the blank entry at the top. This is then returned.	*/
	return	BubbleSort2 (list) ;
}


/*  INTERNET PROTOCOLS							*/
/*  ------------------							*/

/*  FindAllIProto: Load all internet protocols				*/
/*  (returns)	 : SArray *	: List of protocols			*/

SArray	*FindAllIProto ()
{
	/* Libpcap seems to know only about TCP/IP and UDP/IP, so there	*/
	/* is no point scanning /etc/protocols.				*/
	SArray	*list	= new QArray<char *> (3) ;

	(*list)[0]	= ""	;
	(*list)[1]	= "tcp"	;
	(*list)[2]	= "udp"	;

	return	list	;
}


/*  SERVICES								*/
/*  ---------								*/

/*  As well as generating a list of all services, we maintain a hash	*/
/*  table of service names, hashed on the port number, so that we can	*/
/*  convert ports back to service names.				*/

struct	PortMap
{	PortMap	*next	   ;
	int	p	   ;
	char	port[8]	   ;
	char	*serv	   ;
}	;

#define	PMAPSIZ	256
static	PortMap	*portmap[PMAPSIZ] ;

/*  FindAllService: Load all services					*/
/*  (returns)	  : SArray *	: List of services			*/

SArray	*FindAllService ()
{
	SERVENT	*service ;
	int	port	 ;
	int	hashv	 ;
	PortMap	*pmap	 ;
	uint	cnt	 ;
	SArray	*list	 = new QArray<char *> (AINCR) ;

	setservent (1)	 ;

	(*list)[0] = ""	 ;
	cnt	   = 1	 ;

	while ((service = getservent ()) != NULL)
	{
		/* Get the port number in host form, allocate a new	*/
		/* hash table entry, and calculate the hash value.	*/
		port		= ntohs  (service->s_port) ;
		pmap		= new PortMap		;
		hashv		= port & (PMAPSIZ-1)	;

		/* Store the port number, the service name, plus the	*/
		/* port number as a text string. Then insert the new	*/
		/* entry into the port hash table.			*/
		pmap->p		= port			;
		pmap->serv	= strdup (service->s_name) ;
		sprintf	(pmap->port, "%d", port)	;

		pmap->next	= portmap[hashv]	;
		portmap[hashv]	= pmap			;

		/* Lastly, append the service name to the service list,	*/
		/* extending it if neccessary.				*/
		if (cnt >= list->size()) list->resize (cnt + AINCR) ;
		(*list)[cnt++]  = pmap->serv		;

	}

	endservent ()  ;
	list->resize (cnt) ;

	/* Sort the service array (as above, leaving the first entry in	*/
	/* place), and return it.					*/
	return	BubbleSort2 (list) ;
}

/*  HOST NAMES								*/
/*  ----------								*/

struct	HostMap
{	HostMap	*next		 ;	/* Next on hash queue		*/
	long	addr		 ;	/* Binary address value		*/
	int	retry		 ;	/* Failure retry count		*/
	char	dots[ 20]	 ;	/* Dot-notation address		*/
	char	name[128]	 ;	/* Full host name when mapped	*/
}	;

#define	HMAPSIZ	256
static	HostMap	*hostmap[HMAPSIZ];	/* Name lookup cache		*/
static	int	nslsock		 ;	/* Socket to name lookup thread	*/
#define	RTRYCNT	(32)			/* Failed lookup retry interval	*/


/*  FindEPoint	: Find end-point information				*/
/*  _addr	: char *	: AF_INET type address			*/
/*  _port	: int		: IP port number			*/
/*  epoint	: EPoint *	: End-point information			*/
/*  (returns)	: bool		: Got host name				*/

bool	FindEPoint
	(	char	*_addr,
		int	_port,
		EPoint	*epoint
	)
{
	/* NOTE:							*/
	/* Since the EPoint structure contains pointers, all names and	*/
	/* addresses must be permenantly stored as strings before this	*/
	/* routine returns.						*/

	long	addr	= *(long *)_addr	;
	long	addr2	= ntohl (addr)		;
	int	hashv	;
	HostMap	*hmap	;
	PortMap	*pmap	;
	bool	rc	= false	;

	/* HOST NAME							*/
	/* ---- ----							*/

	/* Scan the hash table to see if the name is already there, in	*/
	/* which case we do not need to do a lookup and can return the	*/
	/* name or dot address at once.					*/
	hashv	= addr & (HMAPSIZ - 1)	;

	for (hmap = hostmap[hashv] ; hmap != NULL ; hmap = hmap->next)
		if (hmap->addr == addr)
			break	;

	/* If the host is not found allocate a new hash table entry,	*/
	/* and initially set the dot address in place of the name. This	*/
	/* should later be replaced by the real name.			*/
	if (hmap == NULL)
	{
		hmap		= new HostMap	;
		hmap->next	= hostmap[hashv];
		hostmap[hashv]	= hmap		;

		hmap->addr	= addr		;
		hmap->retry	= RTRYCNT	;

		sprintf	(hmap->name, "%ld.%ld.%ld.%ld",
				     (addr2 >> 24) & 0xff,
				     (addr2 >> 16) & 0xff,
				     (addr2 >>  8) & 0xff,
				     (addr2 >>  0) & 0xff) ;
		strcpy	(hmap->dots, hmap->name) ;

		/* Write the IP address into the name lookup request	*/
		/* queue, to be processed in the name lookup thread.	*/
		if (nslpid > 0)
			write	(nslsock, &hmap->addr, sizeof(hmap->addr)) ;

		setMemUsed (sizeof(HostMap)) ;
	}
	else	/* If the retry count has fallen to zero, the it is	*/
		/* time to have another go .... otherwise we have got	*/
		/* it (or we never will).				*/
		if ((hmap->retry == 0) && (nslpid > 0))
			write	(nslsock, &hmap->addr, sizeof(hmap->addr)) ;
		else	rc	= hmap->retry < 0 ;

	/* Set the endpoint address and name from the hash table entry	*/
	/* for the address.						*/
	epoint->a = hmap->dots ;
	epoint->n = hmap->name ;


	/* SERVICE							*/
	/* -------							*/

	/* Special case: If the port value is negative then send the	*/
	/* endpoint port and service values to empty strings. This is	*/
	/* used when decoding IP fragments and non-port protocols.	*/
	if (_port < 0)
	{	epoint->p = "" ;
		epoint->s = "" ;
		return	  rc   ;
	}

	/* Scan the port hash table to see if the we know about this	*/
	/* port.							*/
	hashv	= _port & (PMAPSIZ - 1) ;

	for (pmap = portmap[hashv] ; pmap != NULL ; pmap = pmap->next)
		if (pmap->p == _port)
			break	;

	/* If the port was not found then add a new entry for it, with	*/
	/* the "service name" simply being the port number again.	*/
	if (pmap == NULL)
	{
		pmap		= new PortMap	 ;
		pmap->next	= portmap[hashv] ;
		portmap[hashv]	= pmap		 ;

		pmap->p		= _port		 ;
		pmap->serv	= pmap->port	 ;
		sprintf	(pmap->port, "%d", _port);

		setMemUsed (sizeof(PortMap)) ;
	}

	/* Set the port number and service name in the endpoint from	*/
	/* the hash table entry for the port.				*/
	epoint->p = pmap->port ;
	epoint->s = pmap->serv ;

	return	rc ;
}


/*  nslStart	: Start routine for nameserver lookup thread		*/
/*  sock	: int		: Socket to main process		*/
/*  (returns)	: void *	:					*/

static	void	nslStart
	(	int	sock
	)
{
	long	addr	  ;
	HOSTENT	*host	  ;
	char	name[128] ;
	char	*dotp	  ;

	/* The thread sits in a loop receiving addresses, looking up	*/
	/* the corresponding name, and returning. Exit at any point	*/
	/* that there is a read or write error.				*/
	while (read (sock, &addr, sizeof(addr)) == sizeof(addr))
	{
#if	0
		fprintf	(stderr, "ksnuffle: NSL: %08lx\n", addr) ;
#endif
		if ((host = gethostbyaddr ((char *)&addr, sizeof(addr), AF_INET)) != NULL)
		{	assert	(strlen (host->h_name) < sizeof(name)) ;
			strcpy	(name, host->h_name) ;
		}
		else	strcpy	(name, "") ;
#if	0
		fprintf	(stderr, "ksnuffle: NSL: %s\n", name) ;
#endif	

		/* If domain name stripping is enabled and we have an	*/
		/* hostname which has our domain name, then strip it.	*/
		if (dropDom && (domName != NULL))
			if ((dotp = strchr (name, '.')) != NULL)
				if (strcmp (&dotp[1], domName) == 0)
					*dotp	= 0 ;

		if ( (write (sock, &addr, sizeof(addr)) != sizeof(addr)) ||
		     (write (sock, name,  sizeof(name)) != sizeof(name)) )
			break	;
	}

	fprintf	(stderr, "ksnuffle NSL exits: %s\n", STRERR) ;
	close	(sock)	;
}

/*  KNDApp								*/
/*  sockRead	: Handle nameserver lookup reply			*/
/*  ksock	: KSocket *	: Connection to nameserver thread	*/
/*  (returns)	: void		:					*/

void	KNDApp::sockRead
	(	KSocket	*ksock
	)
{
	long	addr	  ;
	char	name[128] ;

	/* Read the address and the corresponding name, which comes in	*/
	/* a fixed size field.						*/
	if ( (read (ksock->socket(), &addr, sizeof(addr)) != sizeof(addr)) ||
	     (read (ksock->socket(), name,  sizeof(name)) != sizeof(name)) )
		ErrExit	("Ksnuffle nsl socket read error", STRERR) ;


	/* Scan the hash table for the address, and update with the	*/
	/* name. This should not fail, not that it matters.		*/
	int	hash	= addr & (HMAPSIZ - 1)	;
	HostMap	*hmap	;

	for (hmap = hostmap[hash] ; hmap != NULL ; hmap = hmap->next)
		if (hmap->addr == addr)
		{
			/* If the lookup failed then decrement the	*/
			/* retry count. When this reaches zero, the	*/
			/* next request for the address will trigger a	*/
			/* new lookup.					*/
			if (name[0] == 0)
			{	if (hmap->retry > 0) hmap->retry -= 1 ;
				return	;
			}

			/* Store the name, and set the retry count to	*/
			/* indicate that we have got the name.		*/
			STRCPY	(hmap->name, name) ;
			hmap->retry = -1 ;
			break	;
		}
}

/*  sigchld	: SIGCHLD signal catcher				*/
/*  sig		: int		: Signal				*/
/*  (returns)	: void		:					*/

static	void	sigchld
	(	int	sig
	)
{
	int	status	;
	pid_t	pid	;
	extern	void	FreeCmdSlot (pid_t) ;

	while ((pid = waitpid (-1, &status, WNOHANG)) > 0)
	{
		/* If the deceased process is _not_ the nameserver	*/
		/* lookup process then it may be an event command	*/
		/* terminating, so call FreeCmdSlot in case.		*/
		if (pid != nslpid)
		{	fprintf	    (stderr, "ksnuffle: child ") ;
			FreeCmdSlot (pid) ;
		}
		else
		{	fprintf	    (stderr, "ksnuffle: nsl   ") ;
			nslpid	= 0 ;
		}

		if	(WIFEXITED  (status))
		{
			fprintf	(stderr, "exited: %d\n",
					 WEXITSTATUS(status)) ;
		}
		else if (WIFSIGNALED(status))
		{
			fprintf	(stderr, "failed: %d\n",
					 WTERMSIG   (status)) ;
		}
		else	fprintf	(stderr, "aborted: %d\n",
						     status ) ;
	}

	signal	(SIGCHLD, sigchld) ;
}

/*  sigpipe	: SIGPIPE signal catcher				*/
/*  sig		: int		: Signal				*/
/*  (returns)	: void		:					*/

static	void	sigpipe
	(	int	sig
	)
{
	signal	(SIGPIPE, sigpipe) ;
	fprintf	(stderr, "ksnuffle: got SIGPIPE in %d\n", getpid()) ;
}

/*  startNsl	: Start the nameserver lookup process			*/
/*  (returns)	: void		:					*/

void	startNsl ()
{
	int	sockp[2]	;

	signal	(SIGCHLD, sigchld) ;
	signal	(SIGPIPE, sigpipe) ;

	/* Create a socket pair. This will be used to communicate	*/
	/* between the main process and the name lookup process.	*/
	if (socketpair (AF_UNIX, SOCK_STREAM, 0, sockp) < 0)
		ErrExit	("KSnuffle socketpair error", STRERR) ;

	/* Next create the process which will actually handle the	*/
	/* nameserver lookups ...					*/
	if	((nslpid = fork ()) < 0)
	{	ErrExit	("KSnuffle thread error", STRERR) ;
		exit	(1) ;
	}
	else if (nslpid == 0)
	{
		fprintf	 (stderr, "ksnuffle: nsl child %d\n", getpid()) ;
		close	 (sockp[1]) ;
		nslStart (sockp[0]) ;
		exit	 (0) ;
	}

	close	 (sockp[0]) ;		/* Not needed in parent		*/
	nslsock	= sockp[1]  ;		/* Note for later use		*/
}

/*  getDomain	: Get domain name for this host				*/
/*  (returns)	: void		:					*/

void	getDomain ()
{
	char	hname[256]  ;
	HOSTENT	*hent	    ;
	char	*dotp	    ;

	gethostname (hname, sizeof(hname)) ;

	if ((hent = gethostbyname (hname)) == NULL)
	{	fprintf	(stderr, "KSnuffle: gethostbyname(%s) failed\n",
				 hname) ;
		return	;
	}

	if ((dotp = strchr (hent->h_name, '.')) == NULL)
	{	fprintf	(stderr, "KSnuffle: gethostbyname gave %s\n",
				 hent->h_name) ;
		return	;
	}

	domName	= strdup (&dotp[1]) ;
	fprintf	(stderr, "KSnuffle: domain is [%s]\n", domName) ;
}

/*  KNDApp								*/
/*  startNsl	: Attach to the nameserver lookup			*/
/*  (returns)	: void		:					*/

void	KNDApp::startNsl ()
{
	if (nslpid > 0)
	{
		KSocket	*ksock	= new KSocket (nslsock) ;
		ksock->enableRead (true) ;
		connect	(ksock, SIGNAL(readEvent(KSocket *)), this, SLOT(sockRead(KSocket *))) ;
	}
}

/*  IPIsThisHost: See if IP address is one of ours			*/
/*  ipaddr	: cchar *	: IP address in question		*/
/*  (returns)	: bool		: Trie if ours				*/

bool	IPIsThisHost
	(	cchar	*ipaddr
	)
{
	for (uint idx = 0 ; idx < ourips->size() ; idx += 1)
		if (memcmp (ipaddr, (*ourips)[idx], 4) == 0)
			return	true ;

	return	false ;
}
