/***************************************************************************
                          ksnuffle.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<stdio.h>
#include	<stdarg.h>
#include	<errno.h>
#include	<unistd.h>
#include	<pwd.h>
#include	<ctype.h>

#include	<qarray.h>
#include	<qmessagebox.h>
#include	<qtooltip.h>
#include	<qkeycode.h>
#include	<qobjectlist.h>

#include	<kiconloader.h>
#include	<kmsgbox.h>

#include	<dlfcn.h>

#include	"ksnuffle.h"
#include	"knd_multi.h"
#include	"knd_setup.h"
#include	"knd_user.h"
#include	"knd_view.h"
#include	"knd_mon.h"
#include	"knd_replay.h"
#include	"knd_remote.h"
#include	"resource.h"

static	KNDApp	*kndApp	   ;	/* The application ....			*/

global	SArray	*eplist	   ;	/* List of ethernet protocols		*/
global	SArray	*iplist	   ;	/* List of internet protocols		*/
global	SArray	*svlist	   ;	/* List of internet service names	*/
global	QFont	*pktfont   ;	/* Packet list display font		*/
global	bool	vonly	   ;	/* User has view-only privileges	*/
global	char	*dmap	   ;

static	QString	remHost    ;
static	QString	remPort	   ;
static	QString	remPass	   = ""	;

extern	KApplication	*theApp	;


/*  ItoA	: Convert integer to string				*/
/*  val		: int		: Integer				*/
/*  (returns)	: char *	: String				*/

char	*ItoA
	(	int	val
	)
{
	static	char	b[32]  ;
	sprintf	(b, "%d", val) ;
	return	 b ;
}

/*  JoinStr	: Quick string concatenation				*/
/*  buff	: char *	: Result buffer				*/
/*  ....	: char *	: Strings ...				*/
/*  (returns)	: char *	: Result buffer				*/

char	*JoinStr
	(	char	*buff,
		...
	)
{
	va_list	aptr ;
	char	*str ;

	buff[0]	= 0  ;
	va_start (aptr, buff) ;
	while ((str = va_arg (aptr, char *)) != NULL)
		strcat (buff, str) ;
	va_end	 (aptr) ;
	return	 buff	;
}


/*  QStrToStr	: Convert QString to non-null strdup'd (char *)		*/
/*  qstr	: QString &	: QString in question			*/
/*  (returns)	: char *	: Allocated string			*/

char	*QStrToStr
	(	QString	&qstr
	)
{
	return	strdup ((qstr.isNull() || qstr.isEmpty()) ? "" : (const char *)qstr) ;
}

/*  QCBFindItem	: Find item in a combobox				*/
/*  combo	: QComboBox &	: Combbox in question			*/
/*  name	: const char *	: Item required				*/
/*  defv	: int		: Result if not found			*/
/*  (returns)	: int		: Position				*/

int	QCBFindItem
	(	QComboBox	&combo,
		const	char	*name,
		int		defv
	)
{
	for (int slot = 0 ; slot < combo.count() ; slot += 1)
		if (strcmp (name, combo.text(slot)) == 0)
			return	slot ;

	return	defv ;
}

/*  QLBFindItem	: Find item in a listbox				*/
/*  list	: QListBox &	: Listbox in question			*/
/*  name	: const char *	: Item required				*/
/*  defv	: int		: Result if not found			*/
/*  (returns)	: int		: Position				*/

int	QLBFindItem
	(	QListBox	&list,
		const	char	*name,
		int		defv
	)
{
	for (unsigned int slot = 0 ; slot < list.count() ; slot += 1)
		if (strcmp (name, list.text(slot)) == 0)
			return	slot ;

	return	defv ;
}


/*  ErrExit	: Error exit						*/
/*  fmt		: cchar *	: Message format			*/
/*  ...		: ...		: Arguments				*/
/*  (returns)	: void		: Never returns				*/
	
void	ErrExit
	(	cchar	*fmt,
		...
	)
{
	va_list	  aptr 		;
	char	  buff[256]	;

	va_start (aptr, fmt) 	;
	vsprintf (buff, i18n(fmt), aptr)   ;
	va_end	 (aptr)		;

	QMessageBox mb (i18n("Ksnuffle interface error"),
			buff,
			QMessageBox::Critical,
			QMessageBox::Ok, 0, 0) ;
	mb.exec ( ) ;
	exit	(1) ;
}

/*  ErrorBox	: General error box wrapper				*/
/*  icon	: ::Icon	: Popup icon				*/
/*  title	: cchar *	: Title text				*/
/*  fmt		: cchar *	: Message format			*/
/*  aptr	: va_list	: Argument list pointer			*/
/*  (returns)	: void		:					*/

static	void	ErrorBox
	(	QMessageBox::Icon	icon,
		cchar			*title,
		cchar			*fmt,
		va_list			aptr
	)
{
	char	buff	[1024] ;

	vsprintf (buff, i18n(fmt), aptr) ;

	/* The following is a quick hack to get a help button on the	*/
	/* message box. We add a Retry button, then relabel it as Help.	*/
	QMessageBox mb (i18n(title), buff,
				     icon,
				     QMessageBox::Ok,
				     QMessageBox::Retry, 0) ;

	QObjectList	*butlist = mb.queryList ("QButton")  ;
	QButton		*retry	 = (QButton *)butlist->at(1) ;

	if (retry != NULL) retry->setText ("Help") ;

	if (mb.exec () == QMessageBox::Retry)
		theApp->invokeHTMLHelp ((QString)"ksnuffle/index-2.html",
					(QString)"setuid") ;
}

/*  Error	: Display error popup					*/
/*  title	: const char *	: Popup box title			*/
/*  fmt		: const char *	: Error message format			*/
/*  ....	: ....		: Arguments				*/
/*  (returns)	: void		:					*/

void	Error
	(	cchar	  *title,
		cchar	  *fmt,
		...
	)
{
	va_list	aptr	;

	va_start (aptr, fmt) ;
	ErrorBox (QMessageBox::Warning, title, fmt, aptr) ;
	va_end	 (aptr)	;
}

/*  Error	: Display error popup					*/
/*  icon	: ::Icon	: Popup boc icon			*/
/*  title	: const char *	: Popup box title			*/
/*  fmt		: const char *	: Error message format			*/
/*  ....	: ....		: Arguments				*/
/*  (returns)	: void		:					*/

void	Error
	(	QMessageBox::Icon	icon,
		cchar			*title,
		cchar			*fmt,
		...
	)
{
	va_list	aptr	;

	va_start (aptr, fmt) ;
	ErrorBox (icon, title, fmt, aptr) ;
	va_end	 (aptr)	;
}


/*  upToTick	: Make sure we are up to time				*/
/*  timenow	: long		: Time in seconds			*/
/*  (returns)	: long		: Apparent time				*/

long	upToTick
	(	long	timenow
	)
{
	return	kndApp->upToTick (timenow) ;
}




/*  KNDRemDialog							*/
/*  KNDRemDialog: Dialog for remote sniffing				*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KNDRemDialog	:					*/

KNDRemDialog::KNDRemDialog
	(	QWidget	*parent
	)
	:
	QDialog	(parent, "Remote host", true),
	lHost	(this),
	lPort	(this),
	lPass	(this),
	eHost	(this),
	ePort	(this),
	ePass	(this),
	bOK	(this),
	bCancel	(this)
{
	lHost	.setGeometry (  5,   5,   60,  25 ) ;
	lHost	.setText     ("Host"    ) ;
	lPort	.setGeometry (  5,  35,   60,  25 ) ;
	lPort	.setText     ("Port"    ) ;
	lPass	.setGeometry (  5,  65,   60,  25 ) ;
	lPass	.setText     ("Password") ;
	eHost	.setGeometry ( 70,   5,  120,  25 ) ;
	eHost	.setText     (remHost   ) ;
	ePort	.setGeometry ( 70,  35,  120,  25 ) ;
	ePort	.setText     (remPort    ) ;
	ePass	.setGeometry ( 70,  65,  120,  25) ;
	ePass	.setText     (remPass	 ) ;
	ePass	.setEchoMode (QLineEdit::Password) ;
	bOK	.setGeometry (200,   5,  60,  25 ) ;
	bOK	.setText     ("OK"       ) ;
	bCancel	.setGeometry (200,  35,  60,  25 ) ;
	bCancel	.setText     ("Cancel"   ) ;

	connect	(&bOK,     SIGNAL(clicked()), SLOT(accept())) ;
	connect	(&bCancel, SIGNAL(clicked()), SLOT(reject())) ;
}

/*  KNDRemDialog							*/
/*  ~KNDRemDialog: Destructor for remote sniffing dialog		*/
/*  (returns)	 :		:					*/

KNDRemDialog::~KNDRemDialog ()
{
}

/*  KNDRemDialog							*/
/*  done	: Handle dialog box completion				*/
/*  ok		: int		: Non-zero on OK			*/
/*  (returns)	: void		:					*/

void	KNDRemDialog::done
	(	int	ok
	)
{
	if (ok)
	{	remHost	= eHost.text () ;
		remPort	= ePort.text () ;
		remPass	= ePass.text () ;
		if ( remHost.isEmpty() ||
		     remPort.isEmpty() ||
		     remPass.isEmpty() ) ok = 0 ;
	}

	QDialog::done (ok) ;
}




/*  KNDApp								*/
/*  KNDApp	: Constructor for application object			*/
/*  (returns)	: KNDApp	:					*/

KNDApp::KNDApp ()
{
	KConfig	*config	= kapp->getConfig () ;

	memUsed	  = 0   ;
	keystate  = 0	;
	prefilter = NULL;

	/* The dmap array is used to map data bytes to printable	*/
	/* characters. Construct it and fill it in the first time that	*/
	/* we are called.						*/
	dmap	= (char *)malloc (256)	;
	memset	(dmap, '.', 256)	;
	for (int i = 1 ; i < 128 ; i += 1)
		if (isalnum(i) || ispunct(i) || ((strchr ("", i) != NULL)))
			dmap[i]	= i   ;
	dmap[' '] = ' ' ;

	/* Store a pointer at this object, to be used by the "upToTick"	*/
	/* routine. We also note the current time in seconds as a base	*/
	/* from which to work.						*/
	kndApp	 = this ;
	second	 = time (NULL) ;

	/* See if we are running with effective-uid root. If not then	*/
	/* ksnuffle is not installed setuid-root and is not bing run	*/
	/* by root. In this case we will not execute ...		*/
	if (geteuid () != 0)
	{	Error	("KSnuffle execution error",
			 "KSnuffle not being run by root nor setuid-root") ;
		exit	(1) ;
	}

	/* Get the global configuration, which contains the privileged	*/
	/* users ...							*/
	QString	confpath  = KApplication::kde_configdir().data() + 
			    QString((const char *)"/ksnufflerc") ;

	userconf = new KConfig (confpath) ;
	userconf->setGroup  ("Privilege") ;

	/* If the real uid is not root then ksnuffle must be installed	*/
	/* setuid-root and is bing run by a user other than root. In	*/
	/* this case we look to see if the real user is in the lists of	*/
	/* privileged users.						*/
	if (getuid () != 0)
	{
		QString	fulluser  = userconf->readEntry ("full") ;
		QString	viewuser  = userconf->readEntry ("view") ;
		struct	passwd *p = getpwuid  (getuid()) ;
		char	*_pfull	  = QStrToStr (fulluser) ;
		char	*_pview	  = QStrToStr (viewuser) ;
		bool	ok	  = false ;
		bool	view	  = true  ;
		char	*pptr	  ;

		/* First check the full privilege users, who can see	*/
		/* all of each packet, ie., including the application	*/
		/* data.						*/
		if (_pfull[0] != 0)
			for (pptr = strtok (_pfull, ",\n") ; pptr != NULL ; pptr = strtok (NULL, ",\n"))
				if (strcmp (p->pw_name, pptr) == 0)
				{	ok	= true  ;
					view	= false ;
					break	;
				}

		/* Second, look for those users who are allowed to	*/
		/* sniff packets, but who do not get to see any		*/
		/* application data therein.				*/
		if (_pview[0] != 0)
			for (pptr = strtok (_pview, ",\n") ; pptr != NULL ; pptr = strtok (NULL, ",\n"))
				if (strcmp (p->pw_name, pptr) == 0)
				{	ok	= true  ;
					view	= true  ;
					break	;
				}

		free (_pfull) ;
		free (_pview) ;

		if (!ok)
		{	Error	("KSnuffle execution error",
				 "You do not have permission to run KSnuffle") ;
			exit	(1) ;
		}
		if (view)
		{	QMessageBox mb (i18n("Ksnuffle"),
					i18n("You do not have permission to view packet data"),
					QMessageBox::Warning,
					QMessageBox::Ok, 0, 0) ;
			mb.exec () ;
		}

		vonly	= view ;

		/* We also look to see if the user has been stuck with	*/
		/* a pre-filter. If so, remind the user of this.	*/
		char	cfe[1024] ;
		QString	filter	  ;

		sprintf	(cfe, "pre_filter_%s", p->pw_name) ;
		filter	= userconf->readEntry (cfe, "")	   ;

		if (!filter.isNull() && !filter.isEmpty())
		{
			prefilter = strdup (filter) ;
			sprintf	(cfe, "Network traffic will be pre-filtered by\n\"%s\"",
				      (cchar *)filter) ;

			QMessageBox mb (i18n("Ksnuffle"),
					cfe,
					QMessageBox::Warning,
					QMessageBox::Ok, 0, 0) ;
			mb.exec () ;
		}
	}


	/* OK, we are allowed to run. We can now get on with all the	*/
	/* mundane configuration stuff.					*/

	startNsl () ;

	config->setGroup ("APPEARANCE") ;

	int	sXPos	= config->readNumEntry ("XPos"  ) ;
	int	sYPos	= config->readNumEntry ("YPos"  ) ;
	int	sWidth	= config->readNumEntry ("width" ) ;
	int	sHeight	= config->readNumEntry ("height") ;

	if ((sXPos > 0) && (sYPos > 0))
		setGeometry (sXPos, sYPos, sWidth, sHeight) ;

	setMinimumSize ( 650, 335)  ;
	setMaximumSize (1024, 768)  ;

	viewlist.resize (MAXMON)   ;
	viewlist.fill   ((KNDView *)NULL) ;

	LoadArpCache	()	   ;
	iflist	= FindAllIface	() ;
	eplist	= FindAllEProto	() ;
	iplist	= FindAllIProto	() ;
	svlist	= FindAllService() ;

	setCaption    ("Ksnuffle " VERSION);
	readOptions   () ;

	initMenuBar   () ;
	initToolBar   () ;
	initStatusBar () ;

	display = new KNDDisplay (this) ;
	butbar	= new KNDButBar	 (display, this) ;
	display->setButBar (butbar) ;

	initView      (userconf) ;

	if (!bViewToolbar  ) enableToolBar   (KToolBar  ::Hide, 0) ;
	if (!bViewStatusbar) enableStatusBar (KStatusBar::Show ) ;

	menuBar()->setMenuBarPos (menu_bar_pos) ;
	toolBar()->setBarPos	 (tool_bar_pos) ;

	timer.start (1000) ;
	connect	    (&timer, SIGNAL(timeout()), SLOT(timerTick())) ;
}

/*  KNDApp								*/
/*  ~KNDApp	: Destructor for application object			*/
/*  (returns)	:		:					*/

KNDApp::~KNDApp()
{
	for (int slot = 0 ; slot < MAXMON ; slot += 1)
		if (viewlist[slot] != NULL)
			delete	viewlist[slot] ;

	delete	setup	;
	delete	multi	;

	/* NOTE: We do not destroy the userconf object as we want the	*/
	/*	 save function to be under control of the Save button	*/
	/*	 on the setup page.					*/
}

/*  KNDApp								*/
/*  upToTick	: Advance our time to specified value			*/
/*  timenow	: long		: Apparent time now in seconds		*/
/*  (returns)	: long		: Apparent time				*/

long	KNDApp::upToTick
	(	long	timenow
	)
{
	/* Loop until our time catches up with the apparent current	*/
	/* time. Each time round we advance our time by one second, and	*/
	/* then pass this down to the sniffers and replayers.		*/
	while (second < timenow)
	{
		second	+= 1 ;
		for (int slot = 0 ; slot < MAXMON ; slot += 1)
			if (viewlist[slot] != NULL)
				viewlist[slot]->timerTick (second) ;

		multi->timerTick (second) ;

		/* Every minute, rescan the ARP cache. This way we	*/
		/* should accumulate MAC<->IP mappings (and even spot	*/
		/* machines which have been rebooted with a different	*/
		/* network card :-)					*/
		if (second % 60 == 0) LoadArpCache () ;
	}

	/* Return the apparent time, which may be ahead of the value	*/
	/* with which we were called (for instance, a captured packet	*/
	/* has been delayed over a timer tick).				*/
	return	second	;
}

/*  KNDApp								*/
/*  timerTick	: Handle timer tick					*/
/*  (returns)	: void		:					*/

void	KNDApp::timerTick ()
{
	long	t 	;
	char	b[32]	;

	/* Ensure that we have advanced our idea of the time to be that	*/
	/* of the system.						*/
	upToTick (t = time (NULL)) ;

	if ((t & 0x1f) == 0)
	{
		sprintf	(b, "%d KBytes", memUsed / 1024) ;
		memBox->setText (b) ;
	}
}


void	KNDApp::enableCommand
	(	int	id_
	)
{
	menuBar()->setItemEnabled (id_,  true) ;
	toolBar()->setItemEnabled (id_,  true) ;
}

void	KNDApp::disableCommand
	(	int	id_
	)
{
	menuBar()->setItemEnabled (id_, false) ;
	toolBar()->setItemEnabled (id_, false) ;
}


typedef	struct
{	char	*icon	;
	char	*text	;
	int	id	;
	int	accel	;
}	Control	;


/*  buildMenu	: Build menu from control table				*/
/*  menu	: QPopupMenu *	: Menu to construct			*/
/*  control	: Control *	: List of entries			*/
/*  (returns)	: void		:					*/

static	void	buildMenu
	(	QPopupMenu	*menu,
		Control		*control
	)
{
	Control	*cp	;

	for (cp = &control[0] ; cp->id >= 0 ; cp += 1)
	{
		if (cp->id == ID_VIEW_USER)
			if ((getuid() != 0) || (geteuid() != 0))
				continue ;

		if	(cp->id   == 0)
			menu->insertSeparator () ;
		else if (cp->icon == NULL)
			menu->insertItem (i18n(cp->text), cp->id) ;
		else	menu->insertItem (Icon(cp->icon),
					  i18n(cp->text), cp->id) ;

		if (cp->accel > 0)
			menu->setAccel (CTRL+cp->accel, cp->id) ;
	}
}

/*  KNDApp								*/
/*  initMenuBar	: Initialise menu bar					*/
/*  (returns)	: void		:					*/

void KNDApp::initMenuBar()
{
	static	Control	m_monitor[] =
	{
	{	"ksn_sniff.xpm", "&New Sniffer",   ID_MONITOR_NEW,	Key_N	},
	{	"ksn_remote.xpm","New Remo&te",	   ID_MONITOR_REMOTE,	Key_O	},
	{	"ksn_replay.xpm","New Re&play",	   ID_MONITOR_REPLAY,	Key_P	},
	{	"ksn_close.xpm", "&Close",	   ID_MONITOR_CLOSE,	Key_W	},
	{	NULL,		 NULL,		   0,			-1	},
	{	"ksn_start.xpm", "&Run (all)",	   ID_MONITOR_START,	Key_R	},
	{	"ksn_stop.xpm",	 "&Halt (all)",	   ID_MONITOR_STOP,	Key_H	},
	{	"ksn_freeze.xpm","&Freeze (all)",  ID_MONITOR_FREEZE,	Key_F	},
	{	"ksn_resume.xpm","R&esume (all)",  ID_MONITOR_RESUME,	Key_E	},
	{	NULL,		 NULL,		   0,			-1	},
	{	"filefloppy.xpm","&Save",	   ID_MONITOR_SAVE,	Key_S	},
	{	NULL,		 "S&ave and exit", ID_MONITOR_SAVE_QUIT,Key_A	},
	{	"exit.xpm",	 "E&xit",	   ID_MONITOR_QUIT,	Key_Q	},
	{	NULL,		 NULL,		   -1,			0	}  
	}	;

	static	Control	m_view[] =
	{
	{	NULL,		 "Toolbar",	   ID_VIEW_TOOLBAR,	-1	},
	{	NULL,		 "Statusbar",	   ID_VIEW_STATUSBAR,	-1	},
	{	NULL,		 NULL,		   0,			-1	},
	{	"ksn_conf.xpm",  "&Configuration", ID_VIEW_CONFIG,	Key_C	},
	{	"ksn_filter.xpm","Filters/&Triggers",
						   ID_VIEW_TRIGGERS,	Key_T	},
	{	"ksn_events.xpm","E&vent commands",ID_VIEW_EVENTS,	Key_V	},
	{	"ksn_graph.xpm", "&Graphics",	   ID_VIEW_GRAPHIC,	Key_G	},
	{	"ksn_pkt.xpm",	 "&Packets",	   ID_VIEW_PACKETS,	Key_P	},
	{	NULL,		 NULL,		   0,			-1	},
	{	"ksn_mon.xpm",	 "Sn&iffers",	   ID_VIEW_MONITORS,	Key_I	},
	{	"ksn_par.xpm",	 "Para&llel",	   ID_VIEW_MULTI,	Key_L	},
	{	"ksn_setup.xpm", "Set&up",	   ID_VIEW_SETUP,	Key_U	},
	{	"ksn_priv.xpm",  "Use&r privilege",ID_VIEW_USER,	Key_R	},
	{	NULL,		 NULL,		   0,			-1	},
	{	NULL,		 NULL,		   -1,			0	}  
	}	;

	buildMenu (monitor_menu = new QPopupMenu (), &m_monitor[0]) ;
	buildMenu (view_menu    = new QPopupMenu (), &m_view   [0]) ;

	view_menu->setCheckable   (true) ;
	view_menu->setItemChecked (ID_VIEW_TOOLBAR,   bViewToolbar  ) ;
	view_menu->setItemChecked (ID_VIEW_STATUSBAR, bViewStatusbar) ;


	help_menu = new QPopupMenu    () ;
	help_menu = kapp->getHelpMenu (true, i18n(IDS_APP_ABOUT)) ;


	menuBar()->insertItem (i18n("&Monitor"), monitor_menu) ;
	menuBar()->insertItem (i18n("&View"),    view_menu   ) ;
	menuBar()->insertSeparator () ;
	menuBar()->insertItem (i18n("&Help"),    help_menu   ) ;

	connect (monitor_menu, SIGNAL(activated(int)), SLOT(commandCallback(int))) ;
	connect (view_menu,    SIGNAL(activated(int)), SLOT(commandCallback(int))) ;
}

/*  KNDApp								*/
/*  initToolBar	: Initialise toolbar					*/
/*  (returns)	: void		:					*/

void	KNDApp::initToolBar ()
{
	static	Control	tools[] =
	{
	{	"ksn_sniff.xpm", "New Sniffer",	       ID_MONITOR_NEW,	 -1	},
	{	"ksn_remote.xpm","New Remo&te",	       ID_MONITOR_REMOTE,-1	},
	{	"ksn_replay.xpm","New Replay",	       ID_MONITOR_REPLAY,-1	},
	{	"ksn_close.xpm", "Close monitor",      ID_MONITOR_CLOSE, -1	},
	{	NULL,		 NULL,		       0,		 -1	},
	{	"ksn_start.xpm", "Run monitor(s)",     ID_MONITOR_START, -1	},
	{	"ksn_stop.xpm",	 "Halt monitor(s)",    ID_MONITOR_STOP,	 -1	},
	{	"ksn_freeze.xpm","Freeze monitor(s)",  ID_MONITOR_FREEZE,-1	},
	{	"ksn_resume.xpm","Resume monitor(s)",  ID_MONITOR_RESUME,-1	},
	{	NULL,		 NULL,		       0,		 -1	},
	{	NULL,		 NULL,		       ID_MONITOR_LIST,	 -1	},
	{	NULL,		 NULL,		       0,		 -1	},
	{	"filefloppy.xpm","Save configuration", ID_MONITOR_SAVE,	 -1	},
	{	NULL,		 NULL,		       0,		 -1	},
	{	"ksn_mon.xpm",	 "View sniffers",      ID_VIEW_MONITORS, -1	},
	{	"ksn_par.xpm",	 "View parallel display",
					 	       ID_VIEW_MULTI,	 -1	},
	{	"ksn_setup.xpm", "View setup page",    ID_VIEW_SETUP,	 -1	},
	{	"ksn_priv.xpm",  "View user page",     ID_VIEW_USER,	 -1	},
	{	NULL,		 NULL,		       -1,		  0	}  
	}	;

	for (Control *cp = &tools[0] ; cp->id >= 0 ; cp += 1)
		switch (cp->id)
		{
			case 0	:
				toolBar()->insertSeparator () ;
				break	;

			case ID_MONITOR_LIST :
				toolBar()->insertCombo     ("",
							    ID_MONITOR_LIST,
							    true,
							    SIGNAL(activated(int)),
							    this,
							    SLOT(slotMonitorPick(int))) ; 
				break	;

			default	:
				if (cp->id == ID_VIEW_USER)
					if ((getuid() != 0) || (geteuid() != 0))
						break	;

				toolBar()->insertButton    (Icon(cp->icon),
							    cp->id,
							    true,
							    i18n(cp->text)) ;
				break	;
		}

	monList	= toolBar()->getCombo (ID_MONITOR_LIST) ;
	QRect	rect	= monList->geometry () ;
	rect.setRight (rect.left () + 120) ;
	monList->setGeometry (rect) ;
	monList->removeItem  (   0) ;
	QToolTip::add (monList, i18n ("Select sniffer or replayer")) ;

	toolBar()->insertSeparator  () ;
	toolBar()->insertButton     (Icon("help.xpm"),    ID_HELP, SIGNAL(pressed()), kapp, SLOT(appHelpActivated()), true, i18n("Help"));

	connect	(toolBar(), SIGNAL(clicked(int)), SLOT(commandCallback(int))) ;
}

/*  KNDApp								*/
/*  initStatusBar: Initialise status bar				*/
/*  (returns)	 : void		:					*/

void	KNDApp::initStatusBar ()
{
	stateBox  = new QLabel (statusBar()) ;
	statusBar()->insertWidget      (stateBox,  60, ID_MONITOR_STATE ) ;
	stateBox->setFrameStyle        (QFrame::Panel|QFrame::Sunken) ;
	stateBox->setBackgroundMode    (PaletteBackground) ;
	QToolTip::add (stateBox, i18n  ("Current sniffer or replayer state" )) ;

	frozenBox = new QLabel (statusBar()) ;
	statusBar()->insertWidget      (frozenBox, 60, ID_MONITOR_FROZEN) ;
	frozenBox->setFrameStyle       (QFrame::Panel|QFrame::Sunken) ;
	frozenBox->setBackgroundMode   (PaletteBackground) ;
	QToolTip::add (frozenBox, i18n ("Current sniffer or replayer frozen")) ;

	pausedBox = new QLabel (statusBar()) ;
	statusBar()->insertWidget      (pausedBox, 60, ID_MONITOR_PAUSED) ;
	pausedBox->setFrameStyle       (QFrame::Panel|QFrame::Sunken) ;
	pausedBox->setBackgroundMode   (PaletteBackground) ;
	QToolTip::add (pausedBox, i18n ("Current sniffer or replayer paused")) ;

	load1Box = new QLabel (statusBar()) ;
	statusBar()->insertWidget      (load1Box, 170, ID_MONITOR_LOAD1) ;
	load1Box ->setFrameStyle       (QFrame::Panel|QFrame::Sunken) ;
	load1Box ->setBackgroundMode   (PaletteBackground) ;
	QToolTip::add (load1Box, i18n ("First load average")) ;

	load2Box = new QLabel (statusBar()) ;
	statusBar()->insertWidget      (load2Box, 170, ID_MONITOR_LOAD2) ;
	load2Box ->setFrameStyle       (QFrame::Panel|QFrame::Sunken) ;
	load2Box ->setBackgroundMode   (PaletteBackground) ;
	QToolTip::add (load2Box, i18n ("Second load average")) ;

	memBox    = new QLabel (statusBar()) ;
	statusBar()->insertWidget      (memBox  , 100, ID_MONITOR_STATE ) ;
	memBox  ->setFrameStyle        (QFrame::Panel|QFrame::Sunken) ;
	memBox  ->setBackgroundMode    (PaletteBackground) ;
	QToolTip::add (memBox  , i18n  ("Internal memory usage")) ;

	QWidget *dummy = new QWidget(statusBar()) ;
	statusBar()->insertWidget   (dummy, 120, 999) ;
	dummy->hide () ;
}

/*  KNDApp								*/
/*  setView	: Local override fir setView				*/
/*  view	: QWidget *	: New view				*/
/*  (returns)	: void		:					*/

void	KNDApp::setView
	(	QWidget	*view
	)
{
	KTMainWindow::setView (view) ;

	/* The next two calls are needed to make things work properly	*/
	/* with KTMainWindow. Show the new monitor and then force an	*/
	/* update so that it is correctly sized.			*/
	view->show	()	;
	updateRects	()	;
}

/*  KNDApp								*/
/*  showView	: Show a selected view					*/
/*  next	: KNDView *	: Selected view to show next		*/
/*  (returns)	: void		:					*/

void	KNDApp::showView
	(	KNDView	*next
	)
{
	switch (showing)
	{
		case ShowSetup	:
			setup->hide      () ;
			break	;

		case ShowUser	:
			user ->hide	 () ;
			break	;

		case ShowMulti	:
			multi->hide      () ;
			multi->setOnview (false) ;
			break	;

		default	:
			break	;
	}

	if ((view != next) || (showing != ShowMonitors))
	{
		setView (display) ;
		display->setView   (view = next) ;
		view   ->setOnview (true)	 ;

		monList->setCurrentItem (QCBFindItem (*monList, view->getName(), 0)) ;
	}

	showing	= ShowMonitors	;
	setState  (view)	;
}

/*  KNDView								*/
/*  setState	: Display new state of sniffer or replayer		*/
/*  _view	: KNDView *	: Object with new state			*/
/*  (returns)	: void		:					*/

void	KNDApp::setState
	(	KNDView	*_view
	)
{
	if (_view != view) return ;

	MState	state	= view->getState  () ;
	bool	frozen	= view->getFrozen () ;
	bool	paused	= view->getPaused () ;
	MType	mtype	= view->getMType  () ;
	char	*stext	;

	switch (state)
	{
		case MSUnset	:
			stext	= "Unset"	;
			break	;

		case MSSet	:
			stext	= "Set"		;
			break	;

		case MSCleared	:
			stext	= "Cleared"	;
			break	;

		case MSReset	:
			stext	= "Reset"	;
			break	;

		case MSWaiting	:
			stext	= "Waiting"	;
			break	;

		case MSRunning	:
			stext	= "Running"	;
			break	;

		case MSHalted	:
			stext	= "Halted"	;
			break	;

		case MSStopped	:
			stext	= "Stopped"	;
			break	;

		case MSError	:
			stext	= "Error"	;
			break	;

		default		:
			stext	= "Unknown"	;
			break	;
	}

	stateBox ->setText  (i18n (stext)) ;
	frozenBox->setText  (i18n (frozen ? "Frozen" : "")) ;
	pausedBox->setText  (i18n (paused ? "Paused" : "")) ;

	butbar   ->setState (state, frozen, mtype) ;
}

/*  KNDApp								*/
/*  setLoad	: Set load average values				*/
/*  _view	: KNDView *	: Object with new values		*/
/*  value1	: int		: First value				*/
/*  period1	: int		: First load period			*/
/*  value2	: int		: Second value				*/
/*  period2	: int		: Second load period			*/
/*  (returns)	: void		:					*/

void	KNDApp::setLoad
	(	KNDView	*_view,
		int	load1,
		int	period1,
		int	load2,
		int	period2
	)
{
	char	text[32] ;

	if (_view != view) return ;

	if (load1 >= 1024)
		sprintf	(text, "%d KBytes/sec over %d secs", load1 / 1024, period1) ;
	else	sprintf	(text, "%d bytes/sec over %d secs",  load1,        period1) ;
	load1Box->setText (text) ;

	if (load2 >= 1024)
		sprintf	(text, "%d KBytes/sec over %d secs", load2 / 1024, period2) ;
	else	sprintf	(text, "%d bytes/sec over %d secs",  load2,        period2) ;
	load2Box->setText (text) ;
}

/*  KNDApp								*/
/*  initView	: Initialise all views (monitors)			*/
/*  userconf	: KConfig *	: Privileged users configuration	*/
/*  (returns)	: void		:					*/

void	KNDApp::initView
	(	KConfig	*userconf
	)
{
 	KConfig	*config = kapp->getConfig () ;

	config->setGroup   ("MONITORS") ;

	QString	mlist	= config->readEntry ("MonitorList") ;
	QString	mcurr	= config->readEntry ("Current", "") ;
	QString	text	;

	KNDView	*next	;
	char	*mtoks	;
	int	slot	= 0	;
	int	found	= -1	;
	KNDView	*first	= NULL	;
	MType	mtype		;
	view	= NULL		;
	showing	= ShowMonitors	;

	multi	= new KNDMulti	(this) ;
	setup	= new KNDSetup	(this) ;

	if ((getuid() == 0) && (geteuid() == 0))
		user	= new KNDUser (this, userconf) ;
	else	user	= NULL ;

	multi->hide () ;
	setup->hide () ;
	if (user != NULL) user->hide () ;

	/* Provided that we have a monitor list to recreate, scan it	*/
	/* for entries. For simplicity, each is inserted into the list	*/
	/* at the original place. Call "showView" on each so that they	*/
	/* get correctly sized and whatnot. Also, look for the one that	*/
	/* was current last, so that it can be shown first.		*/
	char	*_mlist	= QStrToStr (mlist) ;

	if (_mlist[0] != 0)
		for (mtoks = strtok (_mlist, ",") ; mtoks != NULL ; mtoks = strtok (NULL, ","))
		{
			text.sprintf	 ("MONITOR_%d", atoi(mtoks)) ;
			config->setGroup (text) ;

			mtype	= (MType)atoi(config->readEntry ("Type", "0")) ;

			if (mtype == Remote)
			{
				QString	host = config->readEntry ("RemHost", "") ;
				QString	port = config->readEntry ("RemPort", "") ;
				QString	pass = config->readEntry ("RemPass", "") ;

				if (host.isEmpty() || port.isEmpty() || pass.isEmpty())
					continue ;

				next	= new KNDRemote  (display, this,
								   slot,
								   host,
								   port,
								   pass,
								   multi,
								   prefilter,
								   config) ;
			}
			else	next	= new KNDMonitor (display, this,
								   slot,
								   iflist,
								   multi,
								   prefilter,
								   config) ;
								   

			viewlist[slot]	= next	;
			view_menu->insertItem (next->getName(), ID_VIEW_SELECT + slot) ; 
			monList  ->insertItem (next->getName(), 0) ;

			showView (next) ;

			if (strcmp (view->getName (), mcurr) == 0)
				first	= view ;

			found		= slot	;
			slot	       += 1	;

		}

	free (_mlist) ;

	/* We should have found and recreated the monitor that was last	*/
	/* on view, in which case make sure that it is now displayed	*/
	/* before returning.						*/
	if (first != NULL)
	{
		showView (first) ;
	}
	/* In case we failed above, show one of the monitors that we	*/
	/* have just created (unless, of course, we didn't create any!	*/
	else if (found >= 0)
	{
		showView (view = viewlist[found]) ;
	}
	else
	{
		/* The fallback case covers the situation the first	*/
		/* time that the program is run, or if we exited last	*/
		/* time without saving the configuration, in which	*/
		/* case we create a skeletal monitor.			*/
		next	    = new KNDMonitor (display, this, 0, iflist, multi, prefilter) ;
		viewlist[0] = next ;
		view_menu->insertItem (next->getName(), ID_VIEW_SELECT + 0) ; 
		monList  ->insertItem (next->getName(), 0) ;
		showView (next) ;
	}

	/* Finally, get the last-used remote host and remote sniffer	*/
	/* port.							*/
	config->setGroup   ("REMOTE") ;
	remHost	= config->readEntry ("host", "localhost") ;
	remPort	= config->readEntry ("port", "900"	) ;
}

/*  KNDApp								*/
/*  doExit	: Handle exit						*/
/*  save	: bool		: Save options				*/
/*  (returns)	: void		:					*/

void	KNDApp::doExit
	(	bool	save
	)
{
	saveOptions (save) ;

	view = NULL ;

	for (int slot = 0 ; slot < MAXMON ; slot += 1)
		if (viewlist[slot] != NULL)
		{	viewlist[slot]->viewStop () ;
			delete	viewlist[slot]	    ;
			viewlist[slot] = NULL       ;
		}


	KTMainWindow::deleteAll () ;
	kapp->quit  () ;
}

/*  KNDApp								*/
/*  queryExit	: Query whether to exit or not				*/
/*  (returns)	: bool		: True to exit				*/

bool	KNDApp::queryExit ()
{
	return KMsgBox::yesNo (this, i18n ("KSnuffle " VERSION),
				     i18n ("Really Quit ?")) == 1 ? true : false ;
}

void	KNDApp::closeEvent
	(	QCloseEvent *e
	)
{
	if (queryExit ()) doExit (false) ;
}

/*  KNDApp								*/
/*  saveOptions	: Save settings						*/
/*  sm		: bool		: Save monitor configurations		*/
/*  (returns)	: void		:					*/

void	KNDApp::saveOptions
	(	bool	sm
	)
{
	KConfig *config = kapp->getConfig() ;
	KNDView	*v	;
	QString	mlist	= "" ;
	QString	text	;
	int	slot	;
	bool	first	;

	/* First generate the standard appearance section which records	*/
	/* tool bar and suchlike information.				*/
	config->setGroup   ("APPEARANCE") ;
	config->writeEntry ("ShowToolbar",   toolBar  ()->isVisible()) ;
	config->writeEntry ("ShowStatusbar", statusBar()->isVisible()) ;
	config->writeEntry ("MenuBarPos",    (int)menuBar()->menuBarPos()) ;
	config->writeEntry ("ToolBar_Pos",   (int)toolBar()->barPos    ()) ;

	config->writeEntry ("XPos",	     x     ()) ;
	config->writeEntry ("YPos",	     y     ()) ;
	config->writeEntry ("width",	     width ()) ;
	config->writeEntry ("height",	     height()) ;

	if (sm)
	{
		/* The next section contains two fields. The first is	*/
		/* a list of numbers, identifying slots in the monitor	*/
		/* table for which there are extant monitors; the	*/
		/* second is the name of the monitor which is currently	*/
		/* on view.						*/
		config->setGroup   ("MONITORS") ;
		for (slot = 0, first = true ; slot < MAXMON ; slot += 1)
			if ((v = viewlist[slot]) != NULL)
			{
				if (!v->SaveMe())
					continue ;

				if (first)
					first	= false ;
				else	mlist  += ","	;

				text.sprintf ("%d", slot) ;
				mlist	+= text ;
			}

		config->writeEntry ("MonitorList", mlist) ;
		config->writeEntry ("Current", 	   view->getName()) ;

		/* There now follow a set of sections, each called	*/
		/* MONITOR_# where each # is a number from the monitor	*/
		/* list in the previous section. These sections		*/
		/* contain all the details for a particular monitor.	*/
		for (slot = 0, first = true ; slot < MAXMON ; slot += 1)
			if ((v = viewlist[slot]) != NULL)
			{
				if (!v->SaveMe())
					continue ;

				text.sprintf ("MONITOR_%d", slot) ;
				config->setGroup    (text) ;
				config->writeEntry  ("Type", (int)v->getMType()) ;
				v     ->writeConfig (config) ;
			}
	}

	/* Save the last-used remote host and remote sniffer port.	*/
	config->setGroup   ("REMOTE") ;
	config->writeEntry ("host", remHost) ;
	config->writeEntry ("port", remPort) ;

	config->sync () ;
}

/*  KNDApp								*/
/*  readOptions	: Read common display options				*/
/*  (returns)	    : void	:					*/

void	KNDApp::readOptions ()
{
	KConfig *config = kapp->getConfig() ;

	config->setGroup("APPEARANCE");
	bViewToolbar   = config->readBoolEntry ("ShowToolbar",   true) ;
	bViewStatusbar = config->readBoolEntry ("ShowStatusbar", true) ;
	menu_bar_pos   = (KMenuBar::menuPosition)config->readNumEntry ("MenuBarPos",  KMenuBar::Top) ; 
	tool_bar_pos   = (KToolBar::BarPosition )config->readNumEntry ("ToolBar_Pos", KToolBar::Top) ;

}


/*  KNDApp	    :							*/
/*  slotMonitorNew  : Create a new monitor				*/
/*  (returns)	    : void	:					*/

void	KNDApp::slotMonitorNew ()
{
	int		slot	;
	KNDMonitor	*next	;

	/* Scan the view array for an empty slot into which the new	*/
	/* monitor will be inserted.					*/
	if ((slot = viewlist.find ((KNDView *)NULL)) < 0) return ;

	next		= new KNDMonitor (display, this, slot, iflist, multi, prefilter) ;
	viewlist[slot]	= next ;
	view_menu->insertItem (next->getName(), ID_VIEW_SELECT + slot) ; 
	monList  ->insertItem (next->getName(), 0) ;

	showView (next) ;
}

/*  KNDApp	     :							*/
/*  slotMonitorRemote: Create a new remote monitor			*/
/*  (returns)	     : void	:					*/

void	KNDApp::slotMonitorRemote ()
{
	int		slot	;
	KNDRemote	*next	;

	/* Scan the view array for an empty slot into which the new	*/
	/* monitor will be inserted.					*/
	if ((slot = viewlist.find ((KNDView *)NULL)) < 0) return ;

	KNDRemDialog	rc (this) ;
	if (!rc.exec()) return	  ;

	next		= new KNDRemote  (display, this, slot,
							 remHost,
							 remPort,
							 remPass,
							 multi, prefilter) ;
	viewlist[slot]	= next ;
	view_menu->insertItem (next->getName(), ID_VIEW_SELECT + slot) ; 
	monList  ->insertItem (next->getName(), 0) ;

	showView (next) ;
}

/*  KNDApp	     :							*/
/*  slotMonitorReplay: Create a new replay				*/
/*  (returns)	     : void	:					*/

void	KNDApp::slotMonitorReplay ()
{
	int	slot	;
	KNDView	*next	;

	/* Scan the views array for an empty slot into which the	*/
	/* new replayer will be inserted.				*/
	if ((slot = viewlist.find ((KNDView *)NULL)) < 0) return ;

	next		= new KNDReplay (display, this, multi, slot) ;
	viewlist[slot]	= next ;
	view_menu->insertItem (next->getName(), ID_VIEW_SELECT + slot) ; 
	monList  ->insertItem (next->getName(), 0) ;
	showView (next) ;
}

/*  KNDApp	    :							*/
/*  slotMonitorClose: Close the current monitor				*/
/*  (returns)	    : void	:					*/

void	KNDApp::slotMonitorClose ()
{
	KNDView	*next	;
	KNDView	*last	;
	int	oldslot	;
	int	newslot	;
	int	scan	;

	/* Scan the monitor combo list looking for the current monitor,	*/
	/* and remove it. We also remove it from the list of monitors.	*/
	for (scan = 0 ; scan < monList->count() ; scan += 1)
		if (strcmp (view->getName(), monList->text(scan)) == 0)
		{	monList->removeItem (scan) ;
			break ;
		}

	oldslot	= view->getSlot () ;
	viewlist[oldslot] = NULL   ;

	/* Scan the view array looking for another monitor to		*/
	/* activate. If none is found create a new one and allocate	*/
	/* the slot that the one we are closing occupied.		*/
	for (scan = 0, next = NULL ; scan < MAXMON ; scan += 1)
		if ((viewlist[scan] != NULL) && (viewlist[scan] != view))
		{	next	= viewlist[scan];
			break	;
		}

	if (next == NULL)
	{	newslot	= oldslot == 0 ? 1 : 0 ;
		next	= new KNDMonitor   (display, this, newslot, iflist, multi, prefilter) ;
		view_menu->insertItem (next->getName(), ID_VIEW_SELECT + newslot) ; 
		monList  ->insertItem (next->getName(), 0) ;
	}

	last	= view	;
	showView (next) ;
	delete	  last  ;

	view_menu->removeItem     (ID_VIEW_SELECT + oldslot) ;
	monList  ->setCurrentItem (QCBFindItem (*monList, view->getName(), 0)) ;
}

/*  KNDApp	    :							*/
/*  slotMonitorPick : Pick a monitor for display			*/
/*  slot	    : int	: Index into combo list			*/
/*  (returns)	    : void	:					*/

void	KNDApp::slotMonitorPick
	(	int	slot
	)
{
	const	char	*name	= monList->text (slot) ;
	int		scan	;

	/* Scan the list of viewlist for the required one and display	*/
	/* it. See the comments in slotMonitorNew about making this	*/
	/* work with KTMainWindow.					*/
	for (scan = 0 ; scan < MAXMON ; scan += 1)
		if ((viewlist[scan] != NULL) && (strcmp (name, viewlist[scan]->getName()) == 0))
		{
			showView (viewlist[scan]) ;
			break ;
		}
}

/*  KNDApp	:							*/
/*  rename	: Rename entry in monitor list				*/
/*  oldname	: const char *	: Old name				*/
/*  newname	: const char *	: New name				*/
/*  (returns)	: void		:					*/

void	KNDApp::rename
	(	const	char	*oldname,
		const	char	*newname
	)
{
	int	slot	;

	/* Scan the list of viewlist for the old name and remove it,	*/
	/* then insert the name name and make it current.		*/
	for (slot = 0 ; slot < monList->count() ; slot += 1)
		if (strcmp (oldname, monList->text (slot)) == 0)
		{	monList->removeItem (slot) ;
			break	;
		}

	monList->insertItem	(newname, 0) ;
	monList->setCurrentItem (0) ;
}

void	KNDApp::slotMonitorCloseWindow ()
{
	close () ;
}

/*  KNDApp								*/
/*  slotMonitorStart	: Start (all) idle monitors			*/
/*  (returns)		: void		:				*/

void	KNDApp::slotMonitorStart ()
{
	if ((keystate & ShiftButton) != 0)
	{	/* Pass the request down to all extant monitors to be	*/
		/* handled by them. Note that we have to show each in	*/
		/* turn to ensure that the controls get correctly	*/
		/* sized.						*/
		for (int slot = 0 ; slot < MAXMON ; slot += 1)
			if (viewlist[slot] != NULL)
			{	showView(viewlist[slot])     ;
				viewlist[slot]->viewStart () ;
			}
	}
	else	view->viewStart () ;
}

/*  KNDApp								*/
/*  slotMonitorStop	: Stop all active monitors				*/
/*  (returns)		: void		:				*/

void	KNDApp::slotMonitorStop ()
{
	if ((keystate & ShiftButton) != 0)
	{	/* Pass the request down to all extant monitors to be	*/
		/* handled by them.					*/
		for (int slot = 0 ; slot < MAXMON ; slot += 1)
			if (viewlist[slot] != NULL)
				viewlist[slot]->viewStop () ;
	}
	else	view->viewStop () ;
}

/*  KNDApp								*/
/*  slotMonitorFreeze	: Freeze all active monitors			*/
/*  (returns)	    	: void		:				*/

void	KNDApp::slotMonitorFreeze ()
{
	if ((keystate & ShiftButton) != 0)
	{	/* Pass the request down to all extant monitors to be	*/
		/* handled by them.					*/
		for (int slot = 0 ; slot < MAXMON ; slot += 1)
			if (viewlist[slot] != NULL)
				viewlist[slot]->viewFreeze () ;
	}
	else	view->viewFreeze () ;
}

/*  KNDApp								*/
/*  slotMonitorResume	: Resume all frozen monitors			*/
/*  (returns)	    	: void		:				*/

void	KNDApp::slotMonitorResume ()
{
	if ((keystate & ShiftButton) != 0)
	{	/* Pass the request down to all extant monitors to be	*/
		/* handled by them.					*/
		for (int slot = 0 ; slot < MAXMON ; slot += 1)
			if (viewlist[slot] != NULL)
				viewlist[slot]->viewResume () ;
	}
	else	view->viewResume () ;
}

/*  KNDApp								*/
/*  slotMonitorSave : Save current configuration			*/
/*  (returns)	    : void		:				*/

void	KNDApp::slotMonitorSave()
{
	saveOptions (true) ;
}

/*  KNDApp								*/
/*  slotMonitorSaveQuit: Save current configuration and quit		*/
/*  (returns)	       : void		:				*/

void	KNDApp::slotMonitorSaveQuit ()
{
	if (this->queryExit ()) doExit (true) ;

}

/*  KNDApp								*/
/*  slotMonitorQuit : Quit						*/
/*  (returns)	    : void		:				*/

void	KNDApp::slotMonitorQuit()
{ 
	if (this->queryExit ()) doExit (false) ;
}

/*  KNDApp								*/
/*  slotViewToolBar: Toggle tool bar					*/
/*  (returns)	   : void	:					*/

void KNDApp::slotViewToolBar()
{
	bViewToolbar = !bViewToolbar ;
	menuBar()->setItemChecked (ID_VIEW_TOOLBAR, bViewToolbar) ;
	enableToolBar (KToolBar::Toggle, 0) ;
}

/*  KNDApp								*/
/*  slotViewStatusBar: Toggle status bar				*/
/*  (returns)	     : void	:					*/

void KNDApp::slotViewStatusBar()
{
	bViewStatusbar = !bViewStatusbar ;
	menuBar()->setItemChecked (ID_VIEW_STATUSBAR, bViewStatusbar) ;
	enableStatusBar (KStatusBar::Toggle) ;
}

/*  KNDApp								*/
/*  slotViewConfig: View monitor configuration				*/
/*  (returns)	  : (void)		:				*/

void	KNDApp::slotViewConfig ()
{
	slotViewMonitors () ;
	view->tabSelect  (TAB_CONFIG) ;
}

/*  KNDApp								*/
/*  slotViewTriggers: View trigger configuration			*/
/*  (returns)	    : (void)		:				*/

void	KNDApp::slotViewTriggers ()
{
	slotViewMonitors () ;
	view->tabSelect  (TAB_TRIGGERS) ;
}

/*  KNDApp								*/
/*  slotViewEvents: View events configuration				*/
/*  (returns)	  : (void)		:				*/

void	KNDApp::slotViewEvents ()
{
	slotViewMonitors () ;
	view->tabSelect  (TAB_EVENTS) ;
}

/*  KNDApp								*/
/*  slotViewGraphic: View monitor graphic display			*/
/*  (returns)	   : (void)		:				*/

void	KNDApp::slotViewGraphic ()
{
	slotViewMonitors () ;
	view->tabSelect  (TAB_GRAPHIC) ;
}

/*  KNDApp								*/
/*  slotViewPackets: View monitor packet display			*/
/*  (returns)	   : (void)		:				*/

void	KNDApp::slotViewPackets ()
{
	slotViewMonitors () ;
	view->tabSelect  (TAB_PACKETS) ;
}

/*  KNDApp	    							*/
/*  slotViewMonitors: View monitors					*/
/*  (returns)	    : void		:				*/

void	KNDApp::slotViewMonitors ()
{
	showView (view) ;
}

/*  KNDApp	    							*/
/*  slotViewMulti   : View multiple graphs				*/
/*  (returns)	    : void		:				*/

void	KNDApp::slotViewMulti ()
{
	switch (showing)
	{
		case ShowMonitors :
			view ->hide  () ;
			view ->setOnview (false) ;
			break	;

		case ShowSetup	  :
			setup->hide  () ;
			break	;

		case ShowUser	  :
			user ->hide  () ;
			break	;

		case ShowMulti	  :
			return	;
	}

	setView		 (multi) ;
	multi->setOnview (true)  ;
	showing	= ShowMulti	 ;
}

/*  KNDApp	    							*/
/*  slotViewSetup   : View global setup					*/
/*  (returns)	    : void		:				*/

void	KNDApp::slotViewSetup ()
{
	switch (showing)
	{
		case ShowMonitors :
			view ->hide      () ;
			view ->setOnview (false) ;
			break	;

		case ShowMulti	  :
			multi->hide      () ;
			multi->setOnview (false) ;
			setView (setup) ;
			break	;

		case ShowUser	  :
			user ->hide  () ;
			break	;

		case ShowSetup	  :
			return	;
	}

	setView	(setup) ;
	showing	= ShowSetup ;
}

/*  KNDApp	    							*/
/*  slotViewUser   : View user setup					*/
/*  (returns)	   : void		:				*/

void	KNDApp::slotViewUser ()
{
	if (user == NULL) return ;

	switch (showing)
	{
		case ShowMonitors :
			view ->hide      () ;
			view ->setOnview (false) ;
			break	;

		case ShowMulti	  :
			multi->hide      () ;
			multi->setOnview (false) ;
			setView (setup) ;
			break	;

		case ShowSetup	  :
			setup->hide  () ;
			break	;

		case ShowUser	  :
			return	;
	}

	setView	(user) ;
	showing	= ShowUser ;
}

/*  KNDApp								*/
/*  showClient	: Show a specified client				*/
/*  slot	: int		: Client slot				*/
/*  (returns)	: void		:					*/

void	KNDApp::showClient
	(	int	slot
	)
{
	if (viewlist[slot] != NULL)
	{	showView        (viewlist[slot]) ;
		view->tabSelect (TAB_GRAPHIC   ) ;
	}
}

/*  KNDApp								*/
/*  Finish	: Terminate a sniffer					*/
/*  _view	: KNDView *	: Sniffer to terminate			*/
/*  (returns)	: void		:					*/

void	KNDApp::Finish
	(	KNDView	*_view
	)
{
	showView 	 (_view) ;
	slotMonitorClose ()	 ;
}

/*  KNDApp								*/
/*  commandCallback: Menu/toolbar command callback			*/
/*  id		   : int	: Command identifier			*/
/*  (returns)	   : void	:					*/

void	KNDApp::commandCallback
	(	int	id
	)
{
	if ((id >= ID_VIEW_SELECT) && (id < ID_VIEW_SELECT + MAXMON))
	{	if (viewlist[id -= ID_VIEW_SELECT] != NULL)
			showView (viewlist[id]) ;
		return	;
	}

	switch (id)
	{
		ON_CMD(ID_MONITOR_NEW,              slotMonitorNew      ())
		ON_CMD(ID_MONITOR_REPLAY,           slotMonitorReplay   ())
		ON_CMD(ID_MONITOR_REMOTE,           slotMonitorRemote   ())
		ON_CMD(ID_MONITOR_CLOSE,            slotMonitorClose    ())
		ON_CMD(ID_MONITOR_STOP,		    slotMonitorStop	())
		ON_CMD(ID_MONITOR_START,	    slotMonitorStart	())
		ON_CMD(ID_MONITOR_FREEZE,	    slotMonitorFreeze	())
		ON_CMD(ID_MONITOR_RESUME,	    slotMonitorResume	())
		ON_CMD(ID_MONITOR_SAVE,		    slotMonitorSave     ())
		ON_CMD(ID_MONITOR_SAVE_QUIT,	    slotMonitorSaveQuit ())
		ON_CMD(ID_MONITOR_QUIT,             slotMonitorQuit     ())

		ON_CMD(ID_VIEW_TOOLBAR,             slotViewToolBar     ())
		ON_CMD(ID_VIEW_STATUSBAR,           slotViewStatusBar   ())
		ON_CMD(ID_VIEW_MONITORS,	    slotViewMonitors	())
		ON_CMD(ID_VIEW_MULTI,	    	    slotViewMulti	())
		ON_CMD(ID_VIEW_SETUP,		    slotViewSetup	())
		ON_CMD(ID_VIEW_USER,		    slotViewUser	())
		ON_CMD(ID_VIEW_CONFIG,		    slotViewConfig	())
		ON_CMD(ID_VIEW_TRIGGERS,	    slotViewTriggers	())
		ON_CMD(ID_VIEW_EVENTS,		    slotViewEvents	())
		ON_CMD(ID_VIEW_GRAPHIC,		    slotViewGraphic	())
		ON_CMD(ID_VIEW_PACKETS,		    slotViewPackets	())
	}
}

void	KNDApp::keyPressEvent
	(	QKeyEvent	*e
	)
{
	if (e->key() == Key_Shift) keystate |=  ShiftButton ;
	KTMainWindow::keyPressEvent (e) ;
}

void	KNDApp::keyReleaseEvent
	(	QKeyEvent	*e
	)
{
	if (e->key() == Key_Shift) keystate &= ~ShiftButton ;
	KTMainWindow::keyReleaseEvent (e) ;
}

/*  KNDApp								*/
/*  clickFunc	: Pass button clicks through				*/
/*  (returns)	: void		:					*/

void	KNDApp::clickSet    	() { view->clickSet    () ; }
void	KNDApp::clickReset  	() { view->clickReset  () ; }
void	KNDApp::clickClear  	() { view->clickClear  () ; }
void	KNDApp::clickGo     	() { view->clickGo     () ; }
void	KNDApp::clickFreeze 	() { view->clickFreeze () ; }
void	KNDApp::clickQuery 	() { view->clickQuery  () ; }


/*  setMemUsed	: Pass memory used information to applocation object	*/
/*  u		: int		: Usage delta				*/
/*  (returns)	: void		:					*/

void	setMemUsed
	(	int	u
	)
{
	if (kndApp != NULL) kndApp->setMemUsed (u) ;
}
