/***************************************************************************
                          ksnuffle.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef		KSNUFFLE_H
#define		KSNUFFLE_H 
 

#ifdef		HAVE_CONFIG_H
#include	<config.h>
#endif

#include	<sys/time.h>
#include	<assert.h>

#include	<kapp.h>
#include	<ktmainwindow.h>
#include	<qmessagebox.h>

typedef	struct	timeval		TVAL	;
typedef	struct	ip		IP	;
typedef	struct	icmphdr		ICMP	;
typedef	struct	ether_arp	ETH_ARP	;
typedef	struct	_tcphash	TCPHASH	;

/*  KDE classes referenced in this header by pointer, but for which the	*/
/*  definition is not needed.						*/

class	KSocket		;

/*  All KSnuffle classes						*/

class	KNDPlugin	;		/* Base plugin class		*/
class	KNDPCap		;		/* Packet capture class		*/
class	KNDPCapMon	;		/* Real capture object		*/
class	KNDPCapLoc	;		/* Local packet capture		*/
class	KNDPCapRem	;		/* Remote packet capture	*/
class	KNDPCapRep	;		/* Replay packet capture	*/
class	KNDGraphic	;		/* Graphic display class	*/
class	KNDPacket	;		/* Packet display class		*/
class	KNDTrigger	;		/* Trigger configuration class	*/
class	KNDEvent	;		/* Event configuration class	*/
class	KNDConfMon	;		/* Monitor configuration class	*/
class	KNDConfRem	;		/* Remote configuration class	*/
class	KNDConfRep	;		/* Replay configuration class	*/
class	KNDConf		;		/* Base class for above		*/

class	KNDDisplay	;		/* Display splitter		*/
class	KNDButBar	;		/* Common button bar		*/

class	KNDMonitor	;		/* Network monitor class	*/
class	KNDRemote	;		/* Remote monitor class		*/
class	KNDReplay	;		/* Class to reply recording	*/
class	KNDView		;		/* Base class for above		*/

class	KNDMulti	;		/* Multiple load graph class	*/
class	KNDSetup	;		/* Global setup class		*/
class	KNDUser		;		/* Privileged user setup	*/
class	KNDDetails	;		/* Packet details		*/
class	KNDTCPData	;		/* TCP stream data		*/

class	KNDLoadHist	;		/* Network load histogram	*/
class	KNDLoadBar	;		/* Network load bar		*/
class	KNDPktHist	;		/* Packet count histogram	*/
class	KNDPktBar	;		/* Packet count bar		*/

class	Capture		;
class	NetPCap		;
class	PktInfo		;		/* Packet information		*/

typedef	QArray<char *>	SArray	;	/* Array of strings		*/

#define	STRERR	strerror(errno)
#define	MAXMON	(16)			/* Maximum number of sniffers	*/
#define	NEVENT	(6)			/* Number of available events	*/

#define	global				/* Syntactic sugar		*/
#define	cchar	const char		/* ... ditto			*/


/*  TrigInf								*/
/*  Structure used to pass trigger and filter information about between	*/
/*  objects. NOTE: This assumes that the filter and triggers will not	*/
/*  be changed while the structure is valid.				*/

struct	TrigInf
{	cchar	*filter	;		/* Main packet filter		*/
	cchar	*start	;		/* Start trigger		*/
	cchar	*stop	;		/* Stop trigger			*/
	int	mode	;		/* Trigger mode			*/
}	;

/*  EventInf								*/
/*  Structure used to pass event information about between objects.	*/
/*  NOTE: This assumes that the events will not be changed while the	*/
/*  structure is valid.							*/

struct	EventInf
{	struct
	{	cchar	*event	;	/* Event			*/
		cchar	*exec	;	/* Command to execute		*/
	}	event[NEVENT]   ;
}	;

/*  PCapInf								*/
/*  Structure used to pass packet capture information about between	*/
/*  objects								*/

struct	PCapInf
{	int	ltype	;		/* Datalink type		*/
	int	snapl	;		/* Capture snap length		*/
	ulong	nmask	;		/* Nextwork mask		*/
}	;


/*  KNDApp								*/
/*  This is the main application class, and represents the main window	*/
/*  of the application.							*/

class	KNDApp : public KTMainWindow 
{
	enum	Showing
	{	ShowMonitors,
		ShowMulti,
		ShowSetup,
		ShowUser
	}	;

 	Q_OBJECT

	KCombo		*monList	;	/* List of monitors	*/
	QLabel		*stateBox	;	/* Monitor state	*/
	QLabel		*frozenBox	;	/* Monitor frozen	*/
	QLabel		*pausedBox	;	/* Monitor paused	*/
	QLabel		*load1Box	;	/* Load 1 value		*/
	QLabel		*load2Box	;	/* Load 1 value		*/
	QLabel		*memBox		;	/* Memory usage box	*/
	QTimer		timer		;	/* Display update timer	*/
	QArray<char *>	*iflist		;	/* Interface list	*/

	KNDMulti	*multi		;
	KNDSetup	*setup		;
	KNDUser		*user		;
	KNDDisplay	*display	;
	KNDButBar	*butbar		;
	Showing		showing		;
	long		second		;
	int		keystate	;
	int		memUsed		;

	KConfig		*userconf	;
	char		*prefilter 	;

	QPopupMenu	*monitor_menu	;
	QPopupMenu	*view_menu	;
	QPopupMenu	*help_menu	;

	KNDView		*view		;
	bool		bViewToolbar	;
	bool		bViewStatusbar	;

	KMenuBar::menuPosition menu_bar_pos ;
	KToolBar::BarPosition  tool_bar_pos ;

	QArray<KNDView *> viewlist	;

	void	showView (KNDView *)	;
	void	setView	 (QWidget *)	;
	void		startNsl ()	;

	virtual	void	keyPressEvent	(QKeyEvent *) ;
	virtual	void	keyReleaseEvent	(QKeyEvent *) ;

public:

	KNDApp	() ; 
       ~KNDApp	() ;

	void	initMenuBar		() ;
	void	initToolBar		() ;
	void	initStatusBar		() ;
	void	initView		(KConfig *) ;
	void	saveOptions		(bool) ;
	void	readOptions		() ;
	void	enableCommand		(int id_) ;
	void	disableCommand		(int id_) ;
	void	doExit			(bool) ;

	void	rename			(const char *, const char *) ;
	void	showClient		(int )	;
	void	setState		(KNDView *) ;
	void	setLoad			(KNDView *, int, int, int, int) ;
	void	Finish			(KNDView *) ;

	inline	void	setMemUsed (int u) { memUsed += u ; }

	virtual	bool	queryExit	() ;
	virtual	void	closeEvent	(QCloseEvent *) ;

public slots	:

	void	commandCallback		(int id_) ;
	void	slotMonitorNew		() ;
	void	slotMonitorReplay	() ;
	void	slotMonitorRemote	() ;
	void	slotMonitorClose	() ;
	void	slotMonitorStop		() ;
	void	slotMonitorStart	() ;
	void	slotMonitorFreeze	() ;
	void	slotMonitorResume	() ;
	void	slotMonitorPick		(int) ;
	void	slotMonitorCloseWindow	() ;
	void	slotMonitorSave		() ;
	void	slotMonitorSaveQuit	() ;
	void	slotMonitorQuit		() ;
	void	slotViewToolBar		() ;
	void	slotViewStatusBar	() ;
	void	slotViewMonitors	() ;
	void	slotViewMulti		() ;
	void	slotViewSetup		() ;
	void	slotViewUser		() ;
	void	slotViewConfig		() ;
	void	slotViewTriggers	() ;
	void	slotViewEvents		() ;
	void	slotViewGraphic		() ;
	void	slotViewPackets		() ;

	void	clickSet	    	() ;
	void	clickReset	  	() ;
	void	clickClear	  	() ;
	void	clickGo	     		() ;
	void	clickFreeze	 	() ;
	void	clickQuery	 	() ;

	/* The "timerTick" method is called, nominally, once per second	*/
	/* to update displays, etc. The "upToTick" method is called	*/
	/* each time a packet is captured with the packet capture time	*/
	/* (in seconds) to ensure that we definitely keep synchronised.	*/
	/* Otherwise, a packet could appear to have been captured in	*/
	/* the future.							*/
	void	timerTick		() 	;
	long	upToTick		(long)	;

	void	sockRead (KSocket *) ;
}	;   


/*  PktInfo								*/
/*  This class is used to record decoded packet information for use	*/
/*  by the graphic and packet display objects. The EPoint structure is	*/
/*  used for IP endpoints (source and destination), and _PIHdr for the	*/
/*  libpcap-level capture information.					*/

struct	EPoint
{	char	*a		;	/* Address			*/
	char	*n		;	/* Host name			*/
	char	*p		;	/* Port				*/
	char	*s		;	/* Service			*/
}	;

enum	Direction
{	Incoming	= 0,		/* Packet is incoming		*/
	Outgoing	= 1,		/* Packet is outgoing		*/
	Passing		= 2,		/* Strangers on a train		*/
	Local		= 3,		/* Looping back			*/
	Unknown		= 4		/* We can work it out (not)	*/
}	;

struct	_PIHdr
{
	int	caplen		;	/* Captured data length		*/
	int	pktlen		;	/* Actual packet length		*/
	TVAL	tval		;	/* Packet timestamp		*/
	int	accept		;	/* Accepted packet count	*/
	int	dropped		;	/* Dropped  packet count	*/
	int	missed		;	/* Missed packet count		*/
}	;


struct	PktInfo
{
	PktInfo	  *next		;	/* Next on free queue		*/
	int	  tag		;	/* Block contents tag		*/

	_PIHdr	  piHdr		;	/* Raw header information	*/
	char	  data[1518]	;	/* Raw data buffer		*/

	bool	  decoded	;	/* Packet decoded flag		*/
	int	  datlen	;	/* Data length in packet	*/
	int	  ago		;	/* Age relative to current time	*/
	int	  usecnt	;	/* Structure use count		*/

	/* The next set of fields are set by the packet decode routines	*/
	/* and are displayed by the Packets module.			*/
	char	  time	[20]	;	/* Packet time stamp		*/
	char	  proto	[20]	;	/* Protocol			*/
	EPoint	  srce		;	/* Source			*/
	EPoint	  dest		;	/* Destination			*/
	char	  info	[96]	;	/* Miscellaneous information	*/
	char	  size	[24]	;	/* Sizes as a string		*/

	/* Next follow other fields set by the packet decode routines,	*/
	/* which are used by the packet details and TCP/IP data display	*/
	/* functions.							*/
	ETH_ARP	  *arp		;	/* ARP packet pointer		*/
	IP	  *ip		;	/* IP packet pointer		*/
	char	  *ipdata	;	/* IP data pointer		*/
	TCPHASH	  *tcphash	;	/* TCP conversation		*/
	bool	  rev		;	/* Conversation reversed flag	*/
}	;

#define	PKDATSZ	(sizeof(_PIHdr) + 1518)

/*  LogItem								*/
/*  So that binary log files can be upwardly compatible, header data	*/
/*  items are tagged by type.						*/

enum	LogItem
{
	ItemLast	= 0,		/* Marks end of headers		*/
	ItemMagic	= 1,		/* Magic marker			*/
	ItemIface	= 2,		/* Interface name		*/
	ItemProg	= 3,		/* Filter program		*/
	ItemLType	= 4,		/* Link type			*/
	ItemNMask	= 5,		/* Network mask			*/
	ItemSnapl	= 6,		/* Packet snap length		*/
	ItemStart	= 7,		/* Start trigger		*/
	ItemStop	= 8,		/* Stop trigger			*/
	ItemMode	= 9		/* Trigger mode			*/
}	;

/*  LogMode								*/
/*  File logging mode ...						*/

enum	LogMode
{	LogNone		= 0,
	LogRecordText	= 1,
	LogRecordBin	= 2
}	;

/*  MType								*/
/*  Module type								*/

enum	MType
{	Monitor,
	Remote,
	Replay
}	;


/*  GenConfig								*/
/*  General configuration structure for built-in plugins		*/

struct	GenConfig
{
	int	p_opts	;	
	int	v_load	;
	int	v_bar1	;
	int	v_bar2	;
}	;

/*  Packet display options						*/
/*									*/
#define	PO_TIME		0x0001
#define	PO_PROTO	0x0002
#define	PO_SRC		0x0004
#define	PO_DST		0x0008
#define	PO_INFO		0x0010
#define	PO_SIZE		0x0020

#define	MAGIC		"KSnuffle Magic Marker!"

/*  MAXBPS is set to the maximum number of bytes/second that we can	*/
/*  reasonably expect, and is used in a number of "assert" calls.	*/

#define	MAXBPS		(10000000)

extern	SArray	*eplist		;	/* List of ethernet protocols	*/
extern	SArray	*iplist		;	/* List of IP protocols		*/
extern	SArray	*svlist		;	/* List of service names	*/
extern	int	maxpack		;	/* Maximum number of packets	*/


/*  Extern functions .....						*/
/*									*/

typedef	void	DECODER		(cchar *, PktInfo *) ;

extern	void	ErrExit		(cchar *, ...) ;
extern	void	Error	 	(		    cchar *, cchar *, ...) ;
extern	void	Error		(QMessageBox::Icon, cchar *, cchar *, ...) ;

extern	char	*ItoA		(int) ;
extern	char	*QStrToStr	(QString    &) ;
extern	int	QCBFindItem	(QComboBox  &, const char *, int = -1) ;
extern	int	QLBFindItem	(QListBox   &, const char *, int = -1) ;

extern	DECODER	*getDecoder	(int)	    ;
extern	PktInfo	*getPktInfo	()	    ;
extern	void	freePktInfo	(PktInfo *) ;
extern	void	usePktInfo	(PktInfo *) ;
extern	void	setMemUsed	(int)	    ;

extern	SArray	*FindAllIface	() ;
extern	SArray	*FindAllEProto	() ;
extern	SArray	*FindAllIProto	() ;
extern	SArray	*FindAllService	() ;

extern	void	LoadArpCache	() ;

extern	bool	FindEPoint	(char *,  int, EPoint *) ;
extern	char	*EthToIP	(uchar *, char * = NULL) ;

extern	long	upToTick	(long) ;

extern	char	*JoinStr		(char *, ...) ;

extern	void	WrToLog		(FILE *, LogItem, cchar *) ;
extern	void	WrToLog		(FILE *, LogItem, int    ) ;

#define	StrIsSet(s)	(((s) != NULL) && ((s)[0] != 0))
#define	STRCPY(d,s)	{ strncpy ((d),(s),sizeof(d)) ; (d)[sizeof(d)-1] = 0 ; }
#define	STRCAT(d,s)	{ strncat ((d),(s),sizeof(d)) ; (d)[sizeof(d)-1] = 0 ; }
#endif // KSNUFFLE_H
