/*
 * Copyright (c) 1996, 1997
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#ifndef lint
static const char rcsid[] =
    "@(#) $Header: pcap-linux.c,v 1.15 97/10/02 22:39:37 leres Exp $ (LBL)";
#endif

#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/uio.h>

#include <net/if.h>
#ifdef HAVE_NET_IF_ARP_H
#include <net/if_arp.h>
#else
#include <linux/if_arp.h>
#endif
#include <linux/if_ether.h>
#include <linux/if_packet.h>
#ifndef SOL_PACKET
#define SOL_PACKET  263
#endif

#include <netinet/in.h>

#include <errno.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "pcap-int.h"

#include "gnuc.h"
#ifdef HAVE_OS_PROTO_H
#include "os-proto.h"
#endif

#ifndef MSG_TRUNC
#warning MSG_TRUNC is not defined: your libc is broken. Please, fix it.
#define MSG_TRUNC 0
#endif

#ifdef PACKET_TRECV
#include <linux/poll.h>
#include <linux/mman.h>

static int packet_tring_setup(pcap_t *p)
{
	int i, k, idx;
	int framesz = TPACKET_ALIGN(TPACKET_HDRLEN)+TPACKET_ALIGN(p->snapshot+16);
	int pg_size = getpagesize();
	struct iovec *ring;

	int frames_per_pg = pg_size/framesz;
	int pgs = (128*1024)/pg_size;
	int ct = pgs*frames_per_pg;

	if (ct==0)
		return -1;

	p->md.ringbuf = (void*)mmap(NULL, pgs*pg_size, PROT_READ|PROT_WRITE|PROT_EXEC,
				    MAP_ANONYMOUS|MAP_PRIVATE, -1, 0);
	if ((long)p->md.ringbuf == -1L) {
		perror("mmap");
		p->md.ringbuf=NULL;
		return -1;
	}
	if (mlock(p->md.ringbuf, pgs*pg_size, 1)) {
		perror("mlock");
		munmap(p->md.ringbuf, pgs*pg_size);
		p->md.ringbuf=NULL;
		return -1;
	}
	ring = malloc(ct*sizeof(struct iovec));

	idx=0;
	for (i=0; i<pgs; i++) {
		for (k=0; k<frames_per_pg; k++) {
			ring[idx].iov_base = p->md.ringbuf + pg_size*i + k*framesz;
			ring[idx].iov_len = framesz;
			*(unsigned long*)ring[idx].iov_base = 0;
			idx++;
		}
	}

	if (setsockopt(p->fd, SOL_PACKET, PACKET_TRECV,
		       (void*)ring, ct*sizeof(struct iovec))) {
		perror("Warning: setsockopt(PACKET_TRECV)");
		munmap(p->md.ringbuf, pgs*pg_size);
		p->md.ringbuf=NULL;
		free(ring);
		ring=NULL;
		return -1;
	}
	p->md.ringbufsize = pgs*pg_size;
	p->md.iovhead = 0;
	p->md.iovmax = ct-1;
	p->md.iovec = ring;

	return 0;
}

int
packet_ring_recv(pcap_t *p, int cnt, pcap_handler callback, u_char *user)
{
	struct iovec *ring = p->md.iovec;
	int n = 0;

	if (cnt<=0)
		cnt=0x7FFFFFFF;

	for (;;) {
		while (*(unsigned long*)ring[p->md.iovhead].iov_base) {
			struct tpacket_hdr *h = ring[p->md.iovhead].iov_base;
			struct sockaddr_ll *sll = (void*)h + TPACKET_ALIGN(sizeof(*h));
			unsigned char *bp = (unsigned char*)h + h->tp_mac;

			++p->md.stat.ps_recv;

			if (p->md.use_bpf == 2) {
				if (sll->sll_pkttype != PACKET_OUTGOING) {
					memset(bp-14, 0, 6);
					if (sll->sll_pkttype == PACKET_BROADCAST)
						memset(bp-14, 0xFF, 6);
					else if (sll->sll_pkttype == PACKET_MULTICAST) {
						*(bp-14) = 1;
					} else {
						*(bp-14+5) = 1;
					}
					memcpy(bp-14+6, sll->sll_addr, 6);
				} else {
					memcpy(bp-14, sll->sll_addr, 6);
					memset(bp-14+6, 0, 6);
				}
				*(unsigned short*)(bp-2) = sll->sll_protocol;
			}

			if (p->fcode.bf_insns == NULL ||
			    bpf_filter(p->fcode.bf_insns, bp, h->tp_len, h->tp_snaplen)) {
				struct pcap_pkthdr h1;

				h1.ifindex = (p->md.device == NULL) ? sll->sll_ifindex : 0;
				h1.protocol = sll->sll_protocol;
				h1.pkt_type = sll->sll_pkttype;
				h1.ts.tv_sec = h->tp_sec;
				h1.ts.tv_usec = h->tp_usec;
				h1.len = h->tp_len;
				h1.caplen = h->tp_snaplen;

				if (p->md.use_bpf == 2) {
					h1.caplen += 14;
					h1.len    += 14;
					bp	  -= 14;
				}

				(*callback)(user, &h1, bp);
			}

			h->tp_status = 0;
			p->md.iovhead = (p->md.iovhead == p->md.iovmax) ? 0 : p->md.iovhead+1;
			if (++n > cnt)
				return n;
		}

		while (*(unsigned long*)ring[p->md.iovhead].iov_base == 0) {
			int pres;
			struct pollfd pfd;

			pfd.fd = p->fd;
			pfd.revents = 0;
			pfd.events = POLLIN|POLLRDNORM|POLLERR;

			pres = poll(&pfd, 1, -1);
			if (pres < 0) {
				sprintf(p->errbuf, "poll: %s", pcap_strerror(errno));
				return n ? : -1;
			}
			if (pfd.events&POLLERR) {
				int err;
				int elen = sizeof(err);
				if (getsockopt(p->fd, SOL_SOCKET, SO_ERROR, &err, &elen)) {
					sprintf(p->errbuf, "getsockopt(SO_ERROR): %s", pcap_strerror(errno));
					return n ? : -1;
				}
				if (err) {
					struct ifreq ifr;
					if (err != ENODEV || p->md.device == NULL) {
						sprintf(p->errbuf, "read error: %s", pcap_strerror(err));
						return n ? : -1;
					}
					fprintf(stderr, "Device %s goes down.\n", p->md.device);

					ifr.ifr_ifindex = 0;
					strcpy(ifr.ifr_name, p->md.device);
					if (ioctl(p->fd, SIOCGIFINDEX, &ifr) < 0) {
						if (errno != ENODEV) {
							sprintf(p->errbuf, "ioctl: %s", pcap_strerror(errno));
							return -1;
						}
						fprintf(stderr, "Device %s does not exist more.\n", p->md.device);
						return n ? : -1;
					}
				}
			}
		}
	}
	return n;
}
#endif


int
pcap_stats(pcap_t *p, struct pcap_stat *ps)
{
#ifdef PACKET_TRECV
	struct tpacket_stats tps;
	int olen = sizeof(tps);
	if (getsockopt(p->fd, SOL_PACKET, PACKET_STATISTICS,
		       (void*)&tps, &olen) == 0) {
		if (tps.tp_packets) {
			p->md.stat.ps_ifdrop = tps.tp_packets - p->md.stat.ps_recv - tps.tp_drops;
			p->md.stat.ps_drop = tps.tp_drops;
		}
	}
#endif
	*ps = p->md.stat;
	return (0);
}

int
pcap_read(pcap_t *p, int cnt, pcap_handler callback, u_char *user)
{
	static int overtrunc = 0;
	register int cc;
	register int bufsize;
	register int caplen;
	register u_char *bp;
	struct sockaddr_ll from;
	int fromlen;
	int snaplen = p->snapshot;

#ifdef PACKET_TRECV
	if (p->buffer == NULL)
		return packet_ring_recv(p, cnt, callback, user);
#endif

	bp = p->buffer + p->offset;
	bufsize = p->bufsize;

	for (;;) {
		fromlen = sizeof(from);
		cc = recvfrom(p->fd, bp, snaplen, MSG_TRUNC, (struct sockaddr*)&from, &fromlen);
		if (cc >= 0)
			break;
		switch (errno) {
		case EINTR:
			continue;
		case EWOULDBLOCK:
			return (0);
		case ENODEV:	
			if (p->md.device) {
				struct ifreq ifr;
				fprintf(stderr, "Device %s goes down.\n", p->md.device);
				ifr.ifr_ifindex = 0;
				strcpy(ifr.ifr_name, p->md.device);
				if (ioctl(p->fd, SIOCGIFINDEX, &ifr) < 0) {
					if (errno != ENODEV) {
						sprintf(p->errbuf, "ioctl: %s", pcap_strerror(errno));
						return -1;
					}
					fprintf(stderr, "Device %s does not exist more.\n", p->md.device);
					return -1;
				}
				continue;
			}
		}
		sprintf(p->errbuf, "read: %s", pcap_strerror(errno));
		return (-1);
	}

	++p->md.stat.ps_recv;

	caplen = cc;

	/* Real captured length can't exceed the snapshot length;
	   if it does, it means, that MSG_TRUNC hack in kernel works.
	 */

	if (cc > snaplen) {
		caplen = snaplen;
		overtrunc = 1;
	}

	/* Emulate Ethernet-like MAC header */
	if (p->md.use_bpf == 2) {
		if (from.sll_pkttype != PACKET_OUTGOING) {
			memset(bp-14, 0, 6);
			if (from.sll_pkttype == PACKET_BROADCAST)
				memset(bp-14, 0xFF, 6);
			else if (from.sll_pkttype == PACKET_MULTICAST) {
				*(bp-14) = 1;
			} else {
				*(bp-14+5) = 1;
			}
			memcpy(bp-14+6, from.sll_addr, 6);
		} else {
			memcpy(bp-14, from.sll_addr, 6);
			memset(bp-14+6, 0, 6);
		}
		*(unsigned short*)(bp-2) = from.sll_protocol;
	}

	if (p->fcode.bf_insns == NULL ||
	    bpf_filter(p->fcode.bf_insns, bp, cc, caplen)) {
		struct pcap_pkthdr h1;
#ifdef __alpha__
		/* Yup. glibc hackers + alpha hackers == NUCLEAR MIX */
		struct {
			unsigned long s, us;
		} tts;
#endif
		h1.ifindex = (p->md.device == NULL) ? from.sll_ifindex : 0;
		h1.protocol = from.sll_protocol;
		h1.pkt_type = from.sll_pkttype;
		h1.caplen = caplen;

		if (p->md.soft_bpf || overtrunc)
			h1.len = cc;
		else
			h1.len = (cc < snaplen ? cc : p->bufsize);

		/* Get timestamp */
#ifdef __alpha__
		if (ioctl(p->fd, SIOCGSTAMP, &tts) < 0) {
			sprintf(p->errbuf, "SIOCGSTAMP: %s",
				pcap_strerror(errno));
			return (-1);
		}
		h1.ts.tv_usec = tts.us;
		h1.ts.tv_sec = tts.s;
#else
		if (ioctl(p->fd, SIOCGSTAMP, &h1.ts) < 0) {
			sprintf(p->errbuf, "SIOCGSTAMP: %s",
				pcap_strerror(errno));
			return (-1);
		}
#endif

		if (p->md.use_bpf == 2) {
			h1.caplen += 14;
			h1.len    += 14;
			bp	  -= 14;
		}

		(*callback)(user, &h1, bp);
		return 1;
	}
	return (0);
}

pcap_t *
pcap_open_live_new(char *device, int snaplen, int mlen, int to_ms,
		   char *ebuf, int raw, u_int16_t proto, u_int8_t *maddr)
{
	register int fd;
	register pcap_t *p;
	struct ifreq ifr;

	p = (pcap_t *)malloc(sizeof(*p));
	if (p == NULL) {
		sprintf(ebuf, "malloc: %s", pcap_strerror(errno));
		return (NULL);
	}
	memset(p, 0, sizeof(*p));
	fd = -1;

	fd = socket(PF_PACKET, (raw&1) ? SOCK_RAW : SOCK_DGRAM, 0);
	if (fd < 0) {
		sprintf(ebuf, "socket: %s", pcap_strerror(errno));
		goto bad;
	}
	p->fd = fd;

	ifr.ifr_ifindex = 0;
	if (device) {
		strcpy(ifr.ifr_name, device);
		if (ioctl(p->fd, SIOCGIFINDEX, &ifr) < 0) {
			sprintf(ebuf, "ioctl: %s", pcap_strerror(errno));
			goto bad;
		}

		if (mlen >= -1) {
			struct packet_mreq mr;
			memset(&mr, 0, sizeof(mr));
			mr.mr_ifindex = ifr.ifr_ifindex;
			switch (mlen) {
			case -1:
				mr.mr_type = PACKET_MR_PROMISC;
				break;
			case 0:
				mr.mr_type = PACKET_MR_ALLMULTI;
				break;
			default:
				mr.mr_type = PACKET_MR_MULTICAST;
				mr.mr_alen = mlen;
				memcpy(&mr.mr_address, maddr, mlen);
				break;
			}
			if (setsockopt(p->fd, SOL_PACKET, PACKET_ADD_MEMBERSHIP,
				       (char *)&mr, sizeof(mr)) < 0) {
				sprintf(ebuf, "setsockopt %s: %s", device, pcap_strerror(errno));
				goto bad;
			}
		}
	}

	if (raw&1) {
		if (device == NULL) {
			sprintf(ebuf, "-R assumes device");
			goto bad;
		}
		p->md.use_bpf = 1;
		memset(&ifr, 0, sizeof(ifr));
		strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
		if (ioctl(p->fd, SIOCGIFHWADDR, &ifr) < 0 ) {
			sprintf(ebuf, "SIOCGIFHWADDR: %s", pcap_strerror(errno));
			goto bad;
		}
		switch (ifr.ifr_hwaddr.sa_family) {
		case ARPHRD_ETHER:
		case ARPHRD_LOOPBACK:
		case ARPHRD_METRICOM:
			p->linktype = DLT_EN10MB;
			p->offset = 2;
			break;

		case ARPHRD_EETHER:
			p->linktype = DLT_EN3MB;
			break;

		case ARPHRD_AX25:
			p->linktype = DLT_AX25;
			break;

		case ARPHRD_PRONET:
			p->linktype = DLT_PRONET;
			break;

		case ARPHRD_CHAOS:
			p->linktype = DLT_CHAOS;
			break;

		case ARPHRD_IEEE802:
			p->linktype = DLT_IEEE802;
			break;

		case ARPHRD_ARCNET:
			p->linktype = DLT_ARCNET;
			break;

		case ARPHRD_SLIP:
		case ARPHRD_CSLIP:
		case ARPHRD_SLIP6:
		case ARPHRD_CSLIP6:
		case ARPHRD_PPP:
			p->linktype = DLT_RAW;
			break;

		case ARPHRD_FDDI:
			p->linktype = DLT_FDDI;
			break;

		default:
			fprintf(stderr, "WARNING: unsupported device type 0x%x, assuming raw\n",
				ifr.ifr_hwaddr.sa_family);
			p->linktype = DLT_RAW;
			break;
		}
	} else {
		p->linktype = DLT_EN10MB;
		p->offset = 16;
		p->md.use_bpf = 2;
	}

	p->bufsize = 8096 - p->offset;
	if (device) {
		/* Base the buffer size on the interface MTU */
		memset(&ifr, 0, sizeof(ifr));
		strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
		if (ioctl(p->fd, SIOCGIFMTU, &ifr) < 0 ) {
			sprintf(ebuf, "SIOCGIFMTU: %s", pcap_strerror(errno));
			goto bad;
		}
		/* Leave room for link header (which is never large under linux...) */
		p->bufsize = ifr.ifr_mtu + 64;
	}

	p->md.protocol = proto;
	p->snapshot = snaplen;
	if (snaplen > p->bufsize)
		p->bufsize = snaplen;
	if (device) {
		p->md.device = strdup(device);
		if (p->md.device == NULL) {
			sprintf(ebuf, "malloc: %s", pcap_strerror(errno));
			goto bad;
		}
	}


#ifdef PACKET_TRECV
	p->buffer = NULL;
	if ((raw&2) || packet_tring_setup(p)) {
#endif
		p->buffer = (u_char *)malloc(p->bufsize + p->offset);
		if (p->buffer == NULL) {
			sprintf(ebuf, "malloc: %s", pcap_strerror(errno));
			goto bad;
		}
#ifdef PACKET_TRECV
        }
#endif

	return (p);
bad:
	if (fd >= 0)
		(void)close(fd);
	if (p->buffer != NULL)
		free(p->buffer);
	if (p->md.device != NULL)
		free(p->md.device);
	free(p);
	return (NULL);
}

pcap_t *
pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
{
	return pcap_open_live_new(device, snaplen, promisc ? -1 : -2, to_ms, ebuf,
				  1, 0, NULL);
}

int
pcap_setfilter(pcap_t *p, struct bpf_program *fp)
{
	struct ifreq ifr;
	struct sockaddr_ll sll;
	int type;
	int typelen = sizeof(type);

	if (p->sf.rfile != NULL) {
		p->fcode = *fp;
		p->md.soft_bpf = 1;
		return 0;
	}

	if (p->md.use_bpf && !p->md.soft_bpf) {
		if (setsockopt(p->fd, SOL_SOCKET, SO_ATTACH_FILTER, fp, sizeof(struct bpf_program)) == 0) {
			fprintf(stderr, "Kernel filter, ");
			p->fcode.bf_insns == NULL;
			p->md.soft_bpf = 0;
			goto done;
		}
		if (errno != ENOPROTOOPT && errno != EOPNOTSUPP) {
			sprintf(p->errbuf, "SO_ATTACH_FILTER: %s(%d)", pcap_strerror(errno), errno);
			return -1;
		};
	}
	fprintf(stderr, "User level filter, ");
	p->fcode = *fp;
	p->md.soft_bpf = 1;

done:
	/* Bind only after filter is loaded on socket */
	ifr.ifr_ifindex = 0;
	if (p->md.device) {
		strcpy(ifr.ifr_name, p->md.device);
		if (ioctl(p->fd, SIOCGIFINDEX, &ifr) < 0) {
			sprintf(p->errbuf, "ioctl: %s", pcap_strerror(errno));
			return -1;
		}
	}
	memset(&sll, 0, sizeof(sll));
	sll.sll_family = AF_PACKET;
	sll.sll_ifindex = ifr.ifr_ifindex;
	sll.sll_protocol = p->md.protocol;
	if (p->md.protocol == 0 || p->md.protocol == 0xFFFF) {
		sll.sll_protocol = htons(ETH_P_ALL);
		fprintf(stderr, "protocol ALL, ");
	} else
		fprintf(stderr, "protocol %04x, ", ntohs(sll.sll_protocol));

	if (p->buffer == NULL)
		fprintf(stderr, "TURBO mode (%d frames), ", p->md.iovmax);

	if (1) {
		int type;
		int tlen = sizeof(type);
		if (getsockopt(p->fd, SOL_SOCKET, SO_TYPE, (char*)&type, &tlen))
			fprintf(stderr, "?\n");
		else
			fprintf(stderr, "%s packet socket\n", type==SOCK_DGRAM?"datagram":"raw");
	}

	if (bind(p->fd, (struct sockaddr*)&sll, sizeof(sll)) < 0) {
		sprintf(p->errbuf, "bind %s: %s", p->md.device ? : "ALL", pcap_strerror(errno));
		return -1;
	}

	return 0;
}
