/***************************************************************************
                        knd_deom.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<qtooltip.h>

#include	"ctrlset.h"
#include	"knd_demo.h"
#include	"knd_view.h"
#include	"nethdrs.h"


/*  KNDDemoConf								*/
/*  KNDDemoConf	: Constructor for packet display object			*/
/*  parent	: KNDView *	: Parent				*/
/*  config	: KCOnfig *	: Configuration				*/
/*  (returns)	: KNDDemoConf	:					*/

KNDDemoConf::KNDDemoConf
	(	KNDView	*_parent,
		KConfig	*config
	)
	:
	KNDPlugin  (_parent, "KND_DEMO_CONF"),
	v_value1   (this),
	v_value2   (this)
{
	/* Initialise all the configuration fields. By default, ctrlSet	*/
	/* hides them, so we show them before finishing.		*/
	ctrlSet.addWidget (&v_value1, 120,  30,  80,  20, Entry) ;
	ctrlSet.addWidget (&v_value2, 205,  30,  80,  20, Entry) ;

	QToolTip::add     (&v_value1, i18n("Copied to demo display")) ;
	QToolTip::add     (&v_value2, i18n("Copied to demo display")) ;

	ctrlSet.show      () ;

	s_value1	= "" ;
	s_value2	= "" ;

	/* Set pointers to the configuration values in the structuer	*/
	/* used to pass these to the display plugin.			*/
	democonf.value1 = &s_value1 ;
	democonf.value2 = &s_value2 ;
}

/*  KNDDemoConf								*/
/*  ~KNDDemoConf	: Destructor for packet display object			*/
/*  (returns)	:		:					*/

KNDDemoConf::~KNDDemoConf ()
{
}

/*  KNDDemoConf								*/
/*  clickSet	: Handle set button click				*/
/*  (returns)	: void		:					*/

void	KNDDemoConf::clickSet ()
{
	/* This method is used to pick up the values entered into the	*/
	/* configuration fields. In a real plugin we might do some	*/
	/* checks that they are valid before actually setting them.	*/
	s_value1 = v_value1.text () ;
	s_value2 = v_value2.text () ;
}

/*  KNDDemoConf								*/
/*  clickClear	: Handle clear button click				*/
/*  (returns)	: void		:					*/

void	KNDDemoConf::clickClear ()
{
	/* Clear all configuration fields ...				*/
	v_value1.setText ("") ;
	v_value2.setText ("") ;
}

/*  KNDDemoConf								*/
/*  clickReset	: Handle reset button click				*/
/*  (returns)	: void		:					*/

void	KNDDemoConf::clickReset ()
{
	/* This method is used to reset the configuration to the last	*/
	/* set state ...						*/
	v_value1.setText (s_value1) ;
	v_value2.setText (s_value2) ;
}


/*  KNDDemoConf								*/
/*  setOptions	: Set configuration options				*/
/*  lvconf	: void *	: Nominal configuration pointer		*/
/*  conf	: void *	: Actually pointer at information	*/ 

void	*KNDDemoConf::setOptions
	(	void	*lvconf
	)
{
	/* Just return a pointer at the configuration structure; this	*/
	/* already contains the configuration values.			*/
	return	(void *)&democonf ;
}


/*  KNDDemoDisp								*/
/*  KNDDemoDisp	: Constructor for packet display object			*/
/*  parent	: KNDView *	: Parent				*/
/*  config	: KCOnfig *	: Configuration				*/
/*  (returns)	: KNDDemoDisp	:					*/

KNDDemoDisp::KNDDemoDisp
	(	KNDView	*_parent,
		KConfig	*config
	)
	:
	KNDPlugin  (_parent, "KND_DEMO_DISP"),
	v_value1   (this),
	v_value2   (this),
	v_frozen   (this),
	v_pktcnt   (this)
{
	going	= false	;	/* Not at the moment ...		*/
	frozen	= false	;	/* ... nor this				*/
	onview	= false	;	/* ... not this				*/

	ctrlSet.addWidget (&v_value1, 120,  30,  80,  20,   Text) ;
	ctrlSet.addWidget (&v_value2, 205,  30,  80,  20,   Text) ;
	ctrlSet.addWidget (&v_frozen, 335,  80,  80,  20,   Text) ;
	ctrlSet.addWidget (&v_pktcnt, 450,  80,  80,  20, Number) ;

	QToolTip::add     (&v_value1, i18n("Copied from demo config")) ;
	QToolTip::add     (&v_value2, i18n("Copied from demo config")) ;

	ctrlSet.show      () ;
}

/*  KNDDemoDisp								*/
/*  ~KNDDemoDisp	: Destructor for packet display object			*/
/*  (returns)	:		:					*/

KNDDemoDisp::~KNDDemoDisp ()
{
}

/*  KNDDemoDisp								*/
/*  execute	: Set executing flag					*/
/*  _going	: bool		: Executing				*/
/*  (returns)	: void		:					*/

void	KNDDemoDisp::execute
	(	bool	_going
	)
{
	going	= _going ;	/* Not the requsted execution state	*/
	frozen	= false	 ;	/* Changes always reset frozen state	*/

	if (going)
	{	ctrlSet.show () ;
		s_pktcnt = 0	;
		v_pktcnt.setText (ItoA (s_pktcnt)) ;
	}
}

/*  KNDDemoDisp								*/
/*  addPacket	: Add a new packet to the display			*/
/*  pktinfo	: PktInfo *	: Packet information			*/
/*  (returns)	: void		:					*/

void	KNDDemoDisp::addPacket
	(	PktInfo	*pktinfo
	)
{
	/* Increment the packet count, but _only_ update the display	*/
	/* if we are on view (otherwise, there is no point) and if we	*/
	/* are now frozen (in which case the displayed value should be	*/
	/* static),							*/
	s_pktcnt += 1 ;
	if (onview && !frozen) v_pktcnt.setText (ItoA (s_pktcnt)) ;
}

/*  KNDDemoDisp								*/
/*  setOnview	: Set whether we are on view or not			*/
/*  _onview	: bool		: TRUE if on view			*/
/*  (returns)	: void		:					*/

void	KNDDemoDisp::setOnview
	(	bool	_onview
	)
{
	/* If we are now on view and are not frozen then bring the	*/
	/* packet count up to date.					*/
	if ((onview = _onview) && !frozen)
		v_pktcnt.setText (ItoA (s_pktcnt)) ;
}

/*  KNDDemoDisp								*/
/*  freeze	: Freeze or unfreeze the display			*/
/*  _frozen	: bool		: New state				*/

void	KNDDemoDisp::freeze
	(	bool	_frozen
	)
{
	/* If we are unfreezing and on view then bring the packet count	*/
	/* up to date.							*/
	if (!(frozen = _frozen) && onview)
		v_pktcnt.setText (ItoA (s_pktcnt)) ;

	/* Something else to do ....					*/
	v_frozen.setText (frozen ? "Frozen" : "Liquid") ;
}


/*  KNDDemoDisp								*/
/*  setOptions	: Set options from configuration			*/
/*  lvconf	: void *	: Actually configuration structure	*/
/*  (returns)	: void		:					*/

void	*KNDDemoDisp::setOptions
	(	void	*lvconf
	)
{
	DemoConf *dc = (DemoConf *)lvconf ;

	v_value1.setText (*dc->value1)	  ;
	v_value2.setText (*dc->value2)	  ;

	/* Actually, the return value does not manage, but in general	*/
	/* we return the argument in order that it would pass on to	*/
	/* any other plugins in this set.				*/
	return	lvconf	;
}



/*  get_plugin	: Instantiate a plugin set				*/
/*  _parent	: KNDView *	: Parent object				*/
/*  config	: KConfig *	: Configuration information		*/
/*  (returns)	: PluginSet *	: Instantiated plugin set		*/

/*  NOTE: This routine _must_ be defined as "extern C", otherwise the	*/
/*	  main executable would need to know about the name-mangling	*/
/*	  scheme used by the particular compiler.			*/

extern	"C"
{
PluginSet *getPlugin
	  (	KNDView	*_parent,
		KConfig	*config
	  )
{
	/* Allocate a structure to pass back instantiation information.	*/
	/* Thus must be malloc'ed, as this routine may be called from	*/
	/* more than one sniffer. Note that we size it to be at least	*/
	/* large enough to contain two plugins.				*/
	PluginSet *pl	= (PluginSet *)malloc(sizeof(KNDPlugin *) * 2 + 
					      sizeof(PluginSet)
					     ) ;

	/* Set the plugin cound and create each plugin (in this case,	*/
	/* one for configuration and one for display). The plugins will	*/
	/* appear on the display from left to right.			*/
	pl->nPlugins	= 2	    ;
	pl->plugin[0]	= new KNDDemoConf (_parent, config) ;
	pl->plugin[1]	= new KNDDemoDisp (_parent, config) ;

	getVersion (pl->major, pl->minor) ;
	pl->name   = "demo" ;
	return	pl ;
}
}
