/***************************************************************************
                        knd_dns.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<qtooltip.h>

#include	"ctrlset.h"
#include	"knd_dns.h"
#include	"knd_view.h"
#include	"nethdrs.h"
#include	<arpa/nameser.h>

#define	MAXPKT	512
#define	DNSHDR	HEADER

typedef	struct	servent	SERVENT	;
typedef	struct	udphdr	UDP_HDR	;


extern	int	DNSQueryLen	(u_char *) ;
extern	int	DNSResourceLen	(u_char *) ;
extern	cchar  *DNSGetQType	(int	 ) ;

extern	u_char *DNSBuildName	(QString &, u_char *, u_char *) ;
extern	uint	GetShort	(u_char *) ;
extern	ulong	GetLong		(u_char *) ;


/*  KNDDNSItem	:							*/
/*  KNDDNSItem	: Constructor for DNS request/answer display item	*/
/*  parent	: QListView *	: Parent list view display		*/
/*  last	: QListViewItem*: Last item added (ie., item to follow)	*/
/*  request	: cchar *	: Requestor name or address		*/
/*  server	: cchar *	: DNS server name or address		*/
/*  ident	: int		: DNS request identifier		*/
/*  query	: QString *	: Query string or NULL if none		*/
/*  answer	: QString *	: Answer string or NULL if none		*/
/*  qtime	: cchar *	: Query time (NULL on answer)		*/
/*  atime	: cchar *	: Answer time (NULL on query)		*/
/*  (returns)	: KNDDNSItem	:					*/

KNDDNSItem::KNDDNSItem
	(	QListView	*parent,
		QListViewItem	*last,
		cchar		*request,
		cchar		*server,
		int		ident,
		QString		*query,
		QString		*answer,
		cchar		*qtime,
		cchar		*atime
	)
	:
	QListViewItem	(parent, last),
	request		(request),
	server		(server ),
	ident		(ident  )
{
	setText	(0, request) ;
	setText	(1, server ) ;
	setText	(2, (query  == NULL) ? (cchar *)"" : (cchar *)*query ) ;
	setText (3, (answer == NULL) ? (cchar *)"" : (cchar *)*answer) ;
	setText (4, (qtime  == NULL) ? "" : qtime) ;
	setText (5, (atime  == NULL) ? "" : atime) ;
}

/*  KNDDNSItem	:							*/
/*  isReply	: Attempt to match a reply				*/
/*  _request	: cchar *	: Requestor name or address		*/
/*  _server	: cchar *	: Server name or address		*/
/*  _ident	: int		: DNS request identifier		*/
/*  answer	: QString *	: Answer text				*/
/*  atime	: cchar *	: Answer time				*/
/*  (returns)	: bool		: TRUE is answer is matched		*/

bool	KNDDNSItem::isReply
	(	cchar	*_request,
		cchar	*_server,
		int	_ident,
		QString	*answer,
		cchar	*atime
	)
{
	if ((_request == request) && (_server == server) && (_ident == ident))
	{
		setText	(3, *answer) ;
		setText	(5,  atime ) ;
		return	true	;
	}

	return	false ;
}

/*  KNDDNS								*/
/*  KNDDemoConf	: Constructor for DNS display object			*/
/*  parent	: KNDView *	: Parent				*/
/*  config	: KCOnfig *	: Configuration				*/
/*  (returns)	: KNDDNS	:					*/

KNDDNS::KNDDNS
	(	KNDView	*_parent,
		KConfig	*config
	)
	:
	KNDPlugin  (_parent, "KND_DNS"),
	dnsInfo    (this)
{
	int	w_req	= 100 ;
	int	w_srv	= 100 ;
	int	w_qry	= 150 ;
	int	w_ans	= 150 ;
	int	w_qtime	= 100 ;
	int	w_atime	= 100 ;

	SERVENT	*sv	;

	if (config != NULL)
	{
		w_req	= config->readNumEntry ("w_Requestor", 100) ;
		w_srv	= config->readNumEntry ("w_Server",    100) ;
		w_qry	= config->readNumEntry ("w_Query",     150) ;
		w_ans	= config->readNumEntry ("w_Answer",    150) ;
		w_qtime	= config->readNumEntry ("w_QueryTime", 100) ;
		w_atime	= config->readNumEntry ("w_AnswerTime",100) ;
	}


	dnsInfo.setSorting  (-1) ;
	dnsInfo.setGeometry (10, 10, size().width() - 20, size().height() - 20) ;
	dnsInfo.show        () ;

	dnsInfo.addColumn   ("Requestor",   w_req) ;
	dnsInfo.addColumn   ("Server",      w_srv) ;
	dnsInfo.addColumn   ("Query",       w_qry) ;
	dnsInfo.addColumn   ("Answer",      w_ans) ;
	dnsInfo.addColumn   ("Query Time",  w_qtime) ;
	dnsInfo.addColumn   ("Answer Time", w_atime) ;

	if ((sv = getservbyname ("domain", "udp")) == NULL)
	{
		Error	("DNS plugin cannot locate DNS (domain) service", "") ;
		domain	= -1 ;
		return	;
	}

	domain	= ntohs (sv->s_port) ;
	last	= NULL	;
	nPkt	= 0	;
}

/*  KNDDNS								*/
/*  ~KNDDNS	: Destructor for packet display object			*/
/*  (returns)	:		:					*/

KNDDNS::~KNDDNS ()
{
	clear	() ;
}

/*  KNDDNS								*/
/*  clear	: Clear up pending packets				*/
/*  (returns)	: void		:					*/

void	KNDDNS::clear ()
{
	PktInfo	*ppkt	;

	while ((ppkt = pending.getFirst ()) != NULL)
	{	freePktInfo    (ppkt) ;
		pending.remove (ppkt) ;
	}
}

/*  KNDDNS								*/
/*  resizeEvent	: Handle resize						*/
/*  e		: QResizeEvent *: Resize event information		*/
/*  (returns)	: void		:					*/

void	KNDDNS::resizeEvent
	(	QResizeEvent *e
	)
{
	dnsInfo.resize (e->size().width() - 20, e->size().height() - 40) ;
}

/*  KNDDNS								*/
/*  clickSet	: Handle set button click				*/
/*  (returns)	: void		:					*/

void	KNDDNS::clickSet ()
{
}

/*  KNDDNS								*/
/*  clickClear	: Handle clear button click				*/
/*  (returns)	: void		:					*/

void	KNDDNS::clickClear ()
{
}

/*  KNDDNS								*/
/*  clickReset	: Handle reset button click				*/
/*  (returns)	: void		:					*/

void	KNDDNS::clickReset ()
{
}


/*  KNDDNS								*/
/*  setOptions	: Set configuration options				*/
/*  lvconf	: void *	: Nominal configuration pointer		*/
/*  conf	: void *	: Actually pointer at information	*/ 

void	*KNDDNS::setOptions
	(	void	*lvconf
	)
{
	return	lvconf	;
}

/*  KNDDNS								*/
/*  execute	: Start/stop execution					*/
/*  exec	: bool		: True on execution start		*/
/*  (returns)	: void		:					*/

void	KNDDNS::execute
	(	bool	exec
	)
{
	if (exec)
	{
		clear	      () ;
		dnsInfo.clear () ;
		nPkt	= 0	 ;
		frozen	= false	 ;
	}
}

/*  KNDDNS								*/
/*  freeze	: Set or clear frozen state				*/
/*  frozen	: bool		: True to freeze			*/
/*  (returns)	: void		:					*/

void	KNDDNS::freeze
	(	bool	_frozen
	)
{
	/* If unfreezing then any pending packets are now processed	*/
	/* and released.						*/
	if (! (frozen = _frozen))
	{
		PktInfo	*ppkt	;

		while ((ppkt = pending.getFirst ()) != NULL)
		{
			putPacket      (ppkt) ;
			pending.remove (ppkt) ;
			freePktInfo    (ppkt) ;
		}
	}
}

/*  KNDDNS								*/
/*  addPacket	: Handle incoming packet				*/
/*  pktInfo	: PktInfo *	: Packet information			*/
/*  (returns)	: void		:					*/

void	KNDDNS::addPacket
	(	PktInfo	*pktinfo
	)
{
	IP	*ip	;
	UDP_HDR	*udp	;

	/* Ignore any packets which are not IP packets and then which	*/
	/* are not UDP packets.						*/
	if ((ip = pktinfo->ip) == NULL) return ;
	if (ip->ip_p    != IPPROTO_UDP) return ;

	udp	= (UDP_HDR *)(pktinfo->ipdata) ;

	/* Also skip any packets which are not heading to, or		*/
	/* originating from, the DNS server port.			*/
	if ( (ntohs (udp->uh_sport) != domain) &&
	     (ntohs (udp->uh_dport) != domain) ) return ;

	/* If the display is frozen at present the add the packet to	*/
	/* the pending list, and note that it is in use so that it is	*/
	/* not dropped.							*/
	if (frozen)
	{
		pending.append (pktinfo) ;
		usePktInfo     (pktinfo) ;
		return	;
	}

	putPacket (pktinfo) ;
}

/*  KNDDNS								*/
/*  putPacket	: Add packet to display					*/
/*  pktInfo	: PktInfo *	: Packet information			*/
/*  (returns)	: void		:					*/

void	KNDDNS::putPacket
	(	PktInfo	*pktinfo
	)
{
	UDP_HDR	*udp	;
	DNSHDR	*hdr	;
	u_char	*dptr	;

	udp	= (UDP_HDR *)(pktinfo->ipdata) ;
	hdr	= (DNSHDR  *)(pktinfo->ipdata + sizeof(UDP_HDR)) ;
	dptr	= (u_char  *)(pktinfo->ipdata + sizeof(UDP_HDR) + sizeof(DNSHDR)) ;

	if (ntohs (udp->uh_dport) == domain)
	{
		/* First case if the destiantion port is the DNS	*/
		/* service, ie., this is a request. We just extract	*/
		/* the query string, and add a new display item.	*/
		QString	query	= "" ;

		DNSBuildName (query, (u_char *)hdr, dptr) ;
		last = new KNDDNSItem (&dnsInfo, last,
						 pktinfo->srce.n,
						 pktinfo->dest.n,
						 hdr->id,
						 &query,
						 NULL,
						 pktinfo->time,
						 NULL) ;
		dnsInfo.ensureItemVisible (last) ;
		nPkt	+= 1 ;
	}
	else if (ntohs (udp->uh_sport) == domain)
	{
		/* Second case is an answer, ie., the source port is	*/
		/* the DNS service. In this case we attempt to matche	*/
		/* the answer against an earlier request, and if this	*/
		/* fails (we must have missed the request) then just	*/
		/* add in the answer on its own.			*/

		QString		answer	= ""   ;
		QListViewItem	*item	= dnsInfo.firstChild () ;
		int		type	;

		dptr	+= DNSQueryLen  (dptr) ;
		dptr	 = DNSBuildName (answer, (u_char *)hdr, dptr) ;
		type	 = GetShort	(dptr) ;
		dptr	+= 4 + 6	       ;

		switch (type)
		{
			case T_A :
				answer.sprintf ("%d.%d.%d.%d", dptr[0], dptr[1], dptr[2], dptr[3]) ;
				break	;

			case T_NS	:
			case T_PTR	:
			case T_CNAME	:
				answer	= "" ;
				DNSBuildName (answer, (u_char *)hdr, dptr) ;
				break	;

			default	:
				answer.sprintf (DNSGetQType (type), type) ;
				break	;
		}

		while (item != NULL)
		{
			if (((KNDDNSItem *)item)->isReply (pktinfo->dest.n,
							   pktinfo->srce.n,
							   hdr->id,
							   &answer,
							   pktinfo->time))
				break	;

			item	= item->nextSibling() ;
		}

		if (item == NULL)
		{
			last = new KNDDNSItem (&dnsInfo, last,
							 pktinfo->dest.n,
							 pktinfo->srce.n,
							 0xffffffff,
							 NULL,
							 &answer,
							 NULL,
							 pktinfo->time) ;
			dnsInfo.ensureItemVisible (last) ;
			nPkt	+= 1 ;
		}
	}

	while (nPkt > MAXPKT)
	{	delete	dnsInfo.firstChild () ;
		nPkt	-= 1 ;
	}
}

/*  get_plugin	: Instantiate a plugin set				*/
/*  _parent	: KNDView *	: Parent object				*/
/*  config	: KConfig *	: Configuration information		*/
/*  (returns)	: PluginSet *	: Instantiated plugin set		*/

/*  NOTE: This routine _must_ be defined as "extern C", otherwise the	*/
/*	  main executable would need to know about the name-mangling	*/
/*	  scheme used by the particular compiler.			*/

extern	"C"
{
PluginSet *getPlugin
	  (	KNDView	*_parent,
		KConfig	*config
	  )
{
	/* Allocate a structure to pass back instantiation information.	*/
	/* Thus must be malloc'ed, as this routine may be called from	*/
	/* more than one sniffer. Note that we size it to be at least	*/
	/* large enough to contain two plugins.				*/
	PluginSet *pl	= (PluginSet *)malloc(sizeof(KNDPlugin *) * 1 + 
					      sizeof(PluginSet)
					     ) ;

	/* Set the plugin count and create each plugin (in this case,	*/
	/* just one).							*/
	pl->nPlugins	= 1	    ;
	pl->plugin[0]	= new KNDDNS (_parent, config) ;

	getVersion (pl->major, pl->minor) ;
	pl->name   = "dns" ;
	return	pl ;
}
}
