/***************************************************************************
                        knd_endtoend.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include	<qtooltip.h>
#include	<qlistview.h>

#include	"ctrlset.h"
#include	"knd_endtoend.h"
#include	"knd_view.h"
#include	"knd_pcap.h"
#include	"nethdrs.h"

#include	<sys/socket.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>

#undef		_DEBUG


/*  EndToEnd								*/
/*  EndToEnd	: Constructor for end-to-end information object		*/
/*  parent	: DListView *	: Parent list view object		*/
/*  eendtoend	: KNDEndToEnd *	: End-to-end plugin			*/
/*  (returns)	: EndToEnd	:					*/

EndToEnd::EndToEnd
	(	DListView 	*parent
	)
	:
	QListViewItem (parent)
{
	order	= 0x7fff ;
}

/*  EndToEnd								*/
/*  key		: Get sort key						*/
/*  column	: int		: Column number				*/
/*  ascending	: bool		: Sort ascending flag			*/
/*  (returns)	: const char *	: Sort key text				*/

cchar	*EndToEnd::key
	(	int	column,
		bool	ascending
	)
	const
{
	static	char	v[128]	;

	if (column < 2)
		sprintf	(v, "%04x.%s", order, text (column)) ;
	else	sprintf	(v, "%04x.%09lu", order, values[column-2]) ;

	return	v	;
}

/*  EndToEnd								*/
/*  paintCell	: Paint a list view cell				*/
/*  p		: QPainter *	: Paint context				*/
/*  cg		: QColorGroup &	: Colour information			*/
/*  column	: int		: Column number				*/
/*  width	: int		: Column width				*/
/*  align	: int		: Alignment				*/
/*  (returns)	: void		:					*/

void	EndToEnd::paintCell
	(	QPainter		*p,
		const QColorGroup	&cg,
		int			column,
		int			width,
		int			align
	)
{
	/* Just set the alignment; the source and destination columns	*/
	/* are left-aligned, the others are all right-aligned.		*/
	QListViewItem::paintCell (p,
				  cg,
				  column,
				  width,
				  column >= 2 ? AlignRight : AlignLeft) ;
}


/*  DListView								*/
/*  DListView	: Constructor for extended list view			*/
/*  parent	: KNDEndToEnd *	: Parent object				*/
/*  (returns)	: DListView	:					*/

DListView::DListView
	(	KNDEndToEnd *parent
	)
	:
	QListView (parent),
	parent	  (parent)
{
	connect	(this,   SIGNAL(doubleClicked(QListViewItem *)),
		 parent, SLOT  (doubleClicked(QListViewItem *))) ;
}

/*  DListView								*/
/*  setSorting	: Called when sorting is changed			*/
/*  column	: int		: Sort column				*/
/*  ascending	: bool		: Ascending sort order			*/
/*  (returns)	: void		:					*/

void	DListView::setSorting
	(	int	column,
		bool	ascending
	)
{
	/* NAIRY CODE ALERT:						*/
	/* We get here if either (a) the user clicks a column heading	*/
	/* or (b) the user doubleclicks an item. To handle this, and to	*/
	/* prevent recursion, the parent object keeps an "insort" flag.	*/
	parent->colSorting    (column, ascending) ;
	QListView::setSorting (column, ascending) ;
}


/*  KNDEndToEnd								*/
/*  KNDEndToEnd	: Constructor for end-to-end categorisor		*/
/*  parent	: KNDView *	: Parent				*/
/*  config	: KCOnfig *	: Configuration				*/
/*  (returns)	: KNDEndToEnd	:					*/

KNDEndToEnd::KNDEndToEnd
	(	KNDView	*_parent,
		KConfig	*config
	)
	:
	KNDPlugin  (_parent, "KND_ENDTOEND")
{
	going	= false	 ;	/* Not at the moment ...		*/
	frozen	= false	 ;	/* ... nor this				*/
	onview	= false	 ;	/* ... not this				*/
	order	= 0x7fff ;
	sortcol	= 0	 ;
	insort	= false	 ;

	etoelst	= new DListView (this) ;

	etoelst->addColumn  ("Addr1",	110) ;
	etoelst->addColumn  ("Addr2",	110) ;
	etoelst->addColumn  ("Pkt ->>",  55) ;
	etoelst->addColumn  ("Pkt <<-",  55) ;
	etoelst->addColumn  ("Net ->>",  55) ;
	etoelst->addColumn  ("Net <<-",  55) ;
	etoelst->addColumn  ("Dat ->>",  55) ;
	etoelst->addColumn  ("Dat <<-",  55) ;
	memset	(hashtab, 0, sizeof(hashtab)) ;

	etoelst->setSorting (-1) ;
	etoelst->hide 	    ()	 ;
}

/*  KNDEndToEnd								*/
/*  ~KNDEndToEnd	: Destructor for packet display object			*/
/*  (returns)	:		:					*/

KNDEndToEnd::~KNDEndToEnd ()
{
	if (etoelst != NULL)
		delete etoelst ;
}

/*  KNDEndToEnd								*/
/*  execute	: Set executing flag					*/
/*  _going	: bool		: Executing				*/
/*  (returns)	: void		:					*/

void	KNDEndToEnd::execute
	(	bool	_going
	)
{
	if (_going && !going)
	{
		QRect	r = geometry () ;

		etoelst->setGeometry (10, 10, r.width() - 20, r.height() - 40) ;
		etoelst->show 	     () ;
	}

	going	= _going ;
	frozen	= false	 ;
}

/*  KNDEndToEnd								*/
/*  clickSet	: Handle set button click				*/
/*  (returns)	: void		:					*/

void	KNDEndToEnd::clickSet ()
{
	etoelst->clear () ;
	unset   .clear () ;
	memset	(hashtab, 0, sizeof(hashtab)) ;
}

/*  KNDEndToEnd								*/
/*  addPacket	: Add a new packet to the display			*/
/*  pktinfo	: PktInfo *	: Packet information			*/
/*  (returns)	: void		:					*/

void	KNDEndToEnd::addPacket
	(	PktInfo	*pktinfo
	)
{
	ulong	 orig	;
	ulong	 srce	;
	ulong	 dest	;
	ulong	 temp	;
	int	 slot	;
	int	 hval	;
	EndToEnd *etoe	;

	/* We are interested in IP packets, so extract the source and	*/
	/* destination according to type. Any other packets are ignored	*/
	if (pktinfo->ip != NULL)
	{
		srce	= *(ulong *)&((IP *)(pktinfo->ip))->ip_src ;
		dest	= *(ulong *)&((IP *)(pktinfo->ip))->ip_dst ;
	}
	else	return	;

	/* Note the address at which the packet originated. This is	*/
	/* needed since on account of the next bit ...			*/
	orig	= srce	;

	/* Get the destination and source into a canonical order so	*/
	/* that we get one hash table entry for each pair, irrespective	*/
	/* of direction.						*/
	if (dest < srce)
	{	temp	= srce	;
		srce	= dest	;
		dest	= temp	;
	}

	/* The values will display with what is now in "srce" in the	*/
	/* column labeled "Addr1", and "dest" in "Addr2". Hence, set	*/
	/* slot	such that we account for the packet in the appropriate	*/
	/* "1->2" or "1<-2" column.					*/
	slot	= orig == srce ? 0 : 1 ;


	/* Search the hash table to see if we alreay have an entry for	*/
	/* this address pair....					*/
	hval	= (srce ^ (srce >> 16) ^ dest ^ (dest >> 16)) & HASHMSK ;

	for (etoe = hashtab[hval] ; etoe != NULL ; etoe = etoe->next)
		if ((etoe->srce == srce) && (etoe->dest == dest))
			break	;


	/* No current entry so create a new one. Initialise the new	*/
	/* structure, and attempt to get the host names for each end.	*/
	/* If either is not forthcoming then the entry is also added to	*/
	/* the unset list for latter resolution.			*/
	if (etoe == NULL)
	{
		EPoint	ep	;
		bool	uns	= false ;

		etoe	= new EndToEnd (etoelst) ;

		etoe->srce	= srce	;
		etoe->dest	= dest	;
		etoe->next	= hashtab[hval] ;
		hashtab[hval]	= etoe	;

		memset	(etoe->values,   0, sizeof(etoe->values  )) ;

		if (!FindEPoint ((char *)(&etoe->srce), -1, &ep)) uns = true ;
		etoe->setText (0, ep.n) ;
		if (!FindEPoint ((char *)(&etoe->dest), -1, &ep)) uns = true ;
		etoe->setText (1, ep.n) ;

		if (uns)
		{
			unset.append (etoe) ;
#ifdef	_DEBUG
			fprintf (stderr, "add unset: %08lx %08lx: %d\n",
					 srce, dest, unset.count()) ;
#endif	/* _DEBUG */
		}
	}

	/* Add in the new packet plus the traffic and data counts into	*/
	/* the summaries; the display is only updated if we are on view	*/
	/* and not frozen (presmably we are executing, or we would not	*/
	/* have got the packet).					*/
	etoe->values[slot + 0] += 1 ;
	etoe->values[slot + 2] += pktinfo->piHdr.pktlen ;
	etoe->values[slot + 4] += pktinfo->datlen       ;

	if (onview && !frozen)
	{	etoe->setText (slot + 0 + 2, ItoA(etoe->values[slot + 0])) ;
		etoe->setText (slot + 2 + 2, ItoA(etoe->values[slot + 2])) ;
		etoe->setText (slot + 4 + 2, ItoA(etoe->values[slot + 4])) ;
	}
}

/*  KNDEndToEnd								*/
/*  redisplay	: Bring display up to date				*/
/*  (returns)	: void		:					*/

void	KNDEndToEnd::redisplay ()
{
	for (uint idx = 0 ; idx < HASHSIZ ; idx += 1)
		for (EndToEnd *etoe = hashtab[idx] ; etoe != NULL ; etoe = etoe->next)
		{
			etoe->setText (2, ItoA(etoe->values[0])) ;
			etoe->setText (3, ItoA(etoe->values[1])) ;
			etoe->setText (4, ItoA(etoe->values[2])) ;
			etoe->setText (5, ItoA(etoe->values[3])) ;
			etoe->setText (6, ItoA(etoe->values[4])) ;
			etoe->setText (7, ItoA(etoe->values[5])) ;
		}
}

/*  KNDEndToEnd								*/
/*  setOnview	: Set whether we are on view or not			*/
/*  _onview	: bool		: TRUE if on view			*/
/*  (returns)	: void		:					*/

void	KNDEndToEnd::setOnview
	(	bool	_onview
	)
{
	/* If we are now on view and are not frozen then bring the	*/
	/* display up to date.						*/
	if ((onview = _onview) && !frozen) redisplay () ;
}

/*  KNDEndToEnd								*/
/*  freeze	: Freeze or unfreeze the display			*/
/*  _frozen	: bool		: New state				*/

void	KNDEndToEnd::freeze
	(	bool	_frozen
	)
{
	/* If we are unfreezing and on view then bring the packet count	*/
	/* up to date.							*/
	if (!(frozen = _frozen) && onview) redisplay () ;
}

/*  KNDEndToEnd								*/
/*  timerTick	: Handle timer tick					*/
/*  tick	: long		: Time in seconds			*/
/*  (returns)	: void		:					*/

void	KNDEndToEnd::timerTick
	(	long	tick
	)
{
	/* Once every ten seconds, we will scan the unset list and try	*/
	/* to resolve the addresses. This should sort out situations	*/
	/* where a packet with a previously unknown address is captured	*/
	/* (at which point the address lookup will not be complete).	*/
	if (tick % 10 == 0)
	{
		QListIterator<EndToEnd>	iter(unset) ;
		EndToEnd		*etoe	    ;

		while ((etoe = iter.current()) != NULL)
		{
			bool	 uns = false ;
			EPoint	 ep  ;

#ifdef	_DEBUG
			fprintf (stderr, "try unset: %08lx %08lx\n",
					 etoe->srce,
					 etoe->dest) ;
#endif	/* _DEBUG */
			/* Check for source and destination endpoints,	*/
			/* setting he "uns" flag is one or other fails.	*/
			if (FindEPoint ((char *)(&etoe->srce), -1, &ep))
				etoe->setText (0, ep.n) ;
			else	uns  = true ;
			if (FindEPoint ((char *)(&etoe->dest), -1, &ep))
				etoe->setText (1, ep.n) ;
			else	uns  = true ;

#ifdef	_DEBUG
			fprintf (stderr, "got unset: %08lx %08lx: %d -> %d\n",
					 etoe->srce,
					 etoe->dest,
					 unset.count(),
					 uns) ;
#endif	/* _DEBUG */

			/* If we still don't have the host names then	*/
			/* iterate on down the list; if we do then this	*/
			/* entry can be removed from the list.		*/
			if (uns)
				iter += 1 ;
			else	unset.remove (etoe) ;
		}
	}

#ifdef	_DEBUG
	if (unset.count() > 0)
		fprintf (stderr, "cnt unset: %d\n", unset.count()) ;
#endif	/* _DEBUG */
}

/*  KNDEndToEnd								*/
/*  resizeEvent	: Handle resize						*/
/*  e		: QResizeEvent *: Resize event information		*/
/*  (returns)	: void		:					*/

void	KNDEndToEnd::resizeEvent
	(	QResizeEvent *e
	)
{
	if (etoelst != NULL)
		etoelst->resize (e->size().width() - 20, e->size().height() - 40) ;
}

/*  KNDEndToEnd								*/
/*  doubleClicked: Handle double click on list view item		*/
/*  lvi		 : QListViewItem *: Item in list			*/
/*  (returns)	 : void		  :					*/

void	KNDEndToEnd::doubleClicked
	(	QListViewItem	*lvi
	)
{
	/* In order to force this item to the top of the list, set its	*/
	/* ordering prefix to the next prefix down ...			*/
	((EndToEnd *)lvi)->order = (order -= 1) ;

	/* ... and then force a sort. We set the "insort" flag so that	*/
	/* the order prefixes do not get cleared when control passes	*/
	/* through the "colSorting" method. Also, the "sortcol" fudge	*/
	/* is so that we can repeatedly move entries to the top.	*/
	insort	= true  ;
	sortcol = (sortcol + 1) & 1   ;
	etoelst->setSorting (sortcol) ;
	insort	= false ;
}

/*  KNDEndToEnd								*/
/*  colSorting	: Called when sorting is about to take place		*/
/*  column	: int		: Sort column				*/
/*  ascending	: bool		: Ascending sort order			*/
/*  (returns)	: void		:					*/

void	KNDEndToEnd::colSorting
	(	int	column,
		bool	ascending
	)
{
	/* This routine is guarded by the "insort" flag, which is used	*/
	/* to prevent infinite recursion, and to flag when we are	*/
	/* inside "doubleClick".					*/
	if (!insort)
	{
		/* Control gets here if the user has clicked a column	*/
		/* header, in which case we return to normal column	*/
		/* based sorting, hence reset all the order prefixes.	*/
		for (uint idx = 0 ; idx < HASHSIZ ; idx += 1)
			for (EndToEnd *etoe = hashtab[idx] ; etoe != NULL ; etoe = etoe->next)
				etoe->order = 0x7fff ;
		order	= 0x7fff ;

		insort	= true   ;
		etoelst->setSorting (column, ascending) ;
		insort	= false  ;
	}
}

/*  get_plugin	: Instantiate a plugin set				*/
/*  _parent	: KNDView *	: Parent object				*/
/*  config	: KConfig *	: Configuration information		*/
/*  (returns)	: PluginSet *	: Instantiated plugin set		*/

/*  NOTE: This routine _must_ be defined as "extern C", otherwise the	*/
/*	  main executable would need to know about the name-mangling	*/
/*	  scheme used by the particular compiler.			*/

extern	"C"
{
PluginSet *getPlugin
	  (	KNDView	*_parent,
		KConfig	*config
	  )
{
	/* Allocate a structure to pass back instantiation information.	*/
	/* Thus must be malloc'ed, as this routine may be called from	*/
	/* more than one sniffer. Note that we size it to be at least	*/
	/* large enough to contain two plugins.				*/
	PluginSet *pl	= (PluginSet *)malloc(sizeof(KNDPlugin *) + 
					      sizeof(PluginSet)
					     ) ;

	/* Set the plugin cound and create each plugin (in this case,	*/
	/* one for configuration and one for display). The plugins will	*/
	/* appear on the display from left to right.			*/
	pl->nPlugins	= 1	    ;
	pl->plugin[0]	= new KNDEndToEnd (_parent, config) ;

	getVersion (pl->major, pl->minor) ;
	pl->name   = "endtoend" ;

	return	pl ;
}
}
