/***************************************************************************
                         knd_endtoend.h  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999,2000 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef	KND_ENDTOEND_H
#define KND_ENDTOEND_H 

#include	<qlistview.h>

#include	"ksnuffle.h"
#include	"ctrlset.h"
#include	"knd_plugin.h"


class	KNDEndToEnd	;
class	DListView	;
class	EndToEnd	;

/*  EndToEnd								*/
/*  Class used to store and display end-to-end traffic information. It	*/
/*  is derived from QListViewItem so that updates are automatically	*/
/*  directed at the list view.						*/

class	EndToEnd : public QListViewItem
{
public:

	EndToEnd	(DListView *)	;

	EndToEnd 	*next		;	/* Next on hash queue	*/
	ulong	 	srce		;	/* Source IP address	*/
	ulong	 	dest		;	/* Dest IP address	*/
	long	 	values	[6]	;	/* Network counts	*/

	int		order		;	/* Ordering value	*/

	virtual	void		paintCell (QPainter *, const QColorGroup &, int, int, int) ;
	virtual	const char 	*key	  (int, bool) const ;
}	;
#define	HASHSIZ	(1024)
#define	HASHMSK	(HASHSIZ-1)


/*  DListView								*/
/*  Class derived from QListView to allow move-to-top functionality as	*/
/*  well as normal column sorting.					*/

class	DListView : public QListView
{
	KNDEndToEnd	*parent   ;
public	:

	DListView (KNDEndToEnd *) ;

	virtual	void	setSorting (int, bool = true) ;
}	;

/*  KNDEndToEnd								*/
/*  Class to categorise packets according to their source and		*/
/*  destination addresses, and display information for each category.	*/

class	KNDEndToEnd : public KNDPlugin
{
	Q_OBJECT

	DListView	*etoelst	  ;
	EndToEnd	*hashtab[HASHSIZ] ;
	QList<EndToEnd>	unset		  ;	/* Unset names list	*/

	bool		going		  ;
	bool		frozen		  ;
	bool		onview		  ;
	int		order		  ;
	int		sortcol		  ;
	bool		insort		  ;

	void		redisplay ()	  ;

virtual	void	resizeEvent	(QResizeEvent  *) ;

public	:
	KNDEndToEnd	(KNDView *, KConfig *);
       ~KNDEndToEnd	()		;

	void	addPacket	(PktInfo *) ;
	void	setOnview	(bool)	 ;
	void	execute		(bool)	 ;
	void	freeze		(bool)	 ;
	void	timerTick	(long)	 ;
	void	clickSet	()	 ;

	void	colSorting	(int, bool) ;

	virtual	cchar	*getLabel () { return "EndToEnd" ; } ;
	virtual	bool	isConfig  () { return false	 ; } ;

public	slots	:

	void	doubleClicked	(QListViewItem *) ;
}	;


#endif	// KND_ENDTOEND_H
