/***************************************************************************
                          rsnuffle.cpp  -  description                              
                             -------------------                                         
    begin                : Thu Mar 25 14:26:46 GMT 1999
                                           
    copyright            : (C) 1999 by Mike Richardson                         
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include	<stdlib.h>
#include	<sys/time.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<wait.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<string.h>
#include	<errno.h>
#include	<signal.h>

#include	<netdb.h>
#include	<sys/socket.h>
#include	<sys/ioctl.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>

#define		__FAVOR_BSD
#include	<net/if.h>
#include	<netinet/ip.h>
#include	<netinet/if_ether.h>
#include	<netinet/tcp.h>
#include	<netinet/udp.h>
#include	<netdb.h>

#include	"ksnuffle.h"
#include	"netpcap.h"
#include	"capture.h"

typedef	struct	ifconf		IFCONF	;
typedef	struct	ifreq		IFREQ	;
typedef	struct	timeval		TVAL	;
typedef	struct	sockaddr	SOCKADR	;
typedef	struct	sockaddr_in	SOCKIN	;
typedef	struct	in_addr		INADR	;
typedef	struct	stat		STAT	;

#define	STRERR	strerror(errno)

static	FILE	*logfd		;
static	ulong	ifaddr		;
static	char	*prefilter	;

static	char	iface	[ 32]	;
static	int	snaplen = 68 	;

static	TrigInf	 triginf	;
static	EventInf eventif	;


static	int	t_count		;
static	long	t_next		;

static	char	ifname[256][64]	;
static	SOCKADR	ipaddr[256]	;
static	SOCKADR	hwaddr[256]	;
static	SOCKADR	braddr[256]	;
static	bool	ifsame[256]	;
static	int	nif		;

static	CapRemote *capture	;
static	NetPCap	  *netpcap	;
static	int	  pcapfd	= -1 ;

static	SOCKIN	peer		;

extern	void	decNetInit ()	;

/*  TxBlock	: Send message block to client				*/
/*  cmd		: int		: Message command				*/
/*  block	: RBlock &	: Message block				*/
/*  (returns)	: void		:					*/

static	void	TxBlock
	(	int	cmd,
		PktInfo	&block
	)
{
	block.tag = cmd ;
	write	(0, &block, sizeof(PktInfo)) ;

	if (logfd != NULL)
	{	if (cmd == RS_PACKET)
			fprintf	(logfd, "send [P:.....]\n") ;
		else	fprintf	(logfd, "send [%c:%s]\n", cmd, block.data) ;
		fflush	(logfd) ;
	}
}

/*  Error	: Report error to client				*/
/*  c		: int		: Command				*/
/*  w		: cchar *	: Error title				*/
/*  e		: cchar *	: Error message				*/
/*  (returns)	: void		:					*/

static	void	Error
	(	int	c,
		cchar	*w,
		cchar	*e
	)
{
	PktInfo	block ;

	sprintf	((char *)block.data, "E%s|%s", w, e) ;
	TxBlock	(c, block) ;

	if (capture != NULL)
	{	delete	capture	;
		capture	= NULL	;
	}
}

/*  Fatal	: Handle fatal error					*/
/*  w		: cchar *	: Error title				*/
/*  e		: cchar *	: Error message				*/
/*  (returns)	: void		:					*/

static	void	Fatal
	(	cchar	*w,
		cchar	*e
	)
{
	Error	(RS_FATAL, w, e) ;
	sleep	(5) ;
	exit	(1) ;
}

/*  SendAllIface: Send all inteface names to client			*/
/*  (returns)	: void		:					*/

void	SendAllIface ()
{
	PktInfo	block		;
	IFCONF	ifconf		;
	IFREQ	ifreq1		;
	IFREQ	ifreq2		;
	IFREQ	ifreq3		;
	IFREQ	ifreq4		;
	IFREQ	*ifr		;
	char	buff[BUFSIZ]	;
	char	*cp		;
	int	sockfd		;

	int	idx1		;
	int	idx2		;

	nif	= 0 ;
	memset	(ifname, 0, sizeof(ifname)) ;
	memset	(ipaddr, 0, sizeof(ipaddr)) ;
	memset	(hwaddr, 0, sizeof(hwaddr)) ;
	memset	(ifsame, 0, sizeof(ifsame)) ;

	if ((sockfd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
		Error	(RS_ERROR, "RSnuffle interface error", "can't create socket") ;

	ifconf.ifc_len	= sizeof(buff)	;
	ifconf.ifc_buf	= buff		;
	if (ioctl (sockfd, SIOCGIFCONF, (char *)&ifconf) < 0)
		Error	(RS_ERROR, "RSnuffle interface error", "SIOCGIFCONF failed") ;

	/* The loop scans all interfaces, rejecting any that are not	*/
	/* up. For each that is OK, add entries to the interface arrays	*/
	for (nif = 0, cp = buff ; cp < buff + ifconf.ifc_len ;
		      cp += sizeof(ifr->ifr_name) + sizeof(ifr->ifr_addr))
	{
		ifr	= (IFREQ *)cp	;
		ifreq1	= *ifr		;
		ifreq2	= *ifr		;
		ifreq3	= *ifr		;
		ifreq4	= *ifr		;

		if (ioctl (sockfd, SIOCGIFFLAGS, (char *)&ifreq1)  < 0)
		{	Error	(RS_ERROR, "RSnuffle interface error", "SIOCGIFFLAGS failed") ;
			continue;
		}

		if ((ifreq1.ifr_flags & IFF_UP) == 0)
			continue ;

		/* Set and save the IP and hardware (MAC) addresses of	*/
		/* the interface, if possible.				*/
		if (ioctl (sockfd, SIOCGIFADDR,    (char *)&ifreq2) >= 0)
			ipaddr[nif] = ifreq2.ifr_addr      ;
		if (ioctl (sockfd, SIOCGIFHWADDR,  (char *)&ifreq3) >= 0)
			hwaddr[nif] = ifreq3.ifr_hwaddr    ;
		if (ioctl (sockfd, SIOCGIFBRDADDR, (char *)&ifreq4) >= 0)
			braddr[nif] = ifreq4.ifr_broadaddr ;

		strcpy	(ifname[nif], ifr->ifr_name) ;

		if (logfd != 0)
		{
			fprintf	(logfd, "[%8s] ",
					ifname[nif]) ;
			fprintf	(logfd, "[%08x] ",
					((SOCKIN *)&ipaddr[nif])->sin_addr.s_addr) ;
			fprintf	(logfd, "[%02x:%02x:%02x:%02x:%02x:%02x]\n",
					hwaddr[nif].sa_data[0] & 0xff,
					hwaddr[nif].sa_data[1] & 0xff,
					hwaddr[nif].sa_data[2] & 0xff,
					hwaddr[nif].sa_data[3] & 0xff,
					hwaddr[nif].sa_data[4] & 0xff,
					hwaddr[nif].sa_data[5] & 0xff) ;
			fflush	(logfd) ;
		}

		/* Increment the interface count. Check for the sake of	*/
		/* paranoia, but will any machine have this many?	*/
		if ((nif += 1) >= 256) break ;
	}

	/* Scan the interfaces to find any with the same IP address as	*/
	/* that for the socket to the client. For any such, mark all	*/
	/* interfaces that have the same MAC address.			*/
	for (idx1 = 0 ; idx1 < nif ; idx1 += 1)
		if (((SOCKIN *)&ipaddr[idx1])->sin_addr.s_addr == ifaddr)
			for (idx2 = 0 ; idx2 < nif ; idx2 += 1)
				if (memcmp (hwaddr[idx1].sa_data,
				    	    hwaddr[idx2].sa_data, 6) == 0)
					ifsame[idx2] = true ;

	/* Pass the interface list to the client. Those that are on the	*/
	/* same as that which connects us to the client are send marked	*/
	/* with a minus as a warning. Others are marked with a plus.	*/
	for (idx1 = 0 ; idx1 < nif ; idx1 += 1)
	{
		block.data[0] = ifsame[idx1] ? '-' : '+' ;
		strcpy	((char *)&block.data[1], ifname[idx1])   ;
		TxBlock	(RS_IFACE, block) ;
	}

	/* Mark the end with a null name ...				*/
	block.data[0] = 0  ;
	TxBlock	(RS_IFACE, block) ;

	close	(sockfd)   ;
}

/*  SetCapture	: Set up for specified packet capture			*/
/*  (returns)	: void		:					*/

static	void	SetCapture ()
{
	int	idx	;
	PktInfo	block  	;
	long	brd	;

	capture	= new CapRemote (0) ;

	if (!capture->Build (iface, snaplen, &triginf, &eventif, prefilter))
	{	Error	(RS_SETERR, "RSnuffle setup error", capture->getError()) ;
		delete	capture	;
		capture	= NULL	;
	}

	for (idx = 0, brd = 0 ; idx < nif ; idx += 1)
		if (strcmp (iface, ifname[idx]) == 0)
		{	brd	= *(long *)&braddr[nif].sa_data[2] ;
			break	;
		}

	/* All seems OK, so signal the success back to the client. The	*/
	/* link type is specified in the data.				*/
	sprintf	((char *)block.data,
				"%d|%d|%u|%lu",
				capture->getLType(),
				snaplen,
				capture->getNMask(), brd) ;
	TxBlock	(RS_SET, block) ;
}

/*  RunCapture	: Run packet capture					*/
/*  (returns)	: void		:					*/

static	void	RunCapture ()
{
	if (!capture->Execute ())
	{	Error	(RS_GOERR, "RSnuffle execute error", capture->getError()) ;
		delete	capture	;
		capture	= NULL	;
	}

	netpcap	= capture->getNetPCap () ;
	pcapfd	= netpcap->getPCap    () -> fd ;
	fprintf	(stderr, "pcapfd is %d\n", pcapfd) ;	
	fcntl	(pcapfd, F_SETFL, O_NONBLOCK) ;

	/* If we successfully start a child process then send a reply	*/
	/* back to the client.						*/
	PktInfo	block	;
	strcpy	((char *)block.data, "") ;
	TxBlock	(RS_GO,  block) ;
}

/*  ClrCapture	: Clear packet capture					*/
/*  (returns)	: void		:					*/

static	void	ClrCapture ()
{
	if (capture != NULL)
	{	delete	capture	;
		capture	= NULL	;
	}

	netpcap	= NULL	;
	pcapfd	= -1	;
}

/*  sigpipe	: Pipe signal catcher					*/
/*  sig		: int		: Signal				*/
/*  (returns)	: void		:					*/

static	void	sigpipe
	(	int	sig
	)
{
	/* Simply catch this signal so that we can exit cleanly to	*/
	/* flush any logging output.					*/
	exit (0) ;
}

/*  NextBlock	: Get next command block from client			*/
/*  block	: PktData *	: Block buffer				*/
/*  (returns)	: void		:					*/

static	bool	NextBlock
	(	PktInfo	&block
	)
{
	TVAL	tval	;
	fd_set	inset	;
	long	tnow	;
	int	got	;

	static	char	recbuff	[sizeof(PktInfo)] ;
	static	uint	recsize	;
	
	for (;;)
	{
		/* See if we have reached the time for the next timer	*/
		/* message in which case send on. The time is updated	*/
		/* such that we send an initial burst at two second	*/
		/* intervals, then send every ten seconds.		*/
		if ((tnow = time (NULL)) >= t_next)	
		{
			/* We want the time as (seconds-from-epoch,	*/
			/* microseconds. The loop handles the situation	*/
			/* where the second increments inconveniently.	*/
			do
			{	tnow	    = time  (NULL) ;
				gettimeofday (&tval, NULL) ;
			}
			while	(tnow != time (NULL)) ;

			sprintf	((char *)block.data, "%ld:%ld", tnow, tval.tv_usec) ;
			TxBlock	(RS_TIME, block)   ;

			if (t_count < 10)
			{	t_count	+= 1  ;
				t_next	+= 2  ;
			}
			else	t_next	+= 10 ;
		}

		/* Now wait for either a command from the client or the	*/
		/* expiry of the timer period ...			*/
		FD_ZERO	(&inset)    ;
		FD_SET	(0, &inset) ;
		if (pcapfd >= 0) FD_SET (pcapfd, &inset) ;

		tval.tv_usec = 0    ;
		tval.tv_sec  = t_next - time (NULL)  ;
		if (tval.tv_sec < 0) tval.tv_sec = 0 ;

		/* The only error from select that we will allow is the	*/
		/* call being interrupted, which happens when the child	*/
		/* process terminates,					*/
		if (select (pcapfd >= 0 ? pcapfd + 1 : 1, &inset, NULL, NULL, &tval) < 0)
			if (errno != EINTR)
			{	Error	(RS_ERROR, "RSnuffle select error", STRERR) ;
				sleep	(2) ;
				exit	(1) ;
			}
			else	continue ;

		/* If there is a command then read it and exit the loop	*/
		/* Any error is fatal here as we should only read when	*/
		/* either there is a command or the client has closed	*/
		/* the connection.					*/
		if (FD_ISSET (0, &inset))
		{
			if ((got = read (0, &recbuff[recsize], sizeof(PktInfo) - recsize)) <= 0)
			{	fprintf	(stderr, "read: %d -> %d/%s\n", sizeof(block), got, STRERR) ;
				Fatal	("RSnuffle read error", STRERR) ;
			}

			if ((recsize += got) >= sizeof(PktInfo))
			{	recsize	= 0 ;
				memcpy	(&block, recbuff, sizeof(PktInfo)) ;
				break	;
			}
		}


		if ((pcapfd >= 0) && FD_ISSET (pcapfd, &inset))
			netpcap->sockRead (pcapfd) ;
	}

	return	true ;
}

/*  CheckPass	: Check password					*/
/*  pass	: char *	: Putative password			*/
/*  (returns)	: void		: Never returns after error		*/

void	CheckPass
	(	char	*pass
	)
{
	STAT	statb	  ;
	FILE	*fd	  ;
	char	buff[128] ;
	char	*lp1	  ;
	char	*lp2	  ;
	INADR	addr	  ;
	ulong	mask	  ;
	char	*RSE	  = "RSnuffle password error" ;

	/* Check that the configuration file exists, is owned by root,	*/
	/* and that only root can read or write it. If either of the	*/
	/* latter are now true then security could be compromised.	*/
	if (stat ("/etc/rsnuffle.conf", &statb) != 0)
		Fatal	(RSE, "Cannot stat configuration file") ;
	if (statb.st_uid != 0)
		Fatal	(RSE, "Configuration file not owned by root") ;
	if ((statb.st_mode & (S_IRWXG|S_IRWXO)) != 0)
		Fatal	(RSE, "Configuration file world read/writable") ;

	/* Scan the file. Each line should have the format		*/
	/*								*/
	/*	ipaddr/mask:password					*/
	/* or	ipaddr:password						*/
	/*								*/
	/* ipaddr	IP address in dot notation			*/
	/* mask		Network mask, eg., /24 -> 255.255.255.0		*/
	/* password	Plaintext password				*/
	/*								*/
	if ((fd = fopen ("/etc/rsnuffle.conf", "r")) == 0)
		Fatal	(RSE, "Cannot open configuration file") ;

	while (fgets (buff, sizeof(buff), fd) != NULL)
	{
		if ((lp1 = strchr (buff, '\n')) != NULL) *lp1 = 0 ;
		if ((lp1 = strchr (buff, ':' )) == NULL) continue ;
		*lp1++ = 0 ;

		if ((lp2 = strchr (buff, '/' )) != NULL)
		{	*lp2++	= 0 ;
			mask	= htonl(0xffffffff << (32 - atoi(lp2))) ;
		}
		else	mask	= 0xffffffff ;

		if (!inet_aton (buff, &addr))
			continue ;

		if (logfd != NULL)
			fprintf	(logfd, "%08x/%08lx %08x\n",
					addr.s_addr,
					mask,
					peer.sin_addr.s_addr) ;

		/* See if the callers address matches the specified IP	*/
		/* address subject to the mask. If not then skip the	*/
		/* entry.						*/
		if ((addr.s_addr & mask) != (peer.sin_addr.s_addr & mask))
			continue ;

		/* Check the password. If this mismatches then fail	*/
		/* the check.						*/
		if (strcmp (pass, lp1) != 0)
			break	;

		/* Success ....						*/
		fclose	(fd) ;
		return	;
	}

	Fatal	(RSE, "No password entry or bad password") ;
}

/*  setMemUsed	: Dummy routine required for linking			*/
/*  u		: int		:					*/
/*  (returns)	: void		:					*/

void	setMemUsed
	(	int	u
	)
{
}


/*  main	: Standard entry routine				*/
/*  argc	: int		: Number of arguments			*/
/*  argv	: char *[]	: Vector of arguments			*/
/*  (returns)	: int		: Exit code				*/

int	main
	(	int	argc,
		char	*argv[]
	)
{
	PktInfo	block	;
	SOCKIN	sockadr	;
	uint	l	;
	bool	passOK	= false	;


	signal	(SIGPIPE, sigpipe) ;
	t_next	= time (NULL) + 2  ;

	if ((logfd = fopen ("/tmp/rsnuffle", "w")) != NULL)
		dup2 (fileno (logfd), 2) ;
	

	decNetInit () ;

	/* Get the socket name for the socket that the client call	*/
	/* arrived on, and make a note of the associated IP address.	*/
	/* Also get the callers address for use when password checking.	*/
	l = sizeof(SOCKADR) ;
	if (getsockname (0, (SOCKADR *)&sockadr, &l) >= 0)
	{	ifaddr	= *(long *)&sockadr.sin_addr.s_addr ;
		sprintf	((char *)block.data, "%08lx", ifaddr) ;
	}
	else	Fatal	("RSnuffle getsockname error", STRERR) ;

	l = sizeof(SOCKADR) ;
	if (getpeername (0, (SOCKADR *)&peer,    &l)  < 0)
		Fatal	("RSnuffle getpeername error", STRERR) ;

	/* Started up OK so send the initial hello back to the caller.	*/
	block.data[0] = 0	  ;
	TxBlock	(RS_HELLO, block) ;

	/* The remote snuffler runs as a loop which receives requests	*/
	/* from the KSnuffle client, and responds appropriately.	*/
	while (NextBlock (block))
	{
		if (logfd != NULL)
		{	fprintf	(logfd, "got  [%c:%s]\n",
					block.tag,
					(char *)block.data) ;
			fflush	(logfd) ;
		}

		switch (block.tag)
		{
			case RS_QUERY	:
				/* First, check the password if this is	*/
				/* if first call.			*/
				if (!passOK)
				{	CheckPass ((char *)block.data) ;
					passOK	  = true ;
				}

				/* Send all interface names to the	*/
				/* client. The client needs to know	*/
				/* these before it can specify an	*/
				/* interface on which to shiff.		*/
				SendAllIface () ;
				break	;

			case RS_IFACE	:
				/* Received by rsnuffle, this sets the	*/
				/* interface to be used.		*/
				strcpy	(iface,   (char *)block.data) ;
				break	;


			case RS_FILTER	:
				/* Note the main packet filter to be	*/
				/* applied at this end.			*/
				triginf.filter  = strdup ((char *)block.data) ;
				break	;

			case RS_BEGIN	:
				/* Note the start event filter to be	*/
				/* applied at this end.			*/
				triginf.start   = strdup ((char *)block.data) ;
				break	;

			case RS_END	:
				/* Note the stop event filter to be	*/
				/* applied at this end.			*/
				triginf.stop    = strdup ((char *)block.data) ;
				break	;

			case RS_MODE	:
				/* Note the trigger mode.		*/
				triginf.mode = atoi ((char *)block.data) ;
				break	;

			case RS_SNAP	:
				/* Specify the capture snap length to	*/
				/* passed to the capture code.		*/
				snaplen	= atoi ((char *)block.data) ;
				break	;

			case RS_PREFILTER :
				/* Set the user packet pre-filter.	*/
				prefilter = strdup ((char *)block.data) ;
				break	;

			case RS_SET	:
				/* The client sends this once it has	*/
				/* specified the filter and so forth.	*/
				/* At this stage we should set up the	*/
				/* filter.				*/
				ClrCapture () ;
				SetCapture () ;
				break	;

			case RS_GO	:
				/* Start the packet capture running ...	*/
				RunCapture () ;
				break	;

			case RS_STOP	:
				/* ... and stop it			*/
				ClrCapture () ;
				break	;

			default	:
				/* Silently ignore anything else ....	*/
				break	;
		}

		if (logfd != NULL) fflush (logfd) ;
	}
}
